var require = require;

require( "src/Game/Base/GameSettings.js" );
var GameSettings = GameSettings;

/**
 * The Brain Shift settings. Adds convenience functions for determining if things exist in the vowel and consonant lists.
 * @class
 * @name BrainShiftSettings
 */
var BrainShiftOverdriveSettings = GameSettings.extend(
{

    /** @lends BrainShiftOverdriveSettings.prototype */
    
    /**
     * Returns whether or not the even numbers in the settings contain the passed number.
     *
     * @param {string} number The number to check against (as a string).
     * @returns {boolean} Whether or not the passed number is even.
     */
    doEvensContain: function( number )
    {
        var i;
        for ( i = 0; i < this.evenCharacterArray.length; i++ )
        {
            if ( this.evenCharacterArray[ i ] === number )
            {
                return true;
            }
        }
        return false;
    },

    /**
     * Returns whether or not the vowels in the settings contain the passed letter.
     *
     * @param {string} letter The letter to check against.
     * @returns {boolean} Whether or not the passed letter is a vowel.
     */
    doVowelsContain: function( letter )
    {
        var i;
        for ( i = 0; i < this.vowelCharacterArray.length; i++ )
        {
            if ( this.vowelCharacterArray[ i ] === letter )
            {
                return true;
            }
        }
        return false;
    },

     /**
     * Returns whether or not the odd numbers in the settings contain the passed number.
     *
     * @param {string} number The number to check against (as a string).
     * @returns {boolean} Whether or not the passed number is even.
     */
    doOddsContain: function( number )
    {
        return this.doEvensContain(number) ? false : true;
    },

    /**
     * Returns whether or not the consonants in the settings contain the passed letter.
     *
     * @param {string} letter The letter to check against.
     * @returns {boolean} Whether or not the passed letter is a vowel.
     */
    doConsonantsContain: function( letter )
    {
        return this.doVowelsContain(letter) ? false : true;
    }
});