/*global cc*/
/*global require*/
/*global GameView*/
/*global GameWidgetFeedback*/
/*global GameWidgetFlipbook*/
/*global GameWidgetHint*/
/*global LayoutLoader*/
/*global lu*/

require( "src/Game/Base/GameView.js" );
require( "src/Game/Widget/GameWidgetFeedback.js" );
require( "src/Game/Widget/GameWidgetFlipbook.js" );
require( "src/Game/Widget/GameWidgetHint.js" );
require( "src/Animation/LayoutLoader.js" );

var BrainShiftOverdriveView = GameView.extend(
{
    _blueTextColor: new cc.c3b( 48, 191, 235 ),
    _buttonFont: "fonts/MuseoSans-500.ttf",
    _buttonTextSize: 48,
    _buttonTextDownColor: cc.WHITE,
    
    _hintsAreShown: false,

    ctor:function( screenSize, settings, model, interactionResponder, preload )
    {
        this._super( screenSize, settings, model, interactionResponder, preload );
        
        //TODO - move this into a HUD widget
        this.hud.setScore( 0 );
    },
    
    //****************TODO MOVE TO CARD WIDGET****************//
    cue_cardBounceIn:function()
    {
        this.flipbooks[ "cardBounceIn" ].play( this["top left card"] );
        this.flipbooks[ "cardBounceIn" ].play( this["bottom left card"] );
        this.flipbooks[ "cardBounceIn" ].play( this["top right card"] );
        this.flipbooks[ "cardBounceIn" ].play( this["bottom right card"] );
    },

    cue_cardBounceOut:function()
    {
        this.flipbooks[ "cardBounceIn" ].play( this["top left card"], true );
        this.flipbooks[ "cardBounceIn" ].play( this["bottom left card"], true );
        this.flipbooks[ "cardBounceIn" ].play( this["top right card"], true );
        this.flipbooks[ "cardBounceIn" ].play( this["bottom right card"], true );

    },

    //****************TODO MOVE TO CARD WIDGET****************//
    enableButtons:function( isEnabled )
    {
        this.no.setEnabled(isEnabled);
        this.yes.setEnabled(isEnabled);
    },

    /**
     * Updates the stimuli based on a passsed in value. Either the top or the bottom is shown,
     * and the other is given an emptystring.
     *
     * @param {string} value The value to show.
     * @param {boolean} isTop If true, we show the value in the top, and hide the bottom. If false, the opposite.
     */
    updateStimuli:function( number, letter, numberThenLetter, trialType )
    {
        var stringValue = numberThenLetter ? localize( number ) + localize( letter ) : localize( letter ) + localize( number );

        this[ "top left stimuli" ].setString( trialType==this.model.TRIAL_TYPE_EVEN ? stringValue : "" );
        this[ "bottom left stimuli" ].setString( trialType==this.model.TRIAL_TYPE_ODD ? stringValue : "" );
        this[ "top right stimuli" ].setString( trialType==this.model.TRIAL_TYPE_VOWEL ? stringValue : "" );
        this[ "bottom right stimuli" ].setString( trialType==this.model.TRIAL_TYPE_CONSONANT ? stringValue : "" );
        
    },
    
    //@Override
    preload:function()
    {
        this._super();
        this.flipbooks[ "cardBounceIn" ] = new GameWidgetFlipbook( "cardBounceIn", "card/brainShift_cardAnimIn{0}.png", 11 );
        this.flipbooks[ "resetCards" ] = new GameWidgetFlipbook( "resetCards", "card/brainShift_cardAnimIn{0}.png", 1 );
    },


    onRestart: function()
    {

        this._super();

        this.flipbooks[ "resetCards" ].play( this["top left card"] );
        this.flipbooks[ "resetCards" ].play( this["bottom left card"] );
        this.flipbooks[ "resetCards" ].play( this["top right card"] );
        this.flipbooks[ "resetCards" ].play( this["bottom right card"] );

    },


    //@Override
    _createViews:function()
    {
        this["top left hint"].setString( localize( "TopLeftHint" ) );
        this["top right hint"].setString( localize( "TopRightHint" ) );
        this["bottom left hint"].setString( localize( "BottomLeftHint" ) );
        this["bottom right hint"].setString( localize("BottomRightHint") );
    },

    setHintVisibility:function ( visible )
    {
        this["top left hint"].setVisible( visible );
        this["top right hint"].setVisible( visible );
        this["bottom left hint"].setVisible( visible );
        this["bottom right hint"].setVisible( visible );
    },

    showHints:function ( animated )
    {

        if (this._hintsAreShown == false) 
        {
            this.animationController.play( "showHint");
            this._hintsAreShown = true;
        }
    },

    hideHints:function ( animated )
    {

        if (this._hintsAreShown == true) {
            this.animationController.play( "hideHint");
            this._hintsAreShown = false;
        }
    },

    //@Override
    layout_readingData: function( nodeName, nodeData )
    {
        var node = this._super( nodeName, nodeData );
        if ( node )
        {
            return node;
        }

        switch( nodeName )
        {
            case "hintTextColor":
                this.hintColor = nodeData.textData.color;
                this.hintFont = lu.getFontLocation( nodeData.textData.face );
                return false;
        }

    },
    
    //@Override
    layout_creatingNode: function( nodeName, nodeData )
    {

        var node = this._super( nodeName, nodeData );
        if ( node )
        {
            return node;
        }
        
        switch( nodeName )
        {
            case "top left card":
            case "top right card":
            case "bottom left card":
            case "bottom right card":
                return this._createSprite( "card/brainShift_cardAnimIn000.png" );
            case "yes":
                return this._createButton( "buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "yes", localize ( "Yes_Label" ),
                                            this._buttonFont, this._buttonTextSize, this._blueTextColor, this._buttonTextDownColor,
                                            nodeData[ LayoutLoader.sizeKey ] );
            case "no":
                return this._createButton( "buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "no",  localize ( "No_Label" ), 
                                            this._buttonFont, this._buttonTextSize, this._blueTextColor, this._buttonTextDownColor,
                                            nodeData[ LayoutLoader.sizeKey ] );
            case "feedback":
                return this.getFeedbackWidget();
            case "hudBackgroundColor":
                this.hud.setBackgroundColor( nodeData.textData.color );
                return false;
            case "hudTextColor":
                this.hud.setTextColor( nodeData.textData.color );
                return false;
        }
        
        return null;
    }
});