var require = require;

require( "src/Game/Base/GameModel.js" );
var GameModel = GameModel;

var BrainShiftModel = GameModel.extend(
{
    /** @lends BrainShiftModel.prototype */

    /**
     * The Brain Shift model. Mostly adds stuff dealing with top and bottom stimuli, vowels, and consonants.
     * @constructs
     */
    ctor:function()
    {

    },

    isNewMulitplier: false,

    startTime: 0,

    consecutiveCorrectNumberCount: 0,
    consecutiveCorrectLetterCount: 0,

    /* True means correct, false means incorrect. */
    trialResults: [],

    /* Metadata Trial Object */
    currentTrial:
    {
        correct: null,      // Was the trial correctly answered? (true/false)
        multiplier: 0,      // Multiplier applied to the current score increment
        points: 0,          // Number of points awarded
        response: null,     // Was the button input Yes or No?  (true/false)
        response_time: 0,   // Response time
        time_offset: 0,     // At start of trial, number of milliseconds since beginning of game
        response_type: "",  // Subtask type (even/vowel)
        stimulus: "",       // Letter and number displayed
        switch: null        // Was the response type different from the previous trial? (true/false)        
    },

    isUsingTopStimulus: true,
    numberThenLetter: null,
    topValue: "0x",
    bottomValue: "0x",
    topNumber: "0",
    bottomNumber: "0",
    topLetter: "x",
    bottomLetter: "x",

    previousType: "first",    

    setMetadataSwitch: function( response_type )
    {
        if (this.previousType === "first")
        {
            this.currentTrial.switch = false;
            this.previousType = response_type;
            return;
        }

        if (response_type === this.previousType)
        {
            this.currentTrial.switch = false;
        }
        else
        {
            this.currentTrial.switch = true;
        }

        this.previousType = response_type;
    },

    addTrialResult: function( correct )
    {
        this.trialResults.push( correct );
    },

    shouldShowHints: function()
    {
        if (this.trialResults.length > 2)
        {
            return (this.trialResults[this.trialResults.length - 1] === false
                    || this.trialResults[this.trialResults.length - 2] === false);
        }
        return true;
    },

    exportResults:function( stats, bestStatKey )
    {
        var stats = {};
        stats[ this.StatsKeys.CardStat ] = Math.max( this.totalCorrectResponseCount
                                                     - ( this.totalResponseCount
                                                     - this.totalCorrectResponseCount ), 0) ; 
        return this._super( stats, this.StatsKeys.CardStat );
    },

    setTopStimulus: function( fullValue, number, letter, numberThenLetter )
    {
        this.isUsingTopStimulus = true;

        this.topValue = fullValue;
        this.topNumber = number;
        this.topLetter = letter;

        this.bottomValue = " ";
        this.bottomNumber = " ";
        this.bottomLetter = " ";

        this.currentTrial.stimulus = numberThenLetter ? (localize( number ) + localize( letter )) : (localize( letter ) + localize( number ));
    },

    setBottomStimulus: function( fullValue, number, letter, numberThenLetter )
    {
        this.isUsingTopStimulus = false;

        this.bottomValue = fullValue;
        this.bottomNumber = number;
        this.bottomLetter = letter;

        this.topValue = " ";
        this.topNumber = " ";
        this.topLetter = " ";

        this.currentTrial.stimulus = numberThenLetter ? (localize( number ) + localize( letter )) : (localize( letter ) + localize( number ));
    }


});