/*global require*/
/*global cc*/
/*global LumosityButton*/
/*global LumosityUtils*/
/*global DisillusionBoardPiece*/
/*global DisillusionPieceOrientation*/



var DisillusionBoardPieceButton;
(function()
{
    "use strict";

    require( "src/View/LumosityButton.js" );
    require( "src/Utility/LumosityUtils.js" );
    require( "src/Game/DisillusionBoardPiece.js" );

    DisillusionBoardPieceButton = LumosityButton.extend(
    {
        /** @lends DisillusionBoardPieceView.prototype */

        pieceModel: null,

        _downSprite: null,
        _upSprite: null,
        _disabledSprite: null,

        _highlightSprite: null,
        _isHighlighted: false,

        _isSlidIn: false,

        _onPieceTappedManager: null,

        designSize: cc.size ( 140, 140 ),

        /**
         * DisillusionBoardPieceView is the view class for a button at spot.
         * @param    onPieceTappedManager  Expects has function signature:  onPieceTapped(button).
         *
         * @constructs
         */
        ctor: function( settings, onPieceTappedManager )
        {
            this._super();

            this._onPieceTappedManager = onPieceTappedManager;

            this.nodeName = "DisillusionBoardPiece";

            this.setAnchorPoint( cc.p( 0.5, 0.5 ) );

            // Down Sprite
            this._downSprite = cc.Sprite.createWithSpriteFrameName( "tapAreaHorizontalAnimation005.png" );
            this._downSprite.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            this.setBackgroundImageForState( this._downSprite, LumosityButtonState.ButtonStateNormal );

            // Up Sprite
            this._upSprite = cc.Sprite.createWithSpriteFrameName( "tapAreaHorizontalAnimation005.png" );
            this._upSprite.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            this.setBackgroundImageForState( this._upSprite, LumosityButtonState.ButtonStateSelected );

            // Disabled Sprite
            this._disabledSprite = cc.Sprite.createWithSpriteFrameName( "tapAreaHorizontalAnimation005.png" );
            this._disabledSprite.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            this.setBackgroundImageForState( this._disabledSprite, LumosityButtonState.ButtonStateDisabled );

            this.setTarget( this );
            //this.setPressedCallback( this._onPieceTapped );
            this.setReleasedCallback( this._onPieceTapped );

            this.setContentSize( cc.size( 140, 140 ) );
            this.scaleForContentSize( this._downSprite, this.designSize );
            this.scaleForContentSize( this._upSprite, this.designSize );
            this.scaleForContentSize( this._disabledSprite, this.designSize );
        },

        _setupImages: function( image )
        {
            var spriteFrame = cc.SpriteFrameCache.getInstance().getSpriteFrame( image );
            this._downSprite.setDisplayFrame( spriteFrame );
            this._upSprite.setDisplayFrame( spriteFrame );
            this._disabledSprite.setDisplayFrame( spriteFrame );
        },

        setupHorizontal: function()
        {
            this._setupImages( "tapAreaHorizontalAnimation005.png" );
        },

        setupVertical: function()
        {
            this._setupImages( "tapAreaVerticalAnimation005.png" );
        },

        setOrientation: function( orientation )
        {
            if ( orientation === DisillusionPieceOrientation.Horizontal )
            {
                this.setupHorizontal();
            }
            else if ( orientation === DisillusionPieceOrientation.Vertical )
            {
                this.setupVertical();
            }
        },

        _onPieceTapped: function()
        {
            if ( this._onPieceTappedManager && this._onPieceTappedManager.onPieceTapped )
            {
                this._onPieceTappedManager.onPieceTapped( this );
            }
        },

        _getSlideInFrames: function( isVertical )
        {
            var frames = [];

            var frameName = "";
            var frameCount = 6;
            var cache = cc.SpriteFrameCache.getInstance();

            var orientationString = "Horizontal";
            if ( isVertical )
            {
                orientationString = "Vertical";
            }

            var index;
            for( index = 0; index < frameCount; index += 1 )
            {
                frameName = "tapArea" + orientationString + "Animation00" + index.toString() + ".png";
                frames.push( cache.getSpriteFrame( frameName ) );
            }

            return frames;
        },

        _onSlideInComplete: function()
        {
            this.setEnabled( true );
        },

        /**
         * GAME-1581 Immediately enable spot button before slide in.  2014-09-18 Last tile.  Tap spot once.  Kelsey expects tile placed (+Mark Scoptur).  2014-09-30 Wouter expects tap area animation finishes quicker.
         */
        slideIn: function()
        {
            this.setEnabled( true );
            this.setVisible( true );

            var isVertical = false;
            if ( this.pieceModel && this.pieceModel.orientation === DisillusionPieceOrientation.Vertical )
            {
                isVertical = true;
            }

            this._isSlidIn = true;

            this._disabledSprite.stopAllActions();

            var frames = this._getSlideInFrames( isVertical );

            var animation = cc.Animation.create( frames, 0.0 );
            var animationAction = cc.Animate.create( animation );
            var callbackAction = cc.CallFunc.create( this._onSlideInComplete, this );
            var finalSequence = cc.Sequence.create( animationAction, callbackAction );

            this._disabledSprite.runAction( finalSequence );
        },

        _onSlideOutComplete: function()
        {
            this.setVisible( false );
        },

        slideOut: function( instant )
        {
            this.setEnabled( false );

            if ( instant )
            {
                this._onSlideOutComplete();
            }
            else
            {
                var isVertical = false;
                if ( this.pieceModel && this.pieceModel.orientation === DisillusionPieceOrientation.Vertical )
                {
                    isVertical = true;
                }

                this._isSlidIn = false;

                this._disabledSprite.stopAllActions();

                var frames = this._getSlideInFrames( isVertical );
                frames.reverse();

                var animation = cc.Animation.create( frames, 0.0 );
                var animationAction = cc.Animate.create( animation );
                var callbackAction = cc.CallFunc.create( this._onSlideOutComplete, this );
                var finalSequence = cc.Sequence.create( animationAction, callbackAction );

                this._disabledSprite.runAction( finalSequence );
            }
        },

        isSlidIn: function()
        {
            return this._isSlidIn;
        },


        highlight: function( enable )
        {
            if ( this._isHighlighted !== enable )
            {
                this._isHighlighted = enable;
                        
                if ( enable )
                {
                    if ( !this._highlightSprite )
                    {
                        var position = cc.p( this.getContentSize().width / 2, this.getContentSize().height / 2 );

                        this._highlightSprite = cc.Sprite.createWithSpriteFrameName( "pieceTapAreaHint.png" );
                        this._highlightSprite.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                        this._highlightSprite.setPosition( position );
                        this.addChild( this._highlightSprite, 20 );

                        this.scaleForContentSize( this._highlightSprite, this.designSize );
                    }

                    if ( this._highlightSprite )
                    {
                        // Rotate
                        if ( this.pieceModel )
                        {
                            if ( this.pieceModel.orientation === DisillusionPieceOrientation.Horizontal )
                            {
                                this._highlightSprite.setRotation( 90 );
                            }
                            else if ( this.pieceModel.orientation === DisillusionPieceOrientation.Vertical )
                            {
                                this._highlightSprite.setRotation( 0 );
                            }
                        }

                        this._highlightSprite.setOpacity( 0 );
                        this._highlightSprite.setVisible( true );

                        var fadeAction = cc.FadeTo.create( 0.2, 255 );
                        this._highlightSprite.runAction( fadeAction );
                    }
                }
                else
                {
                    if ( this._highlightSprite )
                    {
                        this._highlightSprite.setVisible( false );
                    }
                }
            }
        },

        scaleForContentSize: function( node, size )
        {
            var currentSize = node.getContentSize();
            var currentScale = cc.p( 1, 1 );

            if ( currentSize.x !== 0 && currentSize.y !== 0 )
            {
                node.setScaleX( currentScale.x * ( size.width / currentSize.width ) );
                node.setScaleY( currentScale.y * ( size.height / currentSize.height ) );
            }
        }
    });
}());
