/*global require*/
/*global cc*/
/*global lu*/
/*global localize*/
/*global LumosityButton*/
/*global LumosityLabel*/
/*global LumositySprite*/
/*global LumositySwipeView*/
/*global GameWidgetPageButtons*/


var DisillusionIllusionView;
(function()
{
    "use strict";

    require( "src/Utility/LumosityUtils.js" );
    require( "src/View/LumosityButton.js" );
    require( "src/View/LumosityLabel.js" );
    require( "src/View/LumositySprite.js" );
    require( "src/View/LumositySwipeView.js" );
    require( "src/Game/GameWidgetPageButtons.js" );

    DisillusionIllusionView = LumositySwipeView.extend(
    {
        /** @lends DisillusionBoardPieceView.prototype */

        descriptionTextFont: "fonts/MuseoSans-500.ttf",
        descriptionTextFontSize: 32,
        /**
         * GAME-1722 [~wouter], to fit Japanese illusion description that has 4-lines, I moved illusions up by 40 / 960 (1/24th) of author screen size.
         */
        offset: 40,
                // 60,
                // 80,
                // 120,
        descriptionTextFontColor: cc.c4b( 255, 255, 255, 255 ),
        descriptionWidth: 450,

        headerTextFont: "fonts/MuseoSans-700.ttf",
        headerTextFontSize: 50,
        headerTextFontColor: cc.c4b( 255, 255, 255, 255 ),
        imageSize:  cc.size( 450, 450 ),

        /**
         * GAME-1602 Set illusion description width to 75% of screen width.  iPad Retina.  Expect width about the same as in iPhone version.
         * Button in position, so that when the button is added to the view, its hit area is in the right position.
         *  @constructs
         */
        ctor: function()
        {
            this._leftScreen = 0.0;
            this._rightScreen = 640.0;
            this._slideButtonTime = 1.0 / 3.75;
            this._slideButtonWidth = 40;
            this.descriptionPosition = cc.p( 320, 208 + this.offset );
            this.headerPosition = cc.p( 320, 823 + this.offset );
            this.imagePosition = cc.p( 320, 535 + this.offset );
            this.screenSize = cc.Director.getInstance().getWinSize();
            // ll.log("DisillusionIllusionView: screenSize width " + this.screenSize.width + " height " + this.screenSize.height);
            this.descriptionWidth = 480 / 640 * this.screenSize.width;
            this._super();
            this.nodeName = "IllusionView";
            this.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            if ( this._dotsContainer )
            {
                this._dotsContainer.setVisible( false );
            }
            GameWidgetPageButtons.prototype.create( this, true );
            this._leftButton.setEnabled(false);
            this._leftButton.setVisible(false);
            this._rightButton.setEnabled(false);
            this._rightButton.setVisible(false);
            this._leftButtonVisible = false;
            this._rightButtonVisible = false;
            this.showing = false;
        },

        /**
         * GAME-1405 Nest grandparent of illusion swipe view and the view itself to animate independently.  This avoids a race between animation export that would set position when setting visible on grandparent and grandchild.  Test case:  2014-11-06 Finish.  Expect illusions scroll into view.  Sometimes illusions would not scroll.  
         */
        grandparent: function(illusionView)
        {
            var parent = cc.Node.create();
            parent.setPosition(320, 480);
            parent.addChild(illusionView);
            var grandpa = cc.Node.create();
            grandpa.addChild(parent);
            return grandpa;
        },

        /**
         * Lazy load pages to avoid model being replaced when going to HowToPlay and restarting game.
         * Steps:  Play.  How To Play.  Play.  Level Up.  Expect page at highest milestone.
         *
         * @param   {int}   milestoneUnlocked   Milestone starts at 1 and equals (page + 1).  Unlocked show image.
         * @param   {int}   milestoneMax        Number of pages to create.
         * @param   {object}  model             Object that contains function "milestoneToLevelText"; otherwise function's "this" gets stomped.  
         *                      Expects function returns level to unlock this milestone, or mystery "(?)".
         * @return  Bookmark, which is also set to this._bookmark, to show page of highest milestone unlocked.
         *
         * GAME-1402
         */
        loadPages: function(milestoneUnlocked, milestoneMax, model)
        {
            var i;
            var newPage;
            var pageContentSize = cc.size( 640, 960 );
            var requiredScore = 0;
            var lastSlide = false;
            var headerText;
            var headerLabel;
            var imageFile;
            var imageSprite;
            var descriptionText;
            var descriptionLabel;
            var milestoneMax = milestoneMax;
            var milestoneUnlocked = milestoneUnlocked;
            for ( var milestone = 1; milestone <= milestoneMax; milestone++ )
            {
                newPage = cc.Node.create();
                newPage.setContentSize( pageContentSize );
                newPage.setAnchorPoint( cc.p( 0.5, 0.5 ) );

                if ( milestone <= milestoneUnlocked )
                {
                    var digits = lu.padWithLeadingZeroes( milestone, 3 );
                    var imageTemplate = "illusions/illusion{0}.png";
                    imageFile = imageTemplate.format(digits);

                    var headerKey = "IllusionHeader{0}".format(digits);
                    headerText = localize(headerKey);

                    var descriptionKey = "IllusionDescription{0}".format(digits);
                    descriptionText = localize(descriptionKey);
                }
                else
                {
                    imageFile = "illusions/illusionLocked.png";
                    headerText = "";
                                 //- "Locked";
                    var levelMinText = model.milestoneToLevelText(milestone);
                    descriptionText = localize("LevelUnlockEnd").format(levelMinText);
                    //- lastSlide = true;
                }

                if ( imageFile )
                {
                    imageSprite = new LumositySprite ( imageFile );
                    imageSprite.setContentSize( this.imageSize, true );
                    imageSprite.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                    imageSprite.setPosition( this.imagePosition );
                    newPage.addChild( imageSprite );
                }

                if ( headerText )
                {
                    headerLabel = LumosityLabel.create( headerText, this.headerTextFont, this.headerTextFontSize );
                    headerLabel.setColor( this.headerTextFontColor );
                    headerLabel.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                    headerLabel.setPosition( this.headerPosition );
                    headerLabel.setHorizontalAlignment( cc.TEXT_ALIGNMENT_CENTER );
                    headerLabel.setVerticalAlignment( cc.VERTICAL_TEXT_ALIGNMENT_CENTER );
                    newPage.addChild( headerLabel );
                }

                if ( descriptionText )
                {
                    descriptionLabel = LumosityLabel.create( descriptionText, this.descriptionTextFont, this.descriptionTextFontSize );
                    descriptionLabel.setColor( this.descriptionTextFontColor );
                    descriptionLabel.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                    descriptionLabel.setPosition( this.descriptionPosition );
                    descriptionLabel.setHorizontalAlignment( cc.TEXT_ALIGNMENT_CENTER );
                    descriptionLabel.setVerticalAlignment( cc.VERTICAL_TEXT_ALIGNMENT_CENTER );
                    descriptionLabel.setDimensions( cc.size( this.descriptionWidth, 0 ) );
                    newPage.addChild( descriptionLabel );
                }

                this.addPage( newPage );

                if ( lastSlide )
                {
                    break;
                }
            }
            this._bookmark = milestoneUnlocked - 1;
            return this._bookmark;
        },

        /**
         * Show or hide left or right buttons, like in GameWidgetHowToPlay.
         *
         * @param   {int}       pageNumber              Newly visible index in range [0..pages.length-1].
         * @param   {boolean}   instant                 Do not animate.
         * @param   {boolean}   forceSlideButtons       Force different from previous visibility by overriding previous visibility with -1.
         * GAME-1405 Force different from previous visibility by overriding previous visibility with -1.  Test case: 2014-11-10 Short game.  Answer once or more.  See illusions.  Expect left button slides in smoothly.  Got pops into appearance.
         */
        setPage: function( pageNumber, instant, forceSlideButtons )
        {
            this._leftButtonVisible = this.showLeftButton();
            this._rightButtonVisible = this.showRightButton();
            if (forceSlideButtons)
            {
                this._leftButtonVisible = -1;
                this._rightButtonVisible = -1;
            }
            this._super( pageNumber, instant );
            this.setPageButtons();
        },

        /**
         * GAME-1405 Even if not changed state, enable buttons.  At first, force sliding buttons and set.  Test case:  2014-10-31 Short game.  Answer almost at 0.  Only at first illusion.  Expect not to see scroll to the right.
         */
        setPageButtons: function()
        {
            var showLeftNow = this.showLeftButton();
            if ( this._leftButtonVisible != showLeftNow )
            { 
                this.slideLeftButton( showLeftNow );
            }
            this._leftButton.setEnabled( showLeftNow );
            var showRightNow = this.showRightButton();
            if ( this._rightButtonVisible != showRightNow )
            { 
                this.slideRightButton( showRightNow );
            }
            this._rightButton.setEnabled( showRightNow );
        },

        /**
         * GAME-1405 Workaround:  Animate illusion swipe view on and off screen that program plays along with "illusionBegin" and "illusionEnd".  [~torri]: How can I animate illusions sliding offscreen?  They appear in Flash timeline and keyframes are exported, but at the end there is a jump.
         * @param   inToView    If false, slide out.  Ease back.
         */
        show: function( inToView )
        {
            this.showing = inToView;
            /*
            ll.verbose("DisillusionIllusionView.show: " + inToView + " " 
                + DisillusionView.prototype.represent(this._leftButton.worldToNodeTransform())
                + DisillusionView.prototype.represent(this._rightButton.worldToNodeTransform()));
             */
            this.slideButtons( inToView );
            var easeCreate;
            var x;
            if (inToView) 
            {
                this._leftButton.setVisible( this._leftButtonVisible );
                this._rightButton.setVisible( this._rightButtonVisible );
                this.setPositionX(640.0);
                x = 0.0;
                easeCreate = cc.EaseBackOut.create;
            }
            else
            {
                x = -640.0;
                easeCreate = cc.EaseBackIn.create;
            }
            this.slide( this, inToView, x, easeCreate );
        },

        slideButtons: function( inToView )
        {
            if ( inToView )
            {
                this.setPage( this._bookmark, true, true );
            }
            else
            {
                if ( this._leftButtonVisible )
                {
                    this.slideLeftButton( inToView );
                }
                if ( this._rightButtonVisible )
                {
                    this.slideRightButton( inToView );
                }
            }
        },

        /**
         * @param   {cc.Node}   node        The node will change visibility and move.
         * @param   {boolean}   inToView    Set visible before moving or after moving.
         * @param   {int}       x           Horizontal destination.
         * @param   {cc.easing} easeCreate  Function with signature easeCreate(moveAction);
         */
        slide: function( node, inToView, x, easeCreate )
        {
            var moveAction = cc.MoveTo.create(this._slideButtonTime, cc.p(x, node.getPositionY()) );
            if (easeCreate) {
                easeCreate(moveAction);
            }
            if ( inToView ) {
                node.setVisible( inToView );
                node.runAction( moveAction );
            }
            else {
                var hideFunc = function () {
                    node.setVisible( inToView );
                }
                var hideAction = cc.CallFunc.create( hideFunc, this );
                var sequence = cc.Sequence.create( moveAction, hideAction );
                node.runAction( sequence );
            }
        },

        slideLeftButton: function( inToView )
        {
            this._leftButtonVisible = inToView;
            this._leftButton.setEnabled( inToView );
            var x = this._leftScreen + (inToView ? 0.0 : -this._slideButtonWidth);
            var xOld = this._leftScreen + (inToView ? -this._slideButtonWidth : 0.0);
            this._leftButton.setPositionX(xOld);
            this.slide( this._leftButton, inToView, x );
        },

        slideRightButton: function( inToView )
        {
            this._rightButtonVisible = inToView;
            this._rightButton.setEnabled( inToView );
            var x = this._rightScreen + (inToView ? 0.0 : this._slideButtonWidth);
            var xOld = this._rightScreen + (inToView ? this._slideButtonWidth : 0.0);
            this._rightButton.setPositionX(xOld);
            this.slide( this._rightButton, inToView, x );
        }
    });
}());
