var require = require;
var cc = cc;

require( "src/Game/Base/GameSettings.js" );
var GameSettings = GameSettings;

require( "src/Utility/LumosityUtils.js" );
var lu = LumosityUtils;

require( "src/Game/DisillusionBoardPiece.js" );
var DisillusionPieceType = DisillusionPieceType;

/**
 * GAME-1400 Speed up tile fade duration from 0.5 to 0.1.  2014-07-02 Mark Scoptur expects to show next piece immediately and tap immediately.  (2014-07-03 +Samantha Lieberman)
 *
 * @class
 * @name DisillusionSettings
 */
var DisillusionSettings = GameSettings.extend(
{
    /** @lends DisillusionSettings.prototype */

    /**
     * Returns image name for the specified type index. (Current 3 is max)
     *
     * @param pieceType
     * @returns {string}
     */
    getBoardPieceImage: function( pieceType )
    {
        var image = "pieceSymbol1.png";

        if ( pieceType === DisillusionPieceType.Type1 )
        {
            image = "pieceSymbol1.png";
        }
        else if ( pieceType === DisillusionPieceType.Type2 )
        {
            image = "pieceSymbol2.png";
        }
        else if ( pieceType === DisillusionPieceType.Type3 )
        {
            image = "pieceSymbol3.png";
        }

        return image;
    },


    /**
     * Returns name of the dark version of the image for the specified type index. (Current 3 is max)
     *
     * @param pieceType
     * @returns {string}
     */
    getDarkBoardPieceImage: function( pieceType )
    {
        var image = "pieceDark1.png";

        if ( pieceType === DisillusionPieceType.Type1 )
        {
            image = "pieceDark1.png";
        }
        else if ( pieceType === DisillusionPieceType.Type2 )
        {
            image = "pieceDark2.png";
        }
        else if ( pieceType === DisillusionPieceType.Type3 )
        {
            image = "pieceDark3.png";
        }

        return image;
    },


    /**
     * Returns actual color for the specified color index. (Currently 3 is max)
     *
     * @param colorIndex
     * @returns {*}
     */
    getBoardPieceColor: function( colorIndex )
    {
        var color;

        if ( colorIndex === 0 )
        {
            color = cc.c3b( 255, 255, 255 );
        }
        else if ( colorIndex === 1 )
        {
            color = cc.c3b( 233, 97, 81 );
        }
        else if ( colorIndex === 2 )
        {
            color = cc.c3b( 85, 199, 193 );
        }
        else if ( colorIndex === 3 )
        {
            color = cc.c3b( 253, 184, 18 );
        }
        else
        {
            color = cc.c3b( 125, 125, 125 );
        }

        return color;
    },

    /**
     * Returns settings for the specified difficulty index (which is the same as the level currently).
     *
     * @param difficultyIndex
     * @returns {*}
     */
    getDifficultySetting: function( difficultyIndex )
    {
        var retval;

        if ( this.difficultyProgressionSettings && this.difficultyProgressionSettings.length > 0 && this.difficultyProgressionSettings.length > difficultyIndex )
        {
            retval = this.difficultyProgressionSettings[ difficultyIndex ];
        }

        return retval;
    }
});
