/*global Tutor*/


var require = require;

require( "src/Game/Tutor.js" );

/**
 * GAME-1403 Replace text "CLICK" with "TAP".  2014-10-08 Shelby expects to read "TAP".  Got "CLICK".
 */
var DisillusionTutor = cc.Class.extend(
{
    /** @lends DisillusionTutor.prototype */
    clear: null,
    color: null,
    correctMin: null,
    colorCount: null,
    colorWrong: null,
    shared: null,
    roundIndex: null,
    symbol: null,
    symbolCount: null,
    symbolWrong: null,

    ctor: function()
    {
        this.shared = new Tutor();
        this.shared.getState = this;
        this.init();
    },

    first: function()
    {
        return this.shared.active && this.colorCount <= 0 && this.symbolCount <= 0;
    },

    init: function()
    {
        this.clear = false;
        this.color = false;
        this.colorCount = 0;
        this.correctMin = 1;
        this.colorWrong = false;
        this.symbol = false;
        this.symbolCount = 0;
        this.symbolWrong = false;
        this.roundIndex = 0;
        this.updating = true;
        this.shared.updating = true;
        this.shared.intro = false;
        this.shared.outro = false;
    },

    /**
     * WG-1198 Count color and symbol correct.  14/7/17 Pol may expect to tutor color and symbol twice.
     * WG-1272 Chris Hewitt expects not to tutor "allSides".
     */
    correct: function(vertical)
    {
        this.shared.intro = false;
        if (vertical) {
            this.colorCount++;
        }
        else {
            this.symbolCount++;
        }
        this.colorWrong = false;
        this.symbolWrong = false;
        this.updating = false;
        /*
        ll.verbose("DisillusionTutor.correct " + " this.colorCount " + this.colorCount 
            + " this.symbolCount " + this.symbolCount);
         */
    },

    wrong: function(vertical)
    {
        this.shared.intro = false;
        this.colorWrong = vertical;
        this.symbolWrong = !vertical;
        this.shared.outro = false;
        this.shared.active = true;
        this.updating = false;
    },

    vertical: function(vertical)
    {
        this.color = vertical;
        this.symbol = !vertical;
    },

    /**
     * @return  state string which can refer to a label in a timeline.
     * WG-1196 States  "introColor", "introSymbol", "outro", color[1-3], symbol[1-3], "colorWrong", "symbolWrong".

     */
    getState: function(state)
    {
        if (!this.updating) {
        }
        else if (this.clear) {
            state = "clear";
        }
        else if (this.colorWrong) {
            state = "colorWrong";
        }
        else if (this.symbolWrong) {
            state = "symbolWrong";
        }
        else if (this.color) {
            if (this.first()) {
                state = "intro";
            }
            else {
                state = "color" + this._clamp(this.colorCount);
            }
        }
        else if (this.symbol) {
            if (this.first()) {
                state = "intro";
            }
            else {
                state = "symbol" + this._clamp(this.symbolCount);
            }
        }
        return state;
    },

    /**
     * WG-1272 Clamp symbol or color correct count between [1, 3].  2014-08-01 After intro, Chris Hewitt expects 3 unique states of tutoring color or symbol.
     * WG-1272 Clamp symbol or color correct count between [1, 4].  2014-08-07 After intro, Chris Hewitt expects 4 unique states of tutoring color or symbol.
     */
    _clamp: function(value)
    {
        return Math.max(this.correctMin, Math.min(4, value));
    }
});
