/*global cc*/
/*global ll*/
/*global GameView*/
/*global require*/
/*global GameWidgetHud*/
/*global GameWidgetFeedback*/
/*global GameWidgetPopup*/
/*global GameWidgetLevelCard*/
/*global LumosityLabel*/
/*global LumositySprite*/
/*global LumosityButton*/
/*global LumosityUtils*/
/*global AnimationChanges*/
/*global AnimationController*/
/*global LayoutLoader*/
/*global DisillusionModel*/
/*global DisillusionSettings*/
/*global InteractionType*/
/*global DisillusionRoundModel*/
/*global DisillusionBoard*/
/*global DisillusionBoardPiece*/
/*global DisillusionPieceOrientation*/
/*global DisillusionBoardPieceView*/
/*global DisillusionBoardPieceButton*/
/*global DisillusionIllusionView*/


var DisillusionView;
(function()
{
    "use strict";

    require( "src/Utility/LumosityLogger.js" );
    require( "src/Utility/LumosityUtils.js" );
    require( "src/Game/Base/GameView.js" );
    require( "src/Game/Widget/GameWidgetHud.js" );
    require( "src/Game/Widget/GameWidgetFeedback.js" );
    require( "src/Game/Widget/GameWidgetPopup.js" );
    require( "src/Game/Widget/GameWidgetLevelCard.js" );
    require( "src/View/LumosityLabel.js" );
    require( "src/View/LumositySprite.js" );
    require( "src/View/LumosityButton.js" );
    require( "src/Animation/AnimationController.js" );
    require( "src/Animation/LayoutLoader.js" );
    require( "src/Game/DisillusionModel.js" );
    require( "src/Game/DisillusionSettings.js" );
    require( "src/Game/DisillusionRoundModel.js" );
    require( "src/Game/DisillusionBoard.js" );
    require( "src/Game/DisillusionBoardPiece.js" );
    require( "src/Game/DisillusionBoardPieceView.js" );
    require( "src/Game/DisillusionBoardPieceButton.js" );
    require( "src/Game/DisillusionIllusionView.js" );

    DisillusionView = GameView.extend(
    {
        _boardNodes: [],

        _boardView: null,
        _boardBackgroundView: null,

        // Default settings
        _pieceSize: cc.size( 88, 88 ),

        illusionSwipeView: null,

        /**
         * Load the spritesheet.  Extend and override some images.
         */
        ctor: function( screenSize, settings, model, interactionResponder, preload )
        {
            this._pieceSize = cc.size( settings.pieceSize, settings.pieceSize );

            DisillusionBoardPieceView.prototype.addSpriteFrames();
            this.illusionSwipeView = new DisillusionIllusionView();
            this._super( screenSize, settings, model, interactionResponder, preload );
        },

        //@Override
        _createViews: function()
        {
            // ll.verbose( "Disillusion", "createView" );
        },

        ancestry: function(child, propertySuffix)
        {
            var messages = [];
            while (child) {
                messages.unshift(child.nodeName + ": " + child["get" + propertySuffix]());
                child = child.getParent();
            }
            messages.unshift(propertySuffix);
            return messages.join(", ");
        },

        _debugIncrementScore: function()
        {
            this.model.score += this.model.scorePerTrial;
            this.setScore( this.model.score );
        },

        //@Override
        preload:function()
        {
            this._super();

            // nothing here
        },

        startswith: function(text, target)
        {
            return text.substring(0, target.length) === target;
        },

        createButton: function(upPath, downPath, buttonName, buttonText, nodeData)
        {
            var button, color, colorData, textColor, textFace, textSize;
            textFace = undefined;
            textSize = 40;
            textColor = cc.c4b(255, 255, 255, 255);
            button = this._createButton(upPath, downPath,
                buttonName, buttonText, 
                textFace, textSize, textColor, textColor, nodeData.size);
            return button;
        },

        suffix: function(name)
        {
            var names = name.split("_");
            var lastName = names[names.length - 1];
            return lastName;
        },

        /**
         * For each level, show generic pause button and milestone counting number.
         * TODO:  Level button graphic.  Number of tiles on button.  Aaron expects special case "(?)" at last locked milestone.
         * GAME-1602 Delete next button.  2014-10-22 Sushmita expects no next button.
         */
        layout_creatingNode: function( nodeName, nodeData )
        {
            if (this.startswith(nodeName, "milestone_")) {
                return this.createLevelButton(nodeName, nodeData);
            }
            switch( nodeName )
            {
                /*-
                case "next":
                    return this.createButton("buttons/game_buttonNextUp.png", "buttons/game_buttonNextDown.png", 
                        "btnNext", localize("next"), nodeData);
                -*/
                case "settings":
                    return false;

                case "leftButton":
                    var button = this.illusionSwipeView._leftButton;
                    this.addButtons([button]);
                    return button;

                case "rightButton":
                    var button = this.illusionSwipeView._rightButton;
                    this.addButtons([button]);
                    return button;

                case "questionBevel":
                    return cc.Sprite.createWithSpriteFrameName( "pieceDimArea.png" );

                case "questionSymbol":
                    return cc.Sprite.createWithSpriteFrameName( "pieceDimArea.png" );

                case "questionBackground":
                    return cc.Sprite.createWithSpriteFrameName( "pieceShape.png" );

                case "illusionSwipe":
                    return DisillusionIllusionView.prototype.grandparent(this.illusionSwipeView);

                case "illusionViewDoneBtn":
                    if ( nodeData.textData )
                    {
                        this.illusionViewDoneBtnFont = nodeData.textData.face;
                        this.illusionViewDoneBtnSize = nodeData.textData.size;
                        this.illusionViewDoneBtnColor = cc.c4b( nodeData.textData.color.Red, nodeData.textData.color.Green, nodeData.textData.color.Blue, nodeData.textData.color.Alpha );
                    }
                    else
                    {
                        this.illusionViewDoneBtnFont = "fonts/MuseoSans-500.ttf";
                        this.illusionViewDoneBtnSize = 48 * this.scaleText(0.5);
                        this.illusionViewDoneBtnColor = cc.c4b( 47, 190, 234, 255 );
                    }
                    var button = this._createButton( "buttons/game_buttonInputUp.png", 
                                               "buttons/game_buttonInputDown.png", 
                                               this.interactionResponder.hideIllusionView.bind( this.interactionResponder ), 
                                               localize("illusionViewDoneBtn"), 
                                               this.illusionViewDoneBtnFont, 
                                               this.illusionViewDoneBtnSize, 
                                               this.illusionViewDoneBtnColor, 
                                               cc.c4b(255, 255, 255, 255), 
                                               nodeData.size );
                    return button;
                default:
                    return this._super( nodeName, nodeData );
            }
        },

        /**
         * @param   sizeChange {Number} Proportion of scale by total width.  [0.0...1.0]
         * @return  {Number} At least 1.
         * MOBGAMES-1891 [~wouter]: In code I scale "DONE" text size 50% down when scaling parent up by screen width in pixels.  On small device, never more than 100% of original scale.  [~torri] Any info how to export this without resulting to programming would be appreciated.  We tried several configurations of Lumos XJSFL.  2014-11-07 Julie expects scale text size 50%.
         */
        scaleText: function(sizeChange)
        {
            var constancy = 1.0 - sizeChange;
            var screenSize = cc.Director.getInstance().getWinSize();
            var proportion = 640 / screenSize.width;
            var ratio = sizeChange * proportion + constancy * 1.0;
            var scale = Math.min(1, ratio);
            return scale;
        },

        /**
         * New Shalin code for Torri's anims and artwork
         * load this and store away the milestone details
         * Then load this and create the appropriate buttons
         * undo the scale and offset from parenting underneath menuContainer
         * HACK menu position.
         *
         * GAME-1602 [~wouter]: Workaround: In Flash timeline, I aligned centered content to top-left corner and set transform point in center, as in Ebb and Flow.
         * Although Train of Thought has centered content at center and transform point at center, the export places anchor point at 0, 0.
         * iPad scaling expects anchor at 0.5, 0.5.
         */
        _loadLayout: function() 
        {
            this._super();
            LayoutLoader.loadLayoutFile( "layout_menu.json", this.menuContainer, this , this );
            this.menu.setScaleX(1 / this.menuContainer.getScaleX());
            this.menu.setScaleY(1 / this.menuContainer.getScaleY());
            var halfW = this.menu.getContentSize().width * 0.5;
            var halfH = this.menu.getContentSize().height * 0.5;
            this.menu.setPosition(cc.p(0, this.menu.getContentSize().height));
        },

        /**
         * Populate the hud and select the current level.
         * In case restarting, hide pause button and clear tappable pieces.
         *
         * GAME-1403 At start of play or practice, clear previous tiles.
         * Test case:  2014-10-07 Playing with tiles.  Pause.  How To Play.  
         * Expect previous tiles were already cleared.  Got previous tiles are quickly replaced.
         *
         * GAME-1403 Gotcha:  With first-time user app mode "how_to_play" set, the program expects a keyframe for each container.
         * Such as level footer, centered content, top area at the start of "practiceBegin" animation.  
         * Otherwise, when practiceBegin animation plays, the content that is visible in Flash is not manipulated.
         * This is not a problem when selecting "How To Play" from the menu, because the game animations sets those keyframes.
         *
         * GAME-1602 Workaround level button layout:  If in practice mode, do not create level highlights, because the buttons format is not available.
         * Steps to reproduce:  2014-10-20 How To Play mode.  Skip tutorial.  Level select screen.  Expect highlighted button background.  Got no background visible.
         *
         * GAME-1602 Wait a few frames until cue to show pause button.  This avoids a race condition when restart is selected, which will hide the pause button.  2014-10-22 Torri, Sushmita, or Julie West expects pause button on level select.  Test case:  2014-10-23 See level select.  Pause.  Restart.  Expect to see pause button.  Got no pause button.
         *
         * GAME-1405 If practice mode, then disable level buttons.  Animation exports level buttonsas invisible during practice, yet they are not disabled and so occlude practice board buttons.  2014-10-23 iPhone 5:  How To Play.  Tap highest spot.  Ashley expects to fix response.  Got no response.
         */
        onBegin: function()
        {
            this.model.tutor.shared.showPrompt = this;
            this.setScore( this.model.score );
            this.localizeText();
            this.hidePauseButton();
            this.clearNextPiece( 0.0 );
            this.deconstructBoard();
            this.createVignette();
            if (this.model.persistentData.isLevelSelectMode) 
            {
                this.enableLevelButtons();
                this.beginPauseButton();
            }
            else if (this.model.persistentData.isPracticeMode) 
            {
                this.disableLevelButtons();
                this.cue_hidePauseButton();
            }
        },

        /**
         * Tween from pause menu without animation, visually identical to onResume.
         */
        beginPauseButton: function()
        {
            var playAnimation = DisplayState.PauseMenu !== this.getPrevDisplayState();
            var type = GameWidgetPauseButton.prototype.getType();
            var pauseButtonWidget = this._widgets[ type ];
            if ( null != pauseButtonWidget )
            {
                pauseButtonWidget.showPauseButton( true, playAnimation );
            }
        },

        /**
         * GAME-1603 If below highest level, show "levelUnlockText", otherwise show "maxLevelText".
         * Read tap a tile and number of tiles to unlock more tiles.  
         * Test case:  2014-10-23 Level select mode.  Ashley expects to read what the number represents and a call to tap a number.
         */
        localizeText: function()
        {
            this.levelSelectText.setString( localize("levelSelectText") );
            var unlock;
            if (this.model.session_level < this.model.maxLevel) 
            {
                var unlockMin = this.model.currentLevel.toString();
                unlock = localize("levelUnlockText").format(unlockMin);
            }
            else
            {
                unlock = localize("levelMaxText");
            }
            this.levelUnlockText.setString( unlock );
        },

        /**
         * GAME-1602 Workaround:  Program creates standard vignette image and scales it to the screen size.
         * [~torri]: How can I setup layout of vignette?  Disillusion animation FLA had a vignette already with its transformation point at its origin in the center.  When I export that vignette with Lumos XJSFL, the anchor point is (0, 0), and origin is (320, 480).  But in Train of Thought the same layer exports anchor point (0.5, 0.5) and origin (0, 0).  Train of Thought is well aligned in iPhone 5 simulator on Xcode.  Disllusion is not.  I don't see any difference between the two in Flash.  I copied Train of Thought's layer.
         */
        createVignette: function()
        {
            if (!this.vignette) {
                this.vignette = this._createSprite("vignette/vignette.png");
            }
            if (!this.vignette.getParent()) {
                var screenSize = cc.Director.getInstance().getWinSize();
                this.vignette.setPosition(0.5 * screenSize.width, 0.5 * screenSize.height);
                var contentSize = this.vignette.getContentSize();
                this.vignette.setScaleX(screenSize.width / contentSize.width);
                this.vignette.setScaleY(screenSize.height / contentSize.height);
                this.addChild(this.vignette);
                this.vignette.setZOrder(2);
            }
        },

        /**
         * GAME-1602 If skip tutorial, remove dim layer.  Test case:  2014-10-28 Skip tutorial.  Expect dim layer disappears. 
         * GAME-1405 Upon completing tutorial, destroy tile buttons.  Test case:  2014-11-04 First tutorial trial. Skip.  Click where spots were.  Expect no response.  Got second tutorial board appears.
         */
        onCompleteTutorial: function(skipped)
        {
            this.stopTutorialAnimations(skipped);
            this.cue_hideSkipTutorialButton();
            this.removeDimLayerSandwich();
        },

        /**
         * GAME-1405 At game end animation, hide prompts instantly.  Test case:  2014-10-30 Boiler: short game.  finish round.  expect not briefly see prepare text at end.  
         * GAME-1405 At end of game with pieces remaining, immediately remove tappable spots.  Test case:  2014-11-07 Answer in last second.  Piece is fading out.  Expect spots next to piece also disappear.  Got remaining during swipe.
         */
        onEndGameplay: function()
        {
            // ll.log("DisillusionView.onEndGameplay");
            this.clearNextPiece();
            this.destroyTappablePieces( true );
        },

        /**
         * Pause.  Expect illusion swipe view buttons invisible.  Unpause.  Expect visible again.
         */
        onPause: function()
        {
            this._super();
            if (this.illusionSwipeView && this.illusionSwipeView.showing)
            {
                this.illusionSwipeView._leftButton.setEnabled(false);
                this.illusionSwipeView._rightButton.setEnabled(false);
            }
        },

        onResume: function()
        {
            this._super();
            if (this.illusionSwipeView && this.illusionSwipeView.showing)
            {
                this.illusionSwipeView._leftButton.setEnabled(true);
                this.illusionSwipeView._rightButton.setEnabled(true);
            }
        },

        onUpdateTransferTime: function()
        {
            this.setScore( this.model.score );
            this.updateTime();
            this.updateBonusPoints();
        },

        /**
         * GAME-1400 Override anchor to 0.5, 0.5.
         * Rotation expects anchor at 0.5, 0.5, which is what appears in FLA.
         * Export of trialBegin animations set anchor to 0, 0.  
         * JavaScript reports anchor point at 0, 1.
         */
        _overrideQuestionAnchor: function(isVertical)
        {
            this.questionSymbol.setAnchorPoint(0.5, 0.5);
            this.questionBackground.setAnchorPoint(0.5, 0.5);
            var rotation = isVertical ? 0.0 : -90.0;
            this.questionBackground.setRotation(rotation);
            this.questionSymbol.setRotation(rotation);
        },

        updateTime: function()
        {
            if ( this._widgets )
            {
                var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
                if ( hudWidget )
                {
                    hudWidget.setTimeRemaining( Math.floor(this.model.timeRemaining) );
                }
            }
        },

        setBonusRemaining: function( bonusRemaining )
        {
            if ( this._widgets )
            {
                var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
                if ( hudWidget )
                {
                    hudWidget.setTimeRemaining( bonusRemaining );
                }
            }
        },

        updateBonusPoints: function()
        {
            if ( this[ "scorePoints" ] )
            {
                this[ "scorePoints" ].setString( "+" + this.model.bonus );
            }
        },

        /**
         * Save highlight background image from pieceLevelDown.
         * Disable higher level, represented as 25% opacity.
         *
         * Pieces are in sprite sheet, so could refer to sprite sheet frame and save some kilobytes.
         * GameView._createButton function expects paths and does not consider sprite sheet frames.
         * Ideally, create button would check for a sprite sheet before creating a new sprite.
         */
        createLevelButton: function(nodeName, nodeData)
        {
            var buttonName = "btn_" + nodeName;
            var milestone = parseInt(this.suffix(nodeName));
            var buttonText = this.model.milestoneToLevelText(milestone);
            var button = this.createButton(null, null,
                buttonName, buttonText, nodeData);
            this._createLevelButtonHighlight(button);
            this.rotateButtonMilestone(button, milestone, nodeData.size);
            this.levelButtons = (undefined == this["levelButtons"]) ? {} : this.levelButtons; 
            this.levelButtons[buttonName] = button;
            return button;
        },

        /**
         * GAME-1602 After level button node scaled, reuse down image for highlight.  2014-10-17 iPad.  Level select.  Ben expects tile full size.  Got small.
         * GAME-1602 Create duplicate down image.  Do not reuse down state image.  Sometimes this disappears.  Test case:  How to play mode.  Answer three trials.  Skip tutorial.  Level select mode.  Expect to see highlit tile background.  Got invisible.
         * GAME-1602 After buttons are formatted, program creates states "pieceLevelDisabled.png" on unavailable milestones and "pieceLevelHighlighted.png" on highest unlocked milestone.  2014-10-22 Sushmita expects highlight on highest milestone unlocked.  [~wouter] expects disabled image for higher levels.
         * GAME-1602 On each level button, set a disabled font color.  2014-10-23 Button is disabled.  [~wouter] expects hex 0x063E48.
         */
        enableLevelButtons: function()
        {
            var unlocked = this.model.milestoneUnlocked();
            for (var buttonName in this.levelButtons)
            {
                var button = this.levelButtons[buttonName];
                var milestone = parseInt(this.suffix(buttonName));
                var enabled = milestone <= unlocked;
                button.setEnabled(enabled);
                if (enabled) 
                {
                    var highlighted = milestone == unlocked;
                    button.setHighlighted(highlighted);
                }
            }
        },

        _createLevelButtonHighlight: function(button)
        {
            var imageNormal = cc.Sprite.createWithSpriteFrameName( "pieceLevelUp.png" );
            button.setBackgroundImageForState( imageNormal, 
                LumosityButtonState.ButtonStateNormal );
            var imageSelected = cc.Sprite.createWithSpriteFrameName( "pieceLevelDown.png" );
            button.setBackgroundImageForState( imageSelected, 
                LumosityButtonState.ButtonStateSelected );
            var imageHighlighted = cc.Sprite.createWithSpriteFrameName( "pieceLevelHighlighted.png" );
            button.setBackgroundImageForState( imageHighlighted, 
                LumosityButtonState.ButtonStateHighlighted );
            var imageDisabled = cc.Sprite.createWithSpriteFrameName( "pieceLevelDisabled.png" );
            button.setBackgroundImageForState( imageDisabled, 
                LumosityButtonState.ButtonStateDisabled );
            var disabledColor = cc.c3b( 6, 62, 72 );
            button.setTitleColorForState( disabledColor, 
                LumosityButtonState.ButtonStateDisabled );
        },

        rotateButtonMilestone: function(button, milestone, nodeSize)
        {
            var rotates = [      2,     4, 
                             5,      7,
                                10,     12 ];
            // rotates = [];  // debug
            var inhibit = rotates.indexOf(milestone) <= -1; //  || milestone <= 4;  // debug
            this.rotateButton(button, milestone, nodeSize, inhibit);
        },

        /**
         * GAME-1602 Anchor, rotate, position in center of parent node each background image.  2014-10-22 [~wouter] expects to rotate all images in buttons 2, 4, 5, 7, 10, 12 without rotating text.  Test case:  2014-10-22 LumosityMobile iPad Debug Cocos2d: iPad Mini 2 iOS 8.0.  Expect rotate image at center.  Got image is offset, since then content scales (from 140 to 350) but the layout coordinate space does not.  In Boiler, the content size (140) does not scale with higher resolution of iPad mini or iPad 3.  
         */
        rotateButton: function(button, milestone, nodeSize, inhibit)
        {
            var logging = false;
            for (var state in LumosityButtonState) 
            {
                var image = button.backgroundImageForState(state);
                if (image)
                {
                    if (logging)
                    {
                        var positionOld = image.getPosition();
                        var transformOld = image.worldToNodeTransform();
                    }
                    var offset = 0.5;
                    var rotation = -90;
                    if (!inhibit) 
                    {
                        image.setAnchorPoint(0.5, 0.5);
                        image.setRotation(rotation);
                        image.setPosition(offset * nodeSize.width, offset * nodeSize.height);
                    }
                    if (logging) 
                    {
                        logging = false;
                        var positionNew = image.getPosition();
                        var transformNew = image.worldToNodeTransform();
                        ll.log("DisillusionView.rotateButton: " + milestone + ": " + state + ": "
                            + " nodeSize " + this.represent(nodeSize) 
                            + " positionOld " + this.represent(positionOld)
                            + " positionNew " + this.represent(positionNew)
                            + " transformOld " + this.represent(transformOld)
                            + " transformNew " + this.represent(transformNew)
                        );
                    }
                }
            }
        },

        /**
         * @param   hash     Does not support nested keys.
         * @return  String of key and values, sorted by key.
         */
        represent: function(hash)
        {
            var representations = [];
            if (null != hash)
            {
                for (var key in hash)
                {
                    representations.push(key + ": " + hash[key]);
                }
                representations.sort();
            }
            return "{" + representations.join(", ") + "}";
        },

        /**
         * MOBGAMES-1828 Upon selecting level, cleanup level select animations, in case the player has rapidly tapped the level before text animates into view.  2014-11-04 Julie expects delay of some seconds.
         */
        selectLevel: function()
        {
            this.stopAllAnimations();
            this.levelSelectText.cleanup();
            this.levelUnlockText.cleanup();
            this.menuContainer.cleanup();
            this.playAnimation( "transition" );
        },

        /**
         * TODO Deprecate or use.
         */
        selectLevelButton: function(milestoneIndex)
        {
            var buttonName = "btn_milestone_" + milestoneIndex;
            return this.selectLevelName(buttonName);
        },

        /**
         * @return  milestone from suffix of button name.
         * Set image to down by highlight state.  Avoid removing images, to avoid garbage collection:
         * http://stackoverflow.com/questions/25198410/cocos2d-js-error-message-invalid-native-object-using-runaction
         */
        selectLevelName: function(buttonName)
        {
            var milestone = -1;
            if (buttonName in this.levelButtons) {
                var previous = this.selectedButton;
                var next = this.levelButtons[buttonName];
                /*-
                if (next !== previous) {
                    if (null != previous) {
                        previous.setHighlighted(false);
                    }
                }
                next.setHighlighted(true);
                -*/
                milestone = parseInt(this.suffix(buttonName));
                this.selectedButton = next;
            }
            return milestone;
        },

        disableLevelButtons: function()
        {
            for (var buttonName in this.levelButtons) {
                var button = this.levelButtons[buttonName];
                button.setEnabled(false);
            }
        },

        setLevel: function()
        {
            if ( this._widgets )
            {
                var levelCardWidget = this._widgets[ GameWidgetLevelCard.prototype.getType() ];
                if ( levelCardWidget )
                {
                    levelCardWidget.setLevelValue( this.model.currentLevel );
                }
            }
        },

        hidePauseButton: function()
        {
            if ( this._widgets )
            {
                var widget = this._widgets[ GameWidgetPauseButton.prototype.getType() ];
                if ( widget )
                {
                    widget.showPauseButton( false, false );
                }
            }
        },

        /**
         * Keep buttons as children and in dictionary, since they are reused.
         */
        deconstructBoard: function()
        {
            var i;

            // Nodes (not reused)
            var node;
            for ( i = 0; i < this._boardNodes.length; i++ )
            {
                node = this._boardNodes[ i ];
                this[ node.nodeName ] = undefined;

                node.removeFromParent( true );
            }
            this._boardNodes.length = 0;
        },

        /**
         * GAME-1602 Workaround: Program board view vertical position to 380 to align with visual center from export.
         */
        _setupBoardView: function()
        {
            if ( !this._boardView )
            {
                this._boardView = cc.Node.create();
                this._boardView.nodeName = "BoardView";
                this._boardView.setContentSize( cc.size( 580, 580 ) );
                this._boardView.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                this._boardView.setPosition( cc.p( 290, 380
                                                        // 290
                ) );

                if ( this[ "gameTray" ] )
                {
                    this[ "gameTray" ].addChild( this._boardView );
                }
                else
                {
                    ll.warning("DisillusionView._setupBoardView: Expected gameTray");
                }
            }
        },

        isConstructed: function()
        {
            return 1 <= this._boardNodes.length;
        },

        /**
         * Clear previous board.
         * Set up the board view.
         * Get 'animated board' size (specified by flash).
         * Populate the board with each piece.
         * GAME-1403 Practice: Align board toward screen center have room for text at bottom of screen.
         */
        constructBoard: function( boardObject )
        {
            var tileSize = this.settings.pieceSize;
            this.deconstructBoard();
            this._setupBoardView();
            var boardSize = this._boardView.getContentSize();
            var nodeSize = cc.size( tileSize, tileSize );
            var boardContentSize = cc.size( boardObject._boardWidth * nodeSize.width, 
                                            boardObject._boardHeight * nodeSize.height );
            var practiceHeight = 1.0;
            var heightProportion = this.model.persistentData.isPracticeMode ? practiceHeight : 0.5;
            var widthProportion = 0.5;
            var startNodePosition = cc.p( widthProportion * (boardSize.width - boardContentSize.width), 
                                          heightProportion * (boardSize.height - boardContentSize.height) );
            var x, y;
            for ( y = 0; y < boardObject._boardHeight; y++ )
            {
                for ( x = 0; x < boardObject._boardWidth; x++ )
                {
                    var boardPiece = boardObject.getPiece( cc.p( x, y ) );
                    if ( boardPiece )
                    {
                        var newNode = new DisillusionBoardPieceView( boardPiece, cc.p( x, y ), nodeSize, 
                            this.settings, this.animationResponder );
                        newNode.setPosition( cc.p( startNodePosition.x + (x * tileSize) + 0.5 * tileSize, 
                                                   startNodePosition.y + (y * tileSize) + 0.5 * tileSize ) );
                        newNode.setOpacity( 0 );
                        this[ newNode.nodeName ] = newNode;
                        this._boardNodes.push( newNode );
                        this._boardView.addChild( newNode );
                    }
                }
            }
        },

        /**
         * Check all possible spots at all possible board sizes.
         * Check if there is a button in this spot that we can reuse.
         * Or create a new button.
         * If button has changed state, animate spot to tap.
         *
         * Test case:  2014-10-01 Place tile.  Next tile of same type and a spot to tap remains.  This happens in tutorial.  Expect a spot to tap to remain.  Sometimes one spot spot to tap flickers; oftentimes near top, but sometimes on right side.  If correct, a couple spots that remain may flicker.  The timing coincides with the end of feedback animation and replacement of piece to match.  (2014-10-21 saw on LumosityMobile).
         */
        showTappablePieces:function(instant)
        {
            // ll.verbose( "DisillusionView", "showTappablePieces" );
            var position;
            var buttonName;
            var button;
            var x, y;
            var heightMax = this.model.tapHeightMax();
            var widthMax = this.model.tapWidthMax();
            for ( y = 0; y < heightMax; y++ )
            {
                for ( x = 0; x < widthMax; x++ )
                {
                    var column = this.model.currentRound.board._boardSetup[x]
                    if (!column) 
                    {
                        continue;
                    }
                    var pieceModel = column[y];
                    if ( pieceModel && pieceModel.isTappable() )
                    {
                        var nodeName = "boardNode" + x + "x" + y;
                        var node = this[ nodeName ];
                        if ( undefined !== node )
                        {
                            //ll.verbose( "DisillusionView", "showTappablePieces", "Piece at " + x + ", " + y + " is tappable." );
                            position = node.getPosition();
                            buttonName = "boardButton" + x + "x" + y;
                            var button = this[ buttonName ];
                            if (undefined == button)
                            {
                                button = new DisillusionBoardPieceButton( this.settings, this );
                                this[ buttonName ] = button;
                                this.addButton( button );
                                this._boardView.addChild( button );
                                button.nodeName = buttonName;
                                button.x = x;
                                button.y = y;
                            }
                            button.pieceModel = pieceModel;
                            button.setPosition( position );
                            button.setOrientation( pieceModel.orientation );
                            if ( !button.isSlidIn() ) 
                            {
                                button.slideIn(instant);
                            }
                        }
                    }
                    else 
                    {
                        this.clearTappablePiece(x, y, instant);
                    }
                }
            }
        },

        /**
         * Retain button to recycle.
         */
        clearTappablePiece: function(x, y, instant)
        {
            var buttonName = "boardButton" + x + "x" + y;
            if ( this[ buttonName ] !== undefined )
            {
                var button = this[ buttonName ];
                if ( button.isSlidIn() ) 
                {
                    button.slideOut(instant);
                }
            }
        },

        /**
         * Loop through all possible sizes.  Clear pieces.  Destroy reference.
         * @param   instant     {Boolean}  Hide buttons without animation.  
         * Test case:  2014-10-21 Play.  Start tutorial.  Expect previous spots to tap are gone.
         */
        destroyTappablePieces: function()
        {
            var instant = true;
            var heightMax = this.model.tapHeightMax();
            var widthMax = this.model.tapWidthMax();
            for (var y = 0; y < heightMax; y++ )
            {
                for (var x = 0; x < widthMax; x++ )
                {
                    this.clearTappablePiece(x, y, instant);
                    var buttonName = "boardButton" + x + "x" + y;
                    this[buttonName] = undefined;
                }
            }
        },

        /**
         * If valid, place piece and clear piece to place.
         */
        onPieceTapped: function( button )
        {
            var position = this.model.onPieceTapped(button);
            if (position) 
            {
                button.slideOut( true );
                this.placeNextPieceAt( position );
                this.clearNextPiece();
            }
        },

        placeNextPieceAt: function( gridPosition )
        {
            // Get board piece
            var boardPiece = this.model.currentRound.board.getPiece( gridPosition );
            if ( boardPiece )
            {
                var nodeName = "boardNode" + gridPosition.x + "x" + gridPosition.y;
                if ( this[ nodeName ] !== undefined )
                {
                    // ll.verbose( "Disillusion", "updatePiece", "Updating board piece " + nodeName + " from model." );
                    this[ nodeName ].slideIn();
                    this.model.isSlidingIn = true;
                }
            }
        },

        updatePiece:function( gridPosition )
        {
            // Get board piece
            var boardPiece = this.model.currentRound.board.getPiece( gridPosition );
            if ( boardPiece )
            {
                /*
                var buttonName = "boardButton" + gridPosition.x + "x" + gridPosition.y;
                if ( this[ buttonName ] !== undefined )
                {
                    this[ buttonName ].setEnabled( false );
                    this[ buttonName ].setVisible( false );

                    this[ buttonName ] = undefined;
                }
                */

                var nodeName = "boardNode" + gridPosition.x + "x" + gridPosition.y;
                if ( this[ nodeName ] !== undefined )
                {
                    // ll.verbose( "Disillusion", "updatePiece", "Updating board piece " + nodeName + " from model." );
                    this[ nodeName ].updateFromModel();
                }
            }
        },

        updatePieces:function( piecesToUpdate )
        {
            var i;
            for ( i = 0; i < piecesToUpdate.length; i++ )
            {
                this.updatePiece( piecesToUpdate[i] );
            }
        },

        _getFeedbackFrames: function()
        {
            var frameName = "";
            var frameCount = 3;
            var frames = [];
            var cache = cc.SpriteFrameCache.getInstance();
            var index;
            for( index = 0; index < frameCount; index += 1 )
            {
                frameName = "incorrectAnimation00" + index.toString() + ".png";
                frames.push( cache.getSpriteFrame( frameName ) );
            }
            return frames;
        },

        showWrongSelectionFeedback: function( gridPosition )
        {
            var boardNode = this[ "boardNode" + gridPosition.x + "x" + gridPosition.y ];
            if ( boardNode )
            {
                var frames = this._getFeedbackFrames();

                var feedbackSprite = cc.Sprite.createWithSpriteFrame( frames[ 0 ] );
                feedbackSprite.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                feedbackSprite.setPosition( boardNode.getPosition() );
                boardNode.getParent().addChild( feedbackSprite );

                lu.scaleForContentSize( feedbackSprite, cc.size( 64, 64 ) );

                var animation = cc.Animation.create( frames, 0.1 );
                var animationAction = cc.Animate.create( animation );
                var callbackAction = cc.CallFunc.create( this._onShowWrongSelectionFeedbackComplete.bind( this, feedbackSprite ), this );
                var finalSequence = cc.Sequence.create( animationAction, callbackAction );

                feedbackSprite.runAction( finalSequence );
            }
        },

        _onShowWrongSelectionFeedbackComplete: function( feedbackSprite )
        {
            if ( feedbackSprite )
            {
                feedbackSprite.removeFromParent( true );
            }

            if ( this.animationResponder )
            {
                this.animationResponder.onShakePieceComplete();
            }
        },

        /**
         * MOBGAMES-1853 Cleanup previous actions and schedulers on the tile to place.  The tile might be in the middle of a fading out animation that if not cleaned up, could continue to fade out after it was set to show the next tile.  2014-11-04 Test case:  Play 100 sessions on slow device or simulator.  Daniel expects to fix always see symbol and tile.  Got shadow only.  
         */
        setupNextPiece: function()
        {
            if ( this.model.nextPiece !== undefined )
            {
                var nextPiece = this.model.nextPiece;

                var symbolFrameName = this.settings.getBoardPieceImage( nextPiece.pieceType );
                var spriteColor = this.settings.getBoardPieceColor( nextPiece.pieceColor );
                var bevelFrameName = nextPiece.isVertical() 
                    ? "pieceBevelVertical.png" 
                    : "pieceBevelHorizontal.png";
                this.showColor(this.questionBackground, spriteColor);
                this.showFrame(this.questionSymbol, symbolFrameName);
                this.showFrame(this.questionBevel, bevelFrameName);
                this._overrideQuestionAnchor(nextPiece.isVertical());
            }
        },

        showColor: function(sprite, color)
        {
            if ( sprite )
            {
                // sprite.cleanup();
                sprite.setOpacity(255);
                sprite.setColor( color );
                sprite.setVisible( true );
            }
        },

        showFrame: function(sprite, frameName)
        {
            if ( sprite )
            {
                // sprite.cleanup();
                sprite.setOpacity(255);
                sprite.setDisplayFrame( cc.SpriteFrameCache.getInstance().getSpriteFrame( frameName ) );
                sprite.setVisible( true );
            }
        },

        clearNextPiece: function( fadeTime )
        {
            if (undefined === fadeTime) {
                fadeTime = 0.1;
                           // 5.0;  // Inject race with setupNextPiece.
            }
            this.fadeDown(this.questionBackground, fadeTime);
            this.fadeDown(this.questionSymbol, fadeTime);
            if ( this[ "questionBevel" ] )
            {
                this[ "questionBevel" ].setVisible( false );
            }
        },

        fadeDown: function(node, fadeTime)
        {
            if ( node )
            {
                node.setVisible(false);
                /*-
                var fadeAction = cc.FadeTo.create( fadeTime, 0 );
                var moveDownAction = cc.MoveBy.create( fadeTime, cc.p( 0, 0 ) );
                node.runAction( fadeAction );
                node.runAction( moveDownAction );
                -*/
            }
        },

        fadePiecesIn: function( piecesToFadeIn )
        {
            if ( piecesToFadeIn && piecesToFadeIn.length > 0 )
            {
                var i;
                for ( i = 0; i < piecesToFadeIn.length; i++ )
                {
                    var nodeName = "boardNode" + piecesToFadeIn[i].x + "x" + piecesToFadeIn[i].y;
                    if ( this[ nodeName ] )
                    {
                        this[ nodeName ].fadeIn();
                    }
                }
            }
        },

        fadePiecesOut: function( piecesToFadeOut, callback, updateFromModel )
        {
            if ( piecesToFadeOut && piecesToFadeOut.length > 0 )
            {
                var i;
                for ( i = 0; i < piecesToFadeOut.length; i++ )
                {
                    var nodeName = "boardNode" + piecesToFadeOut[i].x + "x" + piecesToFadeOut[i].y;
                    if ( this[ nodeName ] )
                    {
                        this[ nodeName ].fadeOut( callback, updateFromModel );
                    }
                }
            }
        },

        /**
         * GAME-1602 Layout "prompt_prepare" 2 lines lower than "prompt", leaving an extra line for translation.  Layer both prompts above game tray.  Before animating, set the unused prompt to an empty string so it will not be visible.  Test case:  2014-10-20 Large board.  prompt prepare appears.  Expect not overlapping tile.  2014-10-24 During 3-2-1 countdown.  Expect text in front of tile shadow.
         * @param   messageKey  If undefined, then tutor_prepare.  If animation of messageKey exists, play that animation.
         */
        showPrompt: function( show, messageKey )
        {
            ll.log("DisillusionView.showPrompt: " + show + " key " + messageKey);
            messageKey = messageKey || (show ? "tutor_prepare" : "");
            this.model.tutor.shared.stop();
            if ( show )
            {
                var target, ignore;
                if ( "tutor_prepare" == messageKey )
                {
                    target = this.prompt_prepare;
                    ignore = this.prompt;
                }
                else 
                {
                    target = this.prompt;
                    ignore = this.prompt_prepare;
                }
                target.setString( localize(messageKey) );
                ignore.setString( "" );
                if (this.hasAnimation( messageKey ) ) 
                {
                    this.stopTutorialAnimations(true);
                    this.playAnimation( messageKey );
                }
                this.playAnimation( "promptBegin" );
            }
            else
            {
                this.playAnimation( "promptEnd", this.model.tutor.shared.promptEnd, this.model.tutor.shared );
            }
        },

        /**
         * GAME-1403 If skip tutorial, stop all animations, actions, and sequences.  Example:  Tutor.  Animating between boards.  Quickly click skip tutorial.  Expect next board does not appear.
         * GAME-1405 At end of tutorial trials, remove dim layer if it is still present.  Test case:  How to Play mode.  Second trial.  Dim.  Immediately click skip tutorial.  Expect dim layer removed.  
         * GAME-1405 When starting next tutorial animation, clear previous animations and hide orientation hint, prompt, and dim layer.  Test case:  2014-10-20 How to play mode.  Answer quickly.  Expect previous orientation hint and dim layer disappeared.
         */
        stopTutorialAnimations: function(skipped)
        {
            this.stopAllAnimations(skipped);
            this.removeDimLayerSandwich(skipped);
            this.prompt.cleanup();
            this.prompt.setVisible(false);
            this.orientationHintHorizontal.cleanup();
            this.orientationHintHorizontal.setVisible(false);
            this.orientationHintVertical.cleanup();
            this.orientationHintVertical.setVisible(false);
        },

        /**
         * GAME-1403 [~wouter], [~sclark]: Cue "createDimLayer" and "removeDimLayer" plays standard dim animation.  To workaround that the animation FLA does not represent the standard dim layer, the program orders dim layer in front of pieces and spots and behind question, in centered content layer.
         *
         * GAME-1403 Retain dim layer's shaded rectangle before removing images, to avoid Cocos2D-x garbage collection:
         * http://stackoverflow.com/questions/25198410/cocos2d-js-error-message-invalid-native-object-using-runaction
         * If that fails, a fallback workaround was: Do not remove dim layer shaded rectangle.  Xcode throws "Error: Invalid Native Object" when trying to access removed shadedRect, although Safari is okay.
         *
         * GAME-1602 To fit dim layer to iPhone 3 in centered content, set shaded rectangle size to 5x screen size.  Test case:  Boiler.  iPhone 3.  How to Play mode.  Second trial.  Expect dim layer covers full screen.  Got dim layer covers part of screen.
         */
        cue_createDimLayer: function()
        {
            this.removeDimLayerSandwich(true);
            this.dimLayerSandwich = cc.Sprite.create();
            this.dimLayerSandwich.setColor( cc.black() );
            this.dimLayerSandwich.setOpacity( 0 );
            var windowSize = cc.Director.getInstance().getWinSize();
            var rect = cc.rect( 0, 0, windowSize.width * 5, windowSize.height * 5 );
            this.dimLayerSandwich.setTextureRect( rect );
            var fadeIn = cc.FadeTo.create( 0.5, 80 );
            this.dimLayerSandwich.runAction( fadeIn );
            this.centeredContent.addChild(this.dimLayerSandwich);
            var z = this.questionBevel.getZOrder();
            this.dimLayerSandwich.setZOrder( z );
        },

        getChildByNodeName: function(parent, name)
        {
            var children = parent.getChildren();
            var child;
            for (var c = 0; c < children.length; c++)
            {
                if (name === children[c].nodeName) {
                    child = children[c];
                }
            }
            return child;
        },

        /**
         * @param   viewIgnored     CCActionInstant.execute passes in view as first argument.  This argument is ignored.  However, this underlying function is also called with an optional argument.  Test case:  2014-10-20 How to Play.  Dim.  Remove.  [~wouter] expects fade out.  Got snaps out.
         */
        cue_removeDimLayer: function(viewIgnored)
        {
            this.removeDimLayerSandwich();
        },

        /**
         * GAME-1405 To avoid LumosityMobile error, set to invisible and create a new dim layer each time.
         * GAME-1602 The name "removeDimLayer" is reserved for GameView removing the tutorial dim layer.
         * Example Xcode message:
         * 2014-10-22 12:44:03.060 Lumosity[11692:60b] Cocos2d: jsb: ERROR: File /Users/ethan/workspace/LumosityMobile/DoubleWide/cocos2d-x/scripting/javascript/bindings/generated/jsb_cocos2dx_auto.cpp: Line: 4443, Function: js_cocos2dx_CCNode_getParent 
         * 2014-10-22 12:44:03.062 Lumosity[11692:60b] Cocos2d: Invalid Native Object
         */
        removeDimLayerSandwich: function(instant)
        {
            if (this.dimLayerSandwich) 
            {
                if (instant) 
                {
                    this.dimLayerSandwich.setVisible(false);
                    this.dimLayerSandwich = undefined;
                }
                else 
                {
                    var fadeOut = cc.FadeTo.create( 0.5, 0 );
                    this.dimLayerSandwich.runAction( fadeOut );
                }
            }
        },

        cue_hudScoreHighlight: function()
        {
            if ( this._widgets )
            {
                var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
                if ( hudWidget )
                {
                    hudWidget.setScore( this.model.score );
                    hudWidget.setScoreHighlighted( true );
                }
            }
        },

        cue_hudScoreNormal: function()
        {
            if ( this._widgets )
            {
                var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
                if ( hudWidget )
                {
                    hudWidget.setScoreHighlighted( false );
                }
            }
        },

        /**
         * @return  If name ends with ending.  If name is shorter than ending, name cannot end with ending.
         */
        endsWith: function(name, ending)
        {
            return ending.length <= name.length 
                    && name.indexOf(ending) 
                    == name.length - ending.length;
        },

        /**
         * GAME-1405 While score is showing in static popup that cannot be skipped, load illusions.
         * Test case:  2014-11-06 iPad.  Win level 96.  Expect 12th illusion slides in.  Got jerk.
         */
        illusionLoadInBackground: function(delayTime)
        {
            var delay = cc.DelayTime.create(delayTime);
            var load = cc.CallFunc.create(this.illusionLoad, this);
            var sequence = cc.Sequence.create(delay, load);
            this.runAction(sequence);
        },

        illusionLoad: function()
        {
            this.illusionSwipeView.loadPages(this.model.milestoneUnlocked(), 
                this.model.milestoneMax(), this.model);
        },

        illusionBegin: function()
        {
            if ( this.illusionSwipeView && !this.illusionSwipeView.showing )
            {
                /*
                if (lu.isDebugModeOn()) {
                    var screenSize = cc.Director.getInstance().getWinSize();
                    ll.log(screenSize.width, screenSize.height);
                    ll.log(this.ancestry(this.illusionSwipeView._leftButton, "PositionX"));
                    ll.log(this.ancestry(this.illusionSwipeView._leftButton, "PositionY"));
                    ll.log(this.ancestry(this.illusionSwipeView._leftButton, "ScaleY"));
                    ll.log(this.ancestry(this.illusionSwipeView._rightButton, "PositionX"));
                    ll.log(this.ancestry(this.illusionSwipeView._rightButton, "PositionY"));
                    ll.log(this.ancestry(this.illusionSwipeView._rightButton, "ScaleY"));
                }
                 */
                this.playAnimation( "illusionBegin" );
                this.illusionSwipeView.show( true );
            }
        }
    });
}());
