/*global cc*/
/*global ll*/
/*global LumosityButtonState*/

var GameWidgetPageButtons = cc.Class.extend(
{
    /** @lends GameWidgetPageButtons.prototype */

    _zOrder: 500,

    /**
     * Left and right arrow buttons beside how to play and Disillusion illusions.
     * Align to left and right side of screen, compensating for ancestor scaling.
     * Example: View illusion which is in scaled node in iPhone 5, iPad 3.
     *
     * Example @see Disillusion
     *
     * @param   controller  Overwrites properties on controller and addChild:
     *                      _buttons (expects and pushes)
     *                      _leftButton
     *                      _leftButtonContainer
     *                      _rightButton
     *                      _rightButtonContainer
     *                  If controller._buttonParent is defined, then button containers are added to:
     *                      _buttonParent
     *                  Binds button release callbacks:
     *                      _leftButton:  controller.previousPage
     *                      _rightButton:  controller.nextPage
     *                  Parent is a parameter so that Disillusion can use these buttons for illusion.
     *
     * @param   doNotParent     If true, then do not create button containers or parent buttons.  
     *                          Example:  GAME-1602 Disillusion adds scroll buttons from layout.
     *
     * @return  Array of right then left button that were pushed to _buttons.
     *
     * @public @static
     */
    create: function( controller, doNotParent )
    {
        var screenSize = cc.Director.getInstance().getWinSize();
        var y = 0.5 * screenSize.height;
        var sideButtonSize = cc.size ( 40, 120 );

        // Image names
        var buttonUpImageName = null;
        var buttonDownImageName = null;
        var buttonDisabledImageName = null;
        // Image objects
        var buttonUpImage = null;
        var buttonDownImage = null;
        var buttonDisabledImage = null;

        // Settings for right button
        buttonUpImageName = "howToPlays/howToPlayButtonRightUp.png";
        buttonDownImageName = "howToPlays/howToPlayButtonRightDown.png";
        buttonDisabledImageName = "howToPlays/howToPlayButtonRightUp.png";

        buttonUpImage = cc.Sprite.create( buttonUpImageName );
        buttonDownImage = cc.Sprite.create( buttonDownImageName );
        buttonDisabledImage = cc.Sprite.create( buttonDisabledImageName );

        // Create Right button
        controller._rightButton = new LumosityButton();
        controller._rightButton.setBackgroundImageForState( buttonUpImage, LumosityButtonState.ButtonStateNormal );
        controller._rightButton.setBackgroundImageForState( buttonDownImage, LumosityButtonState.ButtonStateSelected );
        controller._rightButton.setBackgroundImageForState( buttonDisabledImage, LumosityButtonState.ButtonStateDisabled );
        controller._rightButton.setTarget( controller );
        controller._rightButton.setReleasedCallback( controller.nextPage );
        controller._rightButton.setAnchorPoint( cc.p( 1.0, 0.5 ) );
        controller._rightButton.setPosition( cc.p( sideButtonSize.width, sideButtonSize.height / 2 ) );
        // buttons are very thin, so expand invisible hit region a little toward the center
        controller._rightButton.setHitRegion(
            [
                cc.p( sideButtonSize.width * -1,  sideButtonSize.height * 0 ),
                cc.p( sideButtonSize.width * -1,  sideButtonSize.height * 1 ),
                cc.p( sideButtonSize.width * 1,   sideButtonSize.height * 1 ),
                cc.p( sideButtonSize.width * 1,   sideButtonSize.height * 0 )
            ]
        );
        controller._rightButton.nodeName = "HowToPlay:RightButton";
        controller._buttons.push( controller._rightButton );

        // Settings for left button
        buttonUpImageName = "howToPlays/howToPlayButtonLeftUp.png";
        buttonDownImageName = "howToPlays/howToPlayButtonLeftDown.png";
        buttonDisabledImageName = "howToPlays/howToPlayButtonLeftUp.png";

        buttonUpImage = cc.Sprite.create( buttonUpImageName );
        buttonDownImage = cc.Sprite.create( buttonDownImageName );
        buttonDisabledImage = cc.Sprite.create( buttonDisabledImageName );

        // Create left button
        controller._leftButton = new LumosityButton();
        controller._leftButton.setBackgroundImageForState( buttonUpImage, LumosityButtonState.ButtonStateNormal );
        controller._leftButton.setBackgroundImageForState( buttonDownImage, LumosityButtonState.ButtonStateSelected );
        controller._leftButton.setBackgroundImageForState( buttonDisabledImage, LumosityButtonState.ButtonStateDisabled );
        controller._leftButton.setTarget( controller );
        controller._leftButton.setReleasedCallback( controller.previousPage );
        controller._leftButton.setAnchorPoint( cc.p( 0.0, 0.5 ) );
        controller._leftButton.setPosition( cc.p( 0.0, sideButtonSize.height / 2 ) );
        // buttons are very thin, so expand invisible hit region a little toward the center
        controller._leftButton.setHitRegion(
            [
                cc.p( sideButtonSize.width * 0,   sideButtonSize.height * 0 ),
                cc.p( sideButtonSize.width * 0,   sideButtonSize.height * 1 ),
                cc.p( sideButtonSize.width * 2,   sideButtonSize.height * 1 ),
                cc.p( sideButtonSize.width * 2,   sideButtonSize.height * 0 )
            ]
        );
        controller._leftButton.nodeName = "HowToPlay:LeftButton";
        controller._buttons.push( controller._leftButton );
        if (!doNotParent)
        {
            // Right Button Container
            controller._rightButtonContainer = cc.Node.create();
            controller._rightButtonContainer.nodeName = "HowToPlay:RightButtonContainer";
            controller._rightButtonContainer.setContentSize( sideButtonSize );
            controller._rightButtonContainer.setAnchorPoint( cc.p( 1, 0.5 ) );
            var right = cc.p( screenSize.width, y);
            controller._rightButtonContainer.setPosition( right );
            controller._rightButtonContainer.addChild( controller._rightButton );

            // Left Button Container
            controller._leftButtonContainer = cc.Node.create();
            controller._leftButtonContainer.nodeName = "HowToPlay:LeftButtonContainer";
            controller._leftButtonContainer.setContentSize( sideButtonSize );
            controller._leftButtonContainer.setAnchorPoint( cc.p( 0, 0.5 ) );
            var left = cc.p( 0.0, y);
            controller._leftButtonContainer.setPosition( left );
            controller._leftButtonContainer.addChild( controller._leftButton );

            var parent = controller._buttonParent || controller;
            parent.addChild( controller._rightButtonContainer, this._zOrder );
            parent.addChild( controller._leftButtonContainer, this._zOrder );
        }
        return [controller._rightButton, controller._leftButton];
    },
});
