/*global ll*/

var require = require;

require( "src/Utility/LumosityLogger.js" );
var ll = LumosityLogger;

/**
 * Manage state of tutor during play.
 */
var Tutor = cc.Class.extend(
{
    /** @lends DisillusionTutor.prototype */

    active: false,
    change: false,
    currentLabel: "none",
    /**
     * Callback container with signature:  getState.getState(previousState)
     * Container preserves "this" scope without needing another "this" parameter.
     */
    getState: null,
    intro: false,
    prepare: false,
    showing: false,
    /**
     * Callback container with signature:  showPrompt.showPrompt(show, message)
     * Container preserves "this" scope without needing another "this" parameter.
     */
    showPrompt: null,
    state: "none",
    updating: true,

    ctor: function()
    {
        this.state = "none";
        this.change = false;
        this.currentLabel = "none";
        this.active = false;
        this.prepare = false;
        this.intro = false;
        this.updating = true;
        this.updatingAt = 0.0;
        this.timeConsumed = 0.0;
        this.outro = false;
    },

    /**
     * @param   timeConsumed    How many seconds since start of play.  Used to enable updating after a delay.
     * @return  state string which can refer to a label in a timeline.
     * WG-1138 If preparing for session, and not in a tutorial level, show message to prepare.
     * Example @see Flash ../games/riverRanger/TestTutor
     *
     * When rebeginning updating, force update to state if not in "none" or "outro" state.
     */
    update: function(timeConsumed)
    {
        this.timeConsumed = timeConsumed;
        if (!this.updating && 0 <= this.updatingAt && this.updatingAt <= timeConsumed) {
            if (this.state != "outro" && this.state != "none") {
                this.state = "none";
            }
            this.updating = true;
        }
        var state = "none";
        if (this.prepare && !this.active) {
            state = "prepare";
        }
        else if (this.active) {
            if (this.intro) {
                state = "intro";
            }
            else if (!this.updating) {
                state = this.state;
            }
            if (this.outro) {
                state = "outro";
            }
            else {
                state = this.getState.getState(state);
            }
        }
        /*-
        if (this.state == "none" && state == "outro")
        {
            state = this.state;
        }
        -*/
        this.change = state != this.state;
        if (this.change) {
            ll.log("Tutor.update: change from " + this.state + " to " + state 
                + " at " + timeConsumed);
            var old = this.state;
            this.state = state;
            if (this.active && "intro" != state) {
                this.intro = false;
            }
            this.changeView(old, state);
        }
        return state;
    },

    nextLabel: function()
    {
        if (this.currentLabel != this.state) {
            this.changeView(this.currentLabel, this.state);
        }
    },

    resume: function()
    {
        this.updatingAt = 0;
    },

    /**
     * Stop updating.
     */
    stop: function()
    {
        this.updating = false;
        this.updatingAt = -1;
    },

    /**
     * Workaround:  Delay updating for another half second after promptEnd animation has finished.
     *
     * GameView.playAnimation of promptBegin and promptEnd race sometimes sets prompt to invisible at end of promptBegin.
     * Your insight into why this race would block an animation's change of visibility would be appreciated.
     * Test case:  2014-10-09 Practice.  Correct.  Wrong.  Correct.  
     * Expect to fix to read next instruction.  Got promptBegin animates but prompt is still set to invisible.
     */
    promptEnd: function()
    {
        this.updatingAt = this.timeConsumed + 0.5;
        if (!this.currentLabel) 
        {
            this.currentLabel = "none";
        }
        this.state = this.currentLabel;
    },

    changeView: function(current, next)
    {
        var show = true;
        if (current != next) 
        {
            if (next == "none" || next == "outro" 
            || (this.showing && this.currentLabel == current)) 
            {
                show = false;
            }
            this.currentLabel = next;
            this.showing = show;
            this.showPrompt.showPrompt(show, "tutor_" + next);
        }
    }
});
