/*global require*/
/*global cc*/
/*global LumosityUtils*/

require( "src/Utility/LumosityUtils.js" );

/**
 * The Bird Formation settings.
 * @class
 * @name BirdFormationSettings
 */
var BirdFormationSettings;
( function()
{
	"use strict";

	BirdFormationSettings = cc.Class.extend(
	{
		/** @lends BirdFormationSettings.prototype */
        
        /**
         *  BirdFormationSettings is the main class for a bird formation settings!
         *
         *  @constructs
         *  @param {object} Settings.
         */
        ctor:function( settings )
        {
            var self = this;
        	// Copies all the properties of settings and makes them
        	// properties of this class.
        	LumosityUtils.forEachIn( settings, function( name, value )
        	{
        		self[ name ] = value;
        	});
        },

        /**
     	* Returns a random bird image rotation.
     	*
     	* @returns {int} Random bird image rotation.
     	*/
	    getRandomBirdImageRotation: function()
	    {
	    	var randomBirdImageRotation = this.birdImageRotations.getRandomElement();
	    	return randomBirdImageRotation;
	    },

        /**
        * Returns a random formation index.
        *
        * @returns {int} Random formation index.
        */
        getRandomFormationIndex: function()
        {
            var randomFormationIndex = Math.floor( Math.random() * this.formations.length );
            return randomFormationIndex;
        },

	    /**
     	* Returns a formation.
        *
        * @param {int} Index.
        * 
     	* @returns {array} Formation.
     	*/
	    getFormation: function( index )
	    {
	    	var randomFormation = this.formations[ index ];
	    	return randomFormation;
	    }
	} );
}() );
