/*global require*/
/*global GameSettings*/

require( "src/Utility/LumosityUtils.js" );

var MemoryMatchCard;
( function()
{
    "use strict";
    
    MemoryMatchCard = cc.Node.extend(
    {
        _CardBackTexture: "stimuli/cardBack.png",///memoryMatch_shapeCardBack.png",
        _CardFrontTexture: "stimuli/cardFront.png", //memoryMatch_shapeBackground.png",

        _CardCircleTexture: "stimuli/cardShapeCircle.png",
        _CardFlowerTexture: "stimuli/cardShapeFlower.png",
        _CardSparkleTexture: "stimuli/cardShapeSparkle.png",
        _CardSquareTexture: "stimuli/cardShapeSquare.png",
        _CardTriangleTexture: "stimuli/cardShapeTriangle.png",
        NUM_SHAPES:5,

        _cardTexture:null,

        _isFront: false,
        _currSide:null,
        _currShape:null,
        typeShape:0,

        _cardPos0:cc.p(-400,520),
        _cardPos1:cc.p(100,520),
        _cardPos2:cc.p(340,520),
        _cardPos3:cc.p(340,240),
        _cardPos4:cc.p(100,240),
        _cardPos5:cc.p(-400,240),

        _duration: 0.25,

        //need to make sure it is used for scaling and initalizing the cards
        _cardSize:cc.p(200,200),
        _isFlipped:false,

        //directions
        LEFT:1,
        RIGHT:2,

        stdSize: cc.p(640,960),

        //0 is off the screen, 1,2,3 is the slots, 4 is off the screen the 2nd time
        slot:0,
        position:null,

        ctor:function(slot, parent, setPos, oppositeSlotVal)
        {
            cc.log("Creating");
            this._super();

            var anchorPoint = cc.p(0.5, 0.5);

            this.nodeName = "MemoryMatchCard";
            
            this.slot = slot;

            this._cardTexture = new LumositySprite(this._CardBackTexture);
            this._cardTexture.setAnchorPoint(anchorPoint);
            this._currSide = this._cardTexture;

            var shape;
            var randShape = Math.floor(Math.random() * (this.NUM_SHAPES - 1)); //generates 0 - 4

            if(oppositeSlotVal != -1){
                var probPullDiffCard = Math.random();
                if( probPullDiffCard > 0.5){
                    this.typeShape = oppositeSlotVal; //matching card
                }
                else{ //don't match the cards
                    randShape = Math.floor(Math.random() * (this.NUM_SHAPES - 1)) + 1; //gen 1-4
                    this.typeShape = (oppositeSlotVal + randShape) % (this.NUM_SHAPES); 
                }
            }
            else{
                this.typeShape = randShape; //Random new card for initializing purposes
            }

            switch(this.typeShape){
                case 0:
                    shape = this._CardTriangleTexture;
                    break;
                case 1:
                    shape = this._CardCircleTexture;
                    break;
                case 2:
                    shape = this._CardSparkleTexture;
                    break;
                case 3:
                    shape = this._CardSquareTexture;
                    break;
                case 4:
                    shape = this._CardFlowerTexture;
                    break;
            }
            this._currShape = new LumositySprite(shape);

            this.position = this._getPosition(this.slot);
            this._setNewAnchorPoint(anchorPoint);
            
            if(setPos){
                this.setPosition(this.position.x + this._cardSize.x/2, this.position.y + this._cardSize.y/2 );
            }
            

            this.addChild(this._currSide);
            this.addChild(this._currShape);
            this._currShape.setVisible(false);

            var resolution = parent.getContentSize().height;
            if(parent.getContentSize().width < parent.getContentSize().height){
                resolution = parent.getContentSize().width;
            }

            if(setPos){
                this.resolutionScale(resolution);
            }

            //cc.log(this.getParent());
        },

        swapSides: function(){
            this._isFront = !this._isFront;
        },

        setToFront:function()
        {
            if(this._isFront){
                this._currSide.setTexture(this._CardFrontTexture);
                this._currShape.setVisible(true);
            }
            else{
                this._currSide.setTexture(this._CardBackTexture);
                this._currShape.setVisible(false);
            }
        },

        wrongCard: function(){
            if(this.slot === 5){
                return;
            }


            var seq1 = this.flipOnSpot();
            var seq2 = this.move();
            var delay = cc.DelayTime.create(this._duration);
            var finalSequence = cc.Sequence.create( delay, seq2 );

            if(!this._isFront){
                finalSequence = cc.Sequence.create( seq1, seq2 );
            }

            if(this.slot == 0){
                finalSequence = cc.Sequence.create( delay, seq2 );   
            }

            
            return finalSequence;
        },

        correctCard: function(){
            var seq1 = null;
            if(this.slot === 5){
                return;
            }

            switch(this.slot){
                case 0:
                    seq1 = cc.Sequence.create(cc.DelayTime.create(0.05/(1+this.slot)), this.move());
                    break;
                case 4:
                    seq1 = cc.Sequence.create(cc.DelayTime.create(0.1/(1+this.slot)), this.move());
                    break;
                case 1:
                    seq1 = cc.Sequence.create(cc.DelayTime.create(0.25/(1+this.slot)), this.flipAcross(this.RIGHT));
                    break;
                case 2:
                    seq1 = cc.Sequence.create(cc.DelayTime.create(0.25/(1+this.slot)), this.flipOnSpot(this._duration/2), this.move(this._duration/2));
                    break;
                case 3:
                    if(this._isFront){
                        seq1 = cc.Sequence.create(cc.DelayTime.create(0.25/(1+this.slot)), this.flipAcross(this.LEFT));
                    }
                    else{
                        seq1 = cc.Sequence.create(cc.DelayTime.create(0.25/(1+this.slot)), this.move());
                    }
                    break;
            }
            return seq1;
        },

        flipAcross: function(direction){
            var offset;

            //based of the flip need to set a different anchor point for the texture.
            var scaleDirectionX = cc.p(0, -1);
            var scaleDirectionY = cc.p(1, 1);
            if(this._isFlipped){
                scaleDirectionX = cc.p(0, 1);
                scaleDirectionY = cc.p(1, 1);
            }

            var shapeOffset;
            if(direction == this.LEFT){
                shapeOffset = -20;
            }
            else if(direction == this.RIGHT){
                shapeOffset = 20;
            }
            var totalFlippingDuration = this._duration;

            var initFlipAction = cc.CallFunc.create( this._setFlipCoordinates, this, direction );

            var flipAction1 = cc.ScaleTo.create( totalFlippingDuration/2, scaleDirectionX.x, scaleDirectionY.x);
            var flipAction1Ease = cc.EaseOut.create ( flipAction1, 5 );

            var swapSideAction = cc.CallFunc.create( this.swapSides, this );
            var textureAction = cc.CallFunc.create( this.setToFront, this );

            var flipAction2 = cc.ScaleTo.create( totalFlippingDuration/2,  scaleDirectionX.y, scaleDirectionY.y);
            var flipAction2Ease = cc.EaseIn.create ( flipAction2, 5 );

            var callbackAction = cc.CallFunc.create( this._onActionFinish, this );

            var actionSequence = cc.Sequence.create(  initFlipAction, flipAction1Ease, swapSideAction, textureAction, flipAction2Ease, callbackAction );
            return actionSequence;
        },

        _setFlipCoordinates: function(from, direction){
            var anchorPoint;
            var newPosition = this._getPosition(this.slot + 1);

            switch(direction){
                case this.LEFT:
                    anchorPoint = cc.p(1, 0.5);
                    if(!this._isFlipped){
                        anchorPoint = cc.p(0,0.5);
                        cc.log("AN1");
                    }
                    else{
                        anchorPoint = cc.p(1, 0.5);
                        cc.log("AN2");
                    }
                    this._setNewAnchorPoint(anchorPoint);
                    this.setPosition(newPosition.x + this._cardSize.x, newPosition.y + this._cardSize.y/2);
                    break;
                case this.RIGHT:
                    anchorPoint = cc.p(1, 0.5);
                    this._isFlipped = true;
                    this._setNewAnchorPoint(anchorPoint);
                    this.setPosition(newPosition.x, newPosition.y + this._cardSize.y/2);                    
                    break;
            }
            this._currShape.setPosition(this._currShape.getPosition().x, this._currShape.getPosition().y);
        },

        flipOnSpot: function(duration){
            duration = typeof duration !== 'undefined' ? duration : this._duration;

            var totalFlippingDuration = duration;

            var flipAction1 = cc.ScaleTo.create( totalFlippingDuration/2, 0, 1);
            var flipAction1Ease = cc.EaseOut.create ( flipAction1, 5 );

            var swapSideAction = cc.CallFunc.create( this.swapSides, this );
            var textureAction = cc.CallFunc.create( this.setToFront, this );

            var flipAction2 = cc.ScaleTo.create( totalFlippingDuration/2, 1, 1);
            var flipAction2Ease = cc.EaseIn.create ( flipAction2, 5 );

            var callbackAction = cc.CallFunc.create( this._onActionFinish, this );

            var actionSequence = cc.Sequence.create( flipAction1Ease, swapSideAction, textureAction, flipAction2Ease );
            return actionSequence;
        },



        move: function(duration){
            duration = typeof duration !== 'undefined' ? duration : this._duration;
            var newPosition = this._getPosition(this.slot + 1);
            var point = cc.p(newPosition.x - this.position.x, newPosition.y - this.position.y);

            var moveToAction1 = cc.MoveBy.create(duration, point);
            var moveActionPart1Ease = cc.EaseIn.create ( moveToAction1, 2 );
            var callbackAction = cc.CallFunc.create( this._onActionFinish, this );
            var sequence = cc.Sequence.create(moveActionPart1Ease, callbackAction);

            return sequence;
        },

        _onActionFinish: function(cls){
            this.slot = this.slot + 1;
            this.position = this._getPosition(this.slot);
            this._setNewAnchorPoint(cc.p(0.5,0.5));
            this.setPosition(this.position.x + this._cardSize.x/2, this.position.y + this._cardSize.y/2);
        },

        _setNewAnchorPoint: function(point){
            var anchorPoint = point;
            this.setAnchorPoint(cc.p(0,0));
            this._currSide.setAnchorPoint(anchorPoint);
            this._currShape.setAnchorPoint(anchorPoint);
        },
        _getPosition: function(slot){
            var newPosition;
            switch(slot){
                case 0:
                    newPosition = this._cardPos0;
                    break;
                case 1:
                    newPosition = this._cardPos1;
                    break;
                case 2:
                    newPosition = this._cardPos2;
                    break;
                case 3:
                    newPosition = this._cardPos3;
                    break;
                case 4:
                    newPosition = this._cardPos4;
                    break;
                case 5:
                    newPosition = this._cardPos5;
                    break;
            }
            return newPosition;
        },

        moveOffScreen: function(){
            var seq1;

            var seq2;
            var duration = 0.6;

            if(this._isFront){
                seq1 = this.flipOnSpot(duration/4);
            }
            else{
                seq1 = cc.DelayTime.create(duration/3.30);
            }   

            if(this.slot == 1 || this.slot == 2 || this.slot == 0){
                seq2 = cc.MoveBy.create(duration, cc.p(0, 800));
            }
            else{
                seq2 = cc.MoveBy.create(duration, cc.p(0, -800));   
            }

            return cc.Sequence.create(seq1, seq2);            
        },

        revertPosition: function(){
            this.setPosition(this.position.x + this._cardSize.x/2, this.position.y + this._cardSize.y/2 );
        },

        resolutionScale: function(resolution){
           //will need to scale this based on the resolution
            var scale = 1;
            cc.log("scaling cards");
            cc.log(resolution);
            if(resolution < 480){ scale = 1.6; } //tested
            else if(resolution >= 480 && resolution <= 640){scale = 1; }  //400
            else if(resolution > 640 && resolution < 800){scale = 0.8; } //640
            else if(resolution >= 800 && resolution < 1200){ scale = 0.53; } //800
            else if(resolution >= 1200 && resolution < 1600){scale = 0.4; } //1200
            else if(resolution >= 1600){scale = 0.2; } //1600

            this._currShape.setScale(scale);
            this._currSide.setScale(scale);
        }

    } );
}() );