/*global cc*/
/*global require*/
/*global GameView*/
/*global GameWidgetFeedback*/
/*global GameWidgetFlipbook*/
/*global GameWidgetHint*/
/*global LayoutLoader*/
/*global HintOrientation*/
/*global localize*/
/*global GameModel*/
/*global LumosityUtils*/

var MemoryMatchModel;
(function ()
{

    "use strict";

    require( "src/Game/Base/GameModel.js" );
    require( "src/Utility/LumosityUtils.js" );

    MemoryMatchModel = GameModel.extend(
    {
        ctor:function()
        {
            this.trialStartTime = 0;
            this.scoreMultiplier = 1;
            this.consecutiveCorrects = 0;
            this.consecutiveIncorrects = 0;
            this.num_match = 0;
            this.response_time_match = 0;
            this.response_time_no_match = 0;
            this.reactionTimes = [];
            this.numPips = 0;
            this.num_tries = 0;
            this.bestStatKey = "cards";
            this.hint  = true;
            this.prevHint = false;
            this.num_no_hint = 0;
            this.num_no_hint_correct = 0 ;
            this.response_time_no_hint_correct = 0;

            /* Metadata Trial Object */
            this.currentTrial = {
                correct: null,      // Was the trial correctly answered? [T/F]
                multiplier: 0,      // Multiplier applied to the current score increment
                response_time: 0,   // Response time in milliseconds
                time_offset: 0,     // At start of trial, number of milliseconds since beginning of game
                shape1: "",
                shape2: "",
                shape3: "",
                shape4: "",
                type: 0
            };
        },

        findAverageReactionTimeMatch: function()
        {
            return this._findAverageReactionTime( this.responseTimesMatch );
        },
        
        findAverageReactionTimeNoMatch: function()
        {
            return this._findAverageReactionTime( this.responseTimesNoMatch );
        },

        markTrialStartTime: function()
        {
            this.trialStartTime = lumosity.game.frametime;
        },

        markTrialFinishTime: function()
        {
            this.reactionTimes.push( lumosity.game.frametime - this.trialStartTime );
        },

        //@Override
        exportResults:function()
        {
            var netCorrects = this.num_match - (this.num_tries - this.num_match);

            if(netCorrects < 0){
                netCorrects = 0;
            }
            var stats = {
                cards: this.num_match,
                total: this.num_tries,
                netCorrect: netCorrects
            };

            if( this.num_tries )
            {
                this.avg_level = (this.avg_level - 1) / this.num_tries;
            }

            this.metadata.num_match = this.num_match;
            this.metadata.num_tries = this.num_tries;
            this.metadata.num_no_hint = this.num_no_hint;
            this.metadata.num_no_hint_correct = this.num_no_hint_correct;

            var averageResponseTimeMatch = Math.round(this.response_time_match / this.num_match);
            var averageResponseTimeNoMatch = Math.round(this.response_time_no_match / (this.num_tries - this.num_match));       
            var averageResponseTimeNoHintMatch = Math.round(this.response_time_no_hint_correct / this.num_no_hint_correct);

            var bonusPerMultiplier = 250;
            this.metadata.end_bonus = this.scoreMultiplier * bonusPerMultiplier;
            this.score = this.score + this.metadata.end_bonus;

            this.metadata.response_time_match = averageResponseTimeMatch;
            this.metadata.response_time_no_match = averageResponseTimeNoMatch;
            this.metadata.response_time_no_hint_correct = averageResponseTimeNoHintMatch;

            //division by zero cases
            if(this.num_match == 0){ this.metadata.response_time_match = 0;}
            if(this.num_match == this.num_tries){ this.metadata.response_time_no_match = 0;}
            if(this.num_no_hint_correct == 0){ this.metadata.response_time_no_hint_correct = 0;}



            return this._super( stats, this.bestStatKey );
        },

        //@Override
        createGameResultsData: function ( stats, bestStatKey )
        {
            var results = this._super( stats, bestStatKey );
            var metadata = this.metadata;

            if ( results && metadata )
            {
                //results[ metadataAggregateKeys.endLevel ] = metadata[ metadataAggregateKeys.endLevel ];
                //results[ metadataAggregateKeys.sessionLevel ] = metadata[ metadataAggregateKeys.sessionLevel ];
            }

            return results;
        },

        updateResponseTime: function(correct, userAnsweredYes, response_time){
            //swap to seconds
            var response_time = Math.round(lumosity.game.frametime - this.trialStartTime ) * 1000;

            if(correct){
                this.response_time_match += response_time;
                if(!this.hint){
                    this.response_time_no_hint_correct += response_time;
                }

            }
            else{
                this.response_time_no_match += response_time;
            }
            this.markTrialStartTime();
        }
    } );
}() );