/*global cc*/
/*global require*/
/*global Game*/
/*global MemoryMatchOverdriveSettings*/
/*global MemoryMatchOverdriveModel*/
/*global MemoryMatchOverdriveView*/

require( "src/Game/Base/Game.js" );
require( "src/Game/MemoryMatchOverdriveSettings.js" );
require( "src/Game/MemoryMatchOverdriveModel.js" );
require( "src/Game/MemoryMatchOverdriveView.js" );
require( "src/Game/MemoryMatch.js" );

var MemoryMatchOverdrive;
( function()
{
    "use strict";
    
    MemoryMatchOverdrive = MemoryMatch.extend(
    {
        /** @lends MemoryMatchOverdrive.prototype */
        
        /**
        *  MemoryMatchOverdrive is the main class for the game, Memory Match Overdrive!
        *
        *  @constructs
        */
        ctor: function()
        {
            this._super();
        },
        
        _createSettings: function()
        {
            return new MemoryMatchOverdriveSettings( "settings.json" );
        },
        
        _createModel: function()
        {
            return new MemoryMatchOverdriveModel();
        },
        
        _createView: function( screenSize )
        {
            return new MemoryMatchOverdriveView( screenSize, this.settings, this.model, this );
        },

        _recordTrialMetadata: function( correct, userAnsweredYes )
        {
            var delta_progress = this.model.updateResponseTime( correct, userAnsweredYes );

            var hinted = 1; //0 - no, 1 - yes, 2 - yes, on the previous trial"
            if(!this.model.hint){ //if hint is false 
                hinted = 0;
            }

            if(this.model.prevHint){ // previous trial is hinted
                hinted = 2;
            }

            var trial_stim = this.view.getSlotVal(1);
            var trial_back = this.view.getSlotVal(4);

            cc.log(trial_stim == trial_back);

            var trial =
            {
                stimulus : trial_stim,
                back : trial_back,
                response : userAnsweredYes ? "T" : "F",
                match : this.view.doCardsMatch() ? "T" : "F",    
                correct : correct ? "T" : "F",
                multiplier : parseInt(this.trial_multiplier),
                progress : parseInt(this.trial_progress),
                hint : hinted
            };
            
            this.model.prevHint = this.model.hint;
            this.model.hint = !correct; //if correct means next trial doesn't have hints, else next trial will be hinted

            this.endTrial( trial, correct, false, false, false, false, true );
            this.beginTrial();
        }
    } );
}() );