/*global cc*/
/*global require*/
/*global GameView*/

require( "src/Game/Base/GameView.js" );
require( "src/Game/MemoryMatchView.js" );
require( "src/Game/MemoryMatchOverdriveCard.js" );

var MemoryMatchOverdriveView;
( function()
{
    "use strict";
    
    MemoryMatchOverdriveView = MemoryMatchView.extend(
    {
        /** @lends MemoryMatchOverdriveView.prototype */
        
        /**
        * The Memory Match Overdrive view.
        * @constructs
        * @param {cc.Size} screenSize The size of the screen that this view will be inserted into.
        * @param {Object} [settings] The settings that we can pull view-based data from.
        * @param {MemoryMatchOverdriveModel} [model] The GameModel that we are linked to.
        * @param {Object} [interactionResponder] The object that we should send interaction events to. We also use this is a default animation event listener.
        * @param {boolean} [preload=true] If true, will preload all flipbooks and other assets upon creation.
        */

        ctor:function( screenSize, settings, model, interactionResponder, preload )
        {
            this._super( screenSize, settings, model, interactionResponder, preload );
        },

        initCards:function(){
            //TODO: fix the z order stuff
            var card0 = new MemoryMatchOverdriveCard(0, this, true, this.RANDOM_SHAPE);
            this._cards.push(card0);
            this.centeredContent.addChild(card0, 20);
            card0.retain();
            this._contentSize = this.getContentSize();
            card0.setVisible(false);

            var card1 = new MemoryMatchOverdriveCard(4, this, false, this.RANDOM_SHAPE);
            card1.setPosition(card1.getPosition().x + card1._currSide.getContentSize().width/2, 
                                card1.getPosition().y + card1._currSide.getContentSize().height/2);
            this.Card1_Transition.addChild(card1);
            this._cards.push(card1);
            card1.retain();

            var card2 = new MemoryMatchOverdriveCard(3, this, false, this.RANDOM_SHAPE);
            card2.setPosition(card2.getPosition().x + card2._currSide.getContentSize().width/2, 
                                card2.getPosition().y + card2._currSide.getContentSize().height/2);
            this.Card2_Transition.addChild(card2);
            this._cards.push(card2);
            card2.retain();

            var card3 = new MemoryMatchOverdriveCard(2, this, false, this.RANDOM_SHAPE);
            card3.setPosition(card3.getPosition().x + card3._currSide.getContentSize().width/2, 
                                card3.getPosition().y + card3._currSide.getContentSize().height/2);
            this.Card3_Transition.addChild(card3);
            this._cards.push(card3);
            card3.retain();

            var card4 = new MemoryMatchOverdriveCard(1, this, false, this.RANDOM_SHAPE);
            card4.setPosition(card4.getPosition().x + card4._currSide.getContentSize().width/2, 
                                card4.getPosition().y + card4._currSide.getContentSize().height/2);
            this.Card4_Transition.addChild(card4);
            this._cards.push(card4);
            card4.retain();

            //TODO: FIX Z Orders
            this.cardHighlightTop.setZOrder(19);
            this.cardHighlightBottom.setZOrder(19);
        },

        //Override
        addNewCard: function(){
            var card;
            var match = Math.random();

            card = new MemoryMatchOverdriveCard(0, this, true, this.getSlotVal(2));//slot 2 because the card in slot 0 is generated already think of slot -1
            this._cards.push(card);
            card.retain();

            //TODO: Fix Z order
            this.centeredContent.addChild(this._cards[this._cards.length-1], 20);
        },


        doCardsMatch: function(){
            //0 1 2 slot layout
            //6 4 3
            var i;
            var slot1;
            var slot4;
            for(i = 0; i < this._cards.length; i++){
                if(this._cards[i].slot == 1){
                    slot1 = this._cards[i].typeShape;
                }
                else if(this._cards[i].slot == 4){
                    slot4 = this._cards[i].typeShape;
                }
            }
            return slot1 == slot4;
        },

        flipInitCards: function(){
            var i;
            for(i = 0; i < this._cards.length; i++){
                if(this._cards[i].slot !== 0 && this._cards[i].slot !== 5){
                    var seq1 = cc.DelayTime.create(0.25);
                    var animationCompletedAction = cc.CallFunc.create( this._cardsAnimationFinished, this);
                    this._cards[i].runAction(cc.Sequence.create(seq1, this._cards[i].flipOnSpot(), animationCompletedAction));
                    this.animCompleted = false;
                    this._animCount += 1;
                }
            }
        }
    } );
}() );
