/*global require*/
/*global GameSettings*/

require( "src/Utility/LumosityUtils.js" );
require( "src/Game/MemoryMatchCard.js" );

var MemoryMatchOverdriveCard;
( function()
{
    "use strict";
    
    MemoryMatchOverdriveCard = MemoryMatchCard.extend(
    {
        ctor:function(slot, parent, setPos, oppositeSlotVal){
            this._super(slot, parent, setPos, oppositeSlotVal);
        },

        wrongCard: function(){
            //this.setPosition(this.position.x, this.position.y);
            if(this.slot === 5){
                return;
            }

            var seq1 = this.flipOnSpot();
            var seq2 = this.move();
            var delay = cc.DelayTime.create(this._duration);
            var finalSequence = cc.Sequence.create( delay, seq2 );

            if(!this._isFront){
                finalSequence = cc.Sequence.create( seq1, seq2 );
            }
            return finalSequence;
        },

        //this is different because the texture that is modified is the same texture
        //it's different from MM because MM modifies the same texture
        flipAcross: function(direction){
            var offset;
            var scaleDirectionX = cc.p(0, -1);
            var scaleDirectionY = cc.p(1, 1);

            var shapeOffset;
            if(direction == this.LEFT){
                shapeOffset = -20;
            }
            else if(direction == this.RIGHT){
                shapeOffset = 20;
            }
            var totalFlippingDuration = this._duration;

            var initFlipAction = cc.CallFunc.create( this._setFlipCoordinates, this, direction );

            var flipAction1 = cc.ScaleTo.create( totalFlippingDuration/2, scaleDirectionX.x, scaleDirectionY.x);
            var flipAction1Ease = cc.EaseOut.create ( flipAction1, 5 );

            var swapSideAction = cc.CallFunc.create( this.swapSides, this );
            var textureAction = cc.CallFunc.create( this.setToFront, this );

            var flipAction2 = cc.ScaleTo.create( totalFlippingDuration/2,  scaleDirectionX.y, scaleDirectionY.y);
            var flipAction2Ease = cc.EaseIn.create ( flipAction2, 5 );

            var callbackAction = cc.CallFunc.create( this._onActionFinish, this );

            var actionSequence = cc.Sequence.create(  initFlipAction, flipAction1Ease, swapSideAction, textureAction, flipAction2Ease, callbackAction );
            return actionSequence;
        },

        _setFlipCoordinates: function(from, direction){
            var anchorPoint;
            var newPosition = this._getPosition(this.slot + 1);

            switch(direction){
                case this.LEFT:
                    anchorPoint = cc.p(0, 0.5);
                    this._setNewAnchorPoint(anchorPoint);
                    this.setPosition(newPosition.x + this._cardSize.x, newPosition.y + this._cardSize.y/2);
                    break;
                case this.RIGHT:
                    anchorPoint = cc.p(1, 0.5);
                    this._isFlipped = true;
                    this._setNewAnchorPoint(anchorPoint);
                    this.setPosition(newPosition.x, newPosition.y + this._cardSize.y/2);                    
                    break;
            }
            this._currShape.setPosition(this._currShape.getPosition().x, this._currShape.getPosition().y);
        },

        correctCard: function(){
            var seq1 = null;
            if(this.slot === 5){
                return;
            }

            switch(this.slot){
                case 0:
                    this._isFront = true;
                    this.setToFront();
                    seq1 = cc.Sequence.create(cc.DelayTime.create(0.05/(1+this.slot)), this.move());
                    break;
                case 4:
                    seq1 = cc.Sequence.create(cc.DelayTime.create(0.1/(1+this.slot)), this.move());
                    break;
                case 1:
                    seq1 = cc.Sequence.create(cc.DelayTime.create(0.25/(1+this.slot)), this.flipAcross(this.RIGHT));
                    break;
                case 2:
                    if(this._isFront){
                        seq1 = cc.Sequence.create(cc.DelayTime.create(0.25/(1+this.slot)), this.flipOnSpot(this._duration/2), this.move(this._duration/2));
                    }
                    else{
                        seq1 = cc.Sequence.create(cc.DelayTime.create(0.25/(1+this.slot)), this.move());
                    }
                    break;
                case 3:
                    if(this._isFront){
                        seq1 = cc.Sequence.create(cc.DelayTime.create(0.25/(1+this.slot)), this.flipAcross(this.LEFT));
                    }
                    else{
                        seq1 = cc.Sequence.create(cc.DelayTime.create(0.25/(1+this.slot)), this.move());
                    }
                    break;
            }
            return seq1;
        },



    } );
}() );