/*global cc*/
/*global require*/
/*global GameView*/



var MemoryMatchView;
( function()
{
    "use strict";

    require( "src/Game/Base/GameView.js" );
    require( "src/Game/Widget/GameWidgetFeedback.js" );
    require( "src/Game/Widget/GameWidgetFlipbook.js" );
    require( "src/Game/Widget/GameWidgetHint.js" );
    require( "src/Animation/LayoutLoader.js" );
    require( "src/Game/MemoryMatchCard.js" );
    
    MemoryMatchView = GameView.extend(
    {
        /** @lends MemoryMatchView.prototype */
        
        /**
        * The Memory Match view.
        * @constructs
        * @param {cc.Size} screenSize The size of the screen that this view will be inserted into.
        * @param {Object} [settings] The settings that we can pull view-based data from.
        * @param {MemoryMatchModel} [model] The GameModel that we are linked to.
        * @param {Object} [interactionResponder] The object that we should send interaction events to. We also use this is a default animation event listener.
        * @param {boolean} [preload=true] If true, will preload all flipbooks and other assets upon creation.
        */
        _blueTextColor: new cc.c3b( 48, 191, 235 ),
        _buttonFont: "fonts/MuseoSans-500.ttf",
        _buttonTextSize: 48,
        _buttonTextDownColor: cc.WHITE,
        hintColor: null,
        hintFont: null,
        _contentSize: null,
        _gameEnded: false,
        animCompleted: true,
        //-----------------------------------------------------------------

        _cards:[],

        _animCount:0,

        RANDOM_SHAPE:-1,

        ctor:function( screenSize, settings, model, interactionResponder, preload )
        {
            this._super( screenSize, settings, model, interactionResponder, preload );
        },

        initCards:function(){
            //TODO: fix the z order stuff
            var card0 = new MemoryMatchCard(0, this, true, this.RANDOM_SHAPE);
            this._cards.push(card0);
            this.centeredContent.addChild(card0, 20);
            card0.retain();
            this._contentSize = this.getContentSize();
            card0.setVisible(false);

            var card1 = new MemoryMatchCard(4, this, false, this.RANDOM_SHAPE);
            card1.setPosition(card1.getPosition().x + card1._currSide.getContentSize().width/2, 
                                card1.getPosition().y + card1._currSide.getContentSize().height/2);
            this.Card1_Transition.addChild(card1);
            this._cards.push(card1);
            card1.retain();

            var card2 = new MemoryMatchCard(3, this, false, this.RANDOM_SHAPE);
            card2.setPosition(card2.getPosition().x + card2._currSide.getContentSize().width/2, 
                                card2.getPosition().y + card2._currSide.getContentSize().height/2);
            this.Card2_Transition.addChild(card2);
            this._cards.push(card2);
            card2.retain();

            var card3 = new MemoryMatchCard(2, this, false, this.RANDOM_SHAPE);
            card3.setPosition(card3.getPosition().x + card3._currSide.getContentSize().width/2, 
                                card3.getPosition().y + card3._currSide.getContentSize().height/2);
            this.Card3_Transition.addChild(card3);
            this._cards.push(card3);
            card3.retain();

            var card4 = new MemoryMatchCard(1, this, false, this.RANDOM_SHAPE);
            card4.setPosition(card4.getPosition().x + card4._currSide.getContentSize().width/2, 
                                card4.getPosition().y + card4._currSide.getContentSize().height/2);
            this.Card4_Transition.addChild(card4);
            this._cards.push(card4);
            card4.retain();

            //TODO: FIX Z Orders
            this.cardHighlightTop.setZOrder(19);
            this.cardHighlightBottom.setZOrder(19);
        },

        enableButtons:function( isEnabled )
        {
            this.no.setEnabled( isEnabled );
            this.yes.setEnabled( isEnabled ); 
        },

        _cardsAnimationFinished: function(){
            this._animCount -= 1;
            cc.log(this.animCompleted);
            //TODO: Make this better
            if(this._animCount <= 0){
                this.animCompleted = true;


                if(this._gameEnded){
                    this.gameEndAnimation();
                }
            }
        },

        //@Override
        layout_readingData: function( nodeName, nodeData )
        {
            var node = this._super( nodeName, nodeData );
            if ( node )
            {
                return node;
            }

            switch( nodeName )
            {
                case "hintTextColor":
                    this.hintColor = nodeData.textData.color;
                    this.hintFont = lu.getFontLocation( nodeData.textData.face );
                    return false;
            }
        },

        //@Override
        layout_creatingNode: function( nodeName, nodeData )
        {
            var node = this._super( nodeName, nodeData );
            if ( node )
            {
                return node;
            }

            switch( nodeName )
            {
                case "yes":
                    return this._createButton( "buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "yes", localize ( "Yes_Label" ),
                                                this._buttonFont, this._buttonTextSize, this._blueTextColor, this._buttonTextDownColor,
                                                nodeData[ LayoutLoader.sizeKey ] );
                case "no":
                    return this._createButton( "buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "no",  localize ( "No_Label" ), 
                                                this._buttonFont, this._buttonTextSize, this._blueTextColor, this._buttonTextDownColor,
                                                nodeData[ LayoutLoader.sizeKey ] );
                case "timeLabel":
                    this.hud.setTextColor( nodeData.textData.color );
                    return false;

                case "scoreLabel":
                    this.hud.setTextColor( nodeData.textData.color );
                    return false;

                case "hudBackgroundColor":
                    this.hud.setBackgroundColor( nodeData.textData.color );
                    return false;
                case "hudTextColor":
                    this.hud.setTextColor( nodeData.textData.color );
                    return false;
                case "cardHighlightTop":
                case "cardHighlightBottom":
                    return this._createSprite("cardHighlight.png");
                case "Card1_Transition":
                case "Card2_Transition":
                case "Card3_Transition":
                case "Card4_Transition":
                    return;
                case "motionTrack_StraightBottom":
                case "motionTrack_StraightTop":
                    return this._createSprite("motionTrack_Straight.png");
                case "motionTrack_Curve":
                    return this._createSprite("motionTrack_Curve.png");
                case "cardSlot1":
                case "cardSlot2":
                case "cardSlot3":
                case "cardSlot4":
                    return this._createSprite("stimuli/cardSlot.png");
                default:
                    return this._super( nodeName, nodeData );

            }
            return null;
        },

        flipCard: function(){
            //this.card.flip();
        },

        cardsToNextPosition: function(correct){
            var i;
            var length = this._cards.length;
            
            var runSequence;
            var tempSeq;

            if(this.no.getIsEnabled() && this.yes.getIsEnabled()){
                for(i = length-1; i >= 0; i--){
                    if(this._cards[i].slot >= 5){
                        this.centeredContent.removeChild(this._cards[i]);
                        this._cards.splice(this._cards.indexOf(this._cards[i]), 1);
                    }
                    else{
                        if(this._cards[i].slot == 0){this._cards[i].setVisible(true);}

                        //TODO: Tidy this stuff up
                        var animationCompletedAction = cc.CallFunc.create( this._cardsAnimationFinished, this);
                        var cardSequence;
                        if(correct){
                            cardSequence = this._cards[i].correctCard();
                        }
                        else{                            
                            cardSequence = this._cards[i].wrongCard();
                        }

                        if(cardSequence){
                                runSequence = cc.Sequence.create(cardSequence, animationCompletedAction);
                                this._cards[i].runAction(runSequence);
                                this._animCount += 1; //keeps track of how many cards are flipping/moving
                                this.animCompleted = false;
                        }
                    }
                }
            }
            //this.enableButtons(false);
        },

        addNewCard: function(){
            var card;
            
            //this is the corresponding card slot for Slot2
            card = new MemoryMatchCard(0, this, true, this.getSlotVal(1)); //it's one because the next card is already generated, think of slot -1 instead
            this._cards.push(card);
            card.retain();

            //TODO: Fix Z order
            this.centeredContent.addChild(this._cards[this._cards.length-1], 20);
        },

        pauseHide: function(){
            this.motionTrack_StraightBottom.setVisible(false);
            this.motionTrack_StraightTop.setVisible(false);
            this.motionTrack_Curve.setVisible(false);

            var i;
            for( i = 0; i < this._cards.length; i++){
                if(this._cards[i].slot == 0 || this._cards[i].slot == 5){
                    this._cards[i].setVisible(false);
                }
            }
        },

        resumeShow: function(){
            //Sequences can only be ran once so it is necessary to create 3 different ones
            var seq1 = cc.Sequence.create(cc.DelayTime.create(0.3), cc.CallFunc.create(this.makeVisible, this), cc.FadeOut.create(0), cc.FadeIn.create(0.25));//cc.CallFunc.create(this.makeVisible, this));
            var seq2 = cc.Sequence.create(cc.DelayTime.create(0.3), cc.CallFunc.create(this.makeVisible, this), cc.FadeOut.create(0), cc.FadeIn.create(0.25));//cc.CallFunc.create(this.makeVisible, this));
            var seq3 = cc.Sequence.create(cc.DelayTime.create(0.3), cc.CallFunc.create(this.makeVisible, this), cc.FadeOut.create(0), cc.FadeIn.create(0.25));//cc.CallFunc.create(this.makeVisible, this));

            this.motionTrack_StraightBottom.runAction(seq1);
            this.motionTrack_StraightTop.runAction(seq2);
            this.motionTrack_Curve.runAction(seq3);
        },

        makeVisible: function(){
            this.motionTrack_StraightBottom.setVisible(true);
            this.motionTrack_StraightTop.setVisible(true);
            this.motionTrack_Curve.setVisible(true);

            //TODO: Could be better
            var i;
            for( i = 0; i < this._cards.length; i++){
                this._cards[i].setVisible(true);
            }
        },

        doCardsMatch: function(){
            //0 1 2 slot layout
            //6 4 3
            var i;
            var slot2;
            var slot4;
            for(i = 0; i < this._cards.length; i++){
                if(this._cards[i].slot == 2){
                    slot2 = this._cards[i].typeShape;
                }
                else if(this._cards[i].slot == 4){
                    slot4 = this._cards[i].typeShape;
                }
            }
            return slot2 == slot4;
        },

        flipAllCards: function(){
            this._gameEnded = true;
            if(this._animCount <= 0){
                this.gameEndAnimation();
            }
        },

        hideInitCards: function(){
            this.Card1_Transition.setVisible(false);
            this.Card2_Transition.setVisible(false);
            this.Card3_Transition.setVisible(false);
            this.Card4_Transition.setVisible(false);

            this.Card1_Transition.removeAllChildren();
            this.Card2_Transition.removeAllChildren();
            this.Card3_Transition.removeAllChildren();
            this.Card4_Transition.removeAllChildren();

            var i = 0;
            for(i = 0; i < this._cards.length; i++){
                if(this._cards[i].slot !== 0 && this._cards[i].slot !== 5){
                    this._cards[i].revertPosition();
                    this._cards[i].resolutionScale(this.getContentSize().width);
                    this.centeredContent.addChild(this._cards[i], 20);
                    this._cards[i].setVisible(true);
                }
            }
        },

        gameEndAnimation: function(){
            var i;
            for(i = this._cards.length-1; i >= 0; i--){
                var actionSeq = this._cards[i].moveOffScreen();
                var runSequence = cc.Sequence.create(actionSeq);
                this._cards[i].runAction(runSequence);
            }
        },

        removeAllCards: function(){
            var i;
            for(i = 0; i < this._cards.length; i++){
                this.centeredContent.removeChild(this._cards[i]);
            }  
            this._cards = [];
        },

        flipInitCards: function(){
            var i;
            for(i = 0; i < this._cards.length; i++){
                if(this._cards[i].slot !== 0 && this._cards[i].slot !== 5 && this._cards[i].slot !== 1){
                    var seq1 = cc.DelayTime.create(0.25);
                    var animationCompletedAction = cc.CallFunc.create( this._cardsAnimationFinished, this);
                    this._cards[i].runAction(cc.Sequence.create(seq1, this._cards[i].flipOnSpot(), animationCompletedAction));
                    this.animCompleted = false;
                    this._animCount += 1;
                }
            }
        },

        getSlotVal: function(slotVal){
            var retVal = -1;
            for(var i = 0; i < this._cards.length; i++){
                if(this._cards[i].slot == slotVal){
                    retVal = this._cards[i].typeShape;
                }
            }
            return retVal;
        }
    } );
}() );