var cc = cc;
var require = require;

require( "src/Entity/AnimatedEntityView.js" );
var AnimatedEntityView = AnimatedEntityView;

var Bump = AnimatedEntity.extend(
{
    /** @lends PenguinCharacter.prototype */

    FRONT_ANIM_NAME:"bumpAnimationFront",
    SIDE_ANIM_NAME:"bumpAnimationSide",

    mazeView:null,

    /**
     *  A bump is the logic behind that wiggly little bump.
     *  Subclass of AnimatedEntity.
     */
    ctor:function( settings )
    {
        var view = new AnimatedEntityView( "penguin/bump", [ this.FRONT_ANIM_NAME, this.SIDE_ANIM_NAME ], null, true );
        this._super( view );
    },

    setMazeView:function( mazeView )
    {
        this.mazeView = mazeView;
    },

    play:function( gridPos, direction, worldDirection, mazeDownDirection, penguinMazeOffset )
    {
        var scaleX = Math.abs( this.view.getScaleX() ) * ( ( worldDirection == MazeDirections.Left ) ? -1 : 1 );
        var animName = ( worldDirection == MazeDirections.Up || worldDirection == MazeDirections.Down ) ? this.FRONT_ANIM_NAME : this.SIDE_ANIM_NAME;

        this.playAnimation( animName );
        this.view.setScaleX( scaleX );

        this.view.mazeOffset = cc.pMult( MazeConstants.positionDeltas[mazeDownDirection],
            ( 1.0 - this.mazeView._wallThickness / this.mazeView._tileSize ) * 0.1 );
        this.view.mazeOffset = cc.pAdd( this.view.mazeOffset, penguinMazeOffset );

        this.view.setGridPosition( gridPos );
        this.view.setRotation( -MazeConstants.directionAngles[mazeDownDirection] * 180.0 / Math.PI - 90 );
    },

    animatedEntity_finishedAnimation:function( animationName )
    {
    },

    /**
     * Pauses bump behavior.
     */
    pause:function()
    {
        this.view.pause();
    },

    /**
     * Resumes bump behavior.
     */
    resume:function()
    {
        this.view.resume();
    },

    /**
     * Sets bump up for pre-trial phase.
     */
    startPreTrial:function()
    {
        this.view.setVisible( true );
    },

    /**
     * Updates bump at end of trial.
     */
    endTrial:function()
    {
        this.view.setVisible( false );
    }
});