var cc = cc;
var require = require;

require( "src/Game/Maze/MazeConstants.js" );
var MazeConstants = MazeConstants;
var MazeDirections = MazeDirections;

var Room = cc.Class.extend(
{
    /** @lends Room.prototype */

    /**
     * The directions in this <code>Room</code> that are open.
     */
    _openInDirection:[],

    /**
     * A <code>Room</code> is a node subclass that contains 4 bools determining where it has
     * walls. These walls can be at the north, south, east, or west. By default, a room is entirely closed.
     * It also contains sprites to draw its own walls.
     *
     * @constructs
     */
    ctor:function()
    {
        this._openInDirection = new Array( MazeDirections.Count );
        var direction;
        for ( direction = 0; direction < MazeDirections.Count; direction++ )
        {
            this._openInDirection[ direction ] = false;
        }
    },
    
    /**
     * Returns whether or not this Room is open in a certain direction.
     *
     * @param {number} direction    The direction to check.
     * @returns {boolean} True or False if this room is open in that direction.
     */
    isOpen:function( direction )
    {
        return !!this._openInDirection[ direction ];
    },
    
    /**
     * Sets whether or not the Room is open in the specified direction.
     *
     * @param {MazeDirections} direction The direction to set.
     * @param {boolean} isOpen Whether it is open or not.
     */
    setIsOpen:function( direction, isOpen )
    {
        this._openInDirection[ direction ] = !!isOpen;
    },
    
    /**
     *  Returns a new array that represents the non-open directions in this Room.
     *  @returns {Array} All directions that are closed.
     */
    createClosedArray:function()
    {
        var arr = [];
        var direction;
        for ( direction = 0; direction < MazeDirections.Count; direction++ )
        {
            if ( !this.isOpen( direction ) )
            {
                arr.push( direction );
            }
        }
        return arr;
    }
});