var cc = cc;
var require = require;

require( "src/Game/PenguinCharacter.js" );
var PenguinCharacter = PenguinCharacter;

require( "src/Game/PenguinCharacterView.js" );
var PenguinCharacterView = PenguinCharacterView;

require( "src/Utility/LumosityUtils.js" );

var PenguinCharacterAI = PenguinCharacter.extend(
{
    /** @lends PenguinCharacterPlayer.prototype */
    
    /** An array of coordinates representing the path this AI will take. */
    _path:[],

    _activeInTrial:true,
    
    /**
     *  A subclass of PenguinCharacter that solves the maze with A*
     *  and then continually moves along the solved path.
     *
     *  @constructs
     *  @param model    The game model.
     *  @param settings The game settings.
     */
    ctor: function( model, settings )
    {
        var view = new PenguinCharacterView( settings.penguinTintComputer );
        view.id = "computerPenguinView";
        this._super( view, false, model, settings.sounds.movementComputer );
    },

    setActiveInTrial:function( active )
    {
        this._activeInTrial = active;
        this.view.setVisible( active );
    },

    /**
     * Called when penguin begins a trial. Overrides base method.
     *
     * @param trial     Current trial data.
     */
    startTrial:function( trial )
    {
        this._super( trial );
        if( this._activeInTrial )
        {
            // setup penguin's path
            var penguinPath = trial.maze.getSolution().reverse();
            if( penguinPath )
            {
                this._path = [];
                this._path.addAll( penguinPath );
                this._path.pop(); // remove the penguin's starting position
            }
        }
    },

    /**
     *  Allows this AI to move along its path. Will continue moving until
     *  all points in the path have been popped.
     *
     */
    beginMoving:function()
    {
        if( this._activeInTrial && this._path )
        {
            this._moveToNextTarget(); // comment out this line to make the ai stay put
        }
    },

    /**
     * Get the remaining path to destination.
     *
     * @returns {*}     List of path grid spaces not yet visited by penguin.
     */
    getRemainingPath:function()
    {
        return this._path;
    },

    /**
     *  An AI has reached its target if its path is empty. Therefore, this will
     *  always be true unless beginMoving() has been called with a path,
     *  so be careful.
     *
     *  @returns {boolean} True if there is no path left to pursue.
     */
    hasReachedTarget:function()
    {
        return this._path.length <= 0;
    },
    
    /**
     *  Move to our next target position if we can. If we don't have one, do nothing.
     */
    _moveToNextTarget:function()
    {
        if( this._activeInTrial && this._movementEnabled && !this.hasReachedTarget() )
        {
            var targetGridPos = this._path[ this._path.length - 1 ];
            var direction = MazeUtility.getDirectionBetween( this._gridPosition, targetGridPos );

            this.moveToGridPosition( targetGridPos );
            return this.movementStarted( targetGridPos, direction );
        }
    },

    /**
     * Overridable callback invoked after penguin has completed moving to his destination.
     */
    movementFinished:function()
    {
        this._path.pop();

        this._super();

        this._moveToNextTarget();

        this.updateMovementSounds();
    }
});