/*global cc*/
/*global require*/
/*global localize*/
/*global Game*/
/*global PetDetectiveSettings*/
/*global PetDetectiveModel*/
/*global PetDetectiveView*/
/*global StageSelect*/
/*global StageSelectButtonState*/
/*global LumosityButtonState*/
/*global Street*/
/*global PracticeStreet*/
/*global GameComponentScore*/
/*global GameComponentTimeBonus*/
/*global LumosityUtils*/
/*global lumosity*/
/*global metadataTrialKeys*/

require( "src/Game/Base/Game.js" );
require( "src/Game/PetDetectiveSettings.js" );
require( "src/Game/PetDetectiveModel.js" );
require( "src/Game/PetDetectiveView.js" );
require( "src/Game/Street.js" );
require( "src/Game/PracticeStreet.js" );
require( "src/Utility/LumosityUtils.js" );

var PetDetective;
( function()
{
    "use strict";    
    PetDetective = Game.extend(
    {
        street: null,

        /** @lends PetDetective.prototype */
        
        /**
        *  PetDetective is the main class for the game, Pet Detective!
        *
        *  @constructs
        */
        ctor: function()
        {
            this._super();

            this.configureMetaDataKeys();

            this.view.addDebugMenuButton( { labelText: "Free Movement", callback: this.debugSetFreeVehicleMovement.bind( this ), target: this, closeMenuOnCallback: true } );
            this.view.addDebugMenuButton( { labelText: "Reached Level 5", callback: this.debugSetCurrentStage.bind( this, 5 ), target: this, closeMenuOnCallback: true } );
            this.view.addDebugMenuButton( { labelText: "Reached Level 10", callback: this.debugSetCurrentStage.bind( this, 10 ), target: this, closeMenuOnCallback: true } );
            this.view.addDebugMenuButton( { labelText: "Reached Level 15", callback: this.debugSetCurrentStage.bind( this, 15 ), target: this, closeMenuOnCallback: true } );
            this.view.addDebugMenuButton( { labelText: "Reached Level 20", callback: this.debugSetCurrentStage.bind( this, 20 ), target: this, closeMenuOnCallback: true } );
        },

        onBegin: function()
        {
            // F**K the AudioMixer! Worst thing ever. Doing our own fading in and out of music.
            // Delete the audio mixer component so on begin cannot be called right away.
            delete this._components[ GameComponentAudioMixer.prototype.getType() ];
            LumosityUtils.callFunctionOnCollection( this._components, "onBegin" );

            this.model.isOnBeginCalled = true;
            this.model.isGamePaused = false;

            this.model.clippingMaskMaxScale = this._calculateClippingMaskMaxScale();
            this.model.clippingMaskSquareScale = this._calculateClippingMaskSquareScale();

            this.view.clippingMaskSquare.setScaleX( this.model.clippingMaskSquareScale.x );
            this.view.clippingMaskSquare.setScaleY( this.model.clippingMaskSquareScale.y );

            this.view.clippingMask.setScaleX( this.model.clippingMaskMaxScale );
            this.view.clippingMask.setScaleY(  this.model.clippingMaskMaxScale );

            this._createStreet();

            this.stopMusic();
            cc.AudioEngine.getInstance().stopAllEffects();

            if( this.model.persistentData.isPracticeMode )
            {
                // Setup the first trial.
                this._setupNextTrial();
                this._playStartTutorialAnimation();
            }
            else
            {
                this._playStageSelectInAnimation();

                // Disable trial restart button at the start.
                this.view.enableTrialRestartButton( false );
            }
        },

        _beginGameMetadata: function()
        {
            this.model.metadata[ metadataTrialKeys.startLevel ] = Math.max( 1, this.model.currentStage - 1 );
            this.model.metadata[ metadataTrialKeys.sessionLevel ] = Math.max( 1, this.model.selectedStage - 1 );
        },

        _endGameMetadata: function()
        {
            var i;
            var data = null;
            var skewedvarianceAverage = 0;

            // Initialize end game meta data
            this.model.metadata[ metadataTrialKeys.fuel ] = 0;
            this.model.metadata[ metadataTrialKeys.shortest ] = 0;
            this.model.metadata[ metadataTrialKeys.steps ] = 0;
            this.model.metadata[ metadataTrialKeys.targets ] = 0;
            this.model.metadata[ metadataTrialKeys.pickups ] = 0;
            this.model.metadata[ metadataTrialKeys.dropoffs ] = 0;

            // Add to end game meta data based on round data.
            for( i = 0; i < this.street.model.roundData.length; i++ )
            {
                data = this.street.model.roundData[ i ];
                this.model.metadata[ metadataTrialKeys.fuel ] += data[ metadataTrialKeys.roundCSV_fuel ];
                this.model.metadata[ metadataTrialKeys.shortest ] += data[ metadataTrialKeys.roundCSV_shortest ];
                this.model.metadata[ metadataTrialKeys.steps ] += data[ metadataTrialKeys.roundCSV_steps ];
                this.model.metadata[ metadataTrialKeys.targets ] += data[ metadataTrialKeys.roundCSV_targets ];
                this.model.metadata[ metadataTrialKeys.pickups ] += data[ metadataTrialKeys.roundCSV_pickups ];
                this.model.metadata[ metadataTrialKeys.dropoffs ] += data[ metadataTrialKeys.roundCSV_dropoffs ];

                skewedvarianceAverage += data[ metadataTrialKeys.roundCSV_skewedVariance ];
            }

            skewedvarianceAverage /= this.street.model.roundData.length;
            this.model.metadata[ metadataTrialKeys.skewedvariance ] = skewedvarianceAverage;
            this.model.metadata[ metadataTrialKeys.numTrials ] = this.settings.getTrialCount();
            this.model.metadata[ metadataTrialKeys.reset ] = this.model.resetCount;
            this.model.metadata[ metadataTrialKeys.userLevel ] = this.model.currentStage;
            this.model.metadata[ metadataTrialKeys.numTotal ] = this.model.totalPickupCount; 
            this.model.metadata[ metadataTrialKeys.roundCSV ] = LumosityUtils.jsToCsv( this.street.model.roundData );
        },

        _playStartTutorialAnimation: function()
        {
            this.view.animationController.play( "startTutorial", this.onStartGameplay, this );
        },

        _playStageSelectInAnimation: function()
        {
            this.model.isOnStageSelect = true;

            this.startStageSelectAudio();
            this.view.animationController.play( "stageSelectIn", this._onStageSelectIn, this );
        },

        _onStageSelectIn: function()
        {
            this._addStageSelectButtons();
        },

        _playStageSelectOutAnimation: function()
        {
            var scaleAction = null;
            var easeAction = null;
            var callbackAction = null;
            var sequence = null;

            scaleAction = cc.ScaleTo.create( this.settings.clippingMaskScaleInDuration, 0, 0 );
            easeAction = cc.EaseIn.create( scaleAction, this.settings.clippingMaskEaseRate );
            callbackAction = cc.CallFunc.create( this._onStageSelectOut, this );
            sequence = cc.Sequence.create( easeAction, callbackAction );
            this.view.clippingMask.runAction( sequence );
        },

        _onStageSelectOut: function()
        {
            var scaleAction = null;
            var easeAction = null;
            var callbackAction = null;
            var sequence = null;

            this.view.animationController.play( "stageSelectOut" );

            scaleAction = cc.ScaleTo.create( this.settings.clippingMaskScaleOutDuration, this.model.clippingMaskMaxScale, this.model.clippingMaskMaxScale );
            easeAction = cc.EaseOut.create( scaleAction, this.settings.clippingMaskEaseRate );
            callbackAction = cc.CallFunc.create( this._onStageSelectOutFinish, this );
            sequence = cc.Sequence.create( easeAction, callbackAction );
            this.view.clippingMask.runAction( sequence );

            this.stopStageSelectAudio();
            this.startGameAudio();

            // Setup the first trial.
            this._setupNextTrial();

            this.view.setLevelCardText( localize( "LevelCard_Trial", "TRIAL" ) );
            this.view.setLevel( this.model.currentTrial );
            this.view.setTrial( this.model.currentTrial, this.settings.getTrialCount() );

            this.playStartingAnimation();  

            this.model.isOnStageSelect = false;

            this._beginGameMetadata(); 
        },

        _onStageSelectOutFinish: function()
        {
            this.view.enableClippingNodeSquare();       
        },

        playStartingAnimation: function()
        {
            this._super();
            
            // Moved to _setupNextTrial
            //var audioMixer = this._components[ GameComponentAudioMixer.prototype.getType() ];
            //audioMixer._playAmbientSound();
        },

        onStartGameplay: function()
        {
            this._super();

            this._beginNextTrial();

            cc.log( "PetDetective::onStartGameplay" );     
        },

        onUpdate: function( delta )
        {
            this._super( delta );

            if( ( this.model.isGamePaused === false ) )
            {
                if( this.street )
                {
                    this.street.onUpdate( delta );
                }
            }

            this._fadeOutAudio( delta );
            this._fadeInAudio( delta );
        },

        onPause: function()
        {
            this._super();

            this.view.pauseAnimations();
            this.street.onPause();
            this.model.setFadeOutAudio( true );
        },

        onResume: function()
        {
            this._super();

            this.view.resumeAnimations();
            this.street.onResume();
            this.model.setFadeInAudio( true );
        },

        showHowToPlay: function( instant )
        {
            this._super( instant );

            this.model.setFadeInAudio( true );
        },

        onRestart: function()
        {
            // Cleanup the street and remove buttons.
            if( this.street )
            {
                this._removeStreetButtons();
                this.street.onRestart();
            }

            // Cleanup the stage select and remove buttons.
            if( this.view.stageSelect )
            {
                this.view.stageSelect.onRestart();
                this._removeStageSelectButtons();
            }

            this.model.setFadeOutAudio( true );

            // Call super last because it calls onBegin.
            this._super();
        },

        onResetTrial: function()
        {
            var scoreComponent = null;

            if( this.street )
            {
                this.street.onResetTrial();
            }

            this._hideSlots();

            // Adjust the score back to the start of the trial.
            scoreComponent = this.getScoringComponent();
            scoreComponent.setScore( this.model.currentTrialStartScore );
            this.model.numberOfPetsDelivered = this.model.currentTrialStartPetsDelivered;
            this.model.totalPickupCount = this.model.currentTrialStartPetsPickuped;

            this.model.resetCount++;
        },

        _backButtonTapped: function()
        {
            if( this.model.isOnStageSelect )
            {
                this.onFinish( false );
                return;
            }

            this._super();
        },

        configureMetaDataKeys: function()
        {
            this._super();

            metadataTrialKeys.fuel              = "fuel";           // "Total amount of fuel allocated (number of spaces)"
            metadataTrialKeys.shortest          = "short";          // "Total length of shortest paths (as found by solver program or users)"
            metadataTrialKeys.skewedvariance    = "skewed_variance";// "Average of pre-calculated skewed variance of board difficulty"
            metadataTrialKeys.steps             = "steps";          // "Total number of steps taken"
            metadataTrialKeys.targets           = "targets";        // "Total number of pets"
            metadataTrialKeys.pickups           = "pickups";        // "Total number of pickups completed"
            metadataTrialKeys.dropoffs          = "dropoffs";       // "Total number of dropoffs completed"
            metadataTrialKeys.numTrials         = "num_trials";     // "Number of rounds"
            metadataTrialKeys.reset             = "reset";          // "Total number of times user restarted"
            metadataTrialKeys.userLevel         = "user_level";     // "Unlocked level"
            metadataTrialKeys.sessionLevel      = "session_level";  //  Selected level (most other games have previously unlocked level)"
            metadataTrialKeys.startLevel        = "start_level";    // "Previously unlocked level (most other games call this session_level)"
            metadataTrialKeys.roundCSV          = "round_csv";      // "Data returned for each round."
            metadataTrialKeys.trialCSV          = "trial_csv";      // "Data returned for each trial."
            metadataTrialKeys.numTotal          = "num_total";      // "Total number of pets"

            metadataTrialKeys.roundCSV_diagram              = "diagram";            // "Prototype layout of level at beginning of round"
            metadataTrialKeys.roundCSV_vary_diagram         = "vary_diagram";       // "Layout of level at beginning of round (after transformation)"
            metadataTrialKeys.roundCSV_end_diagram          = "end_diagram";        // "Layout of level at end of round"
            metadataTrialKeys.roundCSV_fuel                 = "fuel";               // "Amount of fuel allocated (number of spaces)"
            metadataTrialKeys.roundCSV_shortest             = "short";              // "Length of shortest path (as found by solver program or users)"
            metadataTrialKeys.roundCSV_skewedVariance       = "skewed_variance";    // "Pre-calculated skewed variance of board difficulty"
            metadataTrialKeys.roundCSV_steps                = "steps";              // "Number of steps taken"
            metadataTrialKeys.roundCSV_time                 = "time";               // "Time taken to complete round in milliseconds"
            metadataTrialKeys.roundCSV_timeOffset           = "time_offset";        // "At start of round, number of milliseconds since beginning of game"
            metadataTrialKeys.roundCSV_reset                = "reset";              // "Number of times user restarted"
            metadataTrialKeys.roundCSV_targets              = "targets";            // "Number of pets"
            metadataTrialKeys.roundCSV_width                = "width";              // "Board width"
            metadataTrialKeys.roundCSV_height               = "height";             // "Board height"
            metadataTrialKeys.roundCSV_pickups              = "pickups";            // "Number of pickups completed"
            metadataTrialKeys.roundCSV_dropoffs             = "dropoffs";           // "Number of dropoffs completed"

            metadataTrialKeys.trialCSV_round        = "round";          // "Round number"
            metadataTrialKeys.trialCSV_address      = "address";        // "Location in grid as concatenated row, column, zero-indexed or -1 for resets"
            metadataTrialKeys.trialCSV_target       = "target";         // "Occupancy of tile (same letter as pickup or dropoff at that tile) (blank for reset)"
            metadataTrialKeys.trialCSV_dropoff      = "dropoff";        // Is move a dropoff, pickup, or neither [T/F/blank]
            metadataTrialKeys.trialCSV_passengers   = "passengers";     // "List of passengers (letter IDs, no delimit, upper case letters of pickups, each letter in sorted order.)"
            metadataTrialKeys.trialCSV_correct      = "correct";        // Was the trial correctly answered? (T/F)

            cc.log( "metadataTrialKeys: " + JSON.stringify( metadataTrialKeys ) );
        },

        onEndGameplay: function()
        {
            var score = 0;
            var currentStage = 0;
            var scoreComponent = this.getScoringComponent();

            score = scoreComponent.getScore();
            score += this._calculateScoreBonus();
            score += this.settings.endGameBonus * this.model.selectedStage;
            scoreComponent.setScore( score );

            if( !this.model.hasFailedTrial )
            {
                // Increment the current stage if the selected stage is the current one.
                if( this.model.selectedStage === this.model.currentStage )
                {
                    currentStage = this.model.currentStage;
                    currentStage += 1;
                    currentStage = Math.min( Math.max( 0, currentStage ), this.settings.stageCount );
                    this.model.currentStage = currentStage;
                }
            }

            this.stopMusic();
            cc.AudioEngine.getInstance().stopAllEffects();
          
            this._endGameMetadata();      

            // Call super last after calculating score.
            this._super();
        },

        _onTrialComplete: function( responder, isCorrect )
        {
            var timeBonusComponent = null;

            // Pause the time bonus component during trial transitions.
            timeBonusComponent = this._components[ GameComponentTimeBonus.prototype.getType() ];
            timeBonusComponent.setIsComponentPaused( true );

            if( isCorrect )
            {
                this.startTrialSucceedAudio();

                // Torri only wants feedback widget on correct answer.
                // Display feedback widget
                this._showFeedbackWidget( true );
            }
            else
            {
                this.model.hasFailedTrial = true;
                this.startTrialFailAudio();
            }

            // Disable trial restart button.
            this.view.enableTrialRestartButton( false );

            // Sarah wants audio to end after each trial.
            this.stopGameAudio();
        },

        _showFeedbackWidget: function( isCorrect )
        {
            var feedbackWidget = null;
            var gameContentSize = cc.SizeZero();
            var position = cc.PointZero();
            
            // Calculate position of feedbackwidget in centered content space.
            feedbackWidget = this.view.getFeedbackWidget();
        
            gameContentSize = this.view.centeredContent.getContentSize();
            position = cc.p( gameContentSize.width / 2, gameContentSize.height / 2 );

            feedbackWidget.setVisible( true );
            feedbackWidget.setPosition( position);
            this.view.showFeedback( isCorrect );
        },

        _beginMetadataTrial: function()
        {
            this.beginTrial();
        },

        _endMetadataTrial: function()
        {
            this.endTrial( this.street.model.trialData, false, false, false, false, false );
        },

        _onMaxVehicleCapacityButtonPressed: function( /* button */ )
        {
            this._playFullCarAnimation();
        },

        _onPracticeTrialComplete: function( responder, isCorrect )
        {
            if( this.model.persistentData.isPracticeMode )
            {
                if( isCorrect )
                {
                    this._setTutorialSucceedMessage();
                    this.startTrialSucceedAudio();

                    // Display feedback widget
                    this._showFeedbackWidget( true );
                }
                else
                {
                    this._setTutorialFailMessage();
                    this.startTrialFailAudio();
                }
            }
        },

        _onTrialCompleteAnimationFinish: function( /* responder, data */ )
        {
            this.model.currentTrial++;

            if( this.model.currentTrial > this.settings.getTrialCount() )
            {
                // Trials completed.
                this.onEndGameplay();
                return;
            }

            // Update level card with new trial info.
            this.view.setLevel( this.model.currentTrial );
            this.view.setTrial( this.model.currentTrial, this.settings.getTrialCount() );

            // Start level card transition.
            this._transitionToNextTrial();
        },

        _onPracticeTrialCompleteAnimationFinish: function( responder, data )
        {
            // Increment the practice trial if the answer is correct.
            if( data.isCorrect )
            {
                this.model.currentPracticeTrial++;
            }

            if( this.model.currentPracticeTrial > this.settings.practiceTrialCount )
            {
                if( this.street.vehicle )
                {
                    this.street.vehicle.stopEngineRunningAudio();
                    this.street.vehicle.stopDriveAudio();
                }

                // Trials completed.
                this.onCompleteTutorial( false );
                return;
            }

            this._playTutorialTextOutAnimation();
            this._playTutorialBoardOutAnimation();
        },

        _playTutorialBoardInAnimation: function()
        { 
            this.view.playAnimation( "tutorialBoardIn", this._beginNextTrial, this );
        },

        _playTutorialBoardOutAnimation: function()
        { 
            this.view.playAnimation( "tutorialBoardOut", this._onTutorailBoardOut, this );
        },

        _onTutorailBoardOut: function()
        {   
            this._playTutorialBoardInAnimation();
        },

        _playTutorialTextInAnimation: function()
        {
            this.view.playAnimation( "tutorialTextIn" );
        },

        _playTutorialTextOutAnimation: function()
        {
            this.view.playAnimation( "tutorialTextOut" );
        },

        _playTutorialDashboardInAnimation: function()
        {
            this.view.playAnimation( "tutorialDashboardIn" );
        },

        _playFullCarAnimation: function()
        { 
            this.view.playAnimation( "fullCar" );
        },

        _setTutorialPickupMessage: function()
        {
            var practiceTrailInfo = this.street.getCurrentPracticeTrialInfo();
            if( !practiceTrailInfo.pickupMessage )
            {
                return;
            }

            this._playTutorialTextInAnimation();
            this.view.setTutorialPromptText( localize( practiceTrailInfo.pickupMessage ) );
        },

        _setTutorialDropoffMessage: function()
        {
            var practiceTrailInfo = this.street.getCurrentPracticeTrialInfo();
            if( !practiceTrailInfo.dropoffMessage )
            {
                return;
            }

            this._playTutorialTextInAnimation();
            this.view.setTutorialPromptText( localize( practiceTrailInfo.dropoffMessage ) );
        },

        _setTutorialStartMessage: function()
        {
            var practiceTrailInfo = this.street.getCurrentPracticeTrialInfo();
            if( !practiceTrailInfo.startMessage )
            {
                return;
            }

            this._playTutorialTextInAnimation();
            this.view.setTutorialPromptText( localize( practiceTrailInfo.startMessage ) );

            cc.log( "tutorialPrompt" );
            this.view.tutorialPrompt.setOpacity( 255 );
        },

        _setTutorialSucceedMessage: function()
        {
            var practiceTrailInfo = this.street.getCurrentPracticeTrialInfo();
            if( !practiceTrailInfo.succeedMessage )
            {
                return;
            }

            this._playTutorialTextInAnimation();
            this.view.setTutorialPromptText( localize( practiceTrailInfo.succeedMessage ) );
        },

        _setTutorialFailMessage: function()
        {
            var practiceTrailInfo = this.street.getCurrentPracticeTrialInfo();
            if( !practiceTrailInfo.failMessage )
            {
                return;
            }

            this._playTutorialTextInAnimation();
            this.view.setTutorialPromptText( localize( practiceTrailInfo.failMessage ) );
        },

        _transitionToNextTrial: function()
        { 
            this.view.playAnimation( "levelUp", this._beginNextTrial, this );
        },

        _setupNextTrial: function()
        {
            var practiceTrialInfo = null;

            // Stop Sound Effects
            //cc.AudioEngine.getInstance().stopAllEffects();
            if( this.street.vehicle )
            {
                this.street.vehicle.stopEngineRunningAudio();
                this.street.vehicle.stopDriveAudio();
            }

             // Remove all the buttons.
            this._removeStreetButtons();

            if( this.model.persistentData.isPracticeMode )
            {
                // Setup the next trial.
                this.street.setupNextTrial( this.model.selectedPracticeStage, this.model.currentPracticeTrial, this.settings.practiceTrialCount );
                practiceTrialInfo = this.street.getCurrentPracticeTrialInfo();
                
                if( practiceTrialInfo.showDashboard )
                {
                    this._playTutorialDashboardInAnimation();
                }
            }
            else
            {
                this.startGameAudio();

                // Setup the next trial.
                this.street.setupNextTrial( this.model.selectedStage, this.model.currentTrial, this.settings.getTrialCount() );

                // Sarah wants audio to start at the begining of each trial.
                this.model.setFadeInAudio( true );
            }

            this._hideSlots();
        },

        _beginNextTrial: function()
        {
            var timeBonusComponent = null;
            var scoreComponent = null;

            if( !this.model.persistentData.isPracticeMode )
            {
                // UnPause the time bonus component during trial transitions.
                timeBonusComponent = this._components[ GameComponentTimeBonus.prototype.getType() ];
                timeBonusComponent.setIsComponentPaused( false );

                scoreComponent = this.getScoringComponent();
                this.model.currentTrialStartScore = scoreComponent.getScore();
                this.model.currentTrialStartPetsDelivered = this.model.numberOfPetsDelivered;
                this.model.currentTrialStartPetsPickuped = this.model.totalPickupCount;

                // Enable the trial restart button.
                this.view.enableTrialRestartButton( true );
            }
            else
            {
                this._setTutorialStartMessage();
            }

            this.street.onStartGameplay();

            // Add buttons.
            this._addStreetButtons();
        },

        cue_startCountdown: function()
        {
            return;
        },

        cue_generateStreet: function()
        {
            this._setupNextTrial();
        },

        _onPickup: function( responder, passengerIndex, pickupIndex )
        {
            var slotIndex = 0;
            var imageName = "";
            var info = null;

            if( passengerIndex < 0 )
            {
                return;
            }

            if( pickupIndex < 0 )
            {
                return;
            }

            slotIndex = ( passengerIndex + 1 );

            // Set the slot's image.
            info = this.settings.passengers[ pickupIndex ];
            imageName = info.image;
            this.view.dashboard.setSlotImage( slotIndex, imageName );
            this.view.dashboard.displaySlot( slotIndex, true );

            // Play audio.
            this.startPickupAudio();

            // Set the tutorials pickup message.
            if( this.model.persistentData.isPracticeMode )
            {
                this._setTutorialPickupMessage();
            }

            this.model.totalPickupCount += 1;
        },

        _onDropoff: function( responder, passengerIndex )
        {
            var slotIndex = 0;
            var score = 0;
            var scoreComponent = this.getScoringComponent();

            if( passengerIndex < 0 )
            {
                return;
            }

            slotIndex = ( passengerIndex + 1 );
            this.view.dashboard.displaySlot( slotIndex, false );

            if( scoreComponent )
            {
                score = scoreComponent.getScore();
                scoreComponent.setScore( score + this.settings.pointsPerCorrect );
            }

            // Play audio.
            this.startDropoffAudio();

            // Set tutorials dropoff message.
            if( this.model.persistentData.isPracticeMode )
            {
                this._setTutorialDropoffMessage();
            }

            this.model.numberOfPetsDelivered += 1;
        },

        _addStageSelectButtons: function()
        {
            var self = this;
            var buttons = null;

            // Add buttons to the view.
            buttons = this.view.stageSelect.buttons;
            buttons.forEach( function( button )
            {
                self.view.addButton( button );
            } );
        },

        _addStreetButtons: function()
        {
            var self = this;
            var interests = null;

            // Add buttons to the view.
            interests = this.street.interests();
            interests.forEach( function( interest )
            {
                self.view.addButton( interest );
            } );
        },

        _removeStageSelectButtons: function()
        {
            var self = this;
            var buttons = null;

            // Remove all the buttons from the view.
            buttons = this.view.stageSelect.buttons;
            buttons.forEach( function( button )
            {
                self.view.removeButton( button );
            } );
        },

        _removeStreetButtons: function()
        {
            var self = this;
            var interests = null;

            // Remove all the buttons from the view.
            interests = this.street.interests();
            interests.forEach( function( interest )
            {
                self.view.removeButton( interest );
            } );
        },

        stageSelectButtonPressed: function( button )
        {
            this.view.enableClippingNodeCircle();

            // Set the current stage from the selected button.
            this.model.selectedStage = button.getTag();

            this.startStageSelectButtonPressedAudio();

            this._removeStageSelectButtons();

            // Set empty string for text on button
            button.setTitleForState( "", LumosityButtonState.ButtonStateNormal );
            
            // Unhighlight the current selected button.
            this.view.stageSelect._unhighlightStageSelectButtons();

            // Special case to change to a 5th state for buttons in the stage selection screen.
            button._setCurrentState( StageSelectButtonState.ButtonStateAnimated );
            button._updateView();

            // Play stage button swing animation.
            this._playStageSelectButtonSwayAnimation( button, this._playStageSelectOutAnimation );
        },

        _hideSlots: function()
        {
            var i;
            var slotIndex = 0;

            for( i = 0; i < this.street.model.vehicleCapacity; i++ )
            {
                slotIndex = i + 1;
                this.view.dashboard.displaySlot( slotIndex, false );
            }
        },

        _calculateScoreBonus: function()
        {
            var timeBonusComponent = this._components[ GameComponentTimeBonus.prototype.getType() ];
            var timeBonusRemaining = 0;

            timeBonusRemaining = timeBonusComponent.getTimeBonusRemaining();
            cc.log( "ScoreBonus: " + timeBonusRemaining );
            return timeBonusRemaining;
        },

        _calculateClippingMaskMaxScale: function()
        {
            var screenSize = cc.Director.getInstance().getWinSize();
            var contentSize = this.view.clippingMask.getContentSize();
            var p1 = cc.PointZero();
            var p2 = cc.PointZero();
            var distance = 0;
            var maxScale = 1;

            screenSize = cc.size( Math.max( 640, screenSize.width ), Math.max( 960, screenSize.height ) );
            p2 = cc.p( screenSize.width, screenSize.height );
            distance = Math.sqrt( ( p2.x - p1.x ) * ( p2.x - p1.x ) + ( p2.y - p1.y ) * ( p2.y - p1.y ) );
            maxScale = ( distance / contentSize.width );

            return maxScale;
        },

        _calculateClippingMaskSquareScale: function()
        {
            var screenSize = cc.Director.getInstance().getWinSize();
            var contentSize = this.view.clippingMaskSquare.getContentSize();
            var scale = cc.p( 1, 1 );

            screenSize = cc.size( Math.max( 640, screenSize.width ), Math.max( 960, screenSize.height ) );
            scale = cc.p( ( screenSize.width / contentSize.width ), ( screenSize.height / contentSize.height ) );

            return scale;
        },

        /**
         * Start the stage select audio.
         */
        startStageSelectAudio: function()
        {
            if( this.model.isPlayingMusic === true )
            {
                return;
            }

            var soundID = cc.AudioEngine.getInstance().playMusic( "sounds/frontEndLoop.mp3", true );
            this.model.stageSelectSoundID = soundID;
            this.model.isPlayingMusic = true;
        },

        /**
         * Stops the stage select audio.
         */
        stopStageSelectAudio: function()
        {
            if( this.model.isPlayingMusic === false )
            {
                return;
            }

            cc.AudioEngine.getInstance().stopMusic();
            this.model.isPlayingMusic = false;
        },

        /**
         * Start the stage select audio.
         */
        startGameAudio: function()
        {
            if( this.model.isPlayingMusic === true )
            {
                return;
            }

            var soundID = cc.AudioEngine.getInstance().playMusic( "sounds/gameMusicLoop.mp3", true );
            this.model.gameSoundID = soundID;
            this.model.isPlayingMusic = true;
        },

        /**
         * Stops the stage select audio.
         */
        stopGameAudio: function()
        {
            if( this.model.isPlayingMusic === false )
            {
                return;
            }

            cc.AudioEngine.getInstance().stopMusic();
            this.model.isPlayingMusic = false;
        },

        /**
         * Start the pickup passenger audio.
         */
        startPickupAudio: function()
        {
            cc.AudioEngine.getInstance().playEffect( "sounds/carPickupPet.mp3", false );
        },

        /**
         * Start the dropoff passenger audio.
         */
        startDropoffAudio: function()
        {
            cc.AudioEngine.getInstance().playEffect( "sounds/deliverPet.mp3", false );
        },

        stopMusic: function()
        {
            this.stopStageSelectAudio();
            this.stopGameAudio();
        },

        /**
         * Start the trial succeed audio.
         */
        startTrialSucceedAudio: function()
        {
            cc.AudioEngine.getInstance().playEffect( "sounds/complete.mp3", false );
        },

        /**
         * Start the trial fail audio.
         */
        startTrialFailAudio: function()
        {
            cc.AudioEngine.getInstance().playEffect( "sounds/outOfGas.mp3", false );
        },

        /**
         * Start the stage select in audio.
         */
        startStageSelectInAudio: function()
        {
            cc.AudioEngine.getInstance().playEffect( "sounds/screenWipe.mp3", false );
        },

        /**
        * Start the stage select button pressed audio.
        */
        startStageSelectButtonPressedAudio: function()
        {
            cc.AudioEngine.getInstance().playEffect( "sounds/select.mp3", false );
        },

        /**
        * Creates the stage select button swing animation
        */
        _createStageSelectButtonSwayAnimation: function( rotateDuration, rotateDegrees, times )
        {
            var rotateClockwiseAction = cc.RotateTo.create( rotateDuration, rotateDegrees );
            var rotateCounterClockwiseAction = cc.RotateTo.create( rotateDuration, -rotateDegrees );
            var rotateOriginAction = cc.RotateTo.create( rotateDuration, 0 );

            var sequence = cc.Sequence.create( rotateClockwiseAction, rotateCounterClockwiseAction, rotateOriginAction );
            var repeatAction = cc.Repeat.create( sequence, times );

            return repeatAction;
        },

        /**
         * Plays the stage select button swing animation on given button.
         * 
         * @param  {LumosityButton}   button   The button.
         * @param  {Function}         callback The method called after the animation plays.
         */
        _playStageSelectButtonSwayAnimation: function( button, callback )
        {
            var stageSelectButtonSwayAnimation = null;
            var stageSelectSettings = null;
            var callbackAction = null;
            var sequence = null;

            stageSelectSettings = this.settings.stageSelectSettings;
            stageSelectButtonSwayAnimation = this._createStageSelectButtonSwayAnimation( stageSelectSettings.stageSelectButtonSwayAnimation.rotateDuration,
                                                                                         stageSelectSettings.stageSelectButtonSwayAnimation.rotateDegrees,
                                                                                         stageSelectSettings.stageSelectButtonSwayAnimation.times );
            callbackAction = cc.CallFunc.create( callback, this );
            sequence = cc.Sequence.create( stageSelectButtonSwayAnimation, callbackAction );
            button.runAction( sequence );
        },

        trialRestartButtonPressed: function( /* button */ )
        {
            this.onResetTrial();
        },

        debugSetFreeVehicleMovement: function()
        {
            this.street.model.freeVehicleMovement = true;
        },

        debugSetCurrentStage: function( currentStage )
        {
            this.model.currentStage = currentStage;
            this.onEndGameplay();
        },

        _fadeOutAudio: function( delta )
        {
            if( this.model.getFadeOutAudio() === false )
            {
                return;
            }

             this._fadeOutMusic( delta );
             this._fadeOutEffects( delta );
        },

        _fadeInAudio: function( delta )
        {
            if( this.model.getFadeInAudio() === false )
            {
                return;
            }

            this._fadeInMusic( delta );
            this._fadeInEffects( delta );
        },

        _fadeOutMusic: function( delta )
        {
            if( this.model.isPlayingMusic === false )
            {
                return;
            }

            var currentMusicVolume = cc.AudioEngine.getInstance().getMusicVolume();
            cc.AudioEngine.getInstance().setMusicVolume( Math.max( 0 , currentMusicVolume - delta ) );
        },

        _fadeInMusic: function( delta )
        {
            if( this.model.isPlayingMusic === false )
            {
                return;
            }

            var currentMusicVolume = cc.AudioEngine.getInstance().getMusicVolume();
            cc.AudioEngine.getInstance().setMusicVolume( Math.min( 1,  currentMusicVolume + delta ) );
        },

        _fadeOutEffects: function( delta )
        {
            var currentEffectsVolume = cc.AudioEngine.getInstance().getEffectsVolume();
            cc.AudioEngine.getInstance().setEffectsVolume( Math.min( 0,  currentEffectsVolume - delta ) );
        },

        _fadeInEffects: function( delta )
        {
            var currentEffectsVolume = cc.AudioEngine.getInstance().getEffectsVolume();
            cc.AudioEngine.getInstance().setEffectsVolume( Math.min( 1,  currentEffectsVolume + delta ) );
        },

         /**
        *   Creates and adds a street to the gameContent.
        */
        _createStreet: function()
        {
            var contentSize = cc.SizeZero();
            var position = cc.PointZero();

            // Calculate the center of the parent view for the new position.
            contentSize = this.view.centeredContent.getContentSize();
            position = cc.p( contentSize.width >> 1, contentSize.height >> 1 );

            cc.log( "_createStreet: " + JSON.stringify( position ) );
            if( this.model.persistentData.isPracticeMode )
            {
                this.street = new PracticeStreet( this.settings.streetSettings, this.settings.practiceStreetSettings );
                this.street.setTrialCompleteCallback( this._onPracticeTrialComplete );
                this.street.setTrialCompleteAnimationFinishCallback( this._onPracticeTrialCompleteAnimationFinish );
            }
            else
            {
                this.street = new Street( this.settings.streetSettings );
                this.street.setTrialCompleteCallback( this._onTrialComplete );
                this.street.setTrialCompleteAnimationFinishCallback( this._onTrialCompleteAnimationFinish );
                this.street.setBeginMetadataTrialCallback( this._beginMetadataTrial );
                this.street.setEndMetadataTrialCallback( this._endMetadataTrial );
            }

            this.street.setMaxVehicleCapacityButtonPressedCallback( this._onMaxVehicleCapacityButtonPressed );

            this.street.view.setPosition( position );
            this.view.centeredContent.addChild( this.street.view );

            // Set the street's view to the view that is animated through the layout.
            this.view.street = this.street.view;

            // Setup callbacks.
            this.street.setTarget( this );
            this.street.setPickupCallback( this._onPickup );
            this.street.setDropoffCallback( this._onDropoff );
        },
        
        _createSettings: function()
        {
            return new PetDetectiveSettings( "settings.json" );
        },
        
        _createModel: function()
        {
            return new PetDetectiveModel();
        },
        
        _createView: function( screenSize )
        {
            return new PetDetectiveView( screenSize, this.settings, this.model, this );
        }
    } );
}() );
