/*global cc*/
/*global require*/
/*global GameModel*/
/*global lumosity*/

require( "src/Game/Base/GameModel.js" );

var PetDetectiveModel;
( function()
{
    "use strict";
    
    PetDetectiveModel = GameModel.extend(
    {
        currentStage: 1,
        selectedStage: 1,
        currentTrial: 1,
        currentTrialStartScore: 0,
        currentTrialStartPetsDelivered: 0,
        currentTrialStartPetsPickuped: 0,

        selectedPracticeStage: 1,
        currentPracticeTrial: 1,

        clippingMaskMaxScale: 1,
        clippingMaskSquareScale: cc.size( 1, 1 ),

        stageSelectSoundID: -1,
        gameSoundID: -1,

        numberOfPetsDelivered: 0,
        bestStatKey: "NumberOfPetsDelivered",

        currentStageKey: "CurrentStage",

        hasFailedTrial: false,

        resetCount: 0,

        isOnStageSelect: false,

        totalPickupCount: 0,

        shouldFadeInMusic: true,
        shouldFadeInEffects: true,
        shouldFadeOutMusic: false,
        shouldFadeOutEffects: false,

        
        /** @lends PetDetectiveModel.prototype */
        
        /**
        * The Pet Detective model.
        * @constructs
        */
        ctor: function()
        {
            this._initGameStats();
        },

        _initGameStats: function()
        {
            if( !lumosity.game.params )
            {
                return;
            }

            if( !lumosity.game.params.last_results )
            {
                return;
            }

            if( !lumosity.game.params.last_results.gameStats )
            {
               return;
            }

             var currentStage = lumosity.game.params.last_results.gameStats[ this.currentStageKey ];
            this.currentStage = currentStage;
        },

        setFadeInAudio: function( shouldFadeIn )
        {
            this.shouldFadeInMusic = this.shouldFadeInEffects = shouldFadeIn;
            this.shouldFadeOutMusic = this.shouldFadeOutEffects = !shouldFadeIn;
        },

        setFadeOutAudio: function( shouldFadeOut )
        {
            this.shouldFadeInMusic = this.shouldFadeInEffects = !shouldFadeOut;
            this.shouldFadeOutMusic = this.shouldFadeOutEffects = shouldFadeOut;
        },

        getFadeInAudio: function()
        {
            return  ( this.shouldFadeInMusic && this.shouldFadeInEffects );
        },

        getFadeOutAudio: function()
        {
            return  ( this.shouldFadeOutMusic && this.shouldFadeOutEffects );
        },

        exportResults: function()
        {
            var stats = {};
            stats[ this.bestStatKey ] = this.numberOfPetsDelivered;
            stats[ this.currentStageKey ] = this.currentStage;

            cc.log( "exportResults: " + " numberOfPetsDelivered " + this.numberOfPetsDelivered + " currentStage " + this.currentStage );
            
            return this._super( stats, this.bestStatKey );
        }
    } );
}() );
