/*global cc*/
/*global require*/
/*global StreetView*/

require( "src/Game/StreetView.js" );

var PracticeStreetView;
( function()
{
    "use strict";
    
    PracticeStreetView = StreetView.extend(
    {
        _tutorialCircle: null,
      
        /** @lends StreetView.prototype */
        
        /**
         *  PracticeStreetView is the main class for a PracticeStreet view!
         *
         *  @constructs
         */
        ctor:function()
        {
            this._super();
        },

        addTutorialCircle: function( tutorialCircle )
        {
            this._tutorialCircle = tutorialCircle;
            this.addChild( tutorialCircle );
        },

        setTutorialCirclePosition: function( position )
        {
            if( !this._tutorialCircle )
            {
                return;
            }

            this._tutorialCircle.setPosition( position );
        },

        calculateTutorialCirclePosition: function( target )
        {
            var targetContentSize = cc.SizeZero();
            var anchorPoint = cc.PointZero();
            var targetAnchorPoint = cc.PointZero();
            var anchorPointDelta = cc.PointZero();
            var targetPosition = cc.PointZero();
            var offset = cc.PointZero();
            var newPosition = cc.PointZero();
            if( !this._tutorialCircle )
            {
                return;
            }

            targetContentSize = target.getContentSize();
            targetAnchorPoint = target.getAnchorPoint();
            targetPosition = target.getPosition();
            anchorPoint = this._tutorialCircle.getAnchorPoint();
            anchorPointDelta = cc.p( anchorPoint.x - targetAnchorPoint.x, anchorPoint.y - targetAnchorPoint.y );
            offset = cc.p( targetContentSize.width * anchorPointDelta.x, targetContentSize.height * anchorPointDelta.y );
            newPosition = cc.p( targetPosition.x + offset.x, targetPosition.y + offset.y );
            return newPosition;
        },

        hideTutorialCircle: function( )
        {
            if( !this._tutorialCircle )
            {
                return;
            }

            this._tutorialCircle.setOpacity( 0 );
        },

        showTutorialCircle: function()
        {
            if( !this._tutorialCircle )
            {
                return;
            }

            this._tutorialCircle.setOpacity( 255 );
        },

        removeTutorialCircle: function()
        {
            if( !this._tutorialCircle )
            {
                return;
            }

            this.removeChild( this._tutorialCircle );
            this._tutorialCircle = null;
        },

        playTutorialCircleRevealAnimation: function()
        {
            var turorialCircleFadeInAnimation = this._createTutorialCircleFadeInAnimation();
            var tutorialCircleRotateAnimimation = this._createTutorialCircleRotateAnimation();
            this._tutorialCircle.runAction( turorialCircleFadeInAnimation );
            this._tutorialCircle.runAction( tutorialCircleRotateAnimimation );
        },

        playTutorialCircleConcealAnimation: function()
        {
            var turorialCircleFadeOutAnimation = this._createTutorialCircleFadeOutAnimation();
            this._tutorialCircle.runAction( turorialCircleFadeOutAnimation );
        },

        _createTutorialCircleFadeInAnimation: function()
        {
            var delayAction = cc.DelayTime.create( 0.2 );
            var fadeToAction = cc.FadeTo.create( 0.2, 255 );
            var sequence = cc.Sequence.create( delayAction, fadeToAction );
            return sequence;
        },

        _createTutorialCircleRotateAnimation: function()
        {
            var rotateAction = cc.RotateBy.create( 30, 360 );
            var repeatAction = cc.RepeatForever.create( rotateAction );
            return repeatAction;
        },

        _createTutorialCircleFadeOutAnimation: function()
        {
            var delayAction = cc.DelayTime.create( 0.2 );
            var fadeToAction = cc.FadeTo.create( 0.2, 0 );
            var sequence = cc.Sequence.create( delayAction, fadeToAction );
            return sequence;
        }
    } );
}() );
