/*global cc*/
/*global require*/
/*global LumosityButtonState*/
/*global LumosityButton*/
/*global LayoutLoader*/
/*global LayoutNodeFlags*/
/*global LumosityUtils*/

require( "src/View/LumosityButton.js" );
require( "src/Animation/LayoutLoader.js" );
require( "src/Utility/LumosityUtils.js" );

var STAGE_SELECT_BUTTON_FLICKER_ANIMATION_TAG = 3;

var StageSelectButtonState = 
{
    ButtonStateAnimated:    "ButtonStateAnimated"
};

var StageSelect;
( function()
{
    "use strict";
    
    StageSelect = cc.Node.extend(
    {
        _currentStage: 1,
        _stageCount: 1,
        buttons: null,
        _highlightImages: null,

        _buttonPosition: cc.PointZero(),
        _buttonLabelPosition: cc.PointZero(),
        _buttonLabelAnchorPoint: cc.PointZero(),
        _buttonAnchorPoint: cc.PointZero(),
        _buttonDesignSize: cc.SizeZero(),
        _buttonSpacing: cc.SizeZero(),
        _buttonColumnCount: 4,
        _numverUnlockedTextData: null,
        _numberLockedTextData: null,

        _target: null,
        _settings: null,


        /** @lends StageSelect.prototype */
        
        /**
         *  StageSelect is the main class for a StageSelect view!
         *
         *  @constructs
         */
        ctor:function( target, contentSize, settings, currentStage, stageCount )
        {
            this._super();

            this._target = target;
            this._settings = settings;
            this._currentStage = currentStage;
            this._stageCount = stageCount;
            this.buttons = [];
            this._highlightImages = [];

            // Must have contentSize before creating layout.
            this.setContentSize( contentSize );

            this._loadLayout();
            this._createViews();

            this.highlightStageSelectButton( currentStage, true );

            this.nodeName = "StageSelect";
        },

        onEnter: function()
        {
            this._super();

            this.startHighlightImageFlickerAnimation( this._currentStage );
        },

        onRestart: function()
        {
            var i;
            var button = null;
            var highlightedImage = null;
            var stageIndex = 0;
            for( i = 0; i < this.buttons.length; i++ )
            {
                stageIndex = ( i + 1 );
                button = this.buttons[ i ];
                highlightedImage = this._highlightImages[ i ];
                if( this._currentStage < stageIndex )
                {
                    button.setEnabled( false );
                }
                else
                {
                    button.setEnabled( true );
                }

                if( button.currentState() === LumosityButtonState.ButtonStateHighlighted )
                {
                    button.setHighlighted( false );
                    highlightedImage.setVisible( false );
                }

                this.stopHighlightImageFlickerAnimation( stageIndex );
                button.setTitleForState( stageIndex, LumosityButtonState.ButtonStateNormal );
            }

            this.highlightStageSelectButton( this._currentStage, true );
            this.startHighlightImageFlickerAnimation( this._currentStage );
        },

        _createViews: function()
        {
            LumosityUtils.printNodeHierarchy( this );

            this._buttonPosition = this.photo.getPosition();
            this._buttonAnchorPoint = this.photo.getAnchorPoint();
            this._buttonLabelPosition = this.number.getPosition();
            this._buttonLabelAnchorPoint = this.number.getAnchorPoint();
            this._buttonDesignSize = this.photo.getContentSize();
            this._buttonSpacing = this.spacing.getContentSize();

            this._createStageSelectButtons();
        },

        _createButton: function( bgUpImage, bgDownImage, bgHighlightedImage, bgAnimatedImage, iconUpImage, iconDownImage, iconDisabledImage, iconHighlightedImage, iconAnimatedImage, text, fontUpColor, fontDownColor, fontDisabledColor, fontHighlightedColor, fontAnimatedColor )
        {   
            var button = null;

            button = new LumosityButton();
            button.setBackgroundImageForState( bgUpImage, LumosityButtonState.ButtonStateNormal );
            button.setBackgroundImageForState( bgDownImage, LumosityButtonState.ButtonStateSelected );
            button.setBackgroundImageForState( bgHighlightedImage, LumosityButtonState.ButtonStateHighlighted ); 
            button.setBackgroundImageForState( bgAnimatedImage, StageSelectButtonState.ButtonStateAnimated );           
            button.setImageForState( iconUpImage, LumosityButtonState.ButtonStateNormal );
            button.setImageForState( iconDownImage, LumosityButtonState.ButtonStateSelected );
            button.setImageForState( iconDisabledImage, LumosityButtonState.ButtonStateDisabled );
            button.setImageForState( iconHighlightedImage, LumosityButtonState.ButtonStateHighlighted );
            button.setImageForState( iconAnimatedImage, StageSelectButtonState.ButtonStateAnimated );   
            button.setTitleForState( text, LumosityButtonState.ButtonStateNormal );
            button.setTitleColorForState( fontUpColor, LumosityButtonState.ButtonStateNormal );
            button.setTitleColorForState( fontDownColor, LumosityButtonState.ButtonStateSelected );
            button.setTitleColorForState( fontDisabledColor, LumosityButtonState.ButtonStateDisabled );
            button.setTitleColorForState( fontHighlightedColor, LumosityButtonState.ButtonStateHighlighted );
            button.setTitleColorForState( fontAnimatedColor, StageSelectButtonState.ButtonStateAnimated );

            return button;
        },

        _createStageSelectButton: function( bgUpImageName, bgDownImageName, bgHighlightedName, bgAnimatedImageName, iconUpImageName, iconDownImageName, iconDisabledImageName, iconHighlightedName, iconAnimatedImageName, text, lockedTextData, unlockedTextData, labelPosition, position, anchorPoint, target, callback )
        {
            var button = null;
            var bgUpImage = null;
            var bgDownImage = null;
            var bgHighlightedImage = null;
            var bgAnimatedImage = null;
            var iconUpImage = null;
            var iconDownImage = null;
            var iconDisabledImage = null;
            var iconHighlightedImage = null;
            var iconAnimatedImage = null;
            
            var font = "";
            var fontSize = "";
            var lockedFontColor = cc.c4b( 255, 255, 255, 63 );
            var unlockedFontColor = cc.c4b( 255, 255, 255, 63 );

            var lockedFontData = null;
            var unlockedFontData = null;

            bgUpImage = cc.Sprite.create( "photos/" + bgUpImageName );
            bgDownImage = cc.Sprite.create( "photos/" + bgDownImageName );
            bgHighlightedImage = cc.Sprite.create( "photos/" + bgHighlightedName );
            bgAnimatedImage = cc.Sprite.create( "photos/" + bgAnimatedImageName );
            iconUpImage = cc.Sprite.create( "photos/" + iconUpImageName );
            iconDownImage = cc.Sprite.create( "photos/" + iconDownImageName );
            iconDisabledImage = cc.Sprite.create( "photos/" + iconDisabledImageName );
            iconHighlightedImage = cc.Sprite.create( "photos/" + iconHighlightedName );
            iconAnimatedImage = cc.Sprite.create( "photos/" + iconAnimatedImageName );

            font = LumosityUtils.getFontLocation( lockedTextData[ LayoutLoader.fontKey ] );
            fontSize = lockedTextData[ LayoutLoader.fontSizeKey ];
            lockedFontData = lockedTextData[ LayoutLoader.fontColorKey ];
            lockedFontColor = LayoutLoader.getColor( lockedFontData );

            unlockedFontData = unlockedTextData[ LayoutLoader.fontColorKey ];
            unlockedFontColor = LayoutLoader.getColor( unlockedFontData );

            button = this._createButton( bgUpImage,
                                         bgDownImage,
                                         bgHighlightedImage,
                                         bgAnimatedImage,
                                         iconUpImage,
                                         iconDownImage,
                                         iconDisabledImage,
                                         iconHighlightedImage,
                                         iconAnimatedImage,
                                         text,
                                         unlockedFontColor,
                                         unlockedFontColor,
                                         lockedFontColor,
                                         unlockedFontColor );

            button.setContentSize( this._buttonDesignSize, false );
            button.setAnchorPoint( anchorPoint );
            button.setPosition( position );
            button.setTarget( target );
            button.setReleasedCallback( callback );

            iconUpImage.setAnchorPoint( cc.PointZero() );
            iconDownImage.setAnchorPoint( cc.PointZero() );
            iconDisabledImage.setAnchorPoint( cc.PointZero() );
            iconHighlightedImage.setAnchorPoint( cc.PointZero() );
            iconAnimatedImage.setAnchorPoint( cc.PointZero() );

             // Setup label.
            button.titleLabel.setFontName( font );
            button.titleLabel.setInitialFontSize( fontSize );
            button.titleLabel.setAnchorPoint( this._buttonLabelAnchorPoint );
            button.titleLabel.setPosition( labelPosition );
            button.titleLabel.setHorizontalAlignment( cc.TEXT_ALIGNMENT_CENTER );

            // Setup icon.
            LumosityUtils.scaleForContentSize( iconUpImage, this._buttonDesignSize );
            LumosityUtils.scaleForContentSize( iconDownImage, this._buttonDesignSize );
            LumosityUtils.scaleForContentSize( iconDisabledImage, this._buttonDesignSize );
            LumosityUtils.scaleForContentSize( iconHighlightedImage, this._buttonDesignSize );
            LumosityUtils.scaleForContentSize( iconAnimatedImage, this._buttonDesignSize );

            return button;
        },

        _createStageSelectButtons: function()
        {
            var i;
            var stageSelectButton = null;
            var startX = 0;
            var buttonPos = cc.PointZero();
            var anchorPoint = cc.PointZero();
            var labelPos = cc.PointZero();
            var lockedTextData = null;
            var unlockedTextData = null;
            var stageIndex = 0;
            var stageButtonSettings = null;
            var highlightedImage = null;

            // Convert button position.
            buttonPos = this._buttonPosition;
            anchorPoint = this._buttonAnchorPoint;
            labelPos = this._buttonLabelPosition;

            lockedTextData = this._numberLockedTextData;
            unlockedTextData = this._numberUnlockedTextData;

            startX = buttonPos.x;

            // Create
            for( i = 0; i < this._stageCount; i++ )
            {
                if( ( i !== 0 ) && !( i % this._buttonColumnCount ) )
                {
                    buttonPos.x = startX;
                    buttonPos.y -= this._buttonSpacing.height;
                }

                stageIndex = ( i + 1 );
                stageButtonSettings = this._settings.stageButtons[ stageIndex ];
                cc.log( "stageButtonSettings: " + JSON.stringify( stageButtonSettings ) );

                stageSelectButton = this._createStageSelectButton( stageButtonSettings.backgroundImageNormal,
                                                                   stageButtonSettings.backgroundImageSelected,
                                                                   stageButtonSettings.backgroundImageHighlighted,
                                                                   stageButtonSettings.backgroundImageAnimated,
                                                                   stageButtonSettings.iconImageNormal,
                                                                   stageButtonSettings.iconImageSelected,
                                                                   stageButtonSettings.iconImageDisabled,
                                                                   stageButtonSettings.iconImageHighlighted,
                                                                   stageButtonSettings.iconImageAnimated,
                                                                   stageIndex,
                                                                   lockedTextData,
                                                                   unlockedTextData,
                                                                   labelPos,
                                                                   buttonPos,
                                                                   anchorPoint,
                                                                   this,
                                                                   this.stageSelectButtonPressed );
                stageSelectButton.nodeName = "StageSelectButton";
                stageSelectButton.setTag( stageIndex );
                this._addStageSelectButton( stageSelectButton );

                highlightedImage = cc.Sprite.create( "photos/photoSelected.png" );
                highlightedImage.setPosition( buttonPos );
                highlightedImage.setAnchorPoint( anchorPoint );
                highlightedImage.setVisible( false );
                this._addHighlightedImage( highlightedImage );
                LumosityUtils.scaleForContentSize( highlightedImage, this._buttonDesignSize );

                if( this._currentStage < stageIndex )
                {
                    stageSelectButton.setEnabled( false );
                }

                buttonPos.x += this._buttonSpacing.width;
            }
        },

        stageSelectButtonPressed: function( button )
        {
            if( !this._target || !this._target.stageSelectButtonPressed )
            {
                return;
            }

            this._target.stageSelectButtonPressed.call( this._target, button );
        },

        /**
         * Adds a button.
         * 
         * @param {LumosityButton} stageSelectButton The button to be added.
         */
        _addStageSelectButton: function( stageSelectButton )
        {
            var boardZOrder = 0;

            boardZOrder = this.board.getZOrder();
            this.addChild( stageSelectButton, boardZOrder + 1 );
            this.buttons.push( stageSelectButton );
        },

        _addHighlightedImage: function( highlightedImage )
        {
            var boardZOrder = 0;

            boardZOrder = this.board.getZOrder();
            this.addChild( highlightedImage, boardZOrder );
            this._highlightImages.push( highlightedImage );
        },

        highlightStageSelectButton: function( stage, isHighlighted )
        {
            var index = Math.min( Math.max( 0, stage - 1 ), this.buttons.length - 1 );
            var button = this.buttons[ index ];
            var highlightedImage = null;

            this._unhighlightStageSelectButtons();

            button.setHighlighted( isHighlighted );
            highlightedImage = this._highlightImages[ index ];
            highlightedImage.setVisible( true );
        },

        startHighlightImageFlickerAnimation: function( stage )
        {
            var index = Math.min( Math.max( 0, stage - 1 ), this.buttons.length - 1 );
            var image = this._highlightImages[ index ];
            var highlightImageFlickerAnimation = null;

            highlightImageFlickerAnimation = this._createHighlightImageFlickerAnimation( 1, 1, 0, 255 );
            highlightImageFlickerAnimation.setTag( STAGE_SELECT_BUTTON_FLICKER_ANIMATION_TAG );

            image.runAction( highlightImageFlickerAnimation );
        },

        stopHighlightImageFlickerAnimation: function( stage )
        {
            var index = Math.min( Math.max( 0, stage - 1 ), this.buttons.length - 1 );
            var image = this._highlightImages[ index ];

            image.stopActionByTag( STAGE_SELECT_BUTTON_FLICKER_ANIMATION_TAG );
        },

        _createHighlightImageFlickerAnimation: function( fadeOutDuration, fadeInDuration, fadeOutOpacity, fadeInOpacity )
        {
            var fadeOutAction = cc.FadeTo.create( fadeOutDuration, fadeOutOpacity );
            var fadeInAction = cc.FadeTo.create( fadeInDuration, fadeInOpacity );
            var sequence = cc.Sequence.create( fadeOutAction, fadeInAction );
            var repeatAction = cc.RepeatForever.create( sequence );

            return repeatAction;
        },

        _unhighlightStageSelectButtons: function()
        {
            var i;
            var button = null;
            var highlightedImage = null;
            for( i = 0; i < this.buttons.length; i++ )
            {
                button = this.buttons[ i ];
                highlightedImage = this._highlightImages[ i ];
                if( button.currentState() === LumosityButtonState.ButtonStateHighlighted )
                {
                    button.setHighlighted( false );
                    highlightedImage.setVisible( false );
                }
            }
        },

        _loadLayout: function()
        {
            LayoutLoader.loadLayoutFile( "layoutBoard.json", this, this , this._layoutCreatingNode.bind( this ) );
        },

        _layoutCreatingNode: function( nodeName, nodeData )
        {
            var result = null;
            if( nodeName === "numberUnlockedFont" )
            {
                this._numberUnlockedTextData = nodeData[ LayoutLoader.textDataKey ];
                result = false;
            }
            else if( nodeName === "numberLockedFont" )
            {
                this._numberLockedTextData = nodeData[ LayoutLoader.textDataKey ];
                result = false;
            }
            return result;
        },   

        layout_settingNodeFlags: function( nodeName, nodeData, node )
        {
            var nodeFlags = 0xFF;
            nodeFlags &= ~( LayoutNodeFlags.RootScale );
            return nodeFlags;
        }  
    } );
}() );
