/*global cc*/
/*global require*/
/*global LayoutLoader*/
/*global LayoutNodeFlags*/
/*global LumosityUtils*/

require( "src/Animation/LayoutLoader.js" );
require( "src/Utility/LumosityUtils.js" );

var VehicleView;
( function()
{
    "use strict";
    
    VehicleView = cc.Node.extend(
    {
        _images: null,
        _imageIndex: 0,
        _opacity: 255,
        animationView: null,

        /** @lends VehicleView.prototype */
        ctor:function()
        {
            this._super();
            this._images = [];

            this.animationView = cc.Node.create();
            this.animationView.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            this.addChild( this.animationView );
        },

        createViews: function()
        {
            var position = cc.PointZero();

            this._loadLayout();

            position = this.getFuelDefaultPosition();
            this.fuel.setPosition( position );

            // Disable wordwrapping for fuel label.
            this.text.setDimensions( cc.size( 0, 0 ) );
        },

        addVehicleImage: function( vehicleImage )
        {
            this._images.push( vehicleImage );
            this.animationView.addChild( vehicleImage );
        },

        /**
        * Pauses all current running animations.
        */
        pauseAnimations: function()
        {
            this.pauseSchedulerAndActions();
            this.animationView.pauseSchedulerAndActions();
            this.fuel.pauseSchedulerAndActions();
        },

        /**
        * Resumes all previous running animations.
        */
        resumeAnimations: function()
        {
            this.resumeSchedulerAndActions();
            this.animationView.resumeSchedulerAndActions();
            this.fuel.resumeSchedulerAndActions();
        },

        stopAnimations: function()
        {
            this.resumeAnimations();
            this.stopAllActions();
            this.animationView.stopAllActions();
            this.fuel.stopAllActions();
        },

        setImage: function( index )
        {
            var image = null;

            // Hide previous image.
            if( this._images )
            {
                image = this._images[ this._imageIndex ];
                image.setOpacity( 0 );
            }

            this._imageIndex = index;

            // Display new image.
            image = this._images[ this._imageIndex ];
            this.setContentSize( image.getContentSize() );
            image.setOpacity( this._opacity );
        },

        getCurrentImage: function()
        {
            var image = null;
            image = this._images[ this._imageIndex ];
            return image;
        },

        setOpacity: function( opacity )
        {
            var image = null;

            this._opacity = opacity;
            image = this._images[ this._imageIndex ];
            image.setOpacity( this._opacity );
        },

        getFuelDefaultPosition: function()
        {
            var contentSize = cc.SizeZero();
            var defaultPosition = cc.PointZero();

            contentSize = this.getContentSize();
            defaultPosition = cc.p( contentSize.width >> 1, contentSize.height );
            return defaultPosition;
        },

        _loadLayout: function()
        {
            LayoutLoader.loadLayoutFile( "layoutVehicle.json", this, this , this._layoutCreatingNode.bind( this ) );
        },

        _layoutCreatingNode: function( nodeName, nodeData )
        {
            var result = null;
            return result;
        },

        layout_settingNodeFlags: function( nodeName, nodeData, node )
        {
            var nodeFlags = 0xFF;
            nodeFlags &= ~( LayoutNodeFlags.RootScale );
            return nodeFlags;
        }
    } );
}() );
