/*global cc*/
/*global lumosity*/
/*global require*/

require( "src/Game/Base/GameModel.js" );
/*global GameModel*/
require( "src/Game/Component/GameComponentMetadata.js");
/*global metadataAggregateKeys*/
require( "src/Utility/LumosityUtils.js" );
/*global lu*/

// var PinballRecallTrialMetadata;
// ( function()
// {
//     "use strict";

//     PinballRecallTrialMetadata = cc.Class.extend(
//     {
//         // bounces - Number of bounces
//         bounces: null,
//         // correct - Was the trial completed correctly? (T/F)
//         correct: null,
//         // decoy_count - Number of decoy bumpers
//         decoy_count: null,
//         // grid - Layout of grid, describes each space. All rows concatenated to a single string without delimiters. [B=empty,N=\ bumper,S=/ bumper]
//         grid: null,
//         // start - Start point (letter/number pair representing side [t=top,r=right,b=bottom,l=left] and position)
//         start: null,
//         // target - End point (letter/number pair representing side [t=top,r=right,b=bottom,l=left] and position)
//         target: null,
//         // guess - Guessed end point (letter/number pair representing side [t=top,r=right,b=bottom,l=left] and position)
//         guess: null,
//         // index - Trial number
//         index: null,
//         // level - Level
//         level: null,
//         // mirror_count - Number of bumpers
//         mirror_count: null,
//         // path - Length of path ball travelled (number of squares)
//         path: null

//         // AUTOMATIC:

//         // time_offset - At start of trial, number of milliseconds since beginning of game [not current on web version]
//         // time_offset: null,
//         // response_time - Response time in milliseconds [time on web version, in seconds!]
//         // response_time: null,
//     } );
// }() );

// var PinballRecallMetadata;
// ( function()
// {
//     "use strict";

//     PinballRecallMetadata = cc.Class.extend(
//     {
//         currentTrial: new PinballRecallTrialMetadata(),
//         trials: []
//     } );
// }() );

var Dir =
{
    Down  : cc.p(  0, -1 ),
    Up    : cc.p(  0,  1 ),
    Left  : cc.p( -1,  0 ),
    Right : cc.p(  1,  0 )
};

var BoardCellType =
{
    Empty       : 1,
    BumperRight : 2,
    BumperLeft  : 3,
    Start       : 4,
    Drain       : 5
};

var CellModel;
( function()
{
    "use strict";
    
    CellModel = cc.Class.extend(
    {
        cellType: null,
        onPath: false
    } );
}() );

var PinballRecallModel;
( function()
{
    "use strict";
    
    PinballRecallModel = GameModel.extend(
    {
        __path: null,

        bestStatKey: null,
        endingLevelNumberKey: null,

        trial: 1,

        _gameLevelNumber: 1,
        _highestLevelNumber: 1,

        _successfulTrialsAtThisLevel: 0,
        _failedTrialsAtThisLevel: 0,

        _width: null,
        _height: null,
        _cells: null,
        _pinballDirection: null,
        _startPosition: null,

        // NOT NEEDED - AUTOMATIC
        // _trialStartTime: null,

        _allowInput: false,
        _finished: false,
        _succeeded: false,
        _failed: false,
        _unusedBumpersRevealed: false,
        _hideBumpers: false,

        // _startPosition: null,
        _endPosition: null,
        _guessPosition: null,
        _currentPosition: null,
        // _pinballDirection: null,

        _bumpersTotal: 0,
        _bumpersOnPath: 0,

        _bumpersNeeded: 0,
        _bumpersPlaced: 0,

        _bumperPathGenerationFailures: 0,
        _bumperPathGenerationSuccesses: 0,
        _currentTrial: null,
        
        ctor: function( settings )
        {
            this._super();

            // this._metadata = new PinballRecallMetadata();

            // AUTOMATIC: nothing to do here

            // time - Time in milliseconds [seconds on web version!]
            // user_level - Start level
            // max_level - Max level reached
            // session_level - End level

            // MANUAL: must track ourselves

            // up - Number of levels changed between start and finish (end level - start level)
            // levels_up - Number of times leveled up [not currently on web version]
            // levels_down - Number of times leveled down [not currently on web version]
            // this.metadata.up = 0;
            this.metadata.levels_up = 0;
            this.metadata.levels_down = 0;

            this.bestStatKey = settings.bestStatKey;
            this.endingLevelNumberKey = settings.endingLevelNumberKey;

            this._gameLevelNumber = settings.startingLevelNumber;

            if ( settings.loadFromLastResults )
            {
                this._loadFromLastResults( settings );
            }
            this._currentTrial = {};
        },

        initHighestLevel: function()
        {
            this._highestLevelNumber = this._gameLevelNumber;
        },

        exportResults: function( /* stats, bestStatKey */ )
        {
            var stats = {};
            stats[ this.bestStatKey ] = this._highestLevelNumber;
            stats[ this.endingLevelNumberKey ] = this._gameLevelNumber;

            this.metadata[ metadataAggregateKeys.maxLevel ] = this._highestLevelNumber;

            // @todo Is this the right way to compute this?
            // this.metadata.up = this.metadata[ metadataAggregateKeys.sessionLevel ] - this.metadata[ metadataAggregateKeys.userLevel ];

            return this._super( stats, this.bestStatKey );
        },

        getTrialMetadata: function()
        {
            return this._currentTrial;
        },

        generateBoard: function( width, height )
        {
            this._allocate( width, height );

            var x;
            var y;

            // Add center tiles.
            for ( x = 0; x < width; x += 1 )
            {
                for ( y = 0; y < height; y += 1 )
                {
                    this.setCellType( x + 1, y + 1, BoardCellType.Empty );
                }
            }

            // Add horizontal buttons.
            for ( x = 0; x < width; x += 1 )
            {
                // top row
                this.setCellType( x + 1, height + 1, BoardCellType.Drain );
                // bottom row
                this.setCellType( x + 1, 0, BoardCellType.Drain );
            }

            // Add vertical buttons.
            for ( y = 0; y < height; y += 1 )
            {
                // left row
                this.setCellType( width + 1, y + 1, BoardCellType.Drain );
                // right row
                this.setCellType( 0, y + 1, BoardCellType.Drain );
            }

            // Set starting position.
            var startX = 0;
            var startY = 0;
            var chance = lu.getRandomInt( 1, 100 );
            if ( chance < 25 )
            {
                // top
                startX = lu.getRandomInt( 1, width );
                startY = height + 1;
                
                this.setPinballDirection( Dir.Down );
            }
            else if ( chance < 50 )
            {
                // bottom
                startX = lu.getRandomInt( 1, width );
                startY = 0;
                
                this.setPinballDirection( Dir.Up );
            }
            else if ( chance < 75 )
            {
                // left side
                startX = 0;
                startY = lu.getRandomInt( 1, height );
                
                this.setPinballDirection( Dir.Right );
            }
            else
            {
                // right side
                startX = width + 1;
                startY = lu.getRandomInt( 1, height );
                
                this.setPinballDirection( Dir.Left );
            }

            this._setStartPosition( cc.p( startX, startY ), true );
            this.setCellType( startX, startY, BoardCellType.Start );
        },

        _allocate: function( width, height )
        {
            this._width = width;
            this._height = height;

            this._cells = [];

            var x;
            var y;
            for ( x = 0; x < this._width + 2; x += 1 )
            {
                // Maybe more efficient:
                // this._cells[ x ] = new Array( this._height + 2 );

                this._cells[ x ] = [];

                for ( y = 0; y < this._height + 2; y += 1 )
                {
                    this._cells[ x ].push( new CellModel() );
                }
            }
        },

        startNextTrial: function()
        {
            // this._metadata.currentTrial = new PinballRecallTrialMetadata();
            // this._metadata.trials.push( this._metadata.currentTrial );
            // this._currentTrial = new PinballRecallTrialMetadata();
            this._currentTrial = {};
        },

        getGameLevel: function()
        {
            return this._gameLevelNumber;
        },

        setGameLevel: function( gameLevelNumber )
        {
            if ( this._gameLevelNumber !== gameLevelNumber )
            {
                this._successfulTrialsAtThisLevel = 0;
                this._failedTrialsAtThisLevel = 0;
            }

            this._gameLevelNumber = gameLevelNumber;

            if ( this._highestLevelNumber < this._gameLevelNumber )
            {
                this._highestLevelNumber = this._gameLevelNumber;
            }
        },

        getBumpersOnPath: function()
        {
            return this._bumpersOnPath;
        },
        
        setBumpersOnPath: function( bumpersOnPath )
        {
            this._bumpersOnPath = bumpersOnPath;
        },

        setBumpersTotal: function( bumpersTotal )
        {
            this._bumpersTotal = bumpersTotal;
        },
        
        setAllowInput: function( allowInput )
        {
            this._allowInput = allowInput;
        },

        isFinished: function()
        {
            return this._finished;
        },
        
        setFinished: function( finished )
        {
            this._finished = finished;
        },

        isFailed: function()
        {
            return this._failed;
        },

        setFailed: function( failed )
        {
            this._failed = failed;

            if ( this._failed )
            {
                this._successfulTrialsAtThisLevel = 0;
                this._failedTrialsAtThisLevel += 1;
            }
        },

        isSucceeded: function()
        {
            return this._succeeded;
        },
        
        setSucceeded: function( succeeded, updateMetadata )
        {
            this._succeeded = succeeded;

            if ( this._succeeded )
            {
                this._failedTrialsAtThisLevel = 0;
                this._successfulTrialsAtThisLevel += 1;
            }
            
            if ( updateMetadata )
            {
                this.getTrialMetadata().correct = succeeded ? 'T' : 'F';
            }
        },
        
        setUnusedBumpersRevealed: function( unusedBumpersRevealed )
        {
            this._unusedBumpersRevealed = unusedBumpersRevealed;
        },

        getBoardSize: function()
        {
            return cc.p( this._width, this._height );
        },
        
        setHideBumpers: function( hideBumpers )
        {
            this._hideBumpers = hideBumpers;
        },

        getHideBumpers: function()
        {
            return this._hideBumpers;
        },
        
        getStartPosition: function()
        {
            return this._startPosition;
        },

        _setStartPosition: function( startPosition, updateMetadata )
        {
            this._startPosition = startPosition;

            if ( updateMetadata )
            {
                this.getTrialMetadata().start = this._coordinatesToString( startPosition );
            }
        },
        
        getEndPosition: function()
        {
            return this._endPosition;
        },

        setEndPosition: function( endPosition, updateMetadata )
        {
            this._endPosition = endPosition;

            if ( updateMetadata )
            {
                this.getTrialMetadata().target = this._coordinatesToString( endPosition );
            }
        },
        
        getGuessPosition: function()
        {
            return this._guessPosition;
        },

        setGuessPosition: function( guessPosition, updateMetadata )
        {
            this._guessPosition = guessPosition;

            if ( updateMetadata )
            {
                this.getTrialMetadata().guess = this._coordinatesToString( guessPosition );
            }
        },
        
        getCurrentPosition: function()
        {
            return this._currentPosition;
        },

        setCurrentPosition: function( currPos )
        {
            this._currentPosition = currPos;
        },

        getPinballDirection: function()
        {
            return this._pinballDirection;
        },
        
        setPinballDirection: function( pinballDir )
        {
            this._pinballDirection = pinballDir;
        },

        getBumpersPlaced: function()
        {
            return this._bumpersPlaced;
        },

        setBumpersPlaced: function( bumpersPlaced )
        {
            this._bumpersPlaced = bumpersPlaced;
        },

        incrementBumpersPlaced: function()
        {
            this._bumpersPlaced += 1;
        },

        decrementBumpersPlaced: function()
        {
            this._bumpersPlaced -= 1;
        },

        getBumpersNeeded: function()
        {
            return this._bumpersNeeded;
        },

        setBumpersNeeded: function( bumpersNeeded )
        {
            this._bumpersNeeded = bumpersNeeded;
        },

        getBoardCellType: function( pos )
        {
            return this._cellAtPos( pos ).cellType;
        },

        setCellType: function( x, y, cellType )
        {
            this._cellAtXY( x, y ).cellType = cellType;
        },

        setBoardCellType: function( pos, cellType )
        {
            this._cellAtPos( pos ).cellType = cellType;
        },

        isBoardCellOnPath: function( pos )
        {
            return this._cellAtPos( pos ).onPath;
        },

        setBoardCellOnPath: function( pos, onPath )
        {
            this._cellAtPos( pos ).onPath = onPath;

            this.__printNumCellsOnPath();
        },

        getBumperPathGenerationFailures: function()
        {
            return this._bumperPathGenerationFailures;
        },

        setBumperPathGenerationFailures: function( failures )
        {
            this._bumperPathGenerationFailures = failures;
        },

        incrementBumperPathGenerationFailures: function()
        {
            this._bumperPathGenerationFailures += 1;
        },

        getBumperPathGenerationSuccesses: function()
        {
            return this._bumperPathGenerationSuccesses;
        },

        setBumperPathGenerationSuccesses: function( successes )
        {
            this._bumperPathGenerationSuccesses = successes;
        },

        incrementBumperPathGenerationSuccesses: function()
        {
            this._bumperPathGenerationSuccesses += 1;
        },

        getRandomBoardPositionNotOnPath: function()
        {
            var retval = cc.p( -1, -1 );
            
            var emptyCells = [];

            var x;
            var y;
            // Scan ALL cells on the board (center cells and edge cells) and only consider the empty (center) cells not on the path.
            for ( x = 0; x < this._cells.length; x += 1 )
            {
                for ( y = 0; y < this._cells[ x ].length; y += 1 )
                {
                    if ( this._cells[ x ][ y ].cellType === BoardCellType.Empty && this._cells[ x ][ y ].onPath === false )
                    {
                        emptyCells.push( cc.p( x, y ) );
                    }
                }
            }
            
            if ( emptyCells.length > 0 )
            {
                retval = emptyCells[ lu.getRandomInt( 0, emptyCells.length - 1 ) ];
            }
            
            return retval;
        },

        getSuccessfulTrialsAtThisLevel: function()
        {
            return this._successfulTrialsAtThisLevel;
        },

        getFailedTrialsAtThisLevel: function()
        {
            return this._failedTrialsAtThisLevel;
        },

        getBumperBouncePosition: function()
        {
            return this._bumperBouncePosition;
        },

        setBumperBouncePosition: function( position )
        {
            this._bumperBouncePosition = position;
        },

        getBumperBounceDirection: function()
        {
            return this._bumperBounceDirection;
        },

        setBumperBounceDirection: function( direction )
        {
            this._bumperBounceDirection = direction;
        },

        // NOT NEEDED - AUTOMATIC
        // getTrialStartTime: function()
        // {
        //     return this._trialStartTime;
        // },

        // NOT NEEDED - AUTOMATIC
        // setTrialStartTime: function( trialStartTime )
        // {
        //     this._trialStartTime = trialStartTime;
        // },

        _cellAtXY: function( x, y )
        {
            lu.assert( x < this._cells.length, x.toString() + " < " + this._cells.length.toString() );
            lu.assert( y < this._cells[ x ].length, y.toString() + " < " + this._cells[ x ].length.toString() );

            return this._cells[ x ][ y ];
        },

        _cellAtPos: function( pos )
        {
            return this._cellAtXY( pos.x, pos.y );
        },

        _coordinatesToString: function ( coords )
        {
            if ( coords.x === 0 )
            {
                // Flip the board upside down.
                return "l" + (this._height + 1 - coords.y);
            }
            
            if ( coords.x === this._width + 1 )
            {
                // Flip the board upside down.
                return "r" + (this._height + 1 - coords.y);
            }
            
            // Flip the board upside down.
            if ( coords.y === this._height + 1 )
            {
                return "t" + coords.x;
            }
            
            // Flip the board upside down.
            if ( coords.y === 0 )
            {
                return "b" + coords.x;
            }
            
            return "";
        },

        __printNumCellsOnPath: function()
        {
            var numOnPath = 0;

            var x;
            var y;
            for ( x = 0; x < this._cells.length; x += 1 )
            {
                for ( y = 0; y < this._cells[ x ].length; y += 1 )
                {
                    if ( this._cells[ x ][ y ].onPath )
                    {
                        numOnPath += 1;
                    }
                }
            }

            cc.log( "cells on path = " + numOnPath );
        },

        getBoardAsString: function()
        {
            var str = "";

            var x;
            var y;
            var type;
            // Hit all the cells, but skip the edge cells.
            // [B=empty,N=\ bumper,S=/ bumper]
            for ( y = this._height + 1; y >= 1; y -= 1 )
            {
                for ( x = 1; x < this._width + 2; x += 1 )
                {
                    type = this.getBoardCellType( cc.p( x, y ) );

                    if ( type === BoardCellType.Empty )
                    {
                        str += "B";
                    }
                    else if ( type === BoardCellType.BumperRight )
                    {
                        str += "N";
                    }
                    else if ( type === BoardCellType.BumperLeft )
                    {
                        str += "S";
                    }
                }
            }

            return str;
        },

        /**
         * Adjusts model state based on last game results (passed in from app).
         *
         * @param settings  Current game settings.
         * @private
         */
        _loadFromLastResults: function( settings )
        {
            // Get level info from last results.
            if ( lumosity.game.params &&
                 lumosity.game.params.last_results &&
                 lumosity.game.params.last_results.gameStats )
            {
                var endingLevelNumber = lumosity.game.params.last_results.gameStats[ this.endingLevelNumberKey ];

                if ( lu.isNumeric( endingLevelNumber ) && endingLevelNumber >= 1 )
                {
                    // Start each playthrough at a level offset from the player's last ending level.
                    this._gameLevelNumber = Math.max( endingLevelNumber + settings.startingLevelOffsetFromEndingLevel, 1 );

                    // ll.verbose( "SETTING LEVEL INDEX FROM STATS: " + this._gameLevelNumber );
                }
            }
        }
    } );
}() );
