/*global cc*/
/*global require*/
/*global localize*/

require( "src/Entity/AnimatedEntityView.js" );
/*global AnimatedEntityView*/
require( "src/Game/PinballRecallModel.js" );
/*global BoardCellType*/
/*global Dir*/
require( "src/Game/PinballRecallUtils.js" );
/*global Point*/
require( "src/Game/Base/GameView.js" );
/*global GameView*/
require( "src/Game/Widget/GameWidgetHud.js" );
/*global GameWidgetHud*/
require( "src/Game/Widget/GameWidgetLevelCard.js" );
/*global GameWidgetLevelCard*/
require( "src/Utility/LumosityUtils.js" );
/*global lu*/
require( "src/View/LumosityButton.js" );
/*global LumosityButtonState*/

var FlashAnimationBoardSlideLeft      = "slideLeft";
var FlashAnimationBoardSlideLevelUp   = "levelUp";
var FlashAnimationBoardSlideRight     = "slideRight";
var FlashAnimationBoardSlideLevelDown = "levelDown";

var PinballRecallState;
( function()
{
    "use strict";

    var i = 0;
    var inc = function() { var temp = i; i += 1; return temp; };

    PinballRecallState =
    {
        Present_ShowBumpers       : inc(),
        Present_Pause             : inc(),
        Present_HideBumpers       : inc(),
        Present_ButtonActiveDelay : inc(),
        Input                     : inc(),
        Simulate_GrowBall         : inc(),
        Simulate                  : inc(),
        Simulate_RevealBumpers    : inc(),
        Simulate_ShrinkBall       : inc(),
        Simulate_Feedback         : inc(),
        Simulate_SuccessSequence  : inc()
    };
}() );

var AnimationListener;
( function()
{
    "use strict";
    
    AnimationListener = cc.Class.extend(
    {
        _callbackFunc: null,
        _callbackObj: null,

        _listeningTo: [],
        _numStillListeningTo: 0,

        ctor: function( callbackFunc, callbackObj )
        {
            this._callbackFunc = callbackFunc;
            this._callbackObj = callbackObj;

            this._listeningTo = [];
            this._numStillListeningTo = 0;
        },

        listenTo: function( animatedEntityView )
        {
            animatedEntityView.addListener( this );

            this._listeningTo.push( animatedEntityView );
        },

        commit: function()
        {
            this._numStillListeningTo = this._listeningTo.length;
        },

        getNumStillListeningTo: function()
        {
            return this._listeningTo.length;
        },

        animatedEntity_finishedAnimation: function( animationName )
        {
            lu.assert( this._numStillListeningTo > 0 );

            this._numStillListeningTo -= 1;

            if ( this._numStillListeningTo === 0 )
            {
                var i;
                for ( i = 0; i < this._listeningTo.length; i += 1 )
                {
                    this._listeningTo[ i ].removeListener( this );
                }

                this._listeningTo = [];

                this._callbackFunc.call( this._callbackObj );
            }
        }
    } );
}() );

var Bumper;
( function()
{
    "use strict";
    
    Bumper = cc.Class.extend(
    {
        aev: null,
        boardPos: null,
        hasBeenRevealed: false,

        ctor: function( aev, x, y )
        {
            this.aev = aev;
            this.boardPos = cc.p( x, y );
            this.hasBeenRevealed = false;
        }
    } );
}() );

var PinballRecallView;
( function()
{
    "use strict";
    
    PinballRecallView = GameView.extend(
    {
        _tiles: null,
        _drains: null,
        _fence: null,
        _bg: null,
        _bumpers: null,

        _ball: null,

        _startArrow: null,
        _startDrain: null,

        _state: null,
        _animListener: null,
        _callback: null,

        _boardElementScale: 1,

        _startX: 0,
        _startY: 0,
        _tileSize: 0,
        _startPos: null,
        _pinballDir: null,

        _guess: null,

        _pausableElements: null,

        ctor: function( screenSize, settings, model, interactionResponder, preload )
        {
            this._super( screenSize, settings, model, interactionResponder, preload );

            this._init();
        },

        _init: function()
        {
            this._tiles = [];
            this._drains = [];
            this._fence = [];
            this._bg = [];
            this._bumpers = [];

            this._ball = null;

            this._state = null;
            this._animListener = null;
            this._callback = null;

            this._pausableElements = [];
        },

        preload: function()
        {
            this._super();
        },

        setLevelCardValue: function( level )
        {
            var levelCard = this.getWidget( GameWidgetLevelCard.prototype.getType(), true );
            levelCard.setLevelLabel( localize( "LevelCard_Level" ) );
            levelCard.setLevelValue( level );
        },

        awardPoints: function( gained )
        {
            //var hud = this.getHudWidget();
            //hud.setScoreAnimated( gained, score );
            
            this.points.setString( "+" + gained );

            this.playAnimation( "updateScore", this.onPointsAwarded, this );
        },

        onPointsAwarded: function()
        {
            var hud = this.getHudWidget();
            hud.setScore( this.model.score );
        },

        destroyBoard: function()
        {
            var i;
            for ( i = 0; i < this._drains.length; i += 1 )
            {
                this.removeButton( this._drains[ i ] );
            }

            for ( i = this._buttons.length - 1; i >= 0; i-- )
            {
                if ( this._buttons[ i ].isDrainButton )
                {
                    cc.log( "HOLY MOLY! Band-aid fix for buttons, this was about to crash!" );
                    this._buttons.splice( i, 1 );
                }
            }

            this._drains = [];

            while ( this.board.getChildren().length > 0 )
            {
                this.board.removeChild( this.board.getChildren()[ 0 ] );
            }
        },

        generateBoard: function()
        {
            //////////////////////////////////////// CLEANUP

            this.destroyBoard();

            lu.assert( this.board.getChildren().length === 0 );

            ////////////////////////////////////////

            var boardSize = this.model.getBoardSize();
            var width = boardSize.x;
            var height = boardSize.y;

            //////////////////////////////////////// COMPUTE

            var tileImage = this.settings.tileImage;
            var tile = cc.Sprite.create( tileImage );
            var tileContentSize = tile.getContentSize();
            lu.assert( tileContentSize.width === tileContentSize.height);
            var rawTileSize = tileContentSize.width;

            // Take away space for the margins.
            var usableScreenWidth = (1.0 - this.settings.boardMarginAsPercentOfScreen ) * this.board.getContentSize().width;
            // Cut up the usable space by number of tiles (width) and left/right border (drain) area.
            var tileSize = usableScreenWidth / (width + 2);

            this._boardElementScale = tileSize / rawTileSize;

            var tiles = [];

            var startX = this.board.getContentSize().width / 2;
            startX -= ( width / 2 ) * tileSize;
            startX = Math.floor( startX );

            var startY = this.board.getContentSize().height / 2;
            startY -= ( height / 2 ) * tileSize;
            startX = Math.floor( startX );

            //////////////////////////////////////// CACHE

            this._startX = startX;
            this._startY = startY;
            this._tileSize = tileSize;
            this._startPos = this.model.getStartPosition();
            this._pinballDir = this.model.getPinballDirection();

            //////////////////////////////////////// TILES

            var posX = startX;
            var posY = startY;

            var x;
            var y;
            for ( x = 0; x < width; x += 1 )
            {
                posY = startY;

                for ( y = 0; y < height; y += 1 )
                {
                    // Above, we created a tile and used it to get size information.
                    // Now, instead of throwing it away, reuse it.
                    if (tile === null)
                    {
                        tile = cc.Sprite.create( tileImage );
                    }

                    this.board.addChild( tile );
                    tile.setZOrder( this.settings.zOrder.tiles );
                    tiles.push( tile );

                    tile.setAnchorPoint( 0.0, 0.0 );
                    tile.setPosition( cc.p( posX, posY ) );

                    this._scaleBoardElement( tile );

                    posY += tileSize;

                    tile = null;
                }

                posX += tileSize;
            }

            this._tiles = tiles;

            //////////////////////////////////////// FENCE

            var fenceImageFormat = this.settings.fenceImageFormat;
            var fenceImage = fenceImageFormat.format( width, height );

            var fence = cc.Sprite.create( fenceImage );
            this.board.addChild( fence );
            fence.setZOrder( this.settings.zOrder.fence );

            fence.setAnchorPoint( 0.5, 0.5 );
            fence.setPosition( this.board.getContentSize().width / 2, this.board.getContentSize().height / 2 );

            lu.scaleForContentSize( fence, { width: tileSize * ( width + 2 ), height: tileSize * ( height + 2 ) } );

            this._fence = fence;

            //////////////////////////////////////// BACKGROUND

            var bgImageFormat = this.settings.bgImageFormat;
            var bgImage = bgImageFormat.format( width, height );

            var bg = cc.Sprite.create( bgImage );
            this.board.addChild( bg );
            bg.setZOrder( this.settings.zOrder.bg );

            bg.setAnchorPoint( 0.5, 0.5 );
            bg.setPosition( this.board.getContentSize().width / 2, this.board.getContentSize().height / 2 );

            lu.scaleForContentSize( bg, { width: tileSize * ( width + 2 ), height: tileSize * ( height + 2 ) } );

            this._bg = bg;

            //////////////////////////////////////// BUMPERS

            var bumperSpriteSheet = this.settings.bumperSpriteSheet;
            var bumperAnimationNames =
            [
                this.settings.bumperBounceAnim,
                this.settings.bumperInAnim,
                this.settings.bumperOutAnim
            ];

            var bumpers = [];

            var cellType;
            var bumperAEV;
            var bumper;
            for ( x = 1; x < width + 1; x += 1 )
            {
                posY = startY;

                for ( y = 1; y < height + 1; y += 1 )
                {
                    cellType = this.model.getBoardCellType( cc.p( x, y ) );

                    bumperAEV = null;

                    if ( cellType === BoardCellType.BumperRight )
                    {
                        bumperAEV = new AnimatedEntityView( bumperSpriteSheet, bumperAnimationNames, false, true );
                        // bumperAEV.setSprite( cc.Sprite.create( "bumperOut000.png") );
                    }
                    else if ( cellType === BoardCellType.BumperLeft )
                    {
                        bumperAEV = new AnimatedEntityView( bumperSpriteSheet, bumperAnimationNames, false, true );
                        // bumperAEV.setSprite( cc.Sprite.create( "bumperOut000.png") );
                        bumperAEV._sprite.setRotation( 90 );
                    }

                    if ( bumperAEV !== null )
                    {
                        this.board.addChild( bumperAEV );
                        bumperAEV.setZOrder( this.settings.zOrder.bumpers );

                        // The AEV has no content size, so we can't change the anchor point to position its _sprite.
                        // Instead, we shift by half dim.
                        // bumperAEV.setAnchorPoint( 0.5, 0.5 );
                        bumperAEV.setPosition(
                            cc.p(
                                startX + ( x - 1 ) * tileSize + tileSize / 2,
                                startY + ( y - 1 ) * tileSize + tileSize / 2
                            )
                        );

                        this._scaleBoardElement( bumperAEV._sprite );

                        // Don't. Use an AnimationListener instead.
                        // bumperAEV.addListener( this );

                        bumper = new Bumper( bumperAEV, x, y );

                        bumpers.push( bumper );

                        // PinballRecallUtils.enableBoundingBox( bumperAEV._sprite, cc.c3b( 0, 255, 0 ), 0.5 );

                        // cc.log( "Placed bumper at " + x + ", " + y );
                    }
                }

                posX += tileSize;
            }

            this._bumpers = bumpers;

            //////////////////////////////////////// DRAINS & START POSITION

            // cc.log( "start pos is " + this.model.getStartPosition().x + ", " + this.model.getStartPosition().y );

            var drainX;
            var drainY;

            var spots = [];

            // HORIZONTAL

            for ( x = 1; x < width + 1; x += 1 )
            {
                drainX = x;
                drainY = 0;
                spots.push( cc.p( drainX, drainY ) );

                drainX = x;
                drainY = height + 1;
                spots.push( cc.p( drainX, drainY ) );
            }

            // VERTICAL

            for ( y = 1; y < height + 1; y += 1 )
            {
                drainX = 0;
                drainY = y;
                spots.push( cc.p( drainX, drainY ) );

                drainX = width + 1;
                drainY = y;
                spots.push( cc.p( drainX, drainY ) );
            }

            var drains = [];

            var drain;
            var start;

            var i;
            var spot;
            // You may look at this code and say,
            // "what the heck, why are we making a drain in the same spot where we're making the start arrow?"
            // How observant you are! In fact, this is intentional, because the start arrow isn't initially visible.
            // So we need a drain in that spot to show first. Then, when the start arrow is shown, we hide this drain.
            for ( i = 0; i < spots.length; i += 1 )
            {
                spot = spots[ i ];
                x = spot.x;
                y = spot.y;

                // Confused about drains and the arrow? See comment at the top of this loop.
                drain = this._createDrain();
                this.board.addChild( drain );
                drain.setZOrder( this.settings.zOrder.drains );
                drains.push( drain );

                // No need for buttons, this is their default.
                // drain.setAnchorPoint( cc.p( 0, 0 ) );

                drain.setPosition( startX + ( x - 1 ) * tileSize, startY + ( y - 1 ) * tileSize );

                // Attach button location info to the object.
                // Not my favorite way to do this, however, it's needed for the onButtonPressed callback to work.
                drain.boardPos = cc.p( x, y );

                if ( Point.equals( spot, this.model.getStartPosition() ) )
                {
                    // Confused about drains and the arrow? See comment at the top of this loop.
                    start = this._createStartArrow( this.settings.startArrowActiveImage, x, y );
                    start.setVisible( false );

                    this._startArrow = start;
                    this._startDrain = drain;
                }
            }

            this._drains = drains;

            //////////////////////////////////////// DEBUG

            if ( lu.isDebugModeOn() )
            {
                this.__visualizePath( this.model, width, height );
                // this.__visualizeEndPosition( this.model, width, height, tileSize, startX, startY );
            }
        },

        _createDrain: function()
        {
            var upFilename = this.settings.buttonActiveImage;
            var downFilename = this.settings.buttonClickedImage;
            // var disabledFilename = "buttonClicked.png";
            var onClickFunctionOrButtonName = null;
            var text = "";
            var font = "";
            var textSize = 0;
            var textColor = cc.c3b( 255, 255, 255 );
            var textDownColor = textColor;
            var contentSize = null;

            var button = this._createButton(
                upFilename,
                downFilename,
                onClickFunctionOrButtonName,
                text,
                font,
                textSize,
                textColor,
                textDownColor,
                contentSize
            );
            //band-aid fix for button crash, some button hangs around in the _buttons array even though it has no native object
            button.isDrainButton = true;

            button.setBackgroundImageForState( cc.Sprite.create( this.settings.buttonInactiveImage ), LumosityButtonState.ButtonStateDisabled );

            button.setTarget( this );
            button.setPressedCallback( this.onButtonPressed );

            this._scaleBoardElement( button );

            return button;
        },

        _createStartArrow: function( imageName, x, y )
        {
            var start = cc.Sprite.create( imageName );
            start.setZOrder( this.settings.zOrder.drains );
            this.board.addChild( start );

            start.setAnchorPoint( cc.p( 0.5, 0.5 ) );

            var rot = 0;

            if ( Point.equals( this._pinballDir, Dir.Right ) )
            {
                rot = 270;
            }
            else if ( Point.equals( this._pinballDir, Dir.Up ) )
            {
                rot = 180;
            }
            else if ( Point.equals( this._pinballDir, Dir.Left ) )
            {
                rot = 90;
            }

            start.setRotation( rot );
            start.setPosition(
                this._startX + ( x - 1 ) * this._tileSize + this._tileSize / 2,
                this._startY + ( y - 1 ) * this._tileSize + this._tileSize / 2
            );

            this._scaleBoardElement( start );

            return start;
        },

        _scaleBoardElement: function( elem )
        {
            // Doesn't work on AEV's because the initial content size of their sprite is zero.
            // lu.scaleForContentSize( elem, { width: dim, height: dim } );

            elem.setScaleX( this._boardElementScale );
            elem.setScaleY( this._boardElementScale );
        },

        __visualizePath: function( model, width, height )
        {
            // The old way, just putting dots on the board:

            // var x;
            // var y;
            // var path;
            // // Traverse only the tile cells.
            // for ( x = 1; x < width + 1; x += 1 )
            // {
            //     for ( y = 1; y < height + 1; y += 1 )
            //     {
            //         if ( model.isBoardCellOnPath( cc.p( x, y ) ) )
            //         {
            //             // cc.log("got on path at " + x + ", " + y );

            //             path = cc.Sprite.create( this.settings.buttonCorrectImage );

            //             this.board.addChild( path );
            //             path.setZOrder( this.settings.zOrder.debug );
            //             path.setOpacity( 64 );
            //             path.setPosition( this.fromTilePositionToCoordinates( x, y ) );
            //         }
            //     }
            // }

            // The new way:

            var first = 1;
            var last = model.__path.length - 2;

            var i;
            var curr;
            var next;
            var move;
            var img;
            var rot;
            var sprite;
            for ( i = first; i <= last; i += 1 )
            {
                curr = model.__path[ i ];
                next = model.__path[ i + 1 ];
                move = Point.subtract( next, curr );

                img = this.settings.startArrowInactiveImage;

                if ( i === last )
                {
                    img = this.settings.startArrowActiveImage;
                }

                sprite = cc.Sprite.create( img );

                this.board.addChild( sprite );
                sprite.setZOrder( this.settings.zOrder.debug );
                sprite.setOpacity( 32 );
                sprite.setPosition( this.fromTilePositionToCoordinates( curr.x, curr.y ) );

                rot = 0;
                if ( Point.equals( move, Dir.Right ) )
                {
                    rot = 270;
                }
                else if ( Point.equals( move, Dir.Up ) )
                {
                    rot = 180;
                }
                else if ( Point.equals( move, Dir.Left ) )
                {
                    rot = 90;
                }
                sprite.setRotation( rot );
            }
        },

        __visualizeEndPosition: function( model, width, height, tileSize, startX, startY )
        {
            var endPos = cc.Sprite.create( this.settings.buttonIncorrectImage );
            this.board.addChild( endPos );
            endPos.setZOrder( this.settings.zOrder.debug );
            endPos.setAnchorPoint( 0.0, 0.0 );
            endPos.setPosition( cc.p( startX + model.getEndPosition().x * tileSize - tileSize, startY + model.getEndPosition().y * tileSize - tileSize ) );
        },

        makeBall: function()
        {
            var ball = cc.Sprite.create( this.settings.ballImage );
            this.board.addChild( ball );
            ball.setZOrder( this.settings.zOrder.ball );

            ball.setAnchorPoint( cc.p( 0.5, 0.5 ) );

            ball.setPosition(
                this._startX + ( this._startPos.x - 1 ) * this._tileSize + this._tileSize / 2,
                this._startY + ( this._startPos.y - 1 ) * this._tileSize + this._tileSize / 2
            );

            ball.setScaleX( 0.0 );
            ball.setScaleY( 0.0 );

            this._ball = ball;
        },

        growBall: function( callback )
        {
            this._toNewState( callback, PinballRecallState.Simulate_GrowBall );

            var duration = this.settings.ballGrowDuration;
            var scaleAction = cc.ScaleTo.create( duration, this._boardElementScale, this._boardElementScale );
            var finishedAction = cc.CallFunc.create( this.onGrowBall, this );

            // Not implemented because ofx version uses a linear curve.
            // var ease = this.settings.ballGrowEase;
            // var easeAction = cc.EaseIn.create( scaleAction, ease );
            // var sequence = cc.Sequence.create( easeAction, finishedAction );
            var sequence = cc.Sequence.create( scaleAction, finishedAction );

            this._ball.runAction( sequence );
            this._registerPause( this._ball );
        },

        onGrowBall: function()
        {
            this._unregisterPause( this._ball );
            lu.assert( this.__noPausableElements() );
            this._notifyCallbackOfStateComplete();
        },

        shrinkBall: function( callback )
        {
            this._toNewState( callback, PinballRecallState.Simulate_ShrinkBall );

            var duration = this.settings.ballShrinkDuration;
            var scaleAction = cc.ScaleTo.create( duration, 0.0, 0.0 );
            var finishedAction = cc.CallFunc.create( this.onShrinkBall, this );

            // Not implemented because ofx version uses a linear curve.
            // var ease = this.settings.ballShrinkEase;
            // var easeAction = cc.EaseIn.create( scaleAction, ease );
            // var sequence = cc.Sequence.create( easeAction, finishedAction );
            var sequence = cc.Sequence.create( scaleAction, finishedAction );

            this._ball.runAction( sequence );
            this._registerPause( this._ball );
        },

        onShrinkBall: function()
        {
            this._unregisterPause( this._ball );
            lu.assert( this.__noPausableElements() );
            this._notifyCallbackOfStateComplete();
        },

        turnOnStartArrow: function()
        {
            this._startDrain.setEnabled( false );
            this._startDrain.setVisible( false );
            this._startArrow.setVisible( true );
        },

        onButtonPressed: function( button )
        {
            this._guess = button;

            this.interactionResponder.onButtonPressed( button );
        },

        deactivateStartArrow: function()
        {
            this._startArrow.setVisible( false );

            this._startArrow = this._createStartArrow(
                this.settings.startArrowInactiveImage,
                this._startPos.x,
                this._startPos.y
            );
        },

        fromTilePositionToCoordinates: function( x, y )
        {
            return cc.p(
                this._startX + ( x - 1 ) * this._tileSize + this._tileSize / 2,
                this._startY + ( y - 1 ) * this._tileSize + this._tileSize / 2
            );
        },

        showFeedback: function( correct, guessPos )
        {
            var guessCoord = this.fromTilePositionToCoordinates( guessPos.x, guessPos.y );
            var feedback = this.getFeedbackWidget();
            var pos = lu.convertPointSpace( guessCoord, this.board, feedback.getParent() );
            feedback.setPosition( pos );

            this._super( correct );
        },

        makeGuessButtonStick: function()
        {
            this._setImageForButton( this._guess, this.settings.buttonClickedImage );
        },

        updateGuessButtonImage: function( correct )
        {
            var imageName = this.settings.buttonIncorrectImage;

            if ( correct )
            {
                imageName = this.settings.buttonCorrectImage;
            }

            this._setImageForButton( this._guess, imageName );
        },

        _setImageForButton: function( button, imageName )
        {
            var sprite;

            sprite = cc.Sprite.create( imageName );
            button.setBackgroundImageForState( sprite, LumosityButtonState.ButtonStateNormal );

            sprite = cc.Sprite.create( imageName );
            button.setBackgroundImageForState( sprite, LumosityButtonState.ButtonStateSelected );
            
            sprite = cc.Sprite.create( imageName );
            button.setBackgroundImageForState( sprite, LumosityButtonState.ButtonStateDisabled );
        },

        enableInput: function()
        {
            var i;
            for ( i = 0; i < this._drains.length; i += 1 )
            {
                this._drains[ i ].setEnabled( true );
            }
        },

        disableInput: function()
        {
            var i;
            for ( i = 0; i < this._drains.length; i += 1 )
            {
                this._drains[ i ].setEnabled( false );
            }
        },

        // @todo move to model?
        getNumHiddenBumpers: function()
        {
            return this._getHiddenBumpers().length;
        },

        // @todo move to model?
        _getHiddenBumpers: function()
        {
            var bumpers = [];

            var i;
            for ( i = 0; i < this._bumpers.length; i += 1 )
            {
                if ( !this._bumpers[ i ].hasBeenRevealed )
                {
                    bumpers.push( this._bumpers[ i ] );
                }
            }

            return bumpers;
        },

        revealHiddenBumpers: function( callback )
        {
            this._showBumpers( this._getHiddenBumpers(), callback, PinballRecallState.Simulate_RevealBumpers );
        },

        showAllBumpers: function( callback )
        {
            this._showBumpers( this._bumpers, callback, PinballRecallState.Present_ShowBumpers );
        },

        _showBumpers: function( bumpers, callback, state )
        {
            this._toNewState( callback, state );

            lu.assert( this._animListener === null );

            var animListener = new AnimationListener( this.onShowBumpersDone, this );

            var i;
            var bumper;
            for ( i = 0; i < bumpers.length; i += 1 )
            {
                bumper = bumpers[ i ];

                animListener.listenTo( bumper.aev );
                bumper.aev.play( this.settings.bumperInAnim );

                bumper.hasBeenRevealed = true;
            }

            animListener.commit();
            lu.assert( animListener.getNumStillListeningTo() === bumpers.length );

            this._animListener = animListener;
        },

        hideAllBumpers: function( callback )
        {
            this._toNewState( callback, PinballRecallState.Present_HideBumpers );

            lu.assert( this._animListener === null );

            var animListener = new AnimationListener( this.onHideBumpersDone, this );

            var i;
            var bumper;
            for ( i = 0; i < this._bumpers.length; i += 1 )
            {
                bumper = this._bumpers[ i ];

                animListener.listenTo( bumper.aev );
                bumper.aev.play( this.settings.bumperOutAnim );

                bumper.hasBeenRevealed = false;
            }

            animListener.commit();
            lu.assert( animListener.getNumStillListeningTo() === this._bumpers.length );

            this._animListener = animListener;
        },

        onShowBumpersDone: function()
        {
            this._animListener = null;
            this._notifyCallbackOfStateComplete();
        },

        onHideBumpersDone: function()
        {
            this._animListener = null;
            this._notifyCallbackOfStateComplete();
        },

        _toNewState: function( callback, state )
        {
            // Not already in a state.
            // lu.assert( this._callback === null );
            // lu.assert( this._state === null );

            // Valid callback.
            lu.assert( callback !== undefined && callback !== null );

            this._state = state;
            this._callback = callback;
        },

        _notifyCallbackOfStateComplete: function()
        {
            lu.assert( this._callback !== undefined && this._callback !== null );

            var callback = this._callback;
            var state = this._state;

            // Clear out for possible immediate state change.
            // this._callback = null;
            // this._state = null;

            callback.stateComplete( state );
        },

        slideBoard: function( success, levelChange, levelNumber, callback )
        {
            var levelCard = this.getWidget( GameWidgetLevelCard.prototype.getType(), true );
            levelCard.setLevelValue( levelNumber );

            if ( success )
            {    
                if ( levelChange )
                {
                    this.playAnimation( FlashAnimationBoardSlideLevelUp, callback );
                }
                else
                {
                    this.playAnimation( FlashAnimationBoardSlideLeft, callback );
                }
            }
            else
            {
                if ( levelChange )
                {
                    this.playAnimation( FlashAnimationBoardSlideLevelDown, callback );
                }
                else
                {
                    this.playAnimation( FlashAnimationBoardSlideRight, callback );
                }
            }
        },

        firePinball: function( fromX, fromY, toX, toY, callback )
        {
            this._toNewState( callback, PinballRecallState.Simulate );

            var duration = this.settings.timeNeededForPinballToMoveToAnotherCell;
            var to = this.fromTilePositionToCoordinates( toX, toY );

            var moveAction = cc.MoveTo.create( duration, to );
            var finishedAction = cc.CallFunc.create( this.onBallMoved, this );

            var sequence = cc.Sequence.create( moveAction, finishedAction );
            this._ball.runAction( sequence );
            this._registerPause( this._ball );

            var from = this.fromTilePositionToCoordinates( fromX, fromY );

            var type;
            var distanceIntoCell;
            
            type = this.model.getBoardCellType( cc.p( fromX, fromY ) );
            distanceIntoCell = this.settings.trailingLightDistanceIntoCell1;
            if ( type === BoardCellType.Empty || type === BoardCellType.BumperLeft || type === BoardCellType.BumperRight )
            {
                this._addTrailingLight( ( to.x - from.x ) * distanceIntoCell + from.x, ( to.y - from.y ) * distanceIntoCell + from.y );
            }
            
            type = this.model.getBoardCellType( cc.p( toX, toY ) );
            distanceIntoCell = this.settings.trailingLightDistanceIntoCell2;
            if ( type === BoardCellType.Empty || type === BoardCellType.BumperLeft || type === BoardCellType.BumperRight )
            {
                this._addTrailingLight( ( to.x - from.x ) * distanceIntoCell + from.x, ( to.y - from.y ) * distanceIntoCell + from.y );
            }
        },

        onBallMoved: function()
        {
            this._unregisterPause( this._ball );
            lu.assert( this.__noPausableElements() );
            this._notifyCallbackOfStateComplete();
        },

        _addTrailingLight: function( posX, posY )
        {
            var trailSpriteSheet = this.settings.trailSpriteSheet;
            var trailAnimationNames = [ this.settings.trailAnim ];

            var trail = new AnimatedEntityView( trailSpriteSheet, trailAnimationNames, false, true );

            this.board.addChild( trail );
            trail.setZOrder( this.settings.zOrder.trail );

            // The AEV has no content size, so we can't change the anchor point to position its _sprite.
            // Instead, we shift by half dim.
            // bumper.setAnchorPoint( 0.5, 0.5 );
            trail.setPosition( cc.p( posX, posY ) );

            this._scaleBoardElement( trail._sprite );

            trail.play( this.settings.trailAnim );

            // cc.log( "Placed trail at " + posX + ", " + posY );
        },

        _getBumperOnTile: function( x, y )
        {
            var pos = cc.p( x, y );

            var i;
            for ( i = 0; i < this._bumpers.length; i += 1 )
            {
                if ( Point.equals( this._bumpers[ i ].boardPos, pos ) )
                {
                    return this._bumpers[ i ];
                }
            }

            return null;
        },

        showBumperBounce: function( pos, dir )
        {
            var bumper = this._getBumperOnTile( pos.x, pos.y );
            bumper.aev.play( this.settings.bumperBounceAnim );
            bumper.hasBeenRevealed = true;

            // @todo Consider direction.

            // if ( x < _boardObjects.size() )
            // {
            //     if ( y < _boardObjects[x].size() )
            //     {
            //         ofPtr< PinballRecallInstance > instance = getInstance();
            //         ofPtr< PinballRecallSettings > settings = getSettings();
            //         ofPtr< ofxGenericAnimatedImageView > image = dynamic_pointer_cast< ofxGenericAnimatedImageView >( _boardObjects[x][y] );
            //         if ( instance && settings && image )
            //         {
            //             if ( instance->getBoardObjectType( x, y ) == PinballRecallInstance::BoardCellTypeBumperRight )
            //             {                
            //                 if ( dir == DirRight || dir == DirDown )
            //                 {
            //                     image->setImageFrames( settings->getBumperRightBottomBounceImages() );
            //                 }
            //                 else if ( dir == DirLeft || dir == DirUp )
            //                 {
            //                     image->setImageFrames( settings->getBumperRightTopBounceImages() );
            //                 }
            //             }
            //             else if ( instance->getBoardObjectType( x, y ) == PinballRecallInstance::BoardCellTypeBumperLeft )
            //             {
            //                 if ( dir == DirRight || dir == DirUp )
            //                 {
            //                     image->setImageFrames( settings->getBumperLeftTopBounceImages() );
            //                 }
            //                 else if ( dir == DirLeft || dir == DirDown )
            //                 {
            //                     image->setImageFrames( settings->getBumperLeftBottomBounceImages() );
            //                 }
            //             }
                        
            //             image->setTag( AnimationTagBumperBounce );
            //             image->setLoopMode( ofxGenericAnimatedImageLoopTypeClamp );
            //             image->setVisible( true );
            //             image->start();
            //         }
            //     }
            // }
        },

        startSuccessSequence: function()
        {
            var start = this.model.getEndPosition();

            // _pinballRecallInstance->setSuccessSequencePosition( start );
            
            var boardSize = this.model.getBoardSize();
            var dir;
            if ( start.y === 0 )
            {
                dir = Dir.Left;
            }
            else if ( start.y === boardSize.y + 1 )
            {
                dir = Dir.Right;
            }
            else if ( start.x === 0 )
            {
                dir = Dir.Up;
            }
            else if ( start.x === boardSize.x + 1 )
            {
                dir = Dir.Down;
            }

            var pos = cc.p( start.x, start.y );
            var delay = 0.0;
            var delaySpacing = this.settings.successSeriesSpacingSeconds;

            // _pinballRecallInstance->setSuccessSequenceDirection( dir );

            var animListener = new AnimationListener( this.onSuccessSequenceDone, this );

            var advance = function()
            {
                // cc.log( "advancing from " + pos.x + ", " + pos.y );

                // pos += dir;
                pos.x += dir.x;
                pos.y += dir.y;

                if ( Point.equals( pos, start ) )
                {
                    return false;
                }

                // cc.log( "at " + pos.x + ", " + pos.y );

                var shouldMove = function( pos )
                {
                    var move = false;

                    if ( pos.x === 0 && pos.y === 0 )
                    {
                        dir = Dir.Up;
                        move = true;

                        // cc.log( "moving up" );
                    }
                    else if ( pos.x === 0 && pos.y === boardSize.y + 1 )
                    {
                        dir = Dir.Right;
                        move = true;

                        // cc.log( "moving right" );
                    }
                    else if ( pos.x === boardSize.x + 1 && pos.y === boardSize.y + 1 )
                    {
                        dir = Dir.Down;
                        move = true;

                        // cc.log( "moving down" );
                    }
                    else if ( pos.x === boardSize.x + 1 && pos.y === 0 )
                    {
                        dir = Dir.Left;
                        move = true;

                        // cc.log( "moving left" );
                    }
                    else if ( Point.equals( pos, this.model.getStartPosition() ) )
                    {
                        // Continue same direction.
                        move = true;

                        // cc.log( "moving past start pos" );
                    }

                    return move;
                }.bind( this );

                while ( shouldMove( pos ) )
                {
                    // cc.log( "moved to " + pos.x + ", " + pos.y );

                    pos.x += dir.x;
                    pos.y += dir.y;

                    if ( Point.equals( pos, start ) )
                    {
                        return false;
                    }
                }

                var spriteSheet = this.settings.successSpriteSheet;
                var animationNames = [ this.settings.successAnim ];

                var aev = new AnimatedEntityView( spriteSheet, animationNames, false, true );

                this.board.addChild( aev );
                aev.setZOrder( this.settings.zOrder.success );

                var position = this.fromTilePositionToCoordinates( pos.x, pos.y );

                // The AEV has no content size, so we can't change the anchor point to position its _sprite.
                // Instead, we shift by half dim.
                // bumper.setAnchorPoint( 0.5, 0.5 );
                aev.setPosition( position );

                this._scaleBoardElement( aev._sprite );

                animListener.listenTo( aev );
                // ll.log("listening to aev");

                var delayAction = cc.DelayTime.create( delay );
                delay += delaySpacing;

                var playFunc = function() { this._unregisterPause( aev ); aev.play( this.settings.successAnim ); };
                var playAction = cc.CallFunc.create( playFunc, this );

                var sequence = cc.Sequence.create( delayAction, playAction );
                aev.runAction( sequence );
                this._registerPause( aev );

                return true;
            }.bind( this );

            var numAnimating = 0;
            while ( advance() )
            {
                numAnimating += 1;
            }

            // cc.log( "listening to " + numAnimating );

            animListener.commit();
            lu.assert( animListener.getNumStillListeningTo() === numAnimating );
        },

        onSuccessSequenceDone: function()
        {
            lu.assert( this.__noPausableElements() );
            this.interactionResponder.stateComplete( PinballRecallState.Simulate_SuccessSequence );
        },

        onRestart: function()
        {
            this.destroyBoard();

            this._super();

            this._init();
        },

        onPause: function()
        {
            this._super();

            var i;
            for ( i = 0; i < this._pausableElements.length; i += 1 )
            {
                this._pausableElements[ i ].pauseSchedulerAndActions();
            }
        },

        onResume: function()
        {
            this._super();

            var i;
            for ( i = 0; i < this._pausableElements.length; i += 1 )
            {
                this._pausableElements[ i ].resumeSchedulerAndActions();
            }
        },

        _registerPause: function( node )
        {
            this._pausableElements.push( node );
        },

        _unregisterPause: function( node )
        {
            this._pausableElements.splice( this._pausableElements.indexOf( node ), 1 );
        },

        __noPausableElements: function()
        {
            return this._pausableElements.length === 0;
        }
    } );
}() );
