/*global cc*/
/*global ll*/
/*global require*/

var Operator =
{
    Addition: "addition",
    Subtraction: "subtraction",
    Multiplication: "multiplication",
    Division: "division"
};

var RaindropsEquation;
( function()
{
    RaindropsEquation = cc.Class.extend(
    {
        _leftOperand: 0,
        _rightOperand: 0,
        _operator: Operator.Addition,
        _answer: 0,

        ctor: function( addition, subtraction, multiplication, division )
        {
            this.generateUsingRanges( addition, subtraction, multiplication, division );
        },

        getLeftOperand: function()
        {
            return this._leftOperand;
        },

        getOperator: function()
        {
            return this._operator;
        },

        getOperatorAsString: function()
        {
            var stringValue;

            switch ( this._operator )
            {
                case Operator.Addition:
                    stringValue = "+";
                    break;
                case Operator.Subtraction:
                    stringValue = "−";
                    break;
                case Operator.Multiplication:
                    stringValue = "×";
                    break;
                case Operator.Division:
                    stringValue = "÷";
                    break;
            }

            return stringValue;
        },

        getRightOperand: function()
        {
            return this._rightOperand;
        },

        getAnswer: function()
        {
            return this._answer;
        },

        generateUsingRanges: function( addition, subtraction, multiplication, division )
        {
            var possibleOperators = [];
            
            if ( addition && addition.isValid() )
            {
                possibleOperators.push( Operator.Addition );
            }

            if ( subtraction && subtraction.isValid() )
            {
                possibleOperators.push( Operator.Subtraction );
            }
            
            if ( multiplication && multiplication.isValid() )
            {
                possibleOperators.push( Operator.Multiplication );
            }

            if ( division && division.isValid() )
            {
                possibleOperators.push( Operator.Division );
            }
            
            if ( possibleOperators.length === 0 )
            {
                // TODO: build simple default equation
                ll.verbose( "Raindrops equation not supplied with any valid operand ranges, cannot build equation!" );

                return;
            }
            
            this._operator = possibleOperators.getRandomElement();

            switch( this._operator )
            {
                case Operator.Addition:
                    this._leftOperand = addition.getLeftOperand();
                    this._rightOperand = addition.getRightOperand();
                    break;
                    
                case Operator.Subtraction:
                    this._leftOperand = subtraction.getLeftOperand();
                    this._rightOperand = subtraction.getRightOperand();
                    if ( this._leftOperand < this._rightOperand )
                    {
                        var swapBuffer = this._leftOperand;
                        this._leftOperand = this._rightOperand;
                        this._rightOperand = swapBuffer;
                    }
                    break;
                    
                case Operator.Multiplication:
                    this._leftOperand = multiplication.getLeftOperand();
                    this._rightOperand = multiplication.getRightOperand();
                    break;
                    
                case Operator.Division:
                    this._rightOperand = division.getRightOperand();
                    this._leftOperand = division.getLeftOperand() * this._rightOperand;
                    break;
            }
            
            this.calculateAnswer();
        },

        calculateAnswer: function()
        {
            switch( this._operator )
            {
                case Operator.Addition:
                    this._answer = this._leftOperand + this._rightOperand;
                    break;
                    
                case Operator.Subtraction:
                    this._answer = this._leftOperand - this._rightOperand;
                    break;
                    
                case Operator.Multiplication:
                    this._answer = this._leftOperand * this._rightOperand;
                    break;
                    
                case Operator.Division:
                    this._answer = this._leftOperand / this._rightOperand;
                    break;
            }
        },

        toString: function()
        {
            var retval = "";
            
            retval += this._leftOperand.toString();
            retval += this.getOperatorAsString();
            retval += this._rightOperand.toString();
            
            return retval;
        }
    } );
}() );

