/*global ll*/
/*global require*/

require( "src/Game/Base/GameSettings.js" );
/*global GameSettings*/
require( "src/Game/RaindropsEquationRange.js" );
/*global RaindropsEquationRange*/

/**
 * The Raindrops settings.
 * @class
 * @name RaindropsSettings
 */
var RaindropsSettings;
( function()
{
    "use strict";
    
    RaindropsSettings = GameSettings.extend(
    {
        /** @lends RaindropsSettings.prototype */

        /**
         *  RaindropsSettings is the main class Raindrops settings.
         *
         *  @constructs
         *  @param {array} Settings.
         */
        ctor: function( settings )
        {
            this._super( settings, true );
        },

        getProperty: function( name )
        {
            //@hack just to get this to work for now, cleanup alter
            //var firstChar = name.charAt( 0 ).toLowerCase();
            //var localKey = firstChar + name.substring( 1 );
            var localKey = name;

            if ( this[ localKey ] === undefined || this[ localKey ] === null )
            {
                ll.verbose( "RaindropsSettings", "Asked for " + localKey + " and we don't have that setting!" );
            }

            return this[ localKey ];
        },

        getAmbientSound: function()
        {
            return this.getProperty( "ambientSound" );
        },

        getMaximumWaterLevel: function()
        {
            return this.getProperty( "maximumWaterLevel" );
        },

        getCloudsPerWaterLevel: function()
        {
            return this.getProperty( "cloudsPerWaterLevel" );
        },

        getClouds: function()
        {
            return this.getProperty( "clouds" );
        },        

        getCloudCount: function()
        {
            var clouds = this.getClouds();

            return clouds.length;
        },

        getCloud: function( cloudIndex )
        {
            var clouds = this.getClouds();

            return clouds[ cloudIndex ];
        },

        getCloudSize: function( cloudIndex )
        {
            var cloud = this.getCloud( cloudIndex );

            return cloud[ "size" ];
        },

        getCloudImageFileName: function( cloudIndex )
        {
            var cloud = this.getCloud( cloudIndex );

            return cloud[ "imageFileName" ];
        },

        getCloudVelocity: function( cloudIndex )
        {
            var cloud = this.getCloud( cloudIndex );

            return cloud[ "xVelocity" ];
        },

        getCloudOpacity: function( cloudIndex )
        {
            var cloud = this.getCloud( cloudIndex );

            return cloud[ "opacity" ];
        },

        getKeypadSmallButtonTopUpImage: function()
        {
            return this.getProperty( "keypadSmallButtonTopUpImage" );
        },

        getKeypadSmallButtonTopDownImage: function()
        {
            return this.getProperty( "keypadSmallButtonTopDownImage" );
        },
        
        getKeypadSmallButtonBottomImage: function()
        {
            return this.getProperty( "keypadSmallButtonBottomImage" );
        },
        
        getKeypadLargeButtonTopUpImage: function()
        {
            return this.getProperty( "keypadLargeButtonTopUpImage" );
        },
        
        getKeypadLargeButtonTopDownImage: function()
        {
            return this.getProperty( "keypadLargeButtonTopDownImage" );
        },
        
        getKeypadLargeButtonBottomImage: function()
        {
            return this.getProperty( "keypadLargeButtonBottomImage" );
        },

        getDisablePauseAtWaterLevel: function()
        {
            return this.getProperty( "disablePauseAtWaterLevel" );
        },

        getSunState: function()
        {
            return this.getProperty( "sunState" );
        },

        getBackgroundOverlaySunColor: function()
        {
            var sunState = this.getSunState();

            return sunState[ "OverlayColor" ];
        },

        getBackgroundOverlaySunOpacity: function()
        {
            var sunState = this.getSunState();

            return sunState[ "Opacity" ];
        },

        getBackgroundOverlaySunTransitionDuration: function()
        {
            var sunState = this.getSunState();

            return sunState[ "TransitionDuration" ];
        },

        getWaterLevelState: function( waterLevel )
        {
            var states = this.getProperty( "waterLevelStates" );

            return states[ waterLevel ];
        },

        getBackgroundOverlayColor: function( waterLevel )
        {
            var state = this.getWaterLevelState( waterLevel );

            return state[ "OverlayColor" ];
        },

        getBackgroundOverlayOpacity: function( waterLevel )
        {
            var state = this.getWaterLevelState( waterLevel );

            return state[ "Opacity" ];
        },

        getBackgroundOverlayTransitionDuration: function( waterLevel )
        {
            var state = this.getWaterLevelState( waterLevel );

            return state[ "TransitionDuration" ];
        },

        getRaindropScoreValue: function()
        {
            return this.getProperty( "raindropScoreValue" );
        },

        getMaxAllowedDropletCount: function( raindropsCount )
        {
            var allowedDropletCount = 1;
            
            var tierData = this.getTierData( raindropsCount );

            if ( tierData )
            {
                allowedDropletCount = tierData[ "MaxAllowedDropletCount" ];
            }
            
            return allowedDropletCount;
        },

        getTiers: function()
        {
            return this.getProperty( "tiers" );
        },

        getTierData: function( raindropsCount )
        {
            var retval = null;

            var tiers = this.getTiers();

            var currentThreshold = 0;

            var i;
            var tier;
            var threshold;
            for ( i = 0; i < tiers.length; i += 1 )
            {
                tier = tiers[ i ];
                threshold = tier[ "Threshold" ];

                if ( threshold <= raindropsCount && currentThreshold <= threshold )
                {
                    retval = tier;
                    currentThreshold = threshold;
                }
                else
                {
                    break;
                }
            }

            return retval;
        },

        getOpRange: function( raindropsCount, op )
        {
            var retval = new RaindropsEquationRange();

            var tierData = this.getTierData( raindropsCount );
            var rangeData = tierData[ "Ranges" ];
            var opData = rangeData[ op ];

            if ( opData )
            {
                var left = opData[ "left" ];
                var right = opData[ "right" ];
                retval.init( left[ "min" ], left[ "max" ], right[ "min" ], right[ "max" ] );
            }
            
            return retval;
        },

        getAdditionRange: function( raindropsCount )
        {
            return this.getOpRange( raindropsCount, "Add" );
        },

        getSubtratctionRange: function( raindropsCount )
        {
            return this.getOpRange( raindropsCount, "Sub" );
        },

        getMultiplicationRange: function( raindropsCount )
        {
            return this.getOpRange( raindropsCount, "Mul" );
        },

        getDivisionRange: function( raindropsCount )
        {
            return this.getOpRange( raindropsCount, "Div" );
        }
    } );
}() );
