/*global cc*/
/*global ll*/
/*global require*/

require( "src/Game/RaindropsSun.js" );
/*global RaindropsSun*/
require( "src/Utility/LumosityUtils.js" );
/*global lu*/

var RaindropsUtils =
{
    containsSun: function( droplets )
    {
        var i;
        var check;
        for( i = 0; i < droplets.length; i += 1 )
        {
            check = droplets[ i ];

            if ( check instanceof RaindropsSun )
            {
                return true;
            }
        }
        
        return false;
    },

    // @todo Everything below should get moved into LumosityUtils or a BoundingBoxUtils.

    bbPrint: function( node )
    {
        var bb = node.getBoundingBox();

        ll.verbose( "RU", "print", "bb for node " + node.nodeName + ":" + JSON.stringify( bb ) );
    },

    bbGetExtents: function( bb )
    {
        var extents =
        {
            tl: cc.p ( bb.x,            bb.y + bb.height ),
            tr: cc.p ( bb.x + bb.width, bb.y + bb.height ),
            br: cc.p ( bb.x + bb.width, bb.y ),
            bl: cc.p ( bb.x,            bb.y )
        };

        return extents;
    },

    bbOverlap: function( a, b )
    {
        return cc.rectIntersectsRect( a, b );
    },

    bbUnion: function( a, b )
    {
        var aExtents = this.bbGetExtents( a );
        var bExtents = this.bbGetExtents( b );

        var bl = cc.p( Math.min( aExtents.bl.x, bExtents.bl.x ), Math.min( aExtents.bl.y, bExtents.bl.y ) );
        var tr = cc.p( Math.max( aExtents.tr.x, bExtents.tr.x ), Math.max( aExtents.tr.y, bExtents.tr.y ) );

        var union = cc.rect( bl.x, bl.y, tr.x - bl.x, tr.y - bl.y );

        return union;
    },

    bbConvertFromTo: function( bb, fromNode, toNode )
    {
        var extents = this.bbGetExtents( bb );

        var blConverted = lu.convertPointSpace( extents.bl, fromNode, toNode );
        var trConverted = lu.convertPointSpace( extents.tr, fromNode, toNode );

        var bbConverted = cc.rect( blConverted.x, blConverted.y, trConverted.x - blConverted.x, trConverted.y - blConverted.y );

        return bbConverted;
    },

    addBoundingBoxToNode: function( node, bb, color, opacity, zOrder )
    {
        var size = cc.size( bb.width, bb.height );
        var sprite = lu.createRectangularSprite( size, color );
        if ( zOrder === undefined )
        {
            var highest = Number.MIN_VALUE;
            var children = node.getChildren();
            var count = node.getChildrenCount();
            var i;
            var child;
            var childZ;
            for ( i = 0; i < count; i += 1 )
            {
                child = children[ i ];
                childZ = child.getZOrder();
                highest = childZ > highest ? childZ : highest;
            }
            zOrder = highest + 1;
        }
        node.addChild( sprite );
        sprite.setAnchorPoint( 0, 0 );
        sprite.setPosition( bb.x, bb.y );
        sprite.setOpacity( opacity );
        sprite.setVisible( true );
        sprite.setZOrder( zOrder );
        return sprite;
    },

    enableBoundingBox: function( node, color, opacity, zOrder )
    {
        var boundingBox = node.getBoundingBox();
        var parent = node.getParent();

        this.addBoundingBoxToNode( parent, boundingBox, color, opacity, zOrder );
    },

    cloneLabel: function( label )
    {
        var clone = cc.LabelTTF.create( label.getString(), label.getFontName(), label.getFontSize() );
        clone.setAnchorPoint( label.getAnchorPoint().x, label.getAnchorPoint().y );
        clone.setPosition( label.getPosition() );
        return clone;
    },

    getTextBounds: function( text, fontName, fontSize, anchorPoint, position )
    {
        var label = cc.LabelTTF.create( text, fontName, fontSize );
        label.setAnchorPoint( anchorPoint.x, anchorPoint.y );
        label.setPosition( position );
        var bounds = label.getBoundingBox();

        return bounds;
    },

    getTextBoundsUsingLabel: function( label )
    {
        return this.getTextBounds(
            label.getString(),
            label.getFontName(),
            label.getFontSize(),
            label.getAnchorPoint(),
            label.getPosition()
        );
    },

    getTextSize: function( text, fontName, fontSize )
    {
        var label = cc.LabelTTF.create( text, fontName, fontSize );
        var size = label.getContentSize();

        return size;
    },

    getTextSizeUsingLabelFont: function( text, label )
    {
        var size = this.getTextSize( text, label.getFontName(), label.getFontSize() );

        return size;
    },

    getFrame: function( view )
    {
        var position = view.getPosition();
        var contentSize = view.getContentSize();
        var anchorPoint = view.getAnchorPoint();
        var scaleX = view.getScaleX();
        var scaleY = view.getScaleY();

        var frame = {
            x: position.x,
            y: position.y,
            contentWidth: contentSize.width,
            contentHeight: contentSize.height,
            width: contentSize.width * scaleX,
            height: contentSize.height * scaleY,
            top: position.y + ( 1.0 - anchorPoint.y ) * contentSize.height * scaleY,
            right: position.x + ( 1.0 - anchorPoint.x ) * contentSize.width * scaleX,
            bottom: position.y - anchorPoint.y * contentSize.height * scaleY,
            left: position.x - anchorPoint.x * contentSize.width * scaleX
        };

        return frame;
    }
};
