/*global cc*/
/*global ll*/
/*global localize*/
/*global require*/

/*global LayoutLoader*/

require( "src/Animation/EaseWrapper.js" );
/*global EaseType*/
/*global EaseWrapper*/
require( "src/Game/Base/GameView.js" );
/*global GameView*/
require( "src/Game/RaindropsCloud.js" );
/*global RaindropsCloud*/
require( "src/Game/RaindropsDroplet.js" );
/*global RaindropsDroplet*/
require( "src/Game/RaindropsSun.js" );
/*global RaindropsSun*/
require( "src/Game/RaindropsUtils.js" );
/*global RaindropsUtils*/
require( "src/Game/Widget/GameWidgetHud.js" );
/*global GameWidgetHud*/
require( "src/Game/Widget/GameWidgetPauseButton.js" );
/*global GameWidgetPauseButton*/
require( "src/Entity/AnimatedEntityView.js" );
/*global AnimatedEntityView*/
require( "src/View/LumosityLabel.js" );
/*global LumosityLabel*/
require( "src/View/LumositySprite.js" );
/*global LumositySprite*/
require( "src/Utility/LumosityUtils.js" );
/*global lu*/

var DropletCreator;
( function()
{
    DropletCreator = cc.Class.extend(
    {
        _gameView: null,
        _equation: null,

        ctor: function( gameView, equation )
        {
            this._gameView = gameView;
            this._equation = equation;
        },

        layout_creatingNode: function( nodeName, nodeData )
        {
            var textData = nodeData[ LayoutLoader.textDataKey ];

            switch( nodeName )
            {
                case "droplet_operandTop":
                    textData[ LayoutLoader.textStringKey ] = this._equation.getLeftOperand();
                    return LayoutLoader.createLabel( textData );

                case "droplet_operandBottom":
                    textData[ LayoutLoader.textStringKey ] = this._equation.getRightOperand();
                    return LayoutLoader.createLabel( textData );

                case "droplet_operator":
                    textData[ LayoutLoader.textStringKey ] = this._equation.getOperatorAsString();
                    return LayoutLoader.createLabel( textData );

                case "sun_operandTop":
                    textData[ LayoutLoader.textStringKey ] = this._equation.getLeftOperand();
                    return LayoutLoader.createLabel( textData );

                case "sun_operandBottom":
                    textData[ LayoutLoader.textStringKey ] = this._equation.getRightOperand();
                    return LayoutLoader.createLabel( textData );

                case "sun_operator":
                    textData[ LayoutLoader.textStringKey ] = this._equation.getOperatorAsString();
                    return LayoutLoader.createLabel( textData );
            }
            
            return null;
        }
    } );
}() );



var RaindropsView;
( function()
{
    "use strict";
    
    RaindropsView = GameView.extend(
    {
        /** @lends RaindropsView.prototype */

        game: null,

        _keypadButtons: null,

        _shouldProcessInput: false,

        _inputBufferLabel: null,

        _keypadDigitTopTemplate: null,
        _keypadDigitBottomTemplate: null,

        _keypadEnterTopTemplate: null,
        _keypadEnterBottomTemplate: null,

        _templates_droplet: null,
        _templates_sun: null,

        _clouds: null,

        _sunIsOut: false,

        _currentWaterLevel: 0,

        _dropletAvoidColumnsFrames: null,
        __visAvoidZones: [],
        __enableVisAvoidZones: false,

        // Based on keypad construction.
        _waterLevelHeights: [],
        
        /**
        * The Raindrops view.
        * @constructs
        * @param {cc.Size} screenSize The size of the screen that this view will be inserted into.
        * @param {Object} [settings] The settings that we can pull view-based data from.
        * @param {RaindropsModel} [model] The GameModel that we are linked to.
        * @param {Object} [interactionResponder] The object that we should send interaction events to. We also use this is a default animation event listener.
        * @param {boolean} [preload=true] If true, will preload all flipbooks and other assets upon creation.
        */
        ctor: function( screenSize, settings, model, interactionResponder, preload )
        {
            this._super( screenSize, settings, model, interactionResponder, preload );

            this._currentWaterLevel = 0;
            this._sunIsOut = false;

            this._clouds = [];

            this._dropletAvoidColumnsFrames = [];

            this.backgroundColorOverlay.setOpacity( 0 );

            // this.waterSurfaceImage1.setAnchorPoint( 0.5, 0 );
            // this.waterSurfaceImage2.setAnchorPoint( 0.5, 0 );

            // var countdown = this._widgets[ GameWidgetCountdown.prototype.getType() ];
            // countdown.setVisible( true );
        },

        preload: function()
        {
            this._super();
        },

        constructKeypad: function()
        {
            var topContainer = this[ "keypad_buttonsTopLayer" ];
            var bottomContainer = this[ "keypad_buttonsBottomLayer" ];

            var containerFrame = RaindropsUtils.getFrame( topContainer );

            //////////////////////////////////////// ENTRY AREA

            var entryArea = new LumositySprite( this.settings.entryBoxImage );
            entryArea.nodeName = "entry";
            this.keypad_entry = entryArea;
            topContainer.addChild( entryArea );

            var entryCurrentSize = entryArea.getContentSize();
            var entryCurrentWidthOverHeight = entryCurrentSize.width / entryCurrentSize.height;
            var entryDesiredWidth = containerFrame.contentWidth;
            var entryDesiredHeight = entryDesiredWidth / entryCurrentWidthOverHeight;
            var entryDesiredSize = cc.size( entryDesiredWidth, entryDesiredHeight );

            entryArea.setAnchorPoint( cc.p( 0, 1 ) );
            entryArea.setContentSize( entryDesiredSize, true, true );
            entryArea.setPosition( 0, containerFrame.contentHeight );
            entryArea.setZOrder( 0 );

            var entryAreaFrame = RaindropsUtils.getFrame( entryArea );

            //////////////////////////////////////// BACKSPACE BUTTON

            var backspace = this._createButton(
                this.settings.backspaceImageDefault,
                this.settings.backspaceImageHighlight,
                null,
                "",
                "",
                0,
                cc.c3b( 255, 255, 255 ),
                cc.c3b( 255, 255, 255 ),
                null
            );
            this._keypadButtons.push( backspace );
            backspace.setTarget( this );
            backspace.setPressedCallback( this.onKeypadBackspacePressed );
            backspace.setAnchorPoint( cc.p( 1, 1 ) );
            entryArea.addChild( backspace );

            var backspaceCurrentSize = backspace.getContentSize();
            var backspaceCurrentWidthOverHeight = backspaceCurrentSize.width / backspaceCurrentSize.height;
            var backspaceDesiredHeight = entryDesiredHeight;
            var backspaceDesiredWidth = backspaceDesiredHeight * backspaceCurrentWidthOverHeight;
            var backspaceDesiredSize = cc.size( backspaceDesiredWidth, backspaceDesiredHeight );

            backspace.setContentSize( backspaceDesiredSize );
            backspace.setPosition( entryAreaFrame.contentWidth, entryAreaFrame.contentHeight );

            //////////////////////////////////////// ENTRY BOX

            var entryBoxFont = this.keypadFontEntryArea.getFontName();
            var entryBoxTextSize = this.keypadFontEntryArea.getFontSize();
            var entryBoxTextColor = this.keypadFontEntryArea.getColor();
            entryBoxTextColor = cc.c3b( entryBoxTextColor.r, entryBoxTextColor.g, entryBoxTextColor.b );

            var entryBox = this[ "keypad_entry" ];
            var entryBoxFrame = RaindropsUtils.getFrame( entryBox );
            this._inputBufferLabel = LumosityLabel.create( "", entryBoxFont, entryBoxTextSize );
            entryBox.addChild( this._inputBufferLabel );
            this._inputBufferLabel.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            this._inputBufferLabel.setPosition( entryBoxFrame.contentWidth / 2, entryBoxFrame.contentHeight / 2 );
            this._inputBufferLabel.setColor( entryBoxTextColor );

            ////////////////////////////////////////
            ////////////////////////////////////////
            ////////////////////////////////////////

            var smallButtonWidth = this._keypadDigitTopTemplate.size.width;
            var smallButtonHeight = this._keypadDigitTopTemplate.size.height;
            var widthRemainingForMargin = containerFrame.contentWidth - ( smallButtonWidth * 3 );
            var heightRemainingForMargin = ( containerFrame.contentHeight - entryAreaFrame.height ) - ( smallButtonHeight * 4 );
            var horiMargin = widthRemainingForMargin / 2;
            var vertMargin = heightRemainingForMargin / 4;
            var hIncrement = smallButtonWidth + horiMargin;
            var vIncrement = smallButtonHeight + vertMargin;

            var keypadHeight = containerFrame.contentHeight - entryAreaFrame.height - vertMargin;

            //this.__debugOutputNodeInfoAndBB( this.keypad_entry );

            // ll.verbose( "x", "x", "this.keypad_entry.getContentSize().height = " + this.keypad_entry.getContentSize().height );
            // ll.verbose( "x", "x", "this.keypad_entry.getContentSize().height scaled = " + this.keypad_entry.getContentSize().height * this.keypad_entry.getScaleX() );
            // ll.verbose( "x", "x", "vertMargin = " + vertMargin );
            // ll.verbose( "x", "x", "keypadHeight = " + keypadHeight );

            // var hMargin = 20;
            // var vMargin = 20;
            // var smallButtonWidth = ( topContainer.getContentSize().width - ( hMargin * 2 ) ) / 3;
            // var smallButtonHeight = ( topContainer.getContentSize().height - ( vMargin * 3 ) ) / 4;
            // var hIncrement = hMargin + smallButtonWidth;
            // var vIncrement = vMargin + smallButtonHeight;

            var smallButtonTopImage = this._keypadDigitTopTemplate.image;

            var font = this.keypadFontDeselected.getFontName();
            var textSize = this.keypadFontDeselected.getFontSize();
            var textColor = this.keypadFontDeselected.getColor();
            var textDownColor = this.keypadFontSelected.getColor();

            // buttons: 1 - 9

            this._keypadBounds = [ [], [], [] ];

            var i;
            var j;

            for ( i = 0; i < 3; i += 1 )
            {
                for ( j = 0; j < 3; j += 1 )
                {
                    this._createKeypadButton(
                        topContainer,
                        bottomContainer,
                        i,
                        j,
                        smallButtonTopImage,
                        "keypad_buttonTopDown.png", //@todo hacky, cleanup, move to settings
                        font,
                        textSize,
                        textColor,
                        textDownColor,
                        keypadHeight,
                        hIncrement,
                        vIncrement,
                        this._keypadDigitTopTemplate.size,
                        this._keypadBounds[j]
                    );
                }
            }

            this._waterLevelHeights = [];

            // It's OK to do this transform now because we haven't started animating the keypad container yet,
            // so it's in the appropriate space on screen according to the layout.
            for ( i = 0; i < 4; i += 1 )
            {
                this._waterLevelHeights.push( lu.convertPointSpace( cc.p(
                    0, smallButtonHeight + i * vIncrement - smallButtonHeight * 0.33 ),
                    topContainer,
                    this._gameStage
                ) );
            }

            var enterOnLeft = false;

            // button: 0

            var bottomLeft = cc.p( 0, 0 );
            var bottomRight = cc.p( 1, 0 );
            var containerBR = cc.p( containerFrame.contentWidth, 0 );

            var buttonZero = this._createButton(
                smallButtonTopImage,
                "keypad_buttonTopDown.png", //@todo hacky, cleanup, move to settings
                null,
                "0",
                font,
                textSize,
                textColor,
                textDownColor,
                null
            );
            this._keypadButtons.push( buttonZero );
            buttonZero.setTarget( this );
            buttonZero.setPressedCallback( this.onKeypadDigitPressed );
            topContainer.addChild( buttonZero );
            var anchorZero = enterOnLeft ? bottomRight : bottomLeft;
            buttonZero.setAnchorPoint( anchorZero );
            // Weird, _createButton just doesn't do anything like this.
            buttonZero.setContentSize( this._keypadDigitTopTemplate.size );
            var posZero = enterOnLeft ? containerBR : cc.PointZero();
            buttonZero.setPosition( posZero );
            var buttonZeroFrame = RaindropsUtils.getFrame( buttonZero );
            buttonZero.titleLabel.setPosition( cc.p( buttonZeroFrame.width / 2, buttonZeroFrame.height / 2 ) );
            buttonZero.nodeName = "0";

            var buttonBackNameZero = "keypadBottom_0";
            var nodesZero = LayoutLoader.createNodeFromData( this._keypadDigitBottomTemplate, buttonBackNameZero, this._gameStage, bottomContainer, bottomContainer, this );
            var nodeZero = nodesZero[ 0 ];
            nodeZero.setAnchorPoint( anchorZero );
            nodeZero.setPosition( posZero );
            nodeZero.setZOrder( 0 );

            // button: enter

            var largeButtonTopImage = this._keypadEnterTopTemplate.image;

            var buttonEnter = this._createButton(
                largeButtonTopImage,
                "keypad_enterTopDown.png", //@todo hacky, cleanup, move to settings
                null,
                localize("enterButtonText", "ENTER"),
                font,
                textSize,
                textColor,
                textDownColor,
                null
            );
            this._keypadButtons.push( buttonEnter );
            buttonEnter.setTarget( this );
            buttonEnter.setPressedCallback( this.onKeypadEnterPressed );
            topContainer.addChild( buttonEnter );
            var anchorEnter = enterOnLeft ? bottomLeft : bottomRight;
            buttonEnter.setAnchorPoint( anchorEnter );
            // Weird, _createButton just doesn't do anything like this.
            buttonEnter.setContentSize( this._keypadEnterTopTemplate.size );
            var posEnter = enterOnLeft ? cc.PointZero() : containerBR;
            buttonEnter.setPosition( posEnter );
            var buttonEnterFrame = RaindropsUtils.getFrame( buttonEnter );
            buttonEnter.titleLabel.setPosition( cc.p( buttonEnterFrame.width / 2, buttonEnterFrame.height / 2 ) );
            buttonEnter.nodeName = "enter";

            var buttonBackNameEnter = "keypadBottom_Enter";
            var nodesEnter = LayoutLoader.createNodeFromData( this._keypadEnterBottomTemplate, buttonBackNameEnter, this._gameStage, bottomContainer, bottomContainer, this );
            var nodeEnter = nodesEnter[ 0 ];
            nodeEnter.setAnchorPoint( anchorEnter );
            nodeEnter.setPosition( posEnter );
            nodeEnter.setZOrder( 0 );
        },

        _enableKeypadButtons: function( onoff )
        {
            var i;
            for ( i = 0; i < this._keypadButtons.length; i += 1 )
            {
                this._keypadButtons[ i ].setEnabled( onoff );
            }
        },

        onBegin: function()
        {
            this._initWater();

            this._shouldProcessInput = false;
            this._enableKeypadButtons( false );

            this.placeKeypad();
        },

        clearArrow: function( arrowView )
        {
            arrowView.setColor( cc.c3b( 255, 255, 255 ) );
        },

        onRestart: function()
        {
            this._super();

            while (this._clouds.length > 0)
            {
                this._removeCloudAtIndex(0);
            }

            this.clearArrow( this.waterLevelIndicator1 );
            this.clearArrow( this.waterLevelIndicator2 );
            this.clearArrow( this.waterLevelIndicator3 );
            this.clearArrow( this.waterLevelIndicator4 );

            // It's possible to restart after the flood begins by pressing the home button (minimizing the app) and then returning to it.
            // If that happens...

            // The pause button has been disabled, so re-enable it.
            this._enablePauseButton();

            // The waves have been hidden, so show them again.
            this.waterSurfaceImage1.setOpacity( 255 );
            this.waterSurfaceImage2.setOpacity( 255 );
        },

        _convertCoordToKeypadDigit: function( i, j )
        {
            var digit = 9 - ( i * 3 + 2 - j );

            return digit;
        },

        _createKeypadButton: function( topContainer, bottomContainer, i, j, img, downImg, font, textSize, textColor, textDownColor, keypadHeight, hInc, vInc, contentSize, bounds )
        {
            var text = this._convertCoordToKeypadDigit( i, j ).toString();

            //ll.verbose("view", "making keypad button " + text);

            //upFilename, downFilename, onClickFunctionOrButtonName, text, font, textSize, textColor, textDownColor, contentSize )
            var button = this._createButton(
                img,
                downImg,
                null,
                text,
                font,
                textSize,
                textColor,
                textDownColor,
                contentSize
            );
            this._keypadButtons.push( button );
            button.setTarget( this );
            button.setPressedCallback( this.onKeypadDigitPressed );

            // button.setBackgroundImageForState( cc.Sprite.create( img ), LumosityButtonState.ButtonStateNormal );
            // button.setBackgroundImageForState( cc.Sprite.create( downImg ), LumosityButtonState.ButtonStateSelected );

            topContainer[ "button_" + text ] = button;

            topContainer.addChild( button );
            button.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            // Weird, _createButton just doesn't do anything like this.
            button.setContentSize( contentSize );

            //var containerFrame = RaindropsUtils.getFrame( topContainer );

            var xPos = j * hInc + contentSize.width / 2;
            var yPos = keypadHeight - i * vInc - contentSize.height / 2;
            button.setPosition( xPos, yPos );

            var buttonFrame = RaindropsUtils.getFrame( button );
            button.titleLabel.setPosition( cc.p( buttonFrame.width / 2, buttonFrame.height / 2 ) );
            button.nodeName = text;

            var textBounds = RaindropsUtils.getTextBounds(
                text,
                font,
                textSize,
                button.titleLabel.getAnchorPoint(),
                button.titleLabel.getPosition()
            );

            bounds.push( textBounds );

            //RaindropsUtils.enableBoundingBox( button, cc.c3b( 0, 255, 0 ), 128 );
            //RaindropsUtils.addBoundingBoxToNode( button, textBounds, cc.c3b( 0, 255, 0 ), 128 );

            // this.__debugOutputNodeInfoAndBB( topContainer );
            // this.__debugOutputNodeInfoAndBB( bottomContainer );

            //this.createKeypadSprite( img, xPos, yPos, topContainer );

            var buttonBackName = "keypadBottom_" + text;
            var nodes = LayoutLoader.createNodeFromData( this._keypadDigitBottomTemplate, buttonBackName, this._gameStage, bottomContainer, bottomContainer, this );
            var node = nodes[ 0 ];
            node.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            node.setPosition( xPos, yPos );
            node.setZOrder( 0 );
            //node.setContentSize( contentSize);

            // this.__debugOutputNodeInfoAndBB( button );
            // this.__debugOutputNodeInfoAndBB( button.backgroundImageForState( LumosityButtonState.ButtonStateNormal ) );
            // this.__debugOutputNodeInfoAndBB( node );

            // NOT A LUMOSITY SPRITE - THIS IS A cc.Sprite!!!!!
            // this.__debugOutputNodeInfoAndBB( node._sprite );
        },

        // createKeypadSprite: function( img, x, y, container )
        // {
        //     var s = cc.Sprite.create( img );
        //     s.setAnchorPoint( cc.p( 0.5, 0.5 ) );
        //     s.setPosition( x, y );
        //     s.setZOrder( 0 );
        //     container.addChild( s );
        // },

        onKeypadDigitPressed: function( button )
        {
            if ( !this._shouldProcessInput )
            {
                return;
            }

            var currValueStr = this.getInputBufferContents();

            if ( currValueStr.length < 3 )
            {
                var newValueStr = currValueStr + button.nodeName;

                var newValue = parseInt( newValueStr, 10 );
                this.model.setInputtedValue( newValue );
                this._inputBufferLabel.setString( newValueStr );
            }
        },

        getInputBufferContents: function()
        {
            var labelStr = this._inputBufferLabel.getString();

            // This check is necessary for DoubleWide, not for Locomotive.
            if ( labelStr === null || labelStr === undefined )
            {
                labelStr = "";
            }

            return labelStr;
        },

        isInputBufferEmpty: function()
        {
            var contents = this.getInputBufferContents();

            // Used to be this, but now getInputBufferContents() sanitizes for us.
            //return ( contents === null || contents === undefined || contents === "" );

            return contents === "";
        },

        onKeypadBackspacePressed: function(
            //button
            )
        {
            if ( !this._shouldProcessInput )
            {
                return;
            }

            var game = this.getRaindropsGame();
            game.backspace();
        },

        onKeypadEnterPressed: function( button )
        {
            if ( !this._shouldProcessInput )
            {
                return;
            }

            if ( this.isInputBufferEmpty() )
            {
                return;
            }

            var game = this.getRaindropsGame();
            game.testValue();
        },

        constructDroplet: function( creationTime, equation, dropletFallSpeed )
        {
            return this._constructTrialElement(
                new RaindropsDroplet( this.settings.operatorPadding ),
                this._templates_droplet,
                "droplet",
                creationTime,
                equation,
                dropletFallSpeed,
                "droplet/droplet",
                [ "droplet_burstAnim", "droplet_splashAnim" ], // @todo duped data, clean this up
                this.settings.dropletBurstSFX
            );
        },

        constructSun: function( creationTime, equation, dropletFallSpeed )
        {
            return this._constructTrialElement(
                new RaindropsSun( this.settings.operatorPadding ),
                this._templates_sun,
                "sun",
                creationTime,
                equation,
                dropletFallSpeed,
                "sun/sun",
                [ "sunIdle", "sunBurst" ], // @todo duped data, clean this up
                this.settings.sunBurstSFX
            );
        },

        _constructTrialElement: function( element, template, prefix, creationTime, equation, dropletFallSpeed, spriteSheet, animNames, burstSFX )
        {
            element._equation = equation;
            element._fallRate = dropletFallSpeed;
            element._creationTime = creationTime;
            element._burstSFX = burstSFX;

            var rootViewName = prefix + "_instance";

            var dropletCreator = new DropletCreator( this, equation );

            var viewDictionary = {};
            LayoutLoader.createNodeFromData( template, rootViewName, this._gameStage, this._gameStage, viewDictionary, dropletCreator );

            // this is the one that matters, the parent of all the individual components
            element._dropletView = viewDictionary[ rootViewName ];

            element._dropletView.removeFromParent( false );
            this.rain.addChild( element._dropletView );

            // the components
            element._operandTop = viewDictionary[ prefix + "_operandTop" ];
            element._operandBottom = viewDictionary[ prefix + "_operandBottom" ];
            element._operator = viewDictionary[ prefix + "_operator" ];
            element._sprite = viewDictionary[ prefix + "_bg" ];

            element._operandTop.nodeName = "operandTop";
            element._operandBottom.nodeName = "operandBottom";
            element._operator.nodeName = "operator";
            element._sprite.nodeName = "bg";

            // this.__debugOutputNodeInfoAndBB( element._dropletView );
            // this.__debugOutputNodeInfoAndBB( element._sprite );

            // element._operandTop.titleLabel.
            // element._operandTop.titleLabel.setDimensions( { width: 200, height: element._operandTop.titleLabel.getContentSize().height } );

            element._dropletView.nodeName = "dropletView";

            element.ae = new AnimatedEntityView( spriteSheet, animNames, null, true );
            element._dropletView.addChild( element.ae );

            element._sprite.removeFromParent( false );
            element.ae.setSprite( element._sprite );

            // @todo figure out how to get the anim to line up with the static image
            
            // element.ae._sprite.setAnchorPoint( cc.p( 0.5, 0.5 ) );
            // var contentSize = element._dropletView;
            // element.ae._sprite.setPosition( contentSize.width / 2, contentSize.height / 2 );

            element.setValues( element._equation.getLeftOperand(), element._equation.getRightOperand(), element._equation.getOperator() );

            element.begin();

            return element;
        },

        clearValue: function()
        {
            this._inputBufferLabel.setString( "" );
        },

        incorrectAnswerFlashDone: function()
        {
            this._shouldProcessInput = true;

            var game = this.getRaindropsGame();
            game.clearValue();
        },

        setIncorrectAnswerEntered: function( incorrectAnswer )
        {
            ll.verbose("RaindropsView", "setIncorrectAnswerEntered");

            this._shouldProcessInput = false;

            // Doesn't look right in Locomotive, but works fine in DoubleWide.
            var red = cc.TintTo.create( 0.0, 255.0, 0, 0 );
            var end = cc.TintTo.create( 0.25, 255.0, 255.0, 255.0 );
            var finishedAction = cc.CallFunc.create( this.incorrectAnswerFlashDone, this );

            // debugging:
            //var moveAction = cc.MoveTo.create( 2.0, cc.p( 320, 600 ) );

            var sequence = cc.Sequence.create( red, end, finishedAction );
            this.keypad_entry.runAction( sequence );
        },

        getDropletSplashHeight: function()
        {
            var waterBB = this.water.getBoundingBox();
            var topY = waterBB.y + waterBB.height;

            return topY;
        },

        placeKeypad: function()
        {
            this._placeKeypadContainer( this.keypad_buttonsTopLayer );
            this._placeKeypadContainer( this.keypad_buttonsBottomLayer );
        },

        _placeKeypadContainer: function( target )
        {
            var bb = target.getBoundingBox();

            target.setPosition( cc.p( target.getPosition().x, -bb.height - 100 ) );
            target.setVisible( true );
        },

        animateShowKeypad: function()
        {
            this._showKeypadContainer( this.keypad_buttonsTopLayer );
            this._showKeypadContainer( this.keypad_buttonsBottomLayer );
        },

        _showKeypadContainer: function( target )
        {
            var bb = target.getBoundingBox();

            var moveUp = cc.MoveTo.create( 10.0 / 30.0, cc.p( target.getPosition().x, 76 ) );
            var easeUp = EaseWrapper.wrapActionWithEase( moveUp, EaseType.Out );
            var moveDown = cc.MoveTo.create( 4.0 / 30.0, cc.p( target.getPosition().x, 52 ) );
            var easeDown = EaseWrapper.wrapActionWithEase( moveDown, EaseType.None );
            var sequence = cc.Sequence.create( easeUp, easeDown );
            target.runAction( sequence );
        },

        animateHideKeypad: function()
        {
            this._hideKeypadContainer( this.keypad_buttonsTopLayer );
            this._hideKeypadContainer( this.keypad_buttonsBottomLayer );
        },

        _hideKeypadContainer: function( target )
        {
            var bb = target.getBoundingBox();

            var move = cc.MoveTo.create( 0.8, cc.p( target.getPosition().x, -bb.height ) );
            var ease = EaseWrapper.wrapActionWithEase( move, EaseType.In );
            var sequence = cc.Sequence.create( ease );
            target.runAction( sequence );
        },

        animateFlood: function()
        {
            var wait = this.settings.floodWaitBeforeStarting;
            var duration = this.settings.floodDuration;
            var finalOpacity = this.settings.floodFinalWaterOpacity;

            var delay;
            var sequence;

            delay = cc.DelayTime.create( wait );
            var fade = cc.FadeTo.create( duration, finalOpacity );
            sequence = cc.Sequence.create( delay, fade );
            this.water.runAction( sequence );

            delay = cc.DelayTime.create( wait );
            var move = cc.MoveTo.create( duration, cc.p ( this.water.getPosition().x, this._gameStage.getContentSize().height ) );
            sequence = cc.Sequence.create( delay, move );
            this.water.runAction( sequence );

            this._fadeOutWaterSurface( this.waterSurfaceImage1, wait, duration, finalOpacity );
            this._fadeOutWaterSurface( this.waterSurfaceImage2, wait, duration, finalOpacity );
        },

        _fadeOutWaterSurface: function( surface, wait, duration, finalOpacity )
        {
            var delay = cc.DelayTime.create( wait );
            var fade = cc.FadeTo.create( duration, finalOpacity );
            var sequence = cc.Sequence.create( delay, fade );
            surface.runAction( sequence );
        },

        fadeOutClouds: function()
        {
            var i;

            // NOTE: this.cloudLayer doesn't implement RGBAProtocol (it's just a cc.Node) so we're doing this on all the clouds:
            for ( i = 0; i < this._clouds.length; i += 1 )
            {
                this._clouds[ i ].fadeOut( 1.0 );
            }
        },

        addDropletAtRandomX: function( droplet )
        {
            lu.assert( droplet );

            var dropletPosition = this._getDropletPositionAtRandomX( droplet );
            droplet._dropletView.setPosition( dropletPosition );

            // ll.verbose( "RaindropsView", "addDropletAtRandomX", "added droplet at x,y " + dropletFrame.x + "    " + dropletFrame.y );
            // this.__debugOutputNodeInfoAndBB( this.rain );

            if ( droplet instanceof RaindropsSun )
            {
                this.setSunIsOut( true );
            }
        },

        _getDropletPositionAtRandomX: function( droplet )
        {
            var viewFrame = RaindropsUtils.getFrame( this.rain );
            var dropletFrame = droplet._dropletView.getBoundingBox();
            
            var placementX = 0;
            
            var avoidKeypadText = true;
            var useTextArea = true;

            var restrictByWidth;
            var xInset;

            var lastDroplet = this.model.getLastDroplet();
            var shouldAvoidLastDropet = lastDroplet !== null;
            var lastDropletBounds = null;

            var i;

            if ( this.__enableVisAvoidZones )
            {
                // Clear visualization.
                for ( i = 0; i < this.__visAvoidZones.length; i += 1 )
                {
                    this.rain.removeChild( this.__visAvoidZones[ i ] );
                }
                this.__visAvoidZones = [];
            }

            // Start with just the columns.
            var avoidZones = [];
            avoidZones = avoidZones.concat( this._dropletAvoidColumnsFrames );

            // Add most recent droplet if exists.
            if ( shouldAvoidLastDropet )
            {
                lastDropletBounds = lastDroplet._dropletView.getBoundingBox();
                avoidZones = this._dropletAvoidColumnsFrames.concat( [ lastDropletBounds ] );
            }

            // Sort.
            avoidZones.sort( function( a, b ) { return a.x - b.x; } );

            //ll.verbose( "RaindropsView", "_getDropletFrameAtRandomX", "Sorted = " + JSON.stringify( avoidZones ) );

            // Merge overlapping avoid zones.
            if ( shouldAvoidLastDropet )
            {
                var curr;
                var next;
                var replaceAt = -1;
                for ( i = 0; i < avoidZones.length - 1; i += 1 )
                {
                    curr = avoidZones[ i ];
                    next = avoidZones[ i + 1 ];

                    if ( RaindropsUtils.bbOverlap( curr, next ) )
                    {
                        replaceAt = i;
                        break;
                    }
                }

                if ( replaceAt > -1 )
                {
                    // ll.verbose( "RaindropsView", "_getDropletFrameAtRandomX", "Merging " + replaceAt + " and " + ( replaceAt + 1 ) );

                    var merged = RaindropsUtils.bbUnion( avoidZones[ replaceAt ], avoidZones[ replaceAt + 1 ] );
                    avoidZones[ replaceAt ] = merged;
                    avoidZones.removeAt( replaceAt + 1 );
                }
            }

            // ll.verbose( "RaindropsView", "_getDropletFrameAtRandomX", avoidZones.length + " zones are active." );

            if ( this.__enableVisAvoidZones )
            {
                var bbSprite;
                for ( i = 0; i < avoidZones.length; i += 1 )
                {
                    bbSprite = RaindropsUtils.addBoundingBoxToNode( this.rain, avoidZones[ i ], cc.c3b( 255, 255, 0 ), 200 );
                    this.__visAvoidZones.push( bbSprite );
                }
            }

            // ll.verbose( "RaindropsView", "_getDropletFrameAtRandomX", "Avoid = " + JSON.stringify( avoidZones ) );
            
            if ( avoidKeypadText )
            {
                // Constrain by either the width of the text in the droplet or by the width of the droplet itself.
                if ( useTextArea )
                {
                    var textArea = droplet.getTextArea();
                    restrictByWidth = textArea.width;

                    var dropletBounds = droplet._dropletView.getBoundingBox();
                    xInset = textArea.x - dropletBounds.x;
                }
                else
                {
                    restrictByWidth = dropletFrame.width;
                    xInset = 0;
                }

                // Start with full width.
                var usableWidth = viewFrame.contentWidth;

                // Now narrow down the usable width.
                var avoid;
                for ( i = 0; i < avoidZones.length; i += 1 )
                {
                    avoid = avoidZones[ i ];

                    usableWidth -= avoid.width + restrictByWidth;
                }

                // Prevent from being cut off of the right side of the screen.
                usableWidth -= dropletFrame.width;

                // Choose position.
                placementX = lu.getRandomInt( 0, Math.floor( usableWidth ) );

                // DEBUG:
                // var shifted = 0;

                // Shift until we're in between the correct columns.
                for ( i = 0; i < avoidZones.length; i += 1 )
                {
                    avoid = avoidZones[ i ];

                    if ( avoid.x - restrictByWidth < placementX + xInset )
                    {
                        placementX += avoid.width + restrictByWidth;

                        // DEBUG:
                        // shifted += 1;
                    }
                }

                // ll.verbose( "RaindropsView", "_getDropletFrameAtRandomX", "Shifted " + shifted + " times." );
            }
            
            // Droplet has a centered anchor point and our computation above was done assuming left anchor, so adjust accordingly.
            var dropletPosition = cc.p(
                placementX + dropletFrame.width / 2,
                viewFrame.contentHeight + dropletFrame.height / 2
            );
            
            return dropletPosition;
        },

        getCloudCount: function()
        {
            return this._clouds.length;
        },

        addCloud: function()
        {
            var cloud = this._createRandomCloud();
            
            var cloudFrame = RaindropsUtils.getFrame( cloud );
            var layerFrame = RaindropsUtils.getFrame( this.cloudLayer );
            
            var topPadding = RaindropsUtils.getFrame( this.hud ).height;
            
            var waterBB = this.water.getBoundingBox();
            var bottomPadding = waterBB.y + waterBB.height;

            ll.verbose( "RaindropsModel", "createCloud", "min cloud height is " + bottomPadding );
            
            cloudFrame.x = layerFrame.contentWidth;

            var usableHeight = layerFrame.contentHeight - topPadding - bottomPadding - cloudFrame.height;

            cloudFrame.y = lu.getRandomInt( 0, Math.floor( usableHeight ) ) + bottomPadding;

            this.cloudLayer.addChild( cloud );

            cloud.setPosition( cloudFrame.x, cloudFrame.y );

            //ll.verbose( "RaindropsModel", "createCloud", "made a cloud " + cloudFrame.x + ", " + cloudFrame.y );

            this._clouds.push( cloud );

            //this.__debugOutputNodeInfoAndBB( this.cloudLayer );
        },

        _createRandomCloud: function()
        {
            var i = lu.getRandomInt( 0, this.settings.getCloudCount(), false, true );

            return this._createCloud( i );
        },

        _createCloud: function( cloudIndex )
        {
            var size = this.settings.getCloudSize( cloudIndex );
            var imageFileName = this.settings.getCloudImageFileName( cloudIndex );
            var xVelocity = this.settings.getCloudVelocity( cloudIndex );
            var opacity = this.settings.getCloudOpacity( cloudIndex );

            var cloud = new RaindropsCloud( size, imageFileName, xVelocity, opacity );

            return cloud;
        },

        onUpdateAlways: function( deltaTime )
        {
            this._updateWater( deltaTime );
            this._updateWaterLevelIndicators();
        },

        onUpdate: function( deltaTime, paused )
        {
            this.updateCloudsMovement( deltaTime );

            // This is called manually by Raindrops so we can do it before the game begins.
            //this.updateWater( deltaTime );
        },

        _updateWaterLevelIndicators: function()
        {
            this.waterLevelIndicator4.setPosition( cc.p( this.waterLevelIndicator4.getPosition().x, this._waterLevelHeights[ 3 ].y ) );
            this.waterLevelIndicator3.setPosition( cc.p( this.waterLevelIndicator3.getPosition().x, this._waterLevelHeights[ 2 ].y ) );
            this.waterLevelIndicator2.setPosition( cc.p( this.waterLevelIndicator2.getPosition().x, this._waterLevelHeights[ 1 ].y ) );
            this.waterLevelIndicator1.setPosition( cc.p( this.waterLevelIndicator1.getPosition().x, this._waterLevelHeights[ 0 ].y ) );
        },

        updateCloudsMovement: function( deltaTime )
        {
            var i = 0;
            var cloud;
            var cloudFrame;

            while ( i < this._clouds.length )
            {
                cloud = this._clouds[ i ];

                if ( cloud )
                {
                    cloud.update( deltaTime );
                    
                    cloudFrame = RaindropsUtils.getFrame( cloud );

                    if ( cloudFrame.right < 0 )
                    {
                        this._removeCloudAtIndex( i );
                    }
                    else
                    {
                        i += 1;
                    }
                }
                else
                {
                    this._removeCloudAtIndex( i );
                }
            }
        },

        _removeCloudAtIndex: function( index )
        {
            var cloud = this._clouds[ index ];
            this.cloudLayer.removeChild( cloud );
            this._clouds.splice( index, 1 );
        },

        updateSurfaceToWaterHeight: function()
        {
            var waterBB = this.water.getBoundingBox();
            var convertedWaterBB = RaindropsUtils.bbConvertFromTo( waterBB, this.water.getParent(), this.waterSurfaceImage1.getParent() );
            var waterHeight = convertedWaterBB.y + convertedWaterBB.height;

            var overlapBuffer = 0;
            var surfaceY = waterHeight - overlapBuffer;

            this.waterSurfaceImage1.setPositionY( surfaceY );
            this.waterSurfaceImage2.setPositionY( surfaceY );
        },

        _initWater: function()
        {
            this.updateSurfaceToWaterHeight();

            // var img1_w = this.waterSurfaceImage1.getBoundingBox().width;
            // var img2_w = this.waterSurfaceImage2.getBoundingBox().width;
            // var trainWidth = img1_w + img2_w;
            // var screenWidth = this.waterSurfaceContainer.getContentSize().width;

            var img1_x = 0;
            var img2_x = img1_x + this.waterSurfaceImage1.getBoundingBox().width;

            this.waterSurfaceImage1.setPositionX( img1_x );
            this.waterSurfaceImage2.setPositionX( img2_x );

            this.waterSurfaceImage1.setVisible( true );
            this.waterSurfaceImage2.setVisible( true );
        },

        updateWaterPiece: function( deltaTime, piece )
        {
            var img1_w = this.waterSurfaceImage1.getBoundingBox().width;
            var img2_w = this.waterSurfaceImage2.getBoundingBox().width;
            var trainWidth = img1_w + img2_w;
            var screenWidth = this.waterSurfaceContainer.getContentSize().width;

            var delta = ( deltaTime / 3 ) * screenWidth;

            var img1_bb = piece.getBoundingBox();
            var img1_x = img1_bb.x + delta;

            if ( img1_x >= img1_bb.width )
            {
                img1_x -= trainWidth;
            }

            piece.setPositionX( img1_x );
        },

        _updateWater: function( deltaTime )
        {
            this.updateSurfaceToWaterHeight();
            this.updateWaterPiece( deltaTime, this.waterSurfaceImage1 );
            this.updateWaterPiece( deltaTime, this.waterSurfaceImage2 );
        },

        getRaindropsSettings: function()
        {
            return this.settings;
        },

        setWaterLevel: function( level, animate )
        {
            if ( animate && this.animationController )
            {
                if ( level > 0 && level !== this._currentWaterLevel )
                {
                    var strAnim = "waterRise" + level.toString();
                    this.animationController.play( strAnim );

                    this._animateWaterRiseToLevel( level, 0.8, EaseType.In );
                }
            }

            this.setBackgroundColorForLevel( level, animate );
            //this.setWaterLevelIndicatorsForLevel( level );
            
            this._currentWaterLevel = level;

            //ll.verbose( "water", "level set to " + level );
        },

        _animateWaterRiseToLevel: function( level, duration, easeType )
        {
            var waveHeight = this.waterSurfaceImage1.getBoundingBox().height;
            var waterLevelHeight = this._waterLevelHeights[ level ].y;
            var waterHeight = waterLevelHeight - waveHeight * 0.5;
            var move = cc.MoveTo.create( duration, cc.p( this.water.getPosition().x, waterHeight ) );

            var sequence = null;

            if ( easeType !== null )
            {
                var ease = EaseWrapper.wrapActionWithEase( move, easeType );
                sequence = cc.Sequence.create( ease );
            }
            else
            {
                sequence = cc.Sequence.create( move );
            }

            this.water.runAction( sequence );
        },

        setBackgroundColorForLevel: function( waterLevel, animate )
        {
            if ( this._sunIsOut )
            {
                return;
            }

            // var level = waterLevel - 1;

            var settings = this.getRaindropsSettings();

            var color = settings.getBackgroundOverlayColor( waterLevel );
            var opacity = settings.getBackgroundOverlayOpacity( waterLevel );
            var duration = settings.getBackgroundOverlayTransitionDuration( waterLevel );

            if ( animate )
            {
                this._animateBackgroundOverlay( duration, color, opacity );
            }
            else
            {
                this.backgroundColorOverlay.setColor( cc.c3b( color.red, color.green, color.blue ) );
                this.backgroundColorOverlay.setOpacity( opacity );
            }
        },

        setSunIsOut: function( isOut )
        {
            this._sunIsOut = isOut;
            
            if ( this._sunIsOut )
            {
                this.setBackgroundColorForSun();
            }
            else
            {
                this.setBackgroundColorForLevel( this._currentWaterLevel );
            }
        },

        setBackgroundColorForSun: function()
        {
            if ( !this._sunIsOut )
            {
                return;
            }

            var settings = this.getRaindropsSettings();

            var color = settings.getBackgroundOverlaySunColor();
            var opacity = settings.getBackgroundOverlaySunOpacity();
            var duration = settings.getBackgroundOverlaySunTransitionDuration();

            this._animateBackgroundOverlay( duration, color, opacity );
        },

        _animateBackgroundOverlay: function( duration, color, opacity )
        {
            var tint = cc.TintTo.create( duration, color.red, color.green, color.blue );
            var tintSequence = cc.Sequence.create( tint );

            var fade = cc.FadeTo.create( duration, opacity );
            var fadeSequence = cc.Sequence.create( fade );
            
            this.backgroundColorOverlay.runAction( tintSequence );
            this.backgroundColorOverlay.runAction( fadeSequence );
        },

        getRaindropsGame: function()
        {
            return this.game;
        },

        _enablePauseButton: function()
        {
            var pauseButtonWidget = this._widgets[ GameWidgetPauseButton.prototype.getType() ];
            pauseButtonWidget.setEnabled( true );
        },

        disablePauseButton: function()
        {
            var pauseButtonWidget = this._widgets[ GameWidgetPauseButton.prototype.getType() ];
            pauseButtonWidget.setEnabled( false );
        },

        onEndGameplay: function()
        {
            this.fadeOutClouds();
        },

        _createViews: function()
        {
            // WARNING: Must be done here for some reason, is NOT OK to do this in onStartGameplay(), you will get this on iPad simulator:
            /*
            2014-04-29 15:56:43.398 Lumosity[38409:90b] Cocos2d: 
            JavaScript error: RaindropsView.js line 1502
            Error: Invalid Native Object
                RaindropsView.onStartGameplay:1502
                Raindrops.onStartGameplay:257
                cc.Class.extend/prototype[name]:675
            */

            // Setup the mask to behave correctly as a child of the clipper.
            var waterSurfaceMask = cc.Sprite.create( "fullscreen.png" );
            this._waterSurfaceMask = waterSurfaceMask;
            waterSurfaceMask.nodeName = "waterMask";
            // waterSurfaceMask.setContentSize( this._gameStage.getContentSize() );
            lu.scaleForContentSize( waterSurfaceMask, this._gameStage.getContentSize() );
            waterSurfaceMask.setAnchorPoint( 0, 0 );
            waterSurfaceMask.setPosition( 0, 0 );
            //waterSurfaceMask.resizing.widthChange = 0;
            //waterSurfaceMask.resizing.heightChange = 0;
            // Don't want this 
            // waterSurfaceMask.alwaysResizes = false;
            // waterSurfaceMask.setScaleX( 1.0 );
            // waterSurfaceMask.setScaleY( 1.0 );
            // waterSurfaceMask.setOpacity( 0 );

            // Setup the clipping node.
            var waterSurfaceClipper = cc.ClippingNode.create( waterSurfaceMask );
            this._waterSurfaceClipper = waterSurfaceClipper;
            waterSurfaceClipper.nodeName = "waterClipper";
            this.waterSurfaceContainer.addChild( waterSurfaceClipper );            
            // This cannot be used because it will try to set the resizing property, which isn't allowed on ClippingNode? (readonly)
            // this.makeFullScreen( waterSurfaceClipper );
            waterSurfaceClipper.setContentSize( this._gameStage.getContentSize() );
            waterSurfaceClipper.setAnchorPoint( 0, 0 );
            waterSurfaceClipper.setPosition( 0, 0 );
            // ClippingNode won't let me do this:
            // waterSurfaceClipper.resizing.widthChange = 0;
            // waterSurfaceClipper.resizing.heightChange = 0;
            waterSurfaceClipper.alwaysResizes = true;
            waterSurfaceClipper.setScaleX( 1.0 );
            waterSurfaceClipper.setScaleY( 1.0 );
            // waterSurfaceClipper.setInverted( true );

            // RaindropsUtils.enableBoundingBox( waterSurfaceClipper, cc.c3b( 0, 255, 0 ), 128 );

            // CANNOT DO: The stencil doesn't actually become a child of the clipper, it's only drawn that way internally by the clipper.
            // RaindropsUtils.enableBoundingBox( waterSurfaceClipper.getStencil(), cc.c3b( 0, 255, 0 ), 128 );
            // RaindropsUtils.addBoundingBoxToNode( waterSurfaceClipper, waterSurfaceClipper.getStencil(), bb, color, opacity, zOrder )

            // #3GSHACK Too early to do this, gameBegin will resize the node, do in onStartGameplay.
            // this.makeFullScreen( this.waterSurfaceContainer );

            // RaindropsUtils.enableBoundingBox( this.waterSurfaceContainer, cc.c3b( 255, 0, 0 ), 128 );

            // Move the water surface images to their container.
            this.waterSurfaceImage1.removeFromParent( false );
            waterSurfaceClipper.addChild( this.waterSurfaceImage1 );
            this.waterSurfaceImage2.removeFromParent( false );
            waterSurfaceClipper.addChild( this.waterSurfaceImage2 );

            // DEBUG:
            // lu.printNode( waterSurfaceMask );
            // lu.printNode( waterSurfaceClipper );
            // lu.printNode( this.waterSurfaceContainer );
            // lu.printNode( this.waterSurfaceImage1 );
            // lu.printNode( this.waterSurfaceImage2 );

            // #3GSHACK Too early to do this, gameBegin will resize the node, do in onStartGameplay.
            // Should be full screen with no scale.
            // this.makeFullScreen( this.rain );
            // this.makeFullScreen( this.cloudLayer );

            // @todo @hack #todo #hack use backgroundContainer in the Flash file to accomplish this.
            // Docs claim it's not automatically hooked up, so that's why we're doing it here.
            this.moveToBackground( this.cloudLayer );
            this.moveToBackground( this.backgroundColorOverlay );

            this._keypadButtons = [];

            this.constructKeypad();
        },

        moveToBackground: function( node )
        {
            node.removeFromParent( false );
            this.addChild( node );
        },

        makeFullScreen: function( node )
        {
            node.setContentSize( this._gameStage.getContentSize() );
            node.setScaleX( 1.0 );
            node.setScaleY( 1.0 );
            node.resizing.widthChange = 0;
            node.resizing.heightChange = 0;

            this.__debugOutputNodeInfoAndBB( node );
        },

        __debugOutputNodeInfoAndBB: function( node )
        {
            lu.printNode( node );
            RaindropsUtils.bbPrint( node );
        },

        onStartGameplay: function()
        {
            // RaindropsUtils.enableBoundingBox( this.waterSurfaceContainer, cc.c3b( 255, 0, 0 ), 64 );
            // RaindropsUtils.enableBoundingBox( this._waterSurfaceClipper, cc.c3b( 0, 255, 0 ), 128 );

            if ( this._menuButton )
            {
                this._menuButton.setVisible( true );
            }
            
            this._setupDropletAvoidColumns();

            this._shouldProcessInput = true;
            this._enableKeypadButtons( true );

            //RaindropsUtils.enableBoundingBox( this.keypad_buttonsTopLayer, cc.c3b( 0, 255, 0 ), 128 );

            // var children = this.keypad_buttonsTopLayer.getChildren();
            // var count = this.keypad_buttonsTopLayer.getChildrenCount();
            // var i;
            // for ( i = 0; i < count; i += 1 )
            // {
            //     RaindropsUtils.enableBoundingBox( children[ i ], cc.c3b( 255, 255, 0 ), 128 );
            // }

            //this.initWater();

            // #3GSHACK Hacky, but works. Needed for iPhone 3GS.
            // Should be done by specifying scale (0, 0) in Flash, but that's broken right now.
            this.makeFullScreen( this.rain );
            this.makeFullScreen( this.cloudLayer );

            // Too late to do this here, water is already visible.
            // Look at cue_waterRise() instead.
            // this.makeFullScreen( this.waterSurfaceContainer );
        },

        _setupDropletAvoidColumns: function()
        {
            // var boundingBox = this.keypad_buttonsTopLayer.getBoundingBox();

            // DEBUGGING:
            // Can't call on iOS... Alright then.
            //var boundingBoxToWorld = this.keypad_buttonsTopLayer.getBoundingBoxToWorld();
            // Can call these:
            // var nodeToParentTransform = this.keypad_buttonsTopLayer.nodeToParentTransform();
            // var nodeToWorldTransform = this.keypad_buttonsTopLayer.nodeToWorldTransform();
            // Unsure what I was doing with this:
            //var transform = this.keypad_buttonsTopLayer.transform();

            //var size = RaindropsUtils.getTextSize( "foobar", this._operandBottom.getFontName(), this._operandBottom.getFontSize() );
            //var transformed = 

            this._dropletAvoidColumnsFrames = [];

            // MAKE NEW COLUMNS OK

            var keypadTextAreas = this.getKeypadTextRegionsInRainSpace();

            var raindropsLayerFrame = RaindropsUtils.getFrame( this.rain );

            var i;
            var area;
            var width;
            var height;
            var showAvoidColumns = false;
            var columnSize;
            var avoidColumn;
            var columnFrame;
            for( i = 0; i < keypadTextAreas.length; i += 1 )
            {
                area = keypadTextAreas[ i ];

                width = area.right - area.left;
                height = raindropsLayerFrame.contentHeight;

                if ( showAvoidColumns )
                {
                    columnSize = cc.size( width, height );
                    avoidColumn = lu.createRectangularSprite( columnSize, cc.c3b( 255, 0, 0 ) );
                    avoidColumn.setPosition( area.left, 0 );
                    avoidColumn.setAnchorPoint( 0, 0 );
                    avoidColumn.setOpacity( 128 );
                    avoidColumn.setVisible( true );
                    this.rain.addChild( avoidColumn );
                }

                columnFrame = cc.rect( area.left, 0, width, height );
                this._dropletAvoidColumnsFrames.push( columnFrame );
            }
        },

        getKeypadTextRegionsInRainSpace: function()
        {
            // var regions = [];

            // var region1 = { left: 110, right: 170 };
            // var region2 = { left: 290, right: 350 };
            // var region3 = { left: 470, right: 530 };

            // regions.push( region1 );
            // regions.push( region2 );
            // regions.push( region3 );

            // return regions;

            var keypadBoundsConverted = [ [], [], [] ];

            var i;
            var j;
            var text;
            var button;
            var convertedTextBounds;
            var columnOfBounds;
            for ( i = 0; i < 3; i += 1 )
            {
                for ( j = 0; j < 3; j += 1 )
                {
                    columnOfBounds = this._keypadBounds[j];
                    text = this._convertCoordToKeypadDigit( i, j ).toString();
                    button = this.keypad_buttonsTopLayer[ "button_" + text ];
                    convertedTextBounds = RaindropsUtils.bbConvertFromTo( columnOfBounds[i], button, this.rain );
                    keypadBoundsConverted[j].push( convertedTextBounds );
                }
            }

            var showKeypadTextColumns = false;

            var regions = [];

            var temps;
            var union;
            var columnBounds = [];
            var region;
            for ( i = 0; i < 3; i += 1 )
            {
                temps = keypadBoundsConverted[i];
                union = RaindropsUtils.bbUnion( RaindropsUtils.bbUnion( temps[0], temps[1] ), temps[2] );

                columnBounds.push( union );

                if ( showKeypadTextColumns )
                {
                    RaindropsUtils.addBoundingBoxToNode( this.rain, union, cc.c3b( 255, 255, 0 ), 255 );
                }

                region = { left: union.x, right: union.x + union.width };
                regions.push( region );
            }

            return regions;
        },

        layout_creatingNode: function( nodeName, nodeData )
        {
            var node = this._super( nodeName, nodeData );

            if ( node )
            {
                return node;
            }
            
            switch( nodeName )
            {
                case "keypad_template_digitButtonBottom":
                    this._keypadDigitBottomTemplate = nodeData;
                    break;

                case "keypad_template_digitButtonTop":
                    this._keypadDigitTopTemplate = nodeData;
                    break;

                case "keypad_template_enterButtonBottom":
                    this._keypadEnterBottomTemplate = nodeData;
                    break;

                case "keypad_template_enterButtonTop":
                    this._keypadEnterTopTemplate = nodeData;
                    break;

                case "droplet_template":
                    this._templates_droplet = nodeData;
                    break;

                case "sun_template":
                    this._templates_sun = nodeData;
                    break;
            }
            
            return null;
        },

        layout_writingNodeToDictionary: function( nodeName, nodeData, node, viewDictionary )
        {
            // if ( nodeName == "raindrops" )
            // {
            //     //alert("got template");
            //     this._dropletTemplate = node;
            // }

            return true;
        },

        cue_showWater: function()
        {
            this._animateWaterRiseToLevel( 0, 1.7, EaseType.In );
        },

        cue_gameOn: function()
        {
            var game = this.getRaindropsGame();
            game.gameOn();
        },

        cue_finale: function()
        {
            var game = this.getRaindropsGame();
            game.gameOver();
        },

        cue_flood: function()
        {
            this.animateFlood();
        },

        cue_showKeypad: function()
        {
            this.animateShowKeypad();
        },

        cue_hideKeypad: function()
        {
            this.animateHideKeypad();
        },

        highlightArrow: function( arrowView )
        {
            arrowView.setColor( cc.c3b( 250, 214, 58 ) );
        },

        cue_highlightLevel1: function()
        {
            this.highlightArrow( this.waterLevelIndicator1 );
        },

        cue_highlightLevel2: function()
        {
            this.highlightArrow( this.waterLevelIndicator2 );
        },

        cue_highlightLevel3: function()
        {
            this.highlightArrow( this.waterLevelIndicator3 );
        },

        cue_highlightLevel4: function()
        {
            this.highlightArrow( this.waterLevelIndicator4 );
        },

        cue_waterRise: function()
        {
            // #3GSHACK Do this now so it happens after gameBegin changes the scale and before the water is visible on screen.
            this.makeFullScreen( this.waterSurfaceContainer );
        }
    } );
}() );
