/*global require*/
/*global cc*/
/*global lumosity*/
/*global LumosityUtils*/
/*global localize*/

require( "src/Game/Base/GameView.js" );
/*global GameView*/

require("src/Game/Widget/GameWidgetFeedback.js");
/*global GameWidgetFeedback*/

require("src/Game/Widget/GameWidgetHud.js");
/*global GameWidgetHud*/

require( "src/Animation/AnimationChanges.js" );
/*global AnimationChanges*/

var SpeedMatchView;
(function()
{
    "use strict";
    
    SpeedMatchView = GameView.extend(
    {
        ctor: function( screenSize, settings, model, interactionResponder, preload )
        {
            this._super( screenSize, settings, model, interactionResponder, preload );
            
            this.lastShapes = {};
            this.currShapes = {};
            this.lastShape = "";
            this.currShape = "";
            this.desiredText = "";
            
            var offset = cc.p(0.5 * this.card1.getContentSize().width, 0.5 * (this.card1.getContentSize().height*(40/39)));            

            //I believe this block creates sprites for every symbol, which will be reused later instead of constantly creating sprites
            var key;
            for ( key in this.model.currentTrialSet.symbols )
            {
                var symbolImage = this.model.currentTrialSet.symbols[ key ];
                
                var curr = this._createSprite( "stimuli/" + symbolImage );
                this.currShapes[ key ] = curr;
                curr.setPosition( offset );
                curr.setVisible( false );
                this.card1.addChild( curr );
                
                var last = this._createSprite( "stimuli/" + symbolImage );
                this.lastShapes[ key ] = last;
                last.setPosition( offset );
                last.setVisible( false );
                this.card2.addChild( last );
            }
            
            if ( this.settings.useHudTextColor && this.settings.hudTextColor )
            {
                this.getHudWidget().setTextColor( this.settings.hudTextColor );
            }

            //html5, we don't want the yes and no buttons
            if ( lumosity.device.id === "html5" )
            {
                AnimationChanges.addNodeIgnoredChanges( this.yesButton, AnimationChanges.Visibility );
                AnimationChanges.addNodeIgnoredChanges( this.noButton , AnimationChanges.Visibility );
                this.yesButton.setVisible( false );
                this.noButton.setVisible( false );
            }
        },

        layout_readingData : function( name, data )
        {
            this._super( name, data );
            
            switch ( name )
            {
                case "btnText":
                    this.btnFace = LumosityUtils.getFontLocation( data.textData.face );
                    this.btnSize = data.textData.size;
                    this.btnColor = cc.c4b(data.textData.color.Red, data.textData.color.Green, data.textData.color.Blue, data.textData.color.Alpha);
                break;
            }
        },

        layout_creatingNode : function( name, data )
        {
            switch (name)
            {
                case "noButton":
                    return this._createButton("buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", this.onButtonPressedCallback, localize("No_Label", "NO"), this.btnFace, this.btnSize, this.btnColor, cc.c4b(255, 255, 255, 255), data.size);
                case "yesButton":
                    return this._createButton("buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", this.onButtonPressedCallback, localize("Yes_Label", "YES"), this.btnFace, this.btnSize, this.btnColor, cc.c4b(255, 255, 255, 255), data.size);
                case "card1":
                    return this._createSprite("speedMatch_shapeBackground.png");
                case "card2":
                    return this._createSprite("speedMatch_shapeBackground.png");
            }
            
            return this._super( name, data );
        },

        onButtonPressedCallback: function( button )
        {
            if( !this.interactionResponder.allowButtonInput || !this.model.isGameRunning )
            {
                return;
            }
                                    
            switch( button.nodeName )
            {
                case "yesButton":
                    return this.interactionResponder.handleResponse( this.model.doSymbolsMatch(), true );
                case "noButton":
                    return this.interactionResponder.handleResponse( !this.model.doSymbolsMatch(), false );
            }
        },

        enableButtons : function(isEnabled)
        {
            this.noButton.setEnabled(isEnabled);
            this.yesButton.setEnabled(isEnabled);
        },

        updateSymbols : function()
        {
            var last = this.model.lastSymbolId;
            var curr = this.model.currentSymbolId;
            if ( last !== this.lastShape )
            {
                if ( this.lastShape !== "" )
                {
                    this.lastShapes[ this.lastShape ].setVisible( false );
                }
                this.lastShape = last;
                this.lastShapes[ last ].setVisible( true );
            }
            
            if ( curr !== this.currShape )
            {
                if ( this.currShape !== "" )
                {
                    this.currShapes[ this.currShape ].setVisible( false );
                }
                this.currShape = curr;
                return this.currShapes[ curr ].setVisible( true );
            }
        },
        
        /**
         * Updates the text in the prompt, using model values to decide what to show.
         *
         * @param {boolean} [immediate=false] If immediate, we set the text immediately. If not, we play the fadePrompt animation, then change the text on a cue.
         * @param {boolean} [wasIncorrect=false] If this was an incorrect trial in the tutorial, we show a message telling the user what they did wrong.
         * @param {boolean} [wasMatch=false] If this was incorrect, we need to know if it was a match or not to decide what message to show.
         */
        updateText: function( immediate, wasIncorrect, wasMatch )
        {
            this.desiredText = localize( "promptCaption" );
            
            //in practice mode, we show different strings on the bottom
            if ( this.model.persistentData.isPracticeMode )
            {
                if ( !wasIncorrect )
                {
                    //basically we show this until the user can enter input from the first card
                    if ( this.model.practiceProgress === 0 && !this.model.hasAllowedInput )
                    {
                        this.desiredText = localize( "tutorialRememberCard" );
                    }
                }
                //when incorrect, tell the user why they are a screw-up
                else
                {
                    if ( wasMatch )
                    {
                        this.desiredText = localize( "tutorialIncorrectMatch" );
                    }
                    else
                    {
                        this.desiredText = localize( "tutorialIncorrectNoMatch" );
                    }
                }
            }
            
            if ( this.prompt.getString() !== this.desiredText )
            {
                if ( immediate )
                {
                    this.cue_changePrompt();
                }
                else
                {
                    this.playAnimation( "fadePrompt" );
                }
            }
            else
            {
                this.desiredText = "";
            }
        },
        
        cue_changePrompt: function()
        {
            if ( !this.desiredText )
            {
                return;
            }
            
            this.prompt.setString( this.desiredText );
            this.desiredText = "";
        }
    } );
}() );
