/*global cc*/
/*global require*/

require( "src/Game/Base/GameView.js" );
/*global GameView*/

require("src/Game/Widget/GameWidgetFeedback.js");
/*global GameWidgetFeedback*/

require("src/Game/Widget/GameWidgetHud.js");
/*global GameWidgetHud*/

require( "src/Animation/AnimationChanges.js" );
/*global AnimationChanges*/


var SpeedMatchOverdriveView;
( function()
{
    "use strict";
    
    SpeedMatchOverdriveView = GameView.extend(
    {
        /** @lends SpeedMatchOverdriveView.prototype */
        
        /**
        * The Speed Match Overdrive view.
        * @constructs
        * @param {cc.Size} screenSize The size of the screen that this view will be inserted into.
        * @param {Object} [settings] The settings that we can pull view-based data from.
        * @param {SpeedMatchOverdriveModel} [model] The GameModel that we are linked to.
        * @param {Object} [interactionResponder] The object that we should send interaction events to. We also use this is a default animation event listener.
        * @param {boolean} [preload=true] If true, will preload all flipbooks and other assets upon creation.
        */
       ctor: function( screenSize, settings, model, interactionResponder, preload )
        {
            this._super( screenSize, settings, model, interactionResponder, preload );
            
            this.lastShapes = {};
            this.currShapes = {};
            this.lastShape = "";
            this.currShape = "";
            this.lastColor = "";
            this.currColor = "";
            this.desiredText = "";
            this.showPartialHint = false;

            var offset = cc.p(0.5 * this.card1.getContentSize().width, 0.5 * (this.card1.getContentSize().height*(40/39)));
            
            //I believe this block creates sprites for every symbol, which will be reused later instead of constantly creating sprites
            var key;
            for ( key in this.model.currentTrialSet.symbols )
            {
                var symbolImage = this.model.currentTrialSet.symbols[ key ];
                
                var curr = this._createSprite( "stimuli/" + symbolImage );
                this.currShapes[ key ] = curr;
                curr.setPosition( offset );
                curr.setVisible( false );
                this.card1.addChild( curr );
                
                var last = this._createSprite( "stimuli/" + symbolImage );
                this.lastShapes[ key ] = last;
                last.setPosition( offset );
                last.setVisible( false );
                this.card2.addChild( last );
            }
            
            if ( this.settings.useHudTextColor && this.settings.hudTextColor )
            {
                this.getHudWidget().setTextColor( this.settings.hudTextColor );
            }

            //html5, we don't want the yes and no buttons
            /*if ( lumosity.device.id === "html5" )
            {
                AnimationChanges.addNodeIgnoredChanges( this.yesButton, AnimationChanges.Visibility );
                AnimationChanges.addNodeIgnoredChanges( this.noButton , AnimationChanges.Visibility );
                AnimationChanges.addNodeIgnoredChanges( this.PartialButton , AnimationChanges.Visibility );
                this.yesButton.setVisible( false );
                this.noButton.setVisible( false );
                this.PartialButton.setVisible( false );
            }*/
        },

        layout_readingData : function( name, data )
        {
            this._super( name, data );
            
            switch ( name )
            {
                case "btnText":
                    this.btnFace = LumosityUtils.getFontLocation( data.textData.face );
                    this.btnSize = data.textData.size;
                    this.btnColor = cc.c4b(data.textData.color.Red, data.textData.color.Green, data.textData.color.Blue, data.textData.color.Alpha);
                break;
            }
        },

        _setButtonStates : function ( buttonUpSymbol, buttonDownSymbol, button, size )
        {
             buttonUpSymbol.setAnchorPoint( cc.PointZero() );
             LumosityUtils.scaleForContentSize(buttonUpSymbol, size); 
             buttonDownSymbol.setAnchorPoint( cc.PointZero() );
             LumosityUtils.scaleForContentSize(buttonDownSymbol, size); 

             button.setImageForState(buttonUpSymbol, LumosityButtonState.ButtonStateDisabled);
             button.setImageForState(buttonUpSymbol, LumosityButtonState.ButtonStateNormal);
             button.setImageForState(buttonDownSymbol, LumosityButtonState.ButtonStateSelected);
             button.setImageForState(buttonDownSymbol, LumosityButtonState.ButtonStateHighlighted);
        },
        layout_creatingNode_en : function( name, data )
        {
            var button = null;
            var symbol = null;
            var buttonUpSymbol = null;
            var buttonDownSymbol = null;

            switch (name)
            { 
                case "noButton":
                    buttonUpSymbol = cc.Sprite.create("buttonSymbolsNo.png");
                    buttonDownSymbol = cc.Sprite.create("buttonSymbolsNoDown.png");
                    button = this._createButton("buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "btnNo");
                    this._setButtonStates(buttonUpSymbol, buttonDownSymbol, button, data.size);
                    return button;
                case "yesButton":
                    buttonUpSymbol = cc.Sprite.create("buttonSymbolsYes.png");
                    buttonDownSymbol = cc.Sprite.create("buttonSymbolsYesDown.png");
                    button = this._createButton("buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "btnYes");
                    this._setButtonStates(buttonUpSymbol, buttonDownSymbol, button, data.size);
                    return button;
                case "partialButton":
                    buttonUpSymbol = cc.Sprite.create("buttonSymbolsPartial.png");
                    buttonDownSymbol = cc.Sprite.create("buttonSymbolsPartialDown.png");
                    button = this._createButton("buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "btnPartial");
                    this._setButtonStates(buttonUpSymbol, buttonDownSymbol, button, data.size);
                    return button;
                case "card1":
                    symbol = this._createSprite("speedMatch_shapeBackground.png");
                    return symbol;
                case "card2":
                    symbol = this._createSprite("speedMatch_shapeBackground.png");
                    return symbol;                    
            }
            
            return this._super( name, data );
        },
        layout_creatingNode_ja : function( name, data )
        {
            switch (name)
            { 
                case "noButton":
                    return this._createButton("buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "btnNo", localize("No_Label", "NO"), this.btnFace, this.btnSize, this.btnColor, cc.c4b(255, 255, 255, 255), data.size);
                case "yesButton":
                    return this._createButton("buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "btnYes", localize("Yes_Label", "YES"), this.btnFace, this.btnSize, this.btnColor, cc.c4b(255, 255, 255, 255), data.size);
                case "partialButton":
                    return this._createButton("buttons/game_buttonInputUp.png", "buttons/game_buttonInputDown.png", "btnPartial", localize("Partial_Label", "PARTIAL"), this.btnFace, this.btnSize, this.btnColor, cc.c4b(255, 255, 255, 255), data.size);
                case "card1":
                    return this._createSprite("speedMatch_shapeBackground.png");
                case "card2":
                    return this._createSprite("speedMatch_shapeBackground.png");
            }
            
            return this._super( name, data );
        },

        layout_creatingNode : function( name, data )
        {
            var node = lumosity.game.locale === "ja" ? this.layout_creatingNode_ja(name, data) : this.layout_creatingNode_en(name, data);
            if(!node) 
            {
                node = this._super(name, data);
            }
            return node;
        },

        enableButtons : function(isEnabled)
        {
            this.noButton.setEnabled(isEnabled);
            this.yesButton.setEnabled(isEnabled);
            this.partialButton.setEnabled(isEnabled);
        },

        updateSymbols : function()
        {
            var last = this.model.lastSymbolId;
            var lastColor = this.model.lastColorId;
            var curr = this.model.currentSymbolId;
            var currColor = this.model.currentColorId;
            if ( last !== this.lastShape || lastColor !== this.lastColor )
            {
                if ( this.lastShape !== "" )
                {
                    this.lastShapes[ this.lastShape ].setVisible( false );
                }
                this.lastShape = last;
                this.lastColor = lastColor;
                var color = this.settings.trials["Color"].getSymbolImageFileName(lastColor); 
                this.lastShapes[ last ].setColor( cc.c3b(color.r, color.g, color.b) );
                this.lastShapes[ last ].setVisible( true );
            }
            
            if ( curr !== this.currShape || currColor !== this.currColor )
            {
                if ( this.currShape !== "" )
                {
                    this.currShapes[ this.currShape ].setVisible( false );
                }
                this.currShape = curr;
                this.currColor = currColor;
                var color = this.settings.trials["Color"].getSymbolImageFileName(currColor); 
                this.currShapes[ curr ].setColor( cc.c3b(color.r, color.g, color.b ) );
                return this.currShapes[ curr ].setVisible( true );
            }
        },
        
        /**
         * Updates the text in the prompt, using model values to decide what to show.
         *
         * @param {boolean} [immediate=false] If immediate, we set the text immediately. If not, we play the fadePrompt animation, then change the text on a cue.
         * @param {boolean} [wasIncorrect=false] If this was an incorrect trial in the tutorial, we show a message telling the user what they did wrong.
         * @param {boolean} [wasMatch=false] If this was incorrect, we need to know if it was a match or not to decide what message to show.
         * @param {boolean} [forcedPartialTrial=false] Whether or not the user is forced into partial trials.
         */
        updateText: function( immediate, wasIncorrect, wasMatch )
        {
            this.desiredText = !this.showPartialHint ? localize( "promptCaption" ) : localize( "partial_hint" );

            //in practice mode, we show different strings on the bottom
            if ( this.model.persistentData.isPracticeMode )
            {
                if ( !wasIncorrect )
                {
                    //basically we show this until the user can enter input from the first card
                    if ( this.model.practiceProgress === 0 && !this.model.hasAllowedInput )
                    {
                        this.desiredText = localize( "tutorialRememberCard" );
                    }
                }
                //when incorrect, tell the user why they are a screw-up
                else
                {
                    if ( wasMatch )
                    {
                        this.desiredText = localize( "tutorialIncorrectMatch" );
                    }
                    else
                    {
                        this.desiredText = localize( "tutorialIncorrectNoMatch" );
                    }
                }
            }
            
            if ( this.prompt.getString() !== this.desiredText )
            {
                if ( immediate )
                {
                    this.cue_changePrompt();
                }
                else
                {
                    this.playAnimation( "fadePrompt" );
                }
            }
            else
            {
                this.desiredText = "";
            }
        },
        
        cue_changePrompt: function()
        {
            if ( !this.desiredText )
            {
                return;
            }
            
            this.prompt.setString( this.desiredText );
            this.desiredText = "";
        }
    } );
}() );
