/*global cc*/
/*global require*/
/*global Game*/
/*global audio*/
/*global metadataAggregateKeys*/
/*global metadataTrialKeys*/
/*global SpeedPackSettings*/
/*global SpeedPackModel*/
/*global SpeedPackView*/
/*global LumositySprite*/
/*global LumosityLabel*/
/*global LumosityDebugMenu*/
/*global lu*/

require( "src/Game/Base/Game.js" );
require( "src/Game/SpeedPackSettings.js" );
require( "src/Game/SpeedPackModel.js" );
require( "src/Game/SpeedPackView.js" );
require( "src/View/LumositySprite.js" );
require( "src/View/LumosityLabel.js" );
require( "src/View/LumosityDebugMenu.js" );
var lu = LumosityUtils;

var SpeedPack;
( function()
{
    "use strict";
    SpeedPack = Game.extend(
    {
        _score: 0,
        _trialLevel: 0,
        _trial: null,
        _doors: null,
        _slots: null,

        _gameOver: null,
        _debugMode: null,

        _debugLabel1: null,
        _debugLabel2: null,
        _debugLabel3: null,
        _debugLabel4: null,
        _debugLabel5: null,

        _cheatMode: false,

        ctor: function ()
        {
            this._super();

            this._gameOver = false;
            this._debugMode = this.settings.debugMode;
            this._trialLevel = this.model.current_level;

            // text for debug mode
            if ( lu.isDebugModeOn() )
            {
                var font = "fonts/MuseoSans-700.ttf";
                var fontSize = 25;

                var cheatText;
                if ( this._cheatMode )
                {
                    cheatText = "On";
                }
                else
                {
                    cheatText = "Off";
                }

                this._debugLabel1 = LumosityLabel.create( "Level: " + this.model.current_level.toString(), font, fontSize );
                this._debugLabel2 = LumosityLabel.create( "Response Time: " + this.model._responseTime.toString() + "ms", font, fontSize );
                this._debugLabel3 = LumosityLabel.create( "Delta Progress: " + this.model._deltaProgress.toString(), font, fontSize );
                this._debugLabel4 = LumosityLabel.create( "Level Progress: " + Math.round( this.model.progress ).toString(), font, fontSize );
                this._debugLabel5 = LumosityLabel.create( "Cheat Mode: " + cheatText.toString(), font, fontSize );
                this._debugLabel1.setPosition( 70, 105 );
                this._debugLabel2.setPosition( 180, 80 );
                this._debugLabel3.setPosition( 130, 55 );
                this._debugLabel4.setPosition( 130, 30 );
                this._debugLabel5.setPosition( 130, 130 );
                this.view._gameStage.addChild( this._debugLabel1, 200 );
                this.view._gameStage.addChild( this._debugLabel2, 200 );
                this.view._gameStage.addChild( this._debugLabel3, 200 );
                this.view._gameStage.addChild( this._debugLabel4, 200 );
                this.view._gameStage.addChild( this._debugLabel5, 200 );

                var debugMenuObject = { "labelText": "Cheat Mode",
                    "callback": this.toggleCheatMode,
                    "target": this,
                    "closeMenuOnCallback": false };

                this.view._debugMenu.addDebugButton( debugMenuObject );
            }
        },

        onUpdate: function ( deltaTick )
        {
            this._super( deltaTick );
            this.updateHud();
            if ( lu.isDebugModeOn() )
            {
                this.debugModeText();
            }
        },

        onUpdateEachSecond: function ()
        {
            this._super();
        },

        updateHud: function ()
        {
            // update the score
            this.view.setScore( this.model.score );
        },

        /**
         * called once, before countdown
         */
        onBegin: function ()
        {
            this._super();
            this.setInitialHudState();
            this.calculateStartingDifficulty();
            this.startTrial();
        },

        /**
         * called once, after countdown
         */
        onStartGameplay: function ()
        {
            this._super();
            //bring instruction text up into screen
            this.view.playAnimation( "showPrompt" );
            //slides camera into screen
            this.view.playAnimation( "problem" );

            if ( this.model.max_level === 0 )
            {
                this.model.max_level = this.model.current_level;
            }
            this.model.game_start_time = lumosity.game.frametime;
            this.model.width = this.view.getContentSize().width;
            this.model.height = this.view.getContentSize().height;

            this.beginTrial();
            this.showItems();
            this.enableButtons();
            this.setSprites();
        },

        /**
         * called after timer hits 0
         */
        onEndGameplay: function ()
        {
            this.disableButtons();
            this.model.game_end_time = lumosity.game.frametime;
            this._gameOver = true;
            if ( this.model.animationComplete )
            {
                this.view.camera.setVisible( false );
                this._super();
            }

        },

        onFinish: function( wasQuit )
        {
            this._super( wasQuit );
        },

        onPause: function ()
        {
            this._super();
        },

        onResume: function ()
        {
            this._super();
        },

        onRestart: function ()
        {
            this._super();
            this.view.camera.setVisible( false );
            this._gameOver = false;
            this.model.resetModel();
            this.startTrial();
        },

        /**
         * controls effect of enabling Debug Mode for QA (set in App during runtime)
         */
        debugModeText: function ()
        {
            var cheatText;
            if ( this._cheatMode )
            {
                cheatText = "On";
            }
            else
            {
                cheatText = "Off";
            }

            //display values on screen
            this._debugLabel1.setString( "Level: " + this.model.current_level.toString() );
            this._debugLabel2.setString( "Response Time: " + this.model._responseTime.toString() + "ms" );
            this._debugLabel3.setString( "Delta Progress: " + this.model._deltaProgress.toString() );
            this._debugLabel4.setString( "Level Progress: " + Math.round( this.model.progress ).toString() );
            this._debugLabel5.setString( "Cheat Mode: " + cheatText.toString() );
        },

        /**
         * cheat mode toggling for debug mode
         */
        toggleCheatMode: function ()
        {
            if ( this._cheatMode === true )
            {
                this._cheatMode = false;
            }
            else
            {
                this._cheatMode = true;
            }
        },

        /**
         * calculates users current level based on their previous final level
         */
        calculateStartingDifficulty: function ()
        {
            var startingLevel = 0;
            // look for app params
            if ( lumosity.game.params && lumosity.game.params.last_results && lumosity.game.params.last_results.game_result_data )
            {
                //get user's last level
                var prevLastLevel = lumosity.game.params.last_results.game_result_data[ this.model.previousGameStatKey ];
                if ( LumosityUtils.isNumeric( prevLastLevel ) )
                {
                    // previous max level exists, so pull it in
                    startingLevel = prevLastLevel;
                    // Make sure starting level is not too low
                    if ( startingLevel < this.settings.startingLevel )
                    {
                        startingLevel = this.settings.startingLevel;
                    }
                    // Make sure it is not too high (in case we have users who cheated or achieved higher max tile count on another device or older version)
                    if ( startingLevel > this.settings.maxLevel )
                    {
                        startingLevel = this.settings.maxLevel;
                    }

                    // Set session level in meta data
                    var metadata = this.model.metadata;
                    if ( metadata )
                    {
                        metadata[ metadataAggregateKeys.userLevel ] = this.model.current_level;
                    }
                }
            }

            // No previous results found, start at level 1
            if ( startingLevel === 0 || this.settings.debugLevel )
            {
                startingLevel = this.settings.startingLevel;
                this.model.original_level = 0;
            }
            else
            {
                this.model.original_level = startingLevel;
            }

            this.model.current_level = startingLevel;
            this.model.maxLevel = this.model.current_level;
            this.model.avg_level = this.model.current_level;
        },

        setInitialHudState: function ()
        {
            // Zero the score
            this.view.setScore( 0 );
        },

        /**
         * hides items during the countdown so players cannot preempt the first trial
         */
        hideItems: function ()
        {
            var i;
            //hide all items
            var itemLength = this.view._currentTrialItems.length;
            for ( i = 0; i < itemLength; ++i )
            {
                this.view._currentTrialItems[ i ].item.setVisible( false );
            }

            //hide & deactivate all buttons
            itemLength = this.view._currentTrialButtons.length;
            for ( i = 0; i < itemLength; i++ )
            {
                var button = this.view._currentTrialButtons[i].button;
                if ( !button.enabled )
                {
                    button.setButtonActive( false );
                }
            }
        },

        /**
         * reveals items for first trial after countdown ends
         */
        showItems: function ()
        {
            var i;
            //show all items
            var itemLength = this.view._currentTrialItems.length;
            for ( i = 0; i < itemLength; ++i )
            {
                this.view._currentTrialItems[ i ].item.setVisible( true );
            }

            //show & activate all buttons
            itemLength = this.view._currentTrialButtons.length;
            for ( i = 0; i < itemLength; i++ )
            {
                var button = this.view._currentTrialButtons[i].button;
                if ( !button.enabled )
                {
                    button.setButtonActive( true );
                }
            }
        },

        /**
         * cleans up the trial model and view after each trial ends
         */
        deleteCurrentTrial: function ()
        {
            var i;
            var j;
            var k;
            var l;
            var m;
            var popped;

            if ( this.view._currentTrialDoors.length > 0 )
            {
                //remove all items
                var itemLength = this.view._currentTrialItems.length;
                for ( i = 0; i < itemLength; ++i )
                {
                    popped = this.view._currentTrialItems.pop();
                }
                //remove all empty slots
                var slotLength = this.view._currentTrialSlots.length;
                for ( j = 0; j < slotLength; ++j )
                {
                    popped = this.view._currentTrialSlots.pop();
                }
                //remove all button slots
                var buttonLength = this.view._currentTrialButtons.length;
                for ( k = 0; k < buttonLength; ++k )
                {
                    popped = this.view._currentTrialButtons.pop();

                    this.view.removeButton( popped.button );
                }
                //remove all doors
                var doorLength = this.view._currentTrialDoors.length;
                for ( l = 0; l < doorLength; ++l )
                {
                    popped = this.view._currentTrialDoors.pop();
                }
                //remove all door nodes
                var nodeLength = this.view._doorNodes.length;
                for ( m = 0; m < nodeLength; ++m )
                {
                    popped = this.view._doorNodes.pop();
                    popped.removeAllChildren();
                    this.view.suitcase.removeChild( popped );
                }
            }
            this._trial = null;
        },

        /**
         * loads next incoming trial information
         */
        startTrial: function ()
        {
            this.view.camera.setVisible( false );
            this.deleteCurrentTrial();
            this.loadNextTrial();
            this.setSprites();
            this.hideItems();
            this.disableButtons();
        },

        /**
         * destroys current trial and creates next trial
         */
        resetTrial: function ()
        {
            this.deleteCurrentTrial();
            //only draw the camera if the game is still going (fixed bug where camera stayed after countdown)
            if ( !this._gameOver )
            {
                this.view.playAnimation( "problem" );
                this.loadNextTrial();
                this.enableButtons();
                this.setSprites();
            }
            else
            {
                this.view.camera.setVisible( false );
            }
        },

        /**
         * grabs appropriate trial from model and readies it for the user
         */
        loadNextTrial: function ()
        {
            //if you aren't debugging
            if ( !this._debugMode )
            {
                //if the current deck of usable trials is empty, update the deck with usable trials
                if ( this.model.trial_deck.length <= 0 )
                {
                    this.model.resetTrialDeck( this.settings );
                }
                //pull a new trial off the deck
                this._trial = this.model.trial_deck.pop();
            }
            //if you are debugging, just grab that trial you want so bad
            else
            {
                this._trial = this.settings.getTrials( this.settings.debugTrialDifficulty ) [this.settings.debugTrialId ];
            }
            this.view.drawNextTrial( this._trial );
            this.beginTrial();

            var string = "Indexes: ";
            var i;
            var j;
            for ( i = 0; i < this._trial.doors.length; i++ )
            {
                string += "Door " + i.toString() + ": ";
                for ( j = 0; j < this._trial.doors[i].slots.length; j++ )
                {
                    string += this._trial.doors[i].slots[j].index.toString();
                    if ( this._trial.doors[i].slots[j].filled )
                    {
                        string += "!";
                    }
                    string += ", ";
                }
                string += " ";
            }

        },

        /**
         * enable button functionality
         */
        enableButtons: function()
        {
            var i;
            var buttonLength = this.view._currentTrialButtons.length;
            for (i = 0; i < buttonLength; ++i)
            {
                this.view.addButton( this.view._currentTrialButtons[ i ].button );
            }
        },

        /**
         * disable button functionality - called while cases are flipping / moving
         */
        disableButtons: function()
        {
            var i;
            var buttonLength = this.view._currentTrialButtons.length;
            for ( i = 0; i < buttonLength; ++i )
            {
                this.view.removeButton( this.view._currentTrialButtons[ i ].button );
            }
        },

        /**
         * sets up filled slots with appropriate item sprites depending on layout
         * of filled slots (i.e. single items, 2x1 items, odd shaped items
         */
        setSprites: function()
        {
            var i;
            for ( i = 0; i < this.view._currentTrialItems.length; ++i )
            {
                this.view._currentTrialItems[ i ].item.setItemSprite();
            }

            //reset the trial start time (start time here because it is always when the user first sees the item layout)
            this.model.trial_start_time = lumosity.game.frametime;
        },

        /**
         * callback for button-press action on active yellow case slots
         * @param tile
         * @param door
         */
        tileView_onTileTapped: function( tile, door )
        {
            //get the time taken to answer (convert to milliseconds)
            var response_time = ( lumosity.game.frametime - this.model.trial_start_time ) * 1000;

            this.model.animationComplete = false;
            //mark the clicked tile as filled (with camera)
            tile.filled++;

            this.view.camera.setVisible( false );

            //draw the camera on that tile
            this.view.drawCamera( tile, door );

            //initiate folding algorithm -- returns if collision is detected (direction == 0)
            var success = !this._trial.checkDoorOverlap() || this._cheatMode; // check door overlap before anything (includes flip)

            var i;
            for ( i = 0; i < this._trial.doors.length; ++i )
            {
                if ( this._trial.doors[i].direction !== 0 )
                {
                    this.view.prepareDoors( i, this._trial.doors[i] );
                }
            }

            //call animate doors sequence
            this.view.animateNextDoor( success, tile, this._trial.doors, door );

            //record metadata
            this.model.num_tries++;
            this.model.avg_level += this.model.current_level;

            //calculate points for success
            if( success )
            {
                this.model.increasePoints();
                this.model.num_correct++;

                if( this.model.previous_correct == -1 )
                {
                    this.model.num_bounces++;
                }
                this.model.previous_correct = 1;

            }
            //failure
            else
            {
                if( this.model.previous_correct == 1 )
                {
                    this.model.num_bounces++;
                }
                this.model.previous_correct = -1;
            }
            //update the player's progress
            var delta_progress = this.model.calculateProgress( success, response_time, this.model.current_level, this.settings );

            //update the player's level if necessary
            this.model.calculatePlayerLevel( this.settings );

            //this must be called before the multiplier etc are increased
            this._recordTrialMetadata( success , delta_progress );


        },

        /**
         * trial bookkeeping for metadata sent by the model post-game
         * @param success
         * @param progress
         * @param response_time
         * @private
         */
        _recordTrialMetadata: function ( success, delta_progress )
        {

            //trial metadata
            var trial =
            {
                net : this._trial.net.toString(),
                level : this.model.current_level,
                correct : success ? "T" : "F",
                progress : Math.floor( this.model.progress ),
                delta : Math.floor( delta_progress )
            };

            this.endTrial( trial, success, false, false, false, false );

        },

        /**
         * called after the suitcase has been folded
         * short delay (so the user can see the feedback),
         * then activates next trial
         */
        delayAndReset: function()
        {
            var delay = cc.DelayTime.create( 0.6 );
            var setAnimationComplete = cc.CallFunc.create( this.setAnimationComplete, this );
            var checkGameOver = cc.CallFunc.create( this.checkGameOver, this );
            var resetTrial = cc.CallFunc.create( this.resetTrial, this );

            var sequence = cc.Sequence.create( delay, setAnimationComplete, checkGameOver, resetTrial );
            this.view.runAction( sequence );
        },

        checkGameOver: function ()
        {
            if ( this._gameOver )
            {
                this.onEndGameplay();
            }
        },
        setAnimationComplete: function ()
        {
            this.model.animationComplete = true;
        },

        _createSettings: function()
        {
            return new SpeedPackSettings( "settings.json" );
        },

        _createModel: function()
        {
            return new SpeedPackModel();
        },

        _createView: function( screenSize )
        {
            return new SpeedPackView( screenSize, this.settings, this.model, this );
        }
    } );
}() );
