/*global cc*/
/*global require*/
/*global GameView*/
/*global LumosityButton*/
/*global LumositySprite*/
/*global LumosityButtonState*/

require( "src/Game/Base/GameView.js" );
require( "src/Game/Widget/GameWidgetFeedback.js" );
require( "src/Game/Widget/GameWidgetFlipbook.js" );
require( "src/Game/Widget/GameWidgetHud.js" );
require( "src/View/LumosityButton.js" );
require( "src/Utility/LumosityUtils.js" );
require( "src/View/LumositySprite.js" );

var SpeedPackDoorSlotView;
( function ()
{
    "use strict";

    SpeedPackDoorSlotView = LumosityButton.extend(
        {
            activeTileTexture: "tileUp.png",    //clickable (yellow)
            inactiveTileTexture: "tileDown.png",  //non-clickable (dark yellow)
            neutralTileTexture: "tileDefault.png",   //non-clickable (blue)

            doorModel: null,
            slotArrayIndex: null,
            xPos: null,
            yPos: null,
            offsets: null,
            halfDoorSizes: null,
            enabled: null,
            active: null,

            x: null,
            y: null,

            tileSize: cc.size( 91, 91 ),

            _delegate: null,

            ctor: function ()
            {
                this._super();

                this.setContentSize( this.tileSize, true );
                this.setAnchorPoint( cc.p( 0.5, 0.5 ) );

                var activeSprite = this.createSlotSprite( this.activeTileTexture );
                var inactiveSprite = this.createSlotSprite( this.inactiveTileTexture );
                LumosityUtils.scaleForContentSize( activeSprite, cc.size( 91, 91 ) );
                LumosityUtils.scaleForContentSize( inactiveSprite, cc.size( 91, 91 ) );

                this.setBackgroundImageForState( activeSprite, LumosityButtonState.ButtonStateNormal );
                this.setBackgroundImageForState( inactiveSprite, LumosityButtonState.ButtonStateSelected );

                this.setTarget( this );
                this.setPressedCallback( this._onTileTapped );
            },

            /**
             * sets all of the button's parameters
             * @param door
             * @param index
             * @param x
             * @param y
             * @param doorX
             * @param doorY
             * @param offsets
             * @param halfDoorSizes
             * @param enabled
             * @param active
             * @param delegate
             */
            fillButton: function ( door, index, x, y, doorX, doorY, offsets, halfDoorSizes, enabled, active, delegate )
            {
                this.doorModel = door;
                this.slotArrayIndex = index;
                this.xPos = x;
                this.yPos = y;
                this.offsets = offsets;
                this.halfDoorSizes = halfDoorSizes;
                this.enabled = enabled;
                this.active = active;

                this.x = doorX - this.halfDoorSizes.width + ( this.xPos * 89 ) + this.offsets.x;
                this.y = doorY + this.halfDoorSizes.height - ( this.yPos * 89 ) - this.offsets.y;

                this.doorModel.slots[this.slotArrayIndex].pos = cc.p( this.x, this.y );

                this._delegate = delegate;

                if ( !this.enabled )
                {
                    // Need to set disabled state background here so it doesn't change when enabled/disabled
                    if ( this.active )
                    {
                        // This always happens except for during countdown
                        var downSprite = this.createSlotSprite( this.inactiveTileTexture );
                        LumosityUtils.scaleForContentSize( downSprite, cc.size( 91, 91 ) );
                        this.setBackgroundImageForState( downSprite, LumosityButtonState.ButtonStateDisabled );
                    }
                    else
                    {
                        // For countdown
                        var activeSprite = this.createSlotSprite( this.activeTileTexture );
                        LumosityUtils.scaleForContentSize( activeSprite, cc.size( 91, 91 ) );
                        this.setBackgroundImageForState( activeSprite, LumosityButtonState.ButtonStateDisabled );
                    }

                    this.setEnabled( false );
                }
                else
                {
                    var activeSprite = this.createSlotSprite( this.activeTileTexture );
                    LumosityUtils.scaleForContentSize( activeSprite, cc.size( 91, 91 ) );
                    this.setBackgroundImageForState( activeSprite, LumosityButtonState.ButtonStateDisabled );
                    this.setEnabled( true );
                }

                this.setZOrder( 2 );
                this.setPosition( cc.p( this.x, this.y ) );
            },

            /**
             * handles the button press callback of the button
             * @private
             */
            _onTileTapped: function ()
            {
                if ( this._delegate && this._delegate.tileView_onTileTapped )
                {
                    this._delegate.tileView_onTileTapped( this.doorModel.slots[this.slotArrayIndex], this.doorModel );
                }
            },

            createSlotSprite: function ( textureName )
            {
                //place yellow tile
                var tile = cc.Sprite.create( textureName.toString() );
                //update slot model to show x and y position

                return tile;
            },

            /**
             * controls if the button is active or inactive
             * @param active
             */
            setButtonActive: function ( active )
            {
                this.active = active;
                if ( active )
                {
                    var downSprite = this.createSlotSprite( this.inactiveTileTexture );
                    LumosityUtils.scaleForContentSize( downSprite, cc.size( 91, 91 ) );
                    this.setBackgroundImageForState( downSprite, LumosityButtonState.ButtonStateDisabled );
                }
                else
                {
                    var activeSprite = this.createSlotSprite( this.activeTileTexture );
                    LumosityUtils.scaleForContentSize( activeSprite, cc.size( 91, 91 ) );
                    this.setBackgroundImageForState( activeSprite, LumosityButtonState.ButtonStateDisabled );
                }
            },

            softDisable: function ()
            {
                this.setEnabled( false );
            }
        } );
}() );
