/*global require*/
/*global GameModel*/

require( "src/Game/Base/GameModel.js" );

var SpeedPackModel;
( function()
{
    "use strict";

    SpeedPackModel = GameModel.extend(
    {
        current_level: null,
        original_level: null,
        game_start_time: null,
        game_end_time: null,
        trial_start_time: null,
        score: null,
        progress: null,
        trial_deck: null,

        animationComplete: null,

        // Stats
        width: null,
        height: null,
        bestStatKey: "num_correct",
        previousGameStatKey: "user_level",
        num_pauses: null,
        num_correct: null,
        num_tries: null,
        max_level: null,
        avg_level: null,
        num_bounces: null,
        previous_correct: null,
        trial_csv: null,

        _responseTime: 0,
        _deltaProgress: 0,

        ctor: function()
        {
            this._super();

            //game stats
            this.current_level = 0;
            this.score = 0;
            this.progress = 0;
            this.trial_deck = [];

            this.animationComplete = true;

            //app metadata
            this.num_pauses = 0;
            this.num_correct = 0;
            this.num_tries = 0;
            this.max_level = 0;
            this.avg_level = 0;
            this.num_bounces = 0;
            this.previous_correct = 0;
        },

        //@Override
        exportResults:function()
        {
            cc.log("exporting results")
            var stats = {
                num_correct: this.num_correct
            };

            if( this.num_tries )
            {
                this.avg_level = (this.avg_level - 1) / this.num_tries;
            }

            this.metadata.num_correct = this.num_correct;
            this.metadata.num_tries = this.num_tries;
            this.metadata.max_level = this.max_level;
            this.metadata.avg_level = this.avg_level;
            this.metadata.num_bounces = this.num_bounces;
            this.metadata.user_level = this.current_level;
            this.metadata.session_level = this.original_level;
            this.metadata.width = this.width;
            this.metadata.height = this.height;

            return this._super( stats, this.bestStatKey );
        },

        resetModel: function ()
        {
            if ( this.original_level === 0 )
            {
                this.current_level = 1;
            }
            else
            {
                this.current_level = this.original_level;
            }

            this.progress = 0;
            this.score = 0;
            this.num_tries = 0;
            this.avg_level = 0;
            this.max_level = this.current_level;
            this.num_bounces = 0;
            this.session_level = 0;
        },

        increasePoints: function ()
        {
            this.score += 50 * this.current_level * ( this.current_level + 1 );
        },

        increaseLevel: function ( )
        {
            if( this.current_level < 10 )
            {
                this.current_level++;
                this.progress = this.progress - 100;

                if( this.current_level > this.max_level )
                {
                    this.max_level = this.current_level;
                }
            }
        },

        decreaseLevel: function ()
        {
            if( this.current_level > 1 )
            {
                this.current_level--;
                this.progress = this.progress + 100;
            }
        },

        //bring in a new group of trials based on current level
        resetTrialDeck: function ( settings )
        {
            this.trial_deck = settings.getTrials( this.current_level - 1 );
            this.shuffleArray ( this.trial_deck );
        },

        //shuffle the order of elements in an array
        shuffleArray: function ( myArray )
        {
            for ( var j, x, i = myArray.length; i; j = parseInt(Math.random() * i), x = myArray[--i], myArray[i] = myArray[j], myArray[j] = x);
            return myArray;
        },

        calculateProgress: function ( trialCorrect, response_time, level, settings )
        {
            //calculate the player's progress (line ported from Flash version of Speed Pack)
            var progress;
            if ( trialCorrect )
            {
                //interpolate response time onto current level
                var low = settings.lowLevel0 - ( level * settings.lowPerLevel );
                var high = settings.highLevel0 - ( level * settings.highPerLevel );
                var range = ( 0.0 + response_time - low ) / ( high - low ) ;
                progress = Math.max(-settings.maxProgressDelta,
                    Math.min( settings.maxProgressDelta,
                        2 * settings.maxProgressDelta * ( 0.5 - range ) ) );
            }
            else
            {
                progress = -settings.maxProgressDelta;
            }

            this._deltaProgress = Math.round( progress );
            this._responseTime = Math.round( response_time );

            //clamp user's progress between level 1 and 10
            if( this.current_level === 10 && this.progress >= 80 && progress > 0 )
            {
                cc.log("User has hit high level/progress cap");
            }
            else if(this.current_level === 1 && this.progress <= 20 && progress < 0 )
            {
                cc.log("User has hit low level/progress cap");
            }
            else
            {
                this.progress += progress;
            }

            return this._deltaProgress;
        },

        calculatePlayerLevel: function( settings )
        {
            if( this.progress > 100 )
            {
                this.increaseLevel();
                this.resetTrialDeck( settings );
            }
            else if ( this.progress < 0 )
            {
                this.decreaseLevel();
                this.resetTrialDeck( settings );

            }
        }
    } );
}() );
