/*global require*/
/*global cc*/
/*global GameSettings*/
/*global SpeedPackDoorModel*/
/*global SpeedPackTrialModel*/
/*global SpeedPackDoorSlotModel*/
/*global SpeedPackDoorSlotItemModel*/

require( "src/Game/Base/GameSettings.js" );
require( "src/Game/SpeedPackTrialModel.js" );
require( "src/Game/SpeedPackDoorModel.js" );
require( "src/Game/SpeedPackDoorSlotModel.js" );
require( "src/Game/SpeedPackDoorSlotItemModel.js" );

/**
 * The Speed Pack_pro settings.
 * @class
 * @name SpeedPackSettings
 */
var SpeedPackSettings;
( function ()
{
    "use strict";

    SpeedPackSettings = GameSettings.extend(
        {
            _trials: [],
            _shapes: [],
            _doorImages: [],
            _doorImageSizes: [],
            _shapeImages: [],
            slotSize: 25,
            _itemTint: null,
            _caseOpacity: null,

            ctor: function ( fileLocation, convertKeysToLowercase )
            {
                this._super( fileLocation, convertKeysToLowercase );

                this.parseDoorImageSizes();

                this._itemTint = this.itemTint;
                this._caseOpacity = this.caseOpacity;
            },

            parseDoorImageSizes: function ()
            {
                var i;
                for ( i = 0; i < this.doorImageSizes.length; i++ )
                {
                    this._doorImageSizes.push( cc.size( this.doorImageSizes[ i ].width, this.doorImageSizes[ i ].height ) );
                }
            },

            /**
             * creates a new trial model and fills it with organized trial information
             * @param trial
             * @returns {SpeedPackTrialModel}
             */
            parseTrial: function ( trial )
            {
                var trial_model = new SpeedPackTrialModel();

                trial_model.id = trial.id;
                trial_model.net = trial.net;
                trial_model.totalnum_items = trial.totalnum_items;
                trial_model.doors = [];

                var doors = trial.doors;

                var largestDoorSize = cc.size( 0, 0 );
                var largestDoor;

                var i;
                for ( i = 0; i < doors.length; i++ )
                {
                    // add new parsed door to the trial
                    var newDoor = this.parseDoor( doors[i] );
                    trial_model.doors.push( newDoor );

                    // get the largest door and its size for shuffling items
                    if ( newDoor.width * newDoor.height >= largestDoorSize.width * largestDoorSize.height )
                    {
                        largestDoorSize = cc.size( newDoor.width, newDoor.height );
                        largestDoor = newDoor;
                    }

                }

                // get slot count
                var largestDoorSlotCount = largestDoorSize.width * largestDoorSize.height;

                // if diagonal suitcase, remove 1 slot
                if ( largestDoor.img === 5 )
                {
                    largestDoorSlotCount -= 1;
                }

                // add largest door slot indexes to array
                var largestDoorSlotIndexes = [];
                for ( i = 0; i < largestDoor.slots.length; i++ )
                {
                    if ( largestDoor.img === 5 )
                    {
                        if ( ( largestDoor.rotation !== 0 || largestDoor.slots[i].index !== 10 ) &&
                            ( largestDoor.rotation !== 90 || largestDoor.slots[i].index !== 12 ) &&
                            ( largestDoor.rotation !== 180 || largestDoor.slots[i].index !== 2 ) &&
                            ( largestDoor.rotation !== 270 || largestDoor.slots[i].index !== 0 ) )
                        {
                            largestDoorSlotIndexes.push( largestDoor.slots[ i ].index );
                        }
                    }
                    else
                    {
                        largestDoorSlotIndexes.push( largestDoor.slots[ i ].index );
                    }
                }
                // shuffle item indexes
                largestDoorSlotIndexes = this.shuffle( largestDoorSlotIndexes );

                // pop off item indexes to match the number of items
                while ( largestDoorSlotCount > trial_model.totalnum_items )
                {
                    largestDoorSlotIndexes.pop();
                    largestDoorSlotCount--;
                }

                var randomStart = Math.floor( Math.random() * ( trial_model.doors.length ) );
                // randomly distribute items ***BROKEN***
                for ( i = 0; i < largestDoorSlotIndexes.length; i++ )
                {
                    var index = largestDoorSlotIndexes[i];

                    var randomDoor = ( i + randomStart ) % ( trial_model.doors.length );
                    var direction = trial_model.doors[randomDoor].direction;
                    var offset = -100;

                    // get new randomDoor if index doesn't exist in door
                    while ( !this.hasIndexInDoor( index + offset, trial_model.doors[randomDoor] ) )
                    {
                        randomDoor = ( randomDoor + 1 ) % ( trial_model.doors.length );
                        direction = trial_model.doors[randomDoor].direction;
                        offset = 0;
                        if ( largestDoor.slots.length > trial_model.doors[randomDoor].slots.length )
                        {
                            if ( direction === 2 )
                            {
                                offset = -1;
                            }
                            else if ( direction === 8 )
                            {
                                offset = -5;
                            }
                        }
                    }

                    //distribute the items so that each trial is different
                    if ( i !== ( largestDoorSlotIndexes.length - 1 ) )
                    {
                        this.distributeItems( index + offset, trial_model.doors[randomDoor] );
                    }
                    else
                    {
                        var emptycount = 0;;
                        var emptyindex = -1;
                        var j;
                        for ( j = 0; j < trial_model.doors.length; j++ )
                        {
                            if ( trial_model.doors[j].num_items === 0 )
                            {
                                emptycount++;
                                emptyindex = j;
                            }
                        }

                        if ( emptycount > 1 )
                        {
                            direction = trial_model.doors[emptyindex].direction;
                            offset = 0;
                            if ( largestDoorSlotIndexes.length > trial_model.doors[emptyindex].slots.length )
                            {
                                if ( direction === 2 )
                                {
                                    offset = -1;
                                }
                                else if ( direction === 8 )
                                {
                                    offset = -5;
                                }
                            }

                            this.distributeItems( index + offset, trial_model.doors[emptyindex] );
                        }
                        else
                        {
                            this.distributeItems( index + offset, trial_model.doors[randomDoor] );
                        }
                    }
                }


                this.flipDoorsItems( trial_model );

                //make door with fewest items the active door (yellow)
                var fewest_items = 999;
                var fewest_items_doors = [];

                var numSlots = largestDoor.slots.length;
                var lowNumItemIndex = -1;

                for ( i = 0; i < trial_model.doors.length; i++ )
                {
                    if ( trial_model.doors[i].num_items < fewest_items )
                    {
                        fewest_items = trial_model.doors[i].num_items;
                    }

                    if ( trial_model.doors[i].slots.length < largestDoor.slots.length )
                    {
                        numSlots = trial_model.doors[i].slots.length;
                        lowNumItemIndex = i;
                    }
                }

                for ( i = 0; i < trial_model.doors.length; i++ )
                {
                    if ( i !== lowNumItemIndex )
                    {
                        if ( trial_model.doors[i].num_items === fewest_items )
                        {
                            fewest_items_doors.push( trial_model.doors[i] );
                        }
                    }
                }

                //mark a random fewest item case as active
                if ( numSlots !== largestDoor.slots.length || fewest_items_doors.length === 0 )
                {

                        var lowNum = 99;
                        var lowestIndex = -1;
                        for ( var k = 0; k < trial_model.doors.length; k++ )
                        {
                            if ( k !== lowNumItemIndex )
                            {
                                if ( trial_model.doors[k].num_items < lowNum )
                                {
                                    lowNum = trial_model.doors[k].num_items;
                                    lowestIndex = k;
                                }
                            }
                        }
                        trial_model.doors[lowestIndex].active = true;
                }
                else
                {
                    fewest_items_doors[ Math.floor( Math.random() * ( fewest_items_doors.length ) ) ].active = true;
                }

                return trial_model;
            },

            /**
             * distribute items within a door
             * @param index
             * @param door
             */
            distributeItems: function( index, door )
            {
                var slot;
                var j;
                for ( j = 0; j < door.slots.length; j++ )
                {
                    if ( door.slots[j].index === ( index ) )
                    {
                        slot = door.slots[j];
                    }
                }

                slot.filled = 1;
                slot.item = this.parseItem();
                slot.free = true;
                door.num_items++;

            },

            /**
             * flips a door's items to its partner door
             * @param trial_model
             */
            flipDoorsItems: function ( trial_model )
            {
                for ( var i = 0; i <  trial_model.doors.length; i++ )
                {
                    var door = trial_model.doors[i];
                    this.flipItems( door );
                }
            },

            /**
             * flip items to partner door
             * @param door
             */
            flipItems: function( door )
            {
                // Vertical flip
                if ( door.direction === 2 || door.direction === 6 )
                {
                    this.flipVertical( door.slots );
                }
                // Horizontol flip
                else if ( door.direction === 4 || door.direction === 8 )
                {
                    this.flipHorizontal( door.slots );
                }
                // Diagonal flip
                else
                {
                    if ( door.direction === 1 || door.direction === 5 )
                    {
                        this.flipHorizontal( door.slots );
                        this.rotateRightAngle( door.slots, door.width );
                    }
                    else if ( door.direction === 3 || door.direction === 7 )
                    {
                        var string = "before flip: ";
                        var i;
                        for ( i = 0; i < door.slots.length; i++ )
                        {
                            string += door.slots[i].index.toString() + " ";
                        }
                        this.flipVertical( door.slots );
                        this.rotateRightAngle( door.slots, door.width );
                        string = "after flip: ";
                        var i;
                        for ( i = 0; i < door.slots.length; i++ )
                        {
                            string += door.slots[i].index.toString() + " ";
                        }
                    }
                }
            },

            /**
             * flip door items vertically
             * @param slots
             * @returns {*}
             */
            flipVertical: function ( slots )
            {
                var currentVerSlotIndex = 0;
                var prevVerArrayIndex = 0;
                var i;
                for ( i = 0; i < slots.length; ++i )
                {
                    currentVerSlotIndex = slots[ i ].index;
                    if ( currentVerSlotIndex === 5 ||
                        currentVerSlotIndex === 10 ||
                        currentVerSlotIndex === 15 ||
                        currentVerSlotIndex === 20 )
                    {
                        slots = this.flipArray( slots, prevVerArrayIndex, i - 1 );
                        prevVerArrayIndex = i;
                    }
                }
                return this.flipArray( slots, prevVerArrayIndex, slots.length - 1 );
            },

            /**
             * flip door items horizontally
             * @param slots
             * @returns {*}
             */
            flipHorizontal: function ( slots )
            {
                var currentHorSlotIndex = 0;
                var prevHorArrayIndex = 0;
                var j;
                for ( j = 0; j < slots.length; ++j )
                {
                    currentHorSlotIndex = slots[ j ].index;
                    if ( currentHorSlotIndex === 5 ||
                        currentHorSlotIndex === 10 ||
                        currentHorSlotIndex === 15 ||
                        currentHorSlotIndex === 20 )
                    {
                        slots = this.flipArray( slots, prevHorArrayIndex, j - 1 );
                        prevHorArrayIndex = j;
                    }
                }
                slots = this.flipArray( slots, prevHorArrayIndex, slots.length - 1 );
                return this.flipArray( slots, 0, slots.length - 1 );
            },

            /**
             * rotates the slots using the slot indexes only. slots.filled stay in place
             */
            rotateRightAngle: function ( slots, width )
            {
                // manual rotation for 3x3 matrix (only diagonals)
                var temp1, temp2, temp3, temp4, temp5, temp6;

                temp1 = slots[ 0 ].item;
                temp2 = slots[ 0 ].filled;
                temp3 = slots[ 0 ].free;
                temp4 = slots[ 1 ].item;
                temp5 = slots[ 1 ].filled;
                temp6 = slots[ 1 ].free;
                slots[ 0 ].item = slots[ 2 ].item;
                slots[ 0 ].filled = slots[ 2 ].filled;
                slots[ 0 ].free = slots[ 2 ].free;
                slots[ 1 ].item = slots[ 5 ].item;
                slots[ 1 ].filled = slots[ 5 ].filled;
                slots[ 1 ].free = slots[ 5 ].free;
                slots[ 2 ].item = slots[ 8 ].item;
                slots[ 2 ].filled = slots[ 8 ].filled;
                slots[ 2 ].free = slots[ 8 ].free;
                slots[ 5 ].item = slots[ 7 ].item;
                slots[ 5 ].filled = slots[ 7 ].filled;
                slots[ 5 ].free = slots[ 7 ].free;
                slots[ 8 ].item = slots[ 6 ].item;
                slots[ 8 ].filled = slots[ 6 ].filled;
                slots[ 8 ].free = slots[ 6 ].free;
                slots[ 7 ].item = slots[ 3 ].item;
                slots[ 7 ].filled = slots[ 3 ].filled;
                slots[ 7 ].free = slots[ 3 ].free;
                slots[ 6 ].item = temp1;
                slots[ 6 ].filled = temp2;
                slots[ 6 ].free = temp3;
                slots[ 3 ].item = temp4;
                slots[ 3 ].filled = temp5;
                slots[ 3 ].free = temp6;

                return slots;
            },

            /**
             * reverses an array's order between 2 indexes
             */
            flipArray: function ( array, startIndex, endIndex )
            {
                var new_array = [];
                var temp = null;
                var temp2 = null;
                var temp3 = null;

                var k;
                for ( k = startIndex; k <= endIndex; ++k )
                {
                    new_array.push( array[ k ] );
                }

                var i;
                for ( i = 0; i < new_array.length / 2; ++i )
                {
                    temp = new_array[ i ].item;
                    temp2 = new_array[ i ].filled;
                    temp3 = new_array[ i ].free;
                    new_array[ i ].item = new_array[ new_array.length - 1 - i ].item;
                    new_array[ i ].filled = new_array[ new_array.length - 1 - i ].filled;
                    new_array[ i ].free = new_array[ new_array.length - 1 - i ].free;
                    new_array[ new_array.length - 1 - i ].item = temp;
                    new_array[ new_array.length - 1 - i ].filled = temp2;
                    new_array[ new_array.length - 1 - i ].free = temp3;
                }

                var j;
                for ( j = startIndex; j <= endIndex; ++j )
                {
                    array[ j ] = new_array[ j - startIndex ];
                }

                return array;
            },

            /**
             * checks if a certain index is within a door (for checking door mismatch sizing)
             * @param index
             * @param door
             * @returns {boolean}
             */
            hasIndexInDoor: function ( index, door )
            {
                var i;
                var containsIndex = false;
                for ( i = 0; i < door.slots.length; i++ )
                {
                    if ( door.slots[i].index === index )
                    {
                        containsIndex = true;
                    }
                }

                return containsIndex;
            },

            /**
             * parses a door and organizes its slot / item information
             * @param door
             * @returns {SpeedPackDoorModel}
             */
            parseDoor: function ( door )
            {
                var door_model = new SpeedPackDoorModel();

                door_model.img = door.img;
                door_model.x = door.x;
                door_model.y = door.y;
                door_model.width = door.width;
                door_model.height = door.height;
                door_model.direction = door.direction;
                door_model.rotation = door.rotation;
                door_model.active = false;
                door_model.num_items = 0;
                door_model.slots = [];

                var slots = door.slots;

                var j;
                for ( j = 0; j < slots.length; ++j )
                {
                    door_model.slots.push( this.parseSlot( slots[j] ) );
                }

                return door_model;
            },

            /**
             * parses a slot and organizes its information
             * @param slot
             * @returns {SpeedPackDoorSlotModel}
             */
            parseSlot: function ( slot )
            {
                var slot_model = new SpeedPackDoorSlotModel();

                slot_model.index = slot.index;
                slot_model.filled = 0;

                if ( slot_model.filled >= 1 )
                {
                    slot_model.item = this.parseItem();
                    slot_model.free = true;
                }

                return slot_model;
            },

            parseItem: function ()
            {
                var item_model = new SpeedPackDoorSlotItemModel();
                return item_model;
            },

            getDoorImageSize: function ( i )
            {
                return this._doorImageSizes[ i ];
            },


            /**
             * parses all trials
             * @param i
             * @returns {Array}
             */
            getTrials: function ( i )
            {
                var j;
                var array = [];
                for ( j = 0; j < this.allTrials[ i ].length; j++ )
                {
                    array.push( this.parseTrial( this.allTrials[ i ][ j ] ) );
                }
                return array;
            },

            getTrialCount: function ()
            {
                return this._trials.length;
            },

            getShapeTypeCount: function ()
            {
                return this.shapeTypeCount;
            },

            /**
             * shuffles the items in an array
             * @param array
             * @returns {*}
             */
            shuffle: function ( array )
            {
                var currentIndex = array.length, temporaryValue, randomIndex;

                // While there remain elements to shuffle...
                while ( 0 !== currentIndex )
                {

                    // Pick a remaining element...
                    randomIndex = Math.floor( Math.random() * currentIndex );
                    currentIndex -= 1;

                    // And swap it with the current element.
                    temporaryValue = array[currentIndex];
                    array[currentIndex] = array[randomIndex];
                    array[randomIndex] = temporaryValue;
                }

                return array;
            }

        } );
}() );
