/*global cc*/
/*global require*/
/*global GameView*/
/*global audio*/
/*global GameWidgetHud*/
/*global LumositySprite*/
/*global SpeedPackTrialModel*/
/*global SpeedPackDoorView*/
/*global SpeedPackDoorSlotView*/
/*global SpeedPackDoorSlotItemView*/
/*global SpeedPackDoorSlotItemModel*/
/*global GameWidgetFeedback*/
/*global lu*/

require( "src/Game/Base/GameView.js" );
require( "src/Game/Widget/GameWidgetFeedback.js" );
require( "src/Game/Widget/GameWidgetFlipbook.js" );
require( "src/Game/Widget/GameWidgetHud.js" );
require( "src/View/LumositySprite.js" );
require( "src/Game/SpeedPackTrialModel.js");
require( "src/Game/SpeedPackDoorView.js" );
require( "src/Game/SpeedPackDoorSlotView.js" );
require( "src/Game/SpeedPackDoorSlotItemView.js" );
require( "src/Game/SpeedPackDoorSlotItemModel.js" );
require( "src/Game/Widget/GameWidgetFeedback.js" );
require( "src/Utility/LumosityUtils.js" );

var SpeedPackView;
( function()
{
    "use strict";

    SpeedPackView = GameView.extend(
        {
            /** @lends SpeedPackView.prototype */

            /**
             * The Speed Pack_pro view.
             * @constructs
             * @param {cc.Size} screenSize The size of the screen that this view will be inserted into.
             * @param {Object} [settings] The settings that we can pull view-based data from.
             * @param {SpeedPackModel} [model] The GameModel that we are linked to.
             * @param {Object} [interactionResponder] The object that we should send interaction events to. We also use this is a default animation event listener.
             * @param {boolean} [preload=true] If true, will preload all flipbooks and other assets upon creation.
             */

            _trialModel: null,
            screenSize: null,
            _currentTrialDoors: null,
            _currentTrialSlots: null,
            _currentTrialButtons: null,
            _currentTrialItems: null,

            _readyDoors: null,
            _nextDoor: null,
            _mainDoorIndex: null,
            _camera: null,

            _doorNode: null,

            _doorNodes: null,

            scale: 1,
            animateSprite: null,
            topDoor: null,

            ctor: function( screenSize, settings, model, interactionResponder, preload )
            {

                this._super( screenSize, settings, model, interactionResponder, preload );

                this.setTag( 22 );
                this._currentTrialDoors = [];
                this._currentTrialSlots = [];
                this._currentTrialButtons = [];
                this._currentTrialItems = [];
                this._doorNodes = [];
                this._readyDoors = [];

                //add the HUD widget
                var hudWidget;
                hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];
                if (hudWidget)
                {
                    this.hud = hudWidget;
                }

                this._trialModel = new SpeedPackTrialModel();

                this.screenSize = screenSize;
            },

            /**
             * draws the case camera in a specific door and tile
             * @param tile
             * @param door
             */
            drawCamera: function( tile, door )
            {
                var camera = new LumositySprite( "cameras/cameraPlaced.png" );
                camera.setContentSize( cc.size( 90, 90 ), true, true );
                camera.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                camera.setPosition( cc.p( 0, 0 ) );
                door.node.addChild( camera );
                camera.setPosition( tile.pos );
                camera.setVisible( true );
                camera.setZOrder( 10 );

                this._camera = camera;
                camera.setRotation( -door.rotation );
            },

            /**
             * hides slots and buttons (for flipping visuals)
             * @param doorIndex
             */
            ghostCases: function( doorIndex )
            {
                var i;
                var j;

                for ( i = 0; i < this._currentTrialSlots.length; ++i )
                {
                    if( this._currentTrialSlots[i].doorIndex === doorIndex )
                    {
                        this._currentTrialSlots[i].sprite.setVisible( false );
                    }
                }

                for ( j = 0; j < this._currentTrialButtons.length; ++j )
                {
                    if( this._currentTrialButtons[j].doorIndex === doorIndex )
                    {
                        this._currentTrialButtons[j].button.setVisible( false );
                    }
                }
            },

            /**
             * disables all buttons in a trial
             */
            disableButtons: function()
            {
                var j;
                for ( j = 0; j < this._currentTrialButtons.length; ++j )
                {
                    this._currentTrialButtons[j].button.softDisable();
                }

            },

            prepareDoors: function( i, door )
            {
                this._readyDoors.push({"index": i, "door": door});
            },

            /**
             * folds one door over
             * @param feedback
             * @param tile
             * @param door
             * @param clicked_door
             */
            animateNextDoor: function( feedback, tile, door, clicked_door )
            {
                if( this._readyDoors.length > 0 ) {
                    //setup next door
                    this._nextDoor = this._readyDoors.pop();
                    this.topDoor = this._nextDoor.door;
                    this.animateSprite = this.topDoor.view;

                    this._doorNode = this._doorNodes[this._nextDoor.index];
                    this._doorNode.setZOrder( 10 );
                    this._doorNode.setAnchorPoint( 1, 0.5 );

                    switch (this.topDoor.direction) {
                        case 1:
                            this._doorNode.setPositionX(this._doorNode.getPositionX() - 41 - (Math.cos(45)*this._doorNode.getContentSize().height)/2) ;
                            this._doorNode.setPositionY(this._doorNode.getPositionY() - 7 + (Math.sin(45)*this._doorNode.getContentSize().height)/2);
                            break;
                        case 2:
                            this._doorNode.setPositionY(this._doorNode.getPositionY() + (this._doorNode.getContentSize().width / 2));
                            break;
                        case 3:
                            this._doorNode.setPositionX(this._doorNode.getPositionX() + 41 + (Math.cos(45)*this._doorNode.getContentSize().height)/2);
                            this._doorNode.setPositionY(this._doorNode.getPositionY() - 7 + (Math.sin(45)*this._doorNode.getContentSize().height)/2);
                            break;
                        case 4:
                            this._doorNode.setPositionX(this._doorNode.getPositionX() + (this._doorNode.getContentSize().width / 2));
                            break;
                        case 5:
                            this._doorNode.setPositionX(this._doorNode.getPositionX() + 41 + (Math.cos(45)*this._doorNode.getContentSize().height)/2);
                            this._doorNode.setPositionY(this._doorNode.getPositionY() + 7- (Math.sin(45)*this._doorNode.getContentSize().height)/2);
                            break;
                        case 6:
                            this._doorNode.setPositionY(this._doorNode.getPositionY() - (this._doorNode.getContentSize().width / 2));
                            break;
                        case 7:
                            this._doorNode.setPositionX(this._doorNode.getPositionX() - 41 - (Math.cos(45)*this._doorNode.getContentSize().height)/2);
                            this._doorNode.setPositionY(this._doorNode.getPositionY() + 7 - (Math.sin(45)*this._doorNode.getContentSize().height)/2);
                            break;
                        case 8:
                            this._doorNode.setPositionX(this._doorNode.getPositionX() - (this._doorNode.getContentSize().width / 2));
                            break;
                        default:
                            break;
                    }

                    //play case closed sound
                    var playFoldSound = cc.CallFunc.create( this.playFoldSound , this );
                    var flip1 = cc.OrbitCamera.create( 0.15, 0.5, 0, 0, -90, 0, 0 );
                    //remove flipping tiles
                    var disableButtons = cc.CallFunc.create( this.disableButtons, this );
                    var disableTiles = cc.CallFunc.create( this.ghostCases.bind( this, this._nextDoor.index ), this );
                    //remove main tiles
                    var disableMainTiles = cc.CallFunc.create( this.ghostCases.bind( this, this._mainDoorIndex ), this );
                    //set door opacity to less
                    var changeOpacity = cc.CallFunc.create( this.changeOpacity, this );
                    var flip2 = cc.OrbitCamera.create( 0.15, 0.5, 0, -90, -90, 0, 0 );
                    //remove top door
                    var disableCase = cc.CallFunc.create( this.removeTopDoor, this );
                    //alter tint of current door items
                    var alterCurrentTint = cc.CallFunc.create( this.tintItems.bind (this, this._nextDoor.index ) , this );
                    //alter tint of current door items
                    var alterMainTint = cc.CallFunc.create( this.tintItems.bind (this, this._mainDoorIndex ) , this );
                    //alter tint of current door items
                    var alterCameraTint = cc.CallFunc.create( this.alterCameraTint , this );
                    //signal animation has ended, enables next animation
                    var endAnimation = cc.CallFunc.create( this.endAnimation.bind( this, feedback, tile, door, clicked_door ), this );
                    //play case closed sound
                    var playClosedSound = cc.CallFunc.create( this.playClosedSound , this );
                    //put them together

                    //if this door is the door with the camera added to it or the camera is on the main door, tint the camera this flip, otherwise wait to tint camera
                    var masterSequence;
                    if( this._nextDoor.door === clicked_door || clicked_door.direction === 0 )
                    {
                        masterSequence = cc.Sequence.create( disableButtons, playFoldSound, flip1, disableTiles, disableMainTiles, changeOpacity, alterCurrentTint, alterMainTint, alterCameraTint, flip2, disableCase, playClosedSound, endAnimation );
                    }
                    else
                    {
                        masterSequence = cc.Sequence.create( disableButtons, playFoldSound, flip1, disableTiles, disableMainTiles, changeOpacity, alterCurrentTint, alterMainTint, flip2, disableCase, playClosedSound, endAnimation );
                    }


                    this._doorNode.runAction(masterSequence);
                }
                else
                {
                    var showFeedback = cc.CallFunc.create( this.showFeedbackOverTile.bind( this, feedback, tile, door ), this );
                    var func = cc.CallFunc.create( this.interactionResponder.delayAndReset(), this );
                    var seq = cc.Sequence.create( showFeedback, func );
                    this._doorNode.runAction( seq );
                }
            },

            /**
             * animates the next door after the previous door has been folded
             * @param feedback
             * @param tile
             * @param door
             * @param clicked_door
             */
            endAnimation: function ( feedback, tile, door, clicked_door )
            {
                this.animateNextDoor( feedback, tile, door, clicked_door );
            },

            /**
             *  sets the opacity of a flipping door
             */
            changeOpacity: function ()
            {
                if(!this._doorNode.getChildByTag( 777 ) )
                {

                    this._doorNode.getChildren()[ 0 ].getChildByTag( 777 ).setOpacity( this.settings._caseOpacity );
                }
                else
                {
                    this._doorNode.getChildByTag( 777 ).setOpacity( this.settings._caseOpacity );
                }
            },

            /**
             * hides a door sprite after it has been folded onto the main case
             */
            removeTopDoor: function ()
            {
                this.animateSprite.setOpacity(0);
            },

            /**
             * tints folded items to color indicated in settings.json
             * @param doorIndex
             */
            tintItems: function ( doorIndex )
            {
                var i;
                for( i = 0; i < this._currentTrialItems.length; i++ )
                {
                    if( this._currentTrialItems[i].doorIndex === doorIndex )
                    {
                        var tintAction = cc.TintTo.create( 0.0, this.settings._itemTint.r, this.settings._itemTint.g, this.settings._itemTint.b );
                        this._currentTrialItems[ i ].item.runAction( tintAction );
                    }
                }

            },

            alterCameraTint: function ()
            {
                var tintAction = cc.TintTo.create( 0.0, this.settings._itemTint.r, this.settings._itemTint.g, this.settings._itemTint.b );
                this._camera.runAction( tintAction );
            },

            setScale: function( sender )
            {
                this.scale -= 0.1;
                this.animateSprite.setScaleX( this.scale );
            },

            playFoldSound: function ()
            {
                audio.playEffect( "sounds/caseMove.mp3" );
            },

            playClosedSound: function ()
            {
                audio.playEffect( "sounds/caseClose.mp3" );
            },

            /**
             * creates the trial and adds it to the suitcase animation/layer
             * @param trial
             */
            drawNextTrial: function( trial )
            {
                this._currentTrialDoors = [];
                this._currentTrialSlots = [];
                this._currentTrialButtons = [];
                this._currentTrialItems = [];
                this._doorNodes = [];

                //get trial from settings
                var suitcaseSize = this.suitcase.getContentSize();

                //create & fill doors from trial
                var doors = trial.doors;
                var i;
                var doorSprite;
                var offsets;
                var halfDoorSizes;
                var doorContentSize;
                var index;
                var x;
                var y;
                var button;
                var slotSprite;
                var slot;
                var item;
                var diagonalCounter = 0;
                var diagOffsetX = 0;
                var diagOffsetY = 0;

                for ( i = 0; i < doors.length; i++)
                {
                    this._doorNode = new cc.Node();

                    if ( doors[ i ].direction === 0 )
                    {
                        this._mainDoorIndex = i;
                    }

                    //add door
                    doorSprite = new SpeedPackDoorView( this.settings, doors[ i ], suitcaseSize );
                    doors[ i ].node = this._doorNode;

                    this._doorNode.setContentSize( cc.size( doorSprite.getContentSize().width , doorSprite.getContentSize().height ));

                    this._currentTrialDoors.push( doorSprite );
                    this._doorNode.addChild( doorSprite, 0, 777 );

                    this._doorNode.setPosition( doorSprite.getPosition() );
                    doorSprite.x = this._doorNode.getContentSize().width/2;
                    doorSprite.y = this._doorNode.getContentSize().height/2;

                    diagOffsetX = 0;
                    diagOffsetY = 0;
                    if ( doors[ i ].img === 5 )
                    {
                        diagonalCounter++;
                        if ( diagonalCounter % 2 === 0 )
                        {
                            doorSprite.setRotation( 90 );
                            doorSprite.setScaleX( -1 );
                            doorSprite.x += 23;
                            doorSprite.y -= 23;
                            diagOffsetX = 16;
                            diagOffsetY = 18;
                        }
                    }

                    doorSprite.setDoorSpriteProperties();

                    this._doorNode.setRotation( doors[ i ].rotation );

                    this._doorNode.setAnchorPoint( cc.p( 0.5, 0.5) );

                    doors[ i ].view = doorSprite;

                    //set offsets for drawing on suitcase (since all image suitcases have left-facing handles)
                    offsets = cc.p(0, 0);
                    halfDoorSizes = cc.size(0, 0);
                    doorContentSize = doorSprite.getContentSize();

                    offsets.x = 95 - diagOffsetX;
                    offsets.y = 59 - diagOffsetY;
                    if ( doors[i].img === 8 )
                    {
                        offsets.x = 59;
                        offsets.y = 59;
                    }
                    halfDoorSizes.width = doorContentSize.width / 2;
                    halfDoorSizes.height = doorContentSize.height / 2;

                    //draw suitcase contents
                    index = 0;
                    var numOfItems = 0;
                    for( x = 0; x < doors[ i ].width; x++)
                    {
                        for( y = 0; y < doors[ i ].height; y++)
                        {
                            var gridPosX;
                            var gridPosY;
                            if ( doors[ i ].rotation === 90 )
                            {
                                gridPosX = y;
                                gridPosY = doors[ i ].width - 1 - x;
                            }
                            else if ( doors[ i ].rotation === 270 )
                            {
                                gridPosX = doors[ i ].height - 1 - y;
                                gridPosY = x;
                            }
                            else if ( doors[i].rotation === 180 )
                            {
                                gridPosX = doors[ i ].width - 1 - x;
                                gridPosY = doors[ i ].height - 1 - y;
                            }
                            else
                            {
                                gridPosX = x;
                                gridPosY = y;
                            }
                            //if it is a diagonal flipping case --> disable the tile
                            if(doors[ i ].img === 5)
                            {
                                if( ( doors[ i ].rotation === 0 && x === 2 && y === 0 ) ||
                                    ( doors[ i ].rotation === 90 && x === 2 && y === 2 ) ||
                                    ( doors[ i ].rotation === 180 && x === 0 && y === 2 )||
                                    ( doors[ i ].rotation === 270 && x === 0 && y === 0 ) )
                                    {
                                        doors[ i ].slots[ index ].filled = -1;
                                    }

                            }

                            button = null;
                            slotSprite = null;
                            slot = doors[ i ].slots[ index ];
                            //check if the tile is used in door  (i.e. filled != -1), kept invisible if unused
                            if ( slot.filled >= 0 )
                            {
                                //if active door
                                if( doors[ i ].active )
                                {
                                    if ( doors[ i ].slots[index].filled < 1 )
                                    {
                                        button = new SpeedPackDoorSlotView();
                                        button.fillButton( doors[ i ],
                                            index,
                                            gridPosX,
                                            gridPosY,
                                            doorSprite.getPosition().x,
                                            doorSprite.getPosition().y,
                                            offsets,
                                            halfDoorSizes,
                                            true,
                                            true,
                                            this.interactionResponder );
                                    }
                                    else
                                    {
                                        button = new SpeedPackDoorSlotView();
                                        button.fillButton( doors[ i ],
                                            index,
                                            gridPosX,
                                            gridPosY,
                                            doorSprite.getPosition().x,
                                            doorSprite.getPosition().y,
                                            offsets,
                                            halfDoorSizes,
                                            false,
                                            true,
                                            this.interactionResponder );
                                    }
                                    slot.view = button;
                                    button.setVisible( true );
                                    this._doorNode.addChild( button );

                                    this._currentTrialButtons.push( {"button": button, "doorIndex": i });
                                }
                                else //if inactive door (blue)
                                {
                                    slotSprite = new LumositySprite( "tileDefault.png" );
                                    slotSprite.setContentSize( cc.size( 91, 91 ), true );
                                    slotSprite.setPosition( cc.p (
                                        doorSprite.getPosition().x - halfDoorSizes.width + ( gridPosX * 89 ) + offsets.x,
                                        doorSprite.getPosition().y + halfDoorSizes.height - ( gridPosY * 89 ) - offsets.y
                                    ) );
                                    slotSprite.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                                    slotSprite.setZOrder( 2 );

                                    slot.pos = slotSprite.getPosition();
                                    slot.view = slotSprite;

                                    this._currentTrialSlots.push( {"sprite": slotSprite, "doorIndex": i } );
                                    this._doorNode.addChild( slotSprite );
                                }

                                //if slot filled with item
                                if( slot.filled > 0 )
                                {
                                    numOfItems++;
                                }
                            }
                            index++;
                        }
                    }

                    var itemInfo;
                    while( numOfItems > 0 )
                    {
                        for ( x = 0; x < doors[ i ].slots.length; ++x )
                        {
                            if ( doors[  i].slots[  x].free && doors[ i ].slots[ x ].view )
                            {
                                itemInfo = this.placeItems( doors[ i ].slots, doors[ i ].slots[ x ] );

                                var lowPosition = itemInfo.low.view.getPosition();
                                var highPosition = itemInfo.high.view.getPosition();
                                var itemPosition = cc.p( 0, 0 );

                                if ( itemInfo.direction === 6 )
                                {
                                    itemPosition.x = lowPosition.x;
                                    itemPosition.y = ( lowPosition.y + highPosition.y ) / 2;
                                    if ( doors[ i ].rotation === 90 || doors[ i ].rotation === 270 )
                                    {
                                        itemPosition.x = ( lowPosition.x + highPosition.x ) / 2;
                                        itemPosition.y = lowPosition.y;
                                    }
                                }
                                else if ( itemInfo.direction === 4 )
                                {
                                    itemPosition.x = ( lowPosition.x + highPosition.x ) / 2;
                                    itemPosition.y = lowPosition.y;
                                    if ( doors[ i ].rotation === 90 || doors[ i ].rotation === 270 )
                                    {
                                        itemPosition.x = lowPosition.x;
                                        itemPosition.y = ( lowPosition.y + highPosition.y ) / 2;
                                    }
                                }

                                //place item
                                item = new SpeedPackDoorSlotItemView( doors[ i ],
                                    itemInfo.size,
                                    itemInfo.direction,
                                    itemPosition,
                                    this.settings );
                                //slot.item.view = item;
                                //slot.item.pos = cc.p( item.x, item.y );

                                this._currentTrialItems.push( { "item": item, "doorIndex": i } );
                                this._doorNode.addChild( item );
                            }
                        }

                        numOfItems -= itemInfo.size;
                    }


                    if ( doors[ i ].img === 5 )
                    {
                        var superNode = new cc.Node();

                        superNode.setContentSize( cc.size( doorSprite.getContentSize().width , doorSprite.getContentSize().height ));
                        superNode.setPosition( this._doorNode.getPosition() );
                        var temp = 12;
                        this._doorNode.setPosition(doorSprite.getContentSize().width / 2 + temp, doorSprite.getContentSize().height / 2 );
                        superNode.setAnchorPoint( cc.p( 0.5, 0.5) );
                        superNode.addChild( this._doorNode );

                        switch ( doors[i].rotation )
                        {
                            case 0:
                                superNode.setRotation( superNode.getRotation() + 315 );
                                this._doorNode.setRotation( this._doorNode.getRotation() - 315);
                                break;
                            case 90:
                                superNode.setRotation( superNode.getRotation() + 45 );
                                this._doorNode.setRotation( this._doorNode.getRotation() - 45);
                                break;
                            case 180:
                                superNode.setRotation( superNode.getRotation() + 135 );
                                this._doorNode.setRotation( this._doorNode.getRotation() - 135);
                                break;
                            case 270:
                                superNode.setRotation( superNode.getRotation() + 225 );
                                this._doorNode.setRotation( this._doorNode.getRotation() - 225);
                                break;
                        }

                        this._doorNodes.push( superNode );
                        this.suitcase.addChild( superNode );
                    }
                    else
                    {
                        this._doorNodes.push( this._doorNode );
                        this.suitcase.addChild( this._doorNode );
                    }

                }
            },

            /**
             * controls the placement of items, adding variety to otherwise similar trials
             * @param allSlots
             * @param slot
             * @returns {{size: number, direction: number, low: null, high: null}}
             */
            placeItems: function( allSlots, slot )
            {
                var itemInfo = { "size": 1, "direction": 0, "low": null, "high": null };
                itemInfo.size = 1;
                itemInfo.direction = 4;
                itemInfo.low = slot;
                itemInfo.high = slot;
                slot.free = false;

                var current = slot.index;
                var right = 5;
                var down = 1;
                var limit;

                var breakWhile = false;;

                var i;
                var j;
                for ( j = 0; j < allSlots.length; ++j )
                {
                    if ( allSlots[ j ].index === ( current + down ) && allSlots[ j ].free )
                    {
                        breakWhile = false;
                        if ( current >= 0 && current < 5 )
                        {
                            limit = 5;
                        }
                        else if ( current >= 5 && current < 10 )
                        {
                            limit = 10;
                        }
                        else if ( current >= 10 && current < 15 )
                        {
                            limit = 15;
                        }
                        else if ( current >= 15 && current < 20 )
                        {
                            limit = 20;
                        }
                        else if ( current >= 20 && current < 25 )
                        {
                            limit = 25;
                        }
                        itemInfo.direction = 6;
                        while ( current + down < limit )
                        {
                            current += down;
                            for ( i = 0; i < allSlots.length; ++i )
                            {
                                if ( allSlots[ i ].index === current )
                                {
                                    if ( allSlots[ i ].free )
                                    {
                                        itemInfo.size++;
                                        allSlots[ i ].free = false;
                                        itemInfo.high = allSlots[ i ];
                                    }
                                    else
                                    {
                                        breakWhile = true;
                                    }
                                }
                            }
                            if ( breakWhile )
                            {
                                break;
                            }
                        }
                        break;
                    }
                    else if ( allSlots[ j ].index === (current + right) && allSlots[ j ].free )
                    {
                        breakWhile = false;
                        itemInfo.direction = 4;
                        while ( current + right < 25 )
                        {
                            current += right;
                            for ( i = 0; i < allSlots.length; ++i )
                            {
                                if ( allSlots[ i ].index === current )
                                {
                                    if ( allSlots[ i ].free )
                                    {
                                        itemInfo.size++;
                                        allSlots[ i ].free = false;
                                        itemInfo.high = allSlots[ i ];
                                    }
                                    else
                                    {
                                        breakWhile = true;
                                    }
                                }
                            }
                            if ( breakWhile )
                            {
                                break;
                            }
                        }
                        break;
                    }
                }

                return itemInfo;
            },

            /**
             * controls the score HUD element value
             * @param score
             */
            setScore: function( score )
            {
                this._super( score );
            },

            /**
             * creates a Feedback Widget if none exists
             * @returns {GameWidgetFeedback}
             * @private
             */
            _createExtraFeedback: function()
            {
                var zOrder = 100;

                var newFeedbackWidget = new GameWidgetFeedback( this.model, this.interactionResponder );
                var gameContentNode = this[ "gameContent" ] || this[ "centeredContent" ];
                if ( gameContentNode )
                {
                    gameContentNode.addChild( newFeedbackWidget, zOrder );
                }
                else
                {
                    this._gameStage.addChild( newFeedbackWidget, zOrder );
                }
                this.addWidget( newFeedbackWidget, false, true );
                return newFeedbackWidget;
            },

            /**
             * controls the feedback shows after tile is pressed
             * @param success
             * @param tile
             * @param door
             */
            showFeedbackOverTile: function( success, tile, door )
            {
                var feedbackWidget = null;

                if ( this._widgets[ GameWidgetFeedback.prototype.getType() ] instanceof Array )
                {
                    var feedbackWidgets = this._widgets[ GameWidgetFeedback.prototype.getType() ];

                    var i;
                    for ( i = 0; i < feedbackWidgets.length; ++i )
                    {
                        feedbackWidget = feedbackWidgets[ i ];
                        if ( !feedbackWidget.isBusy )
                        {
                            break;
                        }
                        feedbackWidget = null;
                    }
                }
                else
                {
                    feedbackWidget = this._widgets[ GameWidgetFeedback.prototype.getType() ];
                    if ( feedbackWidget.isBusy )
                    {
                        feedbackWidget = null;
                    }
                }

                if ( feedbackWidget === null )
                {
                    feedbackWidget = this._createExtraFeedback();
                }

                if ( feedbackWidget )
                {
                    var finalDoor;
                    var tilePos;
                    var i;
                    for ( i = 0; i < door.length; ++i )
                    {
                        if ( door[ i ].direction === 0 )
                        {
                            finalDoor = door[ i ];
                        }
                    }
                    if ( finalDoor )
                    {
                        for ( i = 0; i < finalDoor.slots.length; ++i )
                        {
                            if ( tile.index === finalDoor.slots[ i ].index )
                            {
                                tilePos = finalDoor.slots[ i ].view.getPosition();
                            }
                        }
                    }
                    feedbackWidget.setVisible( true );
                    feedbackWidget.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                    var convertedPos = finalDoor.node.convertToWorldSpace( tilePos );
                    var convertednodepos = this.gameContent.convertToNodeSpace( convertedPos );

                    feedbackWidget.setPosition( convertednodepos );

                    feedbackWidget.tileModel = tile;

                    feedbackWidget.isBusy = true;

                    feedbackWidget.play( success );
                }
            },

            layout_creatingNode: function( nodeName, nodeData )
            {
                switch ( nodeName )
                {
                    case "timeLabel":
                        this.hud.setTextColor( nodeData.textData.color );
                        return false;

                    case "scoreLabel":
                        this.hud.setTextColor( nodeData.textData.color );
                        return false;

                    default:
                        return this._super( nodeName, nodeData );
                }
            }
        } );
}() );
