/*global cc*/
/*global require*/
/*global LumosityButtonState*/
/*global LumosityButton*/
/*global LayoutLoader*/
/*global LayoutNodeFlags*/
/*global LumosityUtils*/
/*global AnimationController*/

require( "src/View/LumosityButton.js" );
require( "src/Animation/LayoutLoader.js" );
require( "src/Utility/LumosityUtils.js" );
require( "src/Animation/AnimationController.js" );

var StageSelect;
( function()
{
    "use strict";

    StageSelect = cc.Node.extend(
        {
            controller: null,
            button: null,
            buttonAnimationController: null,


            /** @lends StageSelect.prototype */

            /**
             *  StageSelect is the main class for a StageSelect view!
             *
             *  @constructs
             */
            ctor:function( controller, view, contentSize )
            {
                this._super();

                this.controller = controller;
                this.setContentSize( contentSize );
                this.loadLayout();
                this._createAnimationController( view );
                this.createButton( contentSize );

                this.nodeName = "StageSelect";
            },

            drawLine: function ( pointA, pointB )
            {
                var drawNode = cc.DrawNode.create();
                this.addChild(drawNode,100);
                drawNode.clear();
                drawNode.drawSegment( pointA, pointB, 1, cc.c4b( 10, 10, 10, 128 ));

            },

            loadLayout: function()
            {
                LayoutLoader.loadLayoutFile( "layoutButton.json", this, this , this );
            },

            /**
             *  Creates this view's AnimationController from the game's settings file. That way of doing it
             *  is temporary, later we will want to pass the animation file name into this function. The
             *  controller is also automatically created with the shared game animation data.
             */
            _createAnimationController:function( view )
            {
                var animFile = "animationsButton.json";
                if ( this.settings && this.settings.animationFile )
                {
                    animFile = this.settings.animationFile;
                }
                this.buttonAnimationController = new AnimationController( animFile, this, this );
                if(this.buttonAnimationController)
                {
                }
                else
                {
                }
            },

            createButton: function ( contentSize )
            {
                this.button = new LumosityButton();
                //set what the buttons look like
                var normalSprite = cc.Sprite.create( "menu/circle.png" );
                normalSprite.setColor( cc.RED );
                normalSprite.setOpacity ( 0 );
                this.button.setPosition( cc.p (contentSize.width/2,contentSize.height/2));
                this.button.setBackgroundImageForState( normalSprite, LumosityButtonState.ButtonStateNormal );
                this.button.setAnchorPoint( cc.p( 0.5, 0.5 ) );

                this.addChild( this.button );
            },

            playLockedAnimation: function ()
            {
                this.buttonAnimationController.play("locked", this.animFinished );
            },
            playUnlockedAnimation: function ()
            {
                this.buttonAnimationController.play("unlocked", this.animFinished );
            },
            playActiveInAnimation: function ()
            {
                this.buttonAnimationController.play("activeIn", this.animFinished );
            },
            playActiveOutAnimation: function ()
            {
                this.buttonAnimationController.play("activeOut", this.animFinished );
            },
            playSelectAnimation: function ()
            {
                this.buttonAnimationController.play("select", this.animFinished );
            },
            animFinished: function ()
            {
            }



        } );
}() );
