/*global cc*/
/*global require*/
/*global LumosityButton*/
/*global LumosityButtonState*/
/*global LumosityUtils*/

var StarKey =
{
    Shape: 0,
    Color: 1,
    SingleRotation: 2,
    ConstantRotation: 3,
    Texture: 4
};

var Star;
( function()
{

	"use strict"; 

	require( "src/View/LumosityButton.js" ); 
    require( "src/Utility/LumosityUtils.js" );

   	Star = LumosityButton.extend(
    {

    	//whether or not this is the unique element
    	amIUnique: false,
        isAnimating: false,
        //the thing that holds all of my information
        arrayOfStarElements: [],
        //an index that groups identifies stars (the unique star will have a unique index)
        starIndex: null,
        selected: false,
        hidden: false,
	 	/** @lends Star */
        /**
         *  The actual Stars that are used in Star Search. Extends Lumosity Button with a few additional features and variables
         *
         *  @constructs
         */
		ctor: function ( arrayOfStarElements, amIUnique, position, animationLength, starIndex, screenWidth, screenHeight )
		{
			this._super();

            this.arrayOfStarElements = arrayOfStarElements;
            this.amIUnique = amIUnique;
            this.starIndex = starIndex;
            this.toRotateTo = arrayOfStarElements[ StarKey.SingleRotation ];
            this.constantRotation = arrayOfStarElements[ StarKey.ConstantRotation ];
            this.setAnchorPoint( cc.p( 0.5, 0.5 ) );

            var stimuliButtonImage = cc.Sprite.create( "finalShapes/OrigamiShape" + arrayOfStarElements[ StarKey.Shape ] + "-" + arrayOfStarElements[ StarKey.Texture ] + ".png" );
            lu.scaleForContentSize( stimuliButtonImage, cc.size( 150, 150 ), true ); //the 150x150 is the size of the shapes in the 640 folder
            stimuliButtonImage.setColor( arrayOfStarElements[ StarKey.Color ] );
            stimuliButtonImage.setAnchorPoint( cc.p( 0.5, 0.5 ) );

            if ( amIUnique )
            {
                this.setZOrder(99);
            }
            this.permPosition = position;
            this.centerScreen = cc.p (screenWidth/2, screenHeight/2);
            this.setBackgroundImageForState( stimuliButtonImage, LumosityButtonState.ButtonStateNormal );
            this.setBackgroundImageForState( stimuliButtonImage, LumosityButtonState.ButtonStateSelected );
            this.setBackgroundImageForState( stimuliButtonImage, LumosityButtonState.ButtonStateDisabled );

            this.setTarget( this );
            this.scheduleUpdate();

            //TODO - this will probably be necessary to do in flash so the artists can get to it. This is mostly a temporary way of doing it for prettiness
            if( animationLength )
            {
                this.animationTimer = animationLength;
            }
            else
            {
                this.animationTimer = 0.25;
            }

            if( !this.centerScreen )
            {
                this.setPosition( position );
                this.setRotation( this.toRotateTo );
            }
		},

        update: function( dt )
        {
            this._super( dt );

            //rotate the star if it has a constant rotation
            if ( this.constantRotation !== undefined && this.constantRotation !== 0 )
            {
                this.setRotation( this.getRotation() + this.constantRotation );
            }
        },

        //play the star's intro animation - start small in the middle and grow, rotate, and move to permanent place
        introAnimation: function( callBack, callBackTarget)
        {
            this.isAnimating = true;
            //start the star at the center of the screen

            var currentScale = this.getScale();
            this.setPosition( this.centerScreen );

            this.setRotation( -90 );
            this.setScale( this.getParent().getScale()/100 );

            var randomStaggerTime = LumosityUtils.getRandomInt( 0, 10 ) / 100;

            //There's also got to be a way to call these three actions at once. FER SURE
            var animOver = cc.CallFunc.create( this.animationOver, this );
            var delay = cc.DelayTime.create( randomStaggerTime );
            var moveAnim = cc.EaseOut.create( cc.MoveTo.create( this.animationTimer, this.permPosition ), 2 );
            var enableStar = cc.CallFunc.create( callBack, callBackTarget );
            this.runAction( cc.Sequence.create( delay, moveAnim ));
            this.runAction( cc.Sequence.create( delay, cc.RotateTo.create( this.animationTimer, this.toRotateTo ) ) );
            this.runAction( cc.Sequence.create( delay, cc.ScaleTo.create( this.animationTimer, currentScale ), animOver, enableStar ) );
        },

        // elegantly shrinks star out of scene
        outroAnimation: function( timeToWait, animSpeed, destroyFunction, destroyTarget )
        {
            this.isAnimating = true;
            var currentScale = this.getScale();
            var speed = this.animationTimer/2;

            if ( animSpeed )
            {
                speed = animSpeed;
            }
            var delay = cc.DelayTime.create( timeToWait );
            var rotate = cc.RotateTo.create( speed, (-90) );
            var scale = cc.ScaleTo.create( speed, currentScale/100);
            var animOver = cc.CallFunc.create( this.animationOver, this );
            var hideStar = cc.CallFunc.create( this.hide, this );
            var destroySelf = cc.CallFunc.create( destroyFunction, destroyTarget );

            this.runAction( cc.Sequence.create( delay, rotate));
            this.runAction( cc.Sequence.create( delay, scale, hideStar, animOver, destroySelf ));
        },

        // elegantly shrinks star out of scene
        lastOutroAnimation: function( timeToWait, doneFunction, doneTarget, animSpeed, destroyFunction, destroyTarget  )
        {
            this.isAnimating = true;
            var speed = this.animationTimer/2;

            if ( animSpeed )
            {
                speed = animSpeed;
            }
            var animOver = cc.CallFunc.create( this.animationOver, this );
            var hideStar = cc.CallFunc.create( this.hide, this );
            var triggerReset = cc.CallFunc.create( doneFunction, doneTarget );
            var destroySelf = cc.CallFunc.create( destroyFunction, destroyTarget );

            this.runAction( cc.Sequence.create( cc.DelayTime.create( timeToWait ), cc.RotateTo.create( speed, (-90) ) ) );
            this.runAction( cc.Sequence.create( cc.DelayTime.create( timeToWait ), cc.ScaleTo.create( speed, this.getScale()/100 ), hideStar, animOver, triggerReset, destroySelf ) );

        },

        //bounces the unique star to show users a hint
        bounceAnimation: function ()
        {
            this.isAnimating = true;
            var moveUp = cc.MoveTo.create( .5, cc.p( this.getPosition().x, this.getPosition().y + 25 ) );
            var dropOne = cc.MoveTo.create( .25, cc.p( this.getPosition().x, this.getPosition().y) );
            var bounceOne = cc.MoveTo.create( .12, cc.p( this.getPosition().x, this.getPosition().y + 12 ) );
            var dropTwo = cc.MoveTo.create( .06, cc.p( this.getPosition().x, this.getPosition().y) );
            var bounceTwo = cc.MoveTo.create( .03, cc.p( this.getPosition().x, this.getPosition().y + 5) );
            var dropThree = cc.MoveTo.create( .02, cc.p( this.getPosition().x, this.getPosition().y) );
            var animOver = cc.CallFunc.create( this.animationOver, this );

            this.runAction( cc.Sequence.create( moveUp, dropOne, bounceOne, dropTwo, bounceTwo, dropThree, animOver ));
        },

        animationOver: function ()
        {
            this.isAnimating = false;

        },

        hide: function()
        {
            this.setEnabled( false );

        },

        drawLine: function ( pointA, pointB )
        {
            var drawNode = cc.DrawNode.create();
            this.addChild(drawNode,100);
            drawNode.drawSegment( pointA, pointB, 1, cc.c4b( 10, 10, 10, 255 ));

        }
	} );
}() );