/*global cc*/
/*global LumosityUtils*/
/*global audio*/
/*global require*/
/*global LevelKey*/
/*global Star*/
/*global StarKey*/

var MasterKey =
{
    StarKey: 0,
    LevelKey: 1,
    PossibilitiesKey:2
};

var StarLevelCreator;
( function()
{
    "use strict";

    require( "src/Game/LevelKey.js" );
    require( "src/Game/Star.js" );
    require( "src/Utility/LumosityUtils.js" );

    StarLevelCreator = cc.Class.extend(
    {
        // the columns and rows for the actual array
        _cols: null,
        _rows: null,

        //the literal array that will hold the stimuli
        stimuliArray: null,

        /** @lends StarLevelCreator */
            /**
             *  The StarLevelCreator reads the parsed level information and makes the star array that is fed and added to the view
             *  @constructs
             */
        ctor:function()
        {
        },

        //Create the next level to be loaded into the view
        createAndReturnStarArray:function ( levelArray, arrayOfPalettes, screenWidth, screenHeight, animationLength )
        {
            var columnCount = levelArray[ LevelKey.ColumnCount ];
            var rowCount = levelArray[ LevelKey.RowCount ];

            //figure out how many stars will actually be in the array (the fill of cols times rows)
            var totalNumberOfStars = levelArray[LevelKey.ColumnFill] * levelArray[ LevelKey.RowFill ];

            var currentNumberOfStars = 0;

            //create an empty array of the correct size
            var filled2dArray = this.createEmpty2dArray( columnCount, rowCount );

            //Create an array of all possible stars
            var possibleStars = this._createPossibleStarsArray( levelArray, arrayOfPalettes );       

            //make an array that has all of the possibilities for where a star can be
            var locationPossibilitesArray = this.createArrayOfPossibile2dLocations( filled2dArray, false );

            var possibleLocation;
            var possibleLocationIndex;
            var starIndex;
            var currentDistractor = 1;

            //now that we have that information, fill up dat star array.
            while( currentNumberOfStars <= totalNumberOfStars && locationPossibilitesArray.length > 0 )
            {
                //pick a random spot in the 2d array
                possibleLocationIndex = locationPossibilitesArray.getRandomIndex();
                possibleLocation = locationPossibilitesArray[ possibleLocationIndex ];
                //if we are looking at the unique element
                if( currentNumberOfStars === 0 )
                {
                    starIndex = 0;
                }
                //the first n * 2 stars must be carefully set where n is the number of distractors
                //that's because there MUST be at least two of every kind of star that isn't the unique one
                else if( currentNumberOfStars <= (2 * levelArray[LevelKey.DistractorVariety]) )
                {
                    //if we've made an even number of stars and are still less than the above cap, we know we can
                    //move on to the next variety of stars
                    if( currentNumberOfStars % 2 === 0 )
                    {
                        starIndex = currentDistractor;
                        currentDistractor ++;
                    }
                    //else make another of the same variety so we have at least 2
                    else
                    {   
                        starIndex = currentDistractor;
                    }
                }
                //if we have enough placed so that there is only one unique star,
                //we can just add a couple random stars!
                else
                {
                    starIndex = LumosityUtils.getRandomInt( 1, possibleStars.length-1 );
                }
                filled2dArray[possibleLocation.x][possibleLocation.y] =
                    new Star(
                        possibleStars[ starIndex ].arrayOfStarElements,
                        possibleStars[starIndex].amIUnique,
                        this._getStarLocation( columnCount, rowCount, possibleLocation.x, possibleLocation.y, screenWidth, screenHeight ),
                        animationLength,
                        starIndex,
                        screenWidth,
                        screenHeight );
                //now that we've used that location, remove it as a possibility
                locationPossibilitesArray.splice( possibleLocationIndex, 1);
                currentNumberOfStars++;
            }
            return { "filled2dArray": filled2dArray, "possibleStars": possibleStars};
        },
        //does as advertised
        createEmpty2dArray: function( cols, rows )
        {
            var colsIndex;
            var empty2dArray = new Array( cols );
            for( colsIndex = 0; colsIndex< cols; colsIndex++ )
            {
                empty2dArray[colsIndex] = new Array(rows);
            }
            return empty2dArray;
        },

        //creates an array that contains one star for each possibility we will be selecting from
        _createPossibleStarsArray: function( permLevelArray, arrayOfPalettes )
        {
            var distractorChoice;
            var starElementIndex;
            var distractorChoiceIndex;
            var numberOfPossibleStarsCreated;
            var possibleStars = [];

            //copy this array so that we don't permanenetly alter the level information
            var levelArray = permLevelArray.slice();

            var possibleStarShapes = arrayOfPalettes[ levelArray[LevelKey.Palette] ].shapes.slice( 0 );
            var possibleStarColors = arrayOfPalettes[ levelArray[LevelKey.Palette] ].colors.slice( 0 );
            var possibleSingleRotations = this.makeSingleRotationArray( levelArray[LevelKey.SingleRotation] );
            var possibleConstantRotations = this.makeConstantRotationArray( levelArray[LevelKey.ConstantRotation] );
            var possibleStarTextures = this.makeTextureArray( levelArray[LevelKey.Texture] );

            //The starKeys used inside these master arrays come from an enum in star.js
            var masterShapeArray = [
                                                StarKey.Shape,      //the key inside star.js to refer to
                                                LevelKey.Shape,     //the key inside the level file to refer to 
                                                possibleStarShapes  //the array of possibiltiies to refer to
                                ];
            var masterColorArray = [
                                                StarKey.Color,
                                                LevelKey.Color,
                                                possibleStarColors
                                    ];
            var masterSingleRotationArray = [
                                                StarKey.SingleRotation,
                                                LevelKey.SingleRotation,
                                                possibleSingleRotations
                                            ];
            var masterConstantRotationArray =   [
                                                StarKey.ConstantRotation,
                                                LevelKey.ConstantRotation,
                                                possibleConstantRotations
                                                ];
            var masterTextureArray = [
                                                StarKey.Texture,
                                                LevelKey.Texture,
                                                possibleStarTextures
                                    ];
            //just a list of all of the masterArrays
            var masterDistractorArray = [
                                                masterShapeArray,
                                                masterColorArray,
                                                masterSingleRotationArray,
                                                masterConstantRotationArray,
                                                masterTextureArray
                                        ];
            //MAKE THE UNIQUE STAR!
            possibleStars[0] = this._createUniqueStar( levelArray, masterDistractorArray, possibleStarShapes, possibleStarColors,
                                                possibleSingleRotations, possibleConstantRotations, possibleStarTextures );
            //get the arrayOfStarElements from the uniqu star for use later
            var arrayOfStarElements = possibleStars[0].arrayOfStarElements;
            //create a new possibility for the star every time we have another distractor variety.
            //Only one distractor is changed from the unique star per star distractor
            for( numberOfPossibleStarsCreated = 1; numberOfPossibleStarsCreated <= levelArray[LevelKey.DistractorVariety]; numberOfPossibleStarsCreated ++ )
            {
                //DECIDE THE DISTRACTOR TYPE
                //if we don't have enough distractor options to fill out the number of distractors, just default to shape distraction
                if( masterDistractorArray.length <= 0 )
                {
                    distractorChoiceIndex = 0;
                    distractorChoice = LevelKey.Shape;
                }
                else
                {
                    //decide on a random, viable distractor
                    distractorChoiceIndex = LumosityUtils.getRandomInt( 0, masterDistractorArray.length - 1 );
                    distractorChoice = masterDistractorArray[ distractorChoiceIndex ][ MasterKey.LevelKey ];
                    //subtract one from that distractor option
                    levelArray[ distractorChoice ] --;
                }
                //alter this star to be different from the unique one
                if( masterDistractorArray[ distractorChoiceIndex ].length <= 0 )
                {
                    arrayOfStarElements = possibleStars[ numberOfPossibleStarsCreated - 1 ].arrayOfStarElements.slice(0);
                }
                else
                {
                    //start off this star's array as a copy of the unique star!
                    arrayOfStarElements = possibleStars[0].arrayOfStarElements.slice();

                    //select a random index to change
                    starElementIndex = LumosityUtils.getRandomInt( 0, masterDistractorArray[ distractorChoiceIndex ][ MasterKey.PossibilitiesKey ].length - 1 );

                    //now change just the decided distractor from the unique star's index
                    arrayOfStarElements[ masterDistractorArray[ distractorChoiceIndex ][ MasterKey.StarKey ] ] = masterDistractorArray[ distractorChoiceIndex ][ MasterKey.PossibilitiesKey ][ starElementIndex ];
                }
                possibleStars[ numberOfPossibleStarsCreated ] = new Star( arrayOfStarElements, false, cc.p( 0, 0 ) );
                //now splice out the options we used so that we don't ever have repeat distractors
                masterDistractorArray[ distractorChoiceIndex ][ MasterKey.PossibilitiesKey ].splice( starElementIndex, 1 );
                //and check if things need to be removed as a distractor choice
                if( levelArray[ distractorChoice ] <= 1 )
                {
                    masterDistractorArray.splice( distractorChoiceIndex, 1 );
                }
            }
            return possibleStars;
        },

        //the code in the _createPossibleStarsArray function was getting too long. This is simply a function ot help cut it down
        _createUniqueStar: function( levelArray, masterDistractorArray, possibleStarShapes, possibleStarColors, possibleSingleRotations, possibleConstantRotations, possibleStarTextures )
        {
            //shape and color of unique star are ALWAYS random
            var uniqueStarShape = LumosityUtils.getRandomInt( 0, possibleStarShapes.length - 1 );
            var uniqueStarColor = LumosityUtils.getRandomInt( 0, possibleStarColors.length - 1 );
            //single rotation, constantRotation, and textures only happen when the level info lists them greater than one
            var uniqueStarSingleRotation;
            if( levelArray[LevelKey.SingleRotation] > 1 )
            {
                uniqueStarSingleRotation = LumosityUtils.getRandomInt( 0, possibleSingleRotations.length - 1 );
            }
            else
            {
                uniqueStarSingleRotation = 0;
            }
            var uniqueStarConstantRotation;
            if( levelArray[LevelKey.ConstantRotation] > 1 )
            {
                uniqueStarConstantRotation = LumosityUtils.getRandomInt( 0, possibleConstantRotations.length - 1 );
            }
            else
            {
                uniqueStarConstantRotation = 0;
            }
            var uniqueStarTexture;
            if( levelArray[LevelKey.Texture] > 1 )
            {
                uniqueStarTexture = LumosityUtils.getRandomInt( 0, possibleStarTextures.length - 1 );
            }
            else
            {
                uniqueStarTexture = 0;
            }
            //the array that holds all of the stars options
            var arrayOfStarElements = [ possibleStarShapes[ uniqueStarShape ], 
                                        possibleStarColors[ uniqueStarColor ],
                                        possibleSingleRotations[ uniqueStarSingleRotation ], 
                                        possibleConstantRotations[ uniqueStarConstantRotation ], 
                                        possibleStarTextures[ uniqueStarTexture ] ];
            var uniqueStar = new Star( arrayOfStarElements, true, cc.p( 0, 0 ) );
            //remove the element from each distractor type as an option since the unique star used it already
            possibleStarShapes.splice( uniqueStarShape,1 );
            possibleStarColors.splice( uniqueStarColor,1 );
            possibleSingleRotations.splice( uniqueStarSingleRotation, 1 );
            possibleConstantRotations.splice( uniqueStarConstantRotation, 1);
            possibleStarTextures.splice( uniqueStarTexture, 1 );
            var distractionRemoveIndex;
            //run through the current working distractors and remove them as possibilities if they are listed in the level info at 0 or 1
            for( distractionRemoveIndex = ( masterDistractorArray.length - 1 ); distractionRemoveIndex >=0; distractionRemoveIndex-- )
            {
                if( levelArray[ masterDistractorArray[ distractionRemoveIndex ][ MasterKey.LevelKey ] ] <= 1 )
                {
                    masterDistractorArray.splice( distractionRemoveIndex, 1 );
                }
            }
            return uniqueStar;
        },

        //creates an array to be used to place stars. All possible array locations are placed in an array
        //to be spliced out as possibilites are elimited. Each element in the array is a cc.p (x,y)
        //the checkingForExists parameter controls whether we are checking for objects that exist or dont exist
        createArrayOfPossibile2dLocations: function ( array2d, checkingForExists )
        {
            var cols = array2d.length;
            var rows = array2d[0].length;
            var colsIndex;
            var rowsIndex;
            var possibilitesArray = [];
            for( colsIndex = 0; colsIndex < cols; colsIndex++ )
            {
                for( rowsIndex = 0; rowsIndex < rows; rowsIndex++ )
                {
                    //there may be a more terse way of doing this if statement
                    if( checkingForExists )
                    {
                        //if we only want possibilites that exist, only put in those that exist
                        if( true )
                        {
                            possibilitesArray.push( cc.p( colsIndex, rowsIndex ) );
                        }
                    }
                    else
                    {
                        //if we don't care about whether or not they exist, just add all elements of the array
                        possibilitesArray.push( cc.p( colsIndex, rowsIndex ) );
                    }
                }
            }
            return possibilitesArray;
        },

        makeSingleRotationArray: function( numberOfPossibleRotations )
        {
            var index;
            var singleRotationArray = [];
            var rotationToAdd = Math.floor( 360 / numberOfPossibleRotations );
            //the starting rotation is always 0!
            singleRotationArray[0] = 0;

            for( index = 1; index < numberOfPossibleRotations; index++ )
            {
                singleRotationArray[index] = singleRotationArray[index-1] + rotationToAdd;
            }
            return singleRotationArray;
        },

        makeConstantRotationArray: function( numberOfPossibleRotations )
        {
            var index;
            var constantRotationArray = [];
            var startingRotation = (-1 * Math.floor( numberOfPossibleRotations / 2 ));
            constantRotationArray[0] = startingRotation;
            for( index = 1; index < numberOfPossibleRotations; index++ )
            {
                constantRotationArray[index] = constantRotationArray[index-1] + 1;
            }
            return constantRotationArray;
        },

        makeTextureArray: function( numberOfPossibleTextures )
        {
            //method makes a number of possible textures
            //It will pick a random n options from 0, 1, 2, 3
            var index;
            var possibleTexturesStarting = [ 0,1,2,3 ];
            var randomIndex;
            var textureArray = [];

            //we want to be sure that if there are no textures we never have a default texture
            if( numberOfPossibleTextures <= 1 )
            {
                textureArray = [0];
            }
            else
            {
                for ( index = 0; index < numberOfPossibleTextures; index++ )
                {
                    randomIndex = LumosityUtils.getRandomInt(0, possibleTexturesStarting.length - 1 );

                    textureArray.push( possibleTexturesStarting[randomIndex] );

                    possibleTexturesStarting.splice( randomIndex, 1 );
                }
            }

            return textureArray;
        },

        //figures out the x and y for the star
        _getStarLocation: function( cols, rows, colsIndex, rowsIndex, widthOfScreen, heightOfScreen )
        {
            //Full equations
            // X = ((width/cols) * index) + rand
            // Y = ((height/rows) * index) + ((height - width) / 2) + rand

            //two variables to hold the X and Y
            var tempX;
            var tempY;

            //+1 for spacing
            colsIndex ++;
            rowsIndex ++;

            //spacing of the stars in the grid
            var spacePerItemX = ( widthOfScreen / ( cols +.5) );
            var spacePerItemY = ( heightOfScreen / ( rows + 1 ) );

            //random jitter added to add to the location and break up the grid somewhat
            var horizSpacing = (spacePerItemX/8) - (spacePerItemX/16);
            var vertSpacing = (spacePerItemY/4) - (spacePerItemY/16);

            var jitterX = ( LumosityUtils.getRandomInt( -horizSpacing, horizSpacing ) );
            var jitterY = ( LumosityUtils.getRandomInt( -vertSpacing, vertSpacing ) );

            tempX = ( spacePerItemX * colsIndex ) - (spacePerItemX / (cols + 1));
            tempY = ( spacePerItemY * rowsIndex ) + jitterY;

            return cc.p( tempX, tempY );
        }

    } );
}() );
