/*global cc*/
/*global require*/
/*global GameModel*/
/*global LevelInfoTextParser*/
/*global LumosityUtils*/
/*global lumosity*/
/*global StarLevelCreator*/

var StarSearchModel;
( function ()
{
    "use strict";

    require( "src/Game/Base/GameModel.js" );
    require( "src/Game/LevelInfoTextParser.js" );
    require( "src/Game/StarLevelCreator.js" );
    require( "src/Utility/LumosityUtils.js" );

    StarSearchModel = GameModel.extend(
    {
    	//The array that permanently stores ALL of the level information. All 200!
    	arrayOfLevels: [],
        level_select_buttons: [],

        //game timer. Used for getting exact time spent
        gameTimer: 0,
        trialTimer: 0,
        response_time: 0,
        time_offset: null,
        gameState: null,
        hintGiven: false,
        success: null,
        star: null,
        delayStarsAnimation: null,
        animating: null,

        //metadata
        previousGameStatKey: "unlockedLevel",
        bestStatKey: "maxLevel",
        num_correct: null,
        num_correct_fast: null,
        num_correct_slow: null,
        num_tries: null,
        current_level: null,
        unlockedLevel: null,
        selected_level: null,
        maxLevel: 1,
        width: null,
        height: null,
        starLevelInfo: null,
        distractor_colors: [],
        distractor_pallettes: [],
        distractor_shapes: [],
        distractor_motions: [],
        distractor_rotations: [],
        distractor_textures: [],
        distractor_colors_string: "",
        distractor_pallettes_string: "",
        distractor_shapes_string: "",
        distractor_motions_string: "",
        distractor_rotations_string: "",
        distractor_textures_string: "",
        distractorDelayIndex: 0,
        distractorDiagramKey:
        {
            0: "A",
            1: "!",
            2: "@",
            3: "#",
            4: "$",
            5: "%",
            6: "^",
            7: "&",
            8: "*",
            9: "("
        },
        diagram: "no-diagram",

        ctor:function()
        {
        	this._super();
        	this.parser = new LevelInfoTextParser();
            this.gameState = "setup";
            this.animating = false;
            this.starLevelCreator = new StarLevelCreator();
        	this.arrayOfLevels = this.createLevelArray();
            this.current_level = 0;
            this.num_correct = 0;
            this.num_correct_fast = 0;
            this.num_correct_slow = 0;
            this.num_tries = 0;
        },

        //@Override
        exportResults:function()
        {
            //stats for next playthrough
            var stats = {
                maxLevel: this.maxLevel,
                unlockedLevel: this.unlockedLevel
            };

            //game dependent meta
            this.metadata.max_level = this.maxLevel;
            this.metadata.num_correct = this.num_correct;
            this.metadata.num_correct_fast = this.num_correct_fast;
            this.metadata.num_correct_slow = this.num_correct_slow;
            this.metadata.num_tries = this.num_tries;
//            this.metadata.score = this.score;
            this.metadata.selected_level = this.selected_level;

            //common meta
            this.metadata.session_level = this.selected_level;
            this.metadata.user_level = this.current_level;
            this.metadata.width = this.width;
            this.metadata.height = this.height;

            return this._super( stats, this.bestStatKey );
        },
        
        resetTrialData: function ()
        {
            this.distractor_colors =  [];
            this.distractor_pallettes =  [];
            this.distractor_shapes =  [];
            this.distractor_motions =  [];
            this.distractor_rotations =  [];
            this.distractor_textures =  [];

            this.distractor_colors_string = "";
            this.distractor_pallettes_string = "";
            this.distractor_shapes_string = "";
            this.distractor_motions_string = "";
            this.distractor_rotations_string = "";
            this.distractor_textures_string = "";
        },

        /**
         * Set the current level based on the level select button pressed
         * @param tag - the tag of the button selected
         */
        setStage: function ( tag )
        {
            this.selectedStage = tag;
            this.current_level = this.selectedStage;
            this.maxLevel = this.selectedStage;
            if( this.current_level == 0 )
            {
                this.current_level = 1;
            }
        },

        createLevelArray: function()
        {
            var textFile = lumosity.io_read( "textInfo/star_search_levels.txt" );
            var levelsArray = this.parser.parseTxtFileForLevelInfo( textFile );

            return levelsArray;
        },

        /**
         * @param   hintDelay   Milliseconds until first hint at level 10 or less.
         *                      add half a second to delay timer for every level you're at above 10.
         *                      Also equal to bounceTimer:  delay between unique star bounce animations (for hint 2)
         */
        setHintDelay: function(hintDelay)
        {
            this.bounceTimer = hintDelay;
            if ( this.current_level > 10 )
            {
                this.hintDelay = 500 * (this.current_level - 10) + hintDelay;
            }
            else
            {
                this.hintDelay = hintDelay; 
            }
        },

        /**
         * Give the player hints if they have been stuck on this trial long enough.
         * @return  if starting any hint this instant.  May only start during play.
         *
         * GAME-2406 Set hint delay each trial.  Extract hint rules to model and complete a decision before giving any hint.
         * Test case: 2015-01-07 Level 20.  Look at unique object.  Wait 11 seconds.  [~aaron] expects one variety of distractors exit.  
         * Wait 5.5 seconds.  Expect unique object bounces.  Wait 6 seconds.  Expect unique object bounces again.
         * Tap unique object.  Expect level 19.  Wait 10.5 seconds.  Expect one variety of distractors exit.
         * Wait 5.25 seconds.  Expect unique object bounces.  Wait 6 seconds.  Expect unique object bounces again.
         */
        checkHintTimer: function()
        {
            var isHintingNow = false;
            if ( this.isGameRunning && this.gameState == "game" )
            {
                var hintTime = this.hintDelay / 1000;
                if( this.trialTimer >= hintTime )
                {
                    this.hintGiven = true;
                    this.distractorDelayIndex++;
                    var level_details = this.arrayOfLevels[this.current_level];
                    var variety = level_details[ LevelKey.DistractorVariety ];
                    if ( this.distractorDelayIndex < variety )
                    {
                        this.hintDelay += this.hintDelay / Math.pow( 2, this.distractorDelayIndex );
                    }
                    else
                    {
                        this.hintDelay += this.bounceTimer;
                    }
                    isHintingNow = true;
                }
            }
            return isHintingNow;
        },

        /**
         * GAME-2406 When paused, do not accumulate game timers.  Test case:  2015-01-08 Level 20.  Observe 5 seconds.  Pause 15 seconds.  Unpause.  Observe for 5 seconds.  [~aaron] expects to not see a hint yet.
         */
        onUpdate: function(delta) 
        {
            if (!this.isGamePaused) 
            {
                this.gameTimer += delta;
                this.trialTimer += delta;
                this.isHintingNow = this.checkHintTimer(this.hintDelay);
            }
        },

        /**
         * Adjust score and level based on trial
         * GAME-2368 num_correct is incremented for each correct response, regardless of response time.
         * GAME-2404 As in Flash version, only score points if answer before first hint.  Test case:  2015-01-07 Select level 20 or lower.  Observe any hint.  Answer.  [~aaron] expects no points.  Got points.
         * If response is slow, also adjust level.
         */
        factorResults: function( success, settings )
        {
            this.num_tries++;
            this.response_time = this.trialTimer;
            var pointsEarned = 0;
            pointsEarned = this.getPointsEarned(success, settings);
            this.score += pointsEarned;
            if (success)
            {
                this.num_correct++;
            }
            if (!this.hintGiven)
            {
                if (success)
                {
                    this.num_correct_fast++;
                }
            }
            else
            {
                if (success)
                {
                    this.num_correct_slow++;
                }
            }
            //calculate level
            this.adjustLevel( success, settings.timeToProgress );
            return pointsEarned;
        },

        /**
         * calculate points based on performance
         * GAME-2368 points = (level ^ 2) * 0.25;  
         * Round up to nearest 10.
         * Example: Level 200: 10,000 points.
         * The score is added if the response is before the first hint.
         * Test case:  2014-12-22 [~bahroni] expects score around Level 180:  8100 and Level 200:  10000.  Got 150,000 points.
         */
        getPointsEarned: function( success, settings )
        {
            if ( success && !this.hintGiven )
            {
                var scoreToAdd = Math.max(10, Math.pow(this.current_level, settings.scoringLevelPower) 
                    * settings.scoringScale);
                scoreToAdd = Math.ceil(scoreToAdd / 10) * 10;
                return scoreToAdd;
            }
            return 0;
        },

        //adjust level based on performance
        adjustLevel: function( success, timeToProgress )
        {
            if ( success && this.trialTimer <= timeToProgress / 1000 )
            {
                //go up a level
                if ( this.current_level < this.arrayOfLevels.length - 1 )
                {
                    this.current_level++;
                    //change max level if this is users highest level in game
                    if( this.current_level > this.maxLevel )
                    {
                        this.maxLevel = this.current_level;
                    }
                    //change the unlocked this is the users highest lifetime level!
                    if (this.current_level > this.unlockedLevel )
                    {
                        this.unlockedLevel = this.current_level;
                    }
                }
            }
            else
            {
                //go down a level
                if ( this.current_level > 1 )
                {
                    this.current_level--;
                }
            }
        },

        //rotate star grid because it comes in sideways for some reason (so silly)
        rotateCounterClockwise: function ( )
        {
            var n = this.starLevelInfo.filled2dArray.length;
            for (var i = 0; i < n/2; i++) 
            {
                for (var j = i; j < n - i - 1; j++)
                {
                    var tmp = this.starLevelInfo.filled2dArray[i][j];
                    this.starLevelInfo.filled2dArray[i][j]=this.starLevelInfo.filled2dArray[j][n-i-1];
                    this.starLevelInfo.filled2dArray[j][n-i-1]=this.starLevelInfo.filled2dArray[n-i-1][n-j-1];
                    this.starLevelInfo.filled2dArray[n-i-1][n-j-1]=this.starLevelInfo.filled2dArray[n-j-1][i];
                    this.starLevelInfo.filled2dArray[n-j-1][i] = tmp;
                }
            }
        },

        //metadata - create a single string describing star grid
        createTrialDiagram: function()
        {
            this.rotateCounterClockwise();

            var diagram = "";
            for(var i = 0; i < 5; i++)
            {
                if ( this.starLevelInfo.filled2dArray[i] != undefined)
                {
                    for(var j = 0; j < 5; j++)
                    {
                        //empty space
                        if ( this.starLevelInfo.filled2dArray[i][j] == undefined )
                        {
                            diagram += ".";
                        }
                        else
                        {
                            //unique star
                            if ( this.starLevelInfo.filled2dArray[i][j].amIUnique )
                            {
                                //selected correct star
                                if ( this.starLevelInfo.filled2dArray[i][j].selected )
                                {
                                    diagram += "A";
                                }
                                else
                                {
                                    diagram += "a";
                                }
                            }
                            else
                            {
                                //selected incorrect star
                                if (this.starLevelInfo.filled2dArray[i][j].selected )
                                {
                                    diagram += ( this.distractorDiagramKey[this.starLevelInfo.filled2dArray[i][j].starIndex]) ;
                                }
                                else
                                {
                                    diagram += ( this.starLevelInfo.filled2dArray[i][j].starIndex ).toString();
                                }

                            }
                        }
                    }
                }
                else
                {
                    for( var k = 0; k < 5; k++)
                    {
                        diagram += ".";
                    }
                }
                if(i != 4)
                {
                    diagram += ";";

                }
            }
            diagram = diagram.split("").join("");
            return diagram;
        }

    } );
}() );
