/*global cc*/
/*global require*/
/*global GameView*/
/*global GameWidgetFeedback*/
/*global LayoutLoader*/
/*global LumosityButton*/
/*global localize*/
/*global Star*/
/*global LumosityUtils*/
/*global GameWidgetHud*/
/*global GameWidgetFlipbook*/
/*global Random*/
/*global StageSelect*/
/*global Nebula*/

var StarSearchView;
( function()
{

    "use strict";

    require( "src/Game/Base/GameView.js" );
    require( "src/Game/Widget/GameWidgetFeedback.js" );
    require( "src/Game/Widget/GameWidgetFlipbook.js" );
    require( "src/Animation/LayoutLoader.js" );
    require( "src/View/LumosityButton.js" );
    require( "src/Game/Star.js" );
    require( "src/Game/StageSelect.js" );
    require( "src/Game/Nebula.js" );
    require( "src/Utility/LumosityUtils.js" );
    require( "src/Utility/Random.js" );


    StarSearchView = GameView.extend(
    {
        screen_width: null,
        screen_height: null,
        //array of star information
        starArray: null,
        //array of stars that will be removed when giving user hint #1
        hintStarArray: null,
        //the unique star
        uniqueStar: null,
        remainingStars: null,

        //the level select planet that is currently active (highest record level)
        activePlanet: null,

        //array of level select buttons
        levelSelectButtons: [],

        //array of level select connecting lines
        levelSelectLines: [],

        //feedback sprites for flipbooks
        correctSprite: null,
        incorrectSprite: null,
        indicateSprite: null,


        ctor:function( screenSize, settings, model, interactionResponder, preload )
        {
            this._super( screenSize, settings, model, interactionResponder, preload );
            this.settings = settings;
            this.starArray = [];
            this.screen_width = screenSize.width;
            this.screen_height = screenSize.height;
            this.hud.setScore( 0 );

            this._preloadStarImages();
        },

        disableLevelSelectButtons: function ()
        {
            for ( var i = 0; i < this.levelSelectButtons.length; i++ )
            {
                this.removeButton( this.levelSelectButtons[i] );
            }
        },

        enableLevelSelectButtons: function ()
        {
            for ( var i = 0; i < this.levelSelectButtons.length; i++ )
            {
                this.addButton( this.levelSelectButtons[i] );
            }
        },

        giveHint: function ( )
        {
            if ( !this.hintStarArray)
            {
                return;
            }
            
            if (this.hintStarArray.length > 0)
            {
                this.removeHintStars();
                return 1;
            }
            else
            {
                this.bounceStar(0);
                return 2;
            }

        },

        //removes one type of distractor (as a hint)
        removeHintStars: function ( )
        {
            var lowestIndex = this.hintStarArray[this.hintStarArray.length-1].starIndex;
            //go through last list in hintStarArray and animate all the stars out with shared lowest starIndex
            var arrayIndex = this.findFirstOccurrence( lowestIndex );
            while ( this.hintStarArray.length > 0 && arrayIndex != -1)
            {
                var removedStar = this.hintStarArray.splice(arrayIndex, 1)[0];
                if (removedStar)     //added because of Invalid Native object bug in release 2/6/2015
                {
                    arrayIndex = this.findFirstOccurrence( lowestIndex );
                    removedStar.hidden = true;
                    removedStar.outroAnimation( 0, 1, this.destroyStars.bind( this, removedStar ), this );
                }
            }
        },

        //makes the unique star dance like nobody's watching
        bounceStar: function ()
        {
            this.uniqueStar.bounceAnimation();
        },

        //@Override
        preload:function()
        {
            this._super();

            this.flipbooks.incorrect = new GameWidgetFlipbook( "incorrect", "feedback/feedbackIncorrect{0}.png", 12 );
            this.flipbooks.indicate = new GameWidgetFlipbook( "indicate", "feedback/feedbackIndicate{0}.png", 12 );
            this.flipbooks.correct = new GameWidgetFlipbook( "correct", "feedback/feedbackCorrect{0}.png", 14 );
        },

        onRestart: function()
        {
            this._super();
            this.hud.setScore( 0 );
            if (this.remainingStars.length > 0)
            {
                for(var i = 0; i < this.remainingStars.length; i++)
                {
                    this.removeButton(this.remainingStars[i]);
                    this.centeredContent.removeChild(this.remainingStars[i]);
                }
            }
            this.starArray = [];
            this.activePlanet = null;
            this.removeLines();
            //set up level select buttons
            this.enableLevelSelectButtons();
        },

        //@Override
        _createViews:function()
        {
           this._super(); 
        },

        //fills the array with the specified cols and rows and adds the stimuli, then adds them to the view
        fillStarArray:function ( levelArray, buttonFunction, buttonTarget )
        {
            var colsIndex;
            var rowsIndex;
            var cols;
            var rows;

            //set the current levelArray to the one that was passed in
            this.starArray = levelArray;

            this.hintStarArray = [];

            //the caps to the for loop
            cols = levelArray.length;
            if( levelArray[0] )
            {
                rows = levelArray[0].length;
            }
            else
            {
                return;
            }

            //check through every element in the level array
            for( colsIndex = 0; colsIndex < cols; colsIndex++ )
            {
                for( rowsIndex = 0; rowsIndex < rows; rowsIndex++ )
                {
                    var star = this.starArray[colsIndex][rowsIndex];
                    //only add the star if that space in the array is filled
                    if( star )
                    {
                        //if this is the unique star, the player will get it right
                        if( star.amIUnique )
                        {
                            this.uniqueStar = star;
                            star.setReleasedCallback( buttonFunction.bind( buttonTarget, true ) );
                        }
                        //if it is non unique, the player will get it wrong
                        else
                        {
                            star.setReleasedCallback( buttonFunction.bind( buttonTarget, false ) );
                        }
                        if( star.starIndex > 1 )
                        {
                            this.hintStarArray.push( star );
                        }
                        this.centeredContent.addChild( star );
                        star.introAnimation( this.enableStar.bind(this, star), this);
                    }
                }
            }
        },

        enableStar: function( star )
        {
            this.addButton( star );
        },

        //helper for removing distractor stars for hints
        findFirstOccurrence: function ( lowestIndex )
        {
            for( var i = 0; i < this.hintStarArray.length; i++)
            {
                if( this.hintStarArray[i].starIndex == lowestIndex)
                {
                    return i;
                }
            }
            return -1;
        },

        disableStars:function ( )
        {
            for ( var i = 0; i < this.remainingStars.length; i++ )
            {
                if ( this.remainingStars[i] )
                    {
                        this.removeButton( this.remainingStars[i] );
                        this.remainingStars[i].setEnabled( false );
                    }
            }
        },

        //@Override
        layout_readingData: function( nodeName, nodeData )
        {
            var node = this._super( nodeName, nodeData );
            if ( node )
            {
                return node;
            }
        },
        
        //@Override
        layout_creatingNode: function( nodeName, nodeData )
        {
            var result;
            var contentSize = cc.SizeZero();
            var stageSelect = null;

            switch( nodeName )
            {
                case "feedback":
                    result = this.getFeedbackWidget();
                    break;
                case "hudBackgroundColor":
                    this.hud.setBackgroundColor( nodeData.textData.color );
                    result = false;
                    break;
                case "hudTextColor":
                    this.hud.setTextColor( nodeData.textData.color );
                    result = false;
                    break;
                case "button1":
                case "button2":
                case "button3":
                case "button4":
                case "button5":
                case "button6":
                case "button7":
                case "button8":
                case "button9":
                case "button10":
                case "button11":
                    contentSize = nodeData[ LayoutLoader.sizeKey ];
                    //create a stage select node
                    stageSelect = new StageSelect( this.interactionResponder, this, contentSize );
                    stageSelect.buttonLayout.setPosition( cc.p (nodeData.size.width/2, nodeData.size.height/2));
                    stageSelect.button.setTag(this.levelSelectButtons.length);
                    //add it to the level select button list and the view
                    this.levelSelectButtons.push( stageSelect.button );
                    this.addButton( stageSelect.button );
                    //lock all the levels @ first
                    stageSelect.playLockedAnimation();
                    var tintAction = cc.TintTo.create( 0.0, this.settings.lockedColor.r, this.settings.lockedColor.g, this.settings.lockedColor.b );
                    stageSelect.circle.runAction( tintAction );
                    stageSelect.circle.setOpacity( this.settings.lockedColor.a );
                    result = stageSelect;
                    break;
                case "nebulaParentNode":
                    contentSize = nodeData [ LayoutLoader.sizeKey ];
                    var nebula = new Nebula( this.interactionResponder, this, contentSize );
                    result = nebula;
                    nebula.setPosition( cc.p (contentSize.width/2, contentSize.height/2));
                    break;
                default:
                    result = this._super( nodeName, nodeData );
                    break;
            }

            return result;

        },

        //Taking away teh super call, this should allow a level select before gameplay
        cue_startCountdown: function()
        {
            this.playAnimation( "levelLoaderIn" );
        },

        updateHudScore: function()
        {
            if ( this._widgets )
            {
                var hudWidget = this._widgets[ GameWidgetHud.prototype.getType() ];

                if ( hudWidget )
                {
                    hudWidget.setScore( this.model.score );
                    hudWidget.setScoreHighlighted( true );
                }
            }
        },

        //controls the feedback shows over stars when you select a star
        showFeedbackOverStar: function ( success, star, gameState )
        {
            var starPositionInWorldSpace;
            var feedbackSize = cc.size(star.getContentSize().width*1.5,star.getContentSize().height*1.5);

            if ( !success )
            {
                //setup incorrect feedback invisibly
                starPositionInWorldSpace = star.getParent().convertToWorldSpace( cc.p ( star.permPosition.x, star.permPosition.y));
                var convertednodepos = this.centeredContent.convertToNodeSpace( starPositionInWorldSpace );
                this.incorrectSprite = new LumositySprite( "feedback/feedbackIncorrect000.png");
                this.incorrectSprite.setAnchorPoint( cc.p(.5,.5));
                this.incorrectSprite.setPosition( convertednodepos );
                this.incorrectSprite.setZOrder( 100 );
                this.incorrectSprite.setOpacity(0);
                LumosityUtils.scaleForContentSize(this.incorrectSprite, feedbackSize, false);

                this.centeredContent.addChild(this.incorrectSprite);

                //setup indicate feedback invisibly
                starPositionInWorldSpace = this.uniqueStar.getParent().convertToWorldSpace( cc.p (this.uniqueStar.permPosition.x, this.uniqueStar.permPosition.y));
                var convertednodepos = this.centeredContent.convertToNodeSpace( starPositionInWorldSpace );
                this.indicateSprite = new LumositySprite( "feedback/feedbackIndicate000.png");
                this.indicateSprite.setAnchorPoint( cc.p(.5,.5));
                this.indicateSprite.setPosition( convertednodepos );
                this.indicateSprite.setZOrder( 100 );
                this.indicateSprite.setOpacity(0);
                LumosityUtils.scaleForContentSize(this.indicateSprite, feedbackSize, false);
                this.centeredContent.addChild(this.indicateSprite);

                //play sequence
                var showIncorrectFeedback = cc.CallFunc.create( this.showIncorrectFeedback, this );
                var delay = cc.DelayTime.create(.5);
                var removeFeedbackIncorrect = cc.CallFunc.create( this.removeFeedbackIncorrect, this );
                var showIndicateFeedback = cc.CallFunc.create( this.showIndicateFeedback.bind(this, gameState), this );
                var removeFeedbackIndicate = cc.CallFunc.create( this.removeFeedbackIndicate, this );
                this.runAction( cc.Sequence.create( showIncorrectFeedback, delay, removeFeedbackIncorrect, showIndicateFeedback, delay, removeFeedbackIndicate) );
            }
            else
            {
                //get clicked star position
                starPositionInWorldSpace = this.uniqueStar.getParent().convertToWorldSpace( cc.p (this.uniqueStar.permPosition.x, this.uniqueStar.permPosition.y));
                var convertednodepos = this.centeredContent.convertToNodeSpace( starPositionInWorldSpace );
                this.correctSprite = new LumositySprite( "feedback/feedbackCorrect000.png" );
                this.correctSprite.setAnchorPoint( cc.p( .5, .5 ) );
                this.correctSprite.setPosition( convertednodepos );
                this.correctSprite.setZOrder( 100 );
                LumosityUtils.scaleForContentSize(this.correctSprite, feedbackSize, false);
                this.centeredContent.addChild( this.correctSprite );
                this.flipbooks.correct.play( this.correctSprite, false );
                var showCorrectFeedback = cc.CallFunc.create( this.showCorrectFeedback, this );
                var delay = cc.DelayTime.create(.5);
                var removeFeedbackCorrect = cc.CallFunc.create( this.removeFeedbackCorrect, this );

                this.runAction( cc.Sequence.create( showCorrectFeedback, delay, removeFeedbackCorrect) );
            }

        },

        showIncorrectFeedback: function()
        {
            this.incorrectSprite.setOpacity(255);
            this.flipbooks.incorrect.play( this.incorrectSprite , false );

        },

        showIndicateFeedback: function( gameState )
        {
            if ( gameState == "game")
            {
                this.indicateSprite.setOpacity( 255 );
                this.flipbooks.indicate.play( this.indicateSprite, false );
            }
        },
        showCorrectFeedback: function()
        {
            this.flipbooks.correct.play( this.correctSprite , false );
        },
        removeFeedbackIncorrect: function()
        {
            if(this.incorrectSprite)
            {
                this.centeredContent.removeChild( this.incorrectSprite );
                this.incorrectSprite = null;
            }
        },
        removeFeedbackIndicate: function()
        {
            if(this.indicateSprite)
            {
                this.centeredContent.removeChild( this.indicateSprite );
                this.indicateSprite = null;
            }
        },
        removeFeedbackCorrect: function()
        {
            if(this.correctSprite)
            {
                this.centeredContent.removeChild( this.correctSprite );
                this.correctSprite = null;
            }
        },

        //animate stars out
        animateStarsOut: function( numberOfStars, animationLength, doneFunction, doneTarget )
        {
            //half the length of time you have to wait to make the intro animation
            //because this not only animates out, but waits for other stars to do the same
            animationLength /= 2;
            var timeToWait = 0;
            numberOfStars++;
            var timePerStar = animationLength / numberOfStars;

            for ( var i = 0; i < this.remainingStars.length; i++ )
            {
                timeToWait += timePerStar;
                if ( this.remainingStars[i] ) //added because of Invalid Native object bug in release 2/6/2015
                {
                    if ( i == this.remainingStars.length - 1 )
                    {
                        this.remainingStars[i].lastOutroAnimation( timeToWait, doneFunction, doneTarget, animationLength, this.destroyStars.bind( this, this.remainingStars[i] ), this );

                    }
                    else
                    {
                        this.remainingStars[i].outroAnimation( timeToWait, animationLength, this.destroyStars.bind( this, this.remainingStars[i] ), this );
                    }
                }

            }
            //returns the wait on the longest star so that we can wait in the main class before making a new level
            timeToWait += animationLength;
            return timeToWait;
        },

        destroyStars: function ( star )
        {
            this.centeredContent.removeChild( star );
        },

        //creates and stores connector lines between level select planets
        drawLines: function ( unlockedLevel )
        {
            var width = 6;
            var percentage;
            for ( var i = 0; i < this.levelSelectButtons.length-1; i++ )
            {
                //find the radius of the circle
                var startRadius = this.settings.starRadius;
                var endRadius = this.settings.starRadius;

                //hacky way of altering endpoints for active & locked planets since we can't get their radius from flash file
                if ( this.levelSelectButtons[ i + 1 ].getTag() === this.activePlanet.button.getTag() )
                {
                    endRadius *= 1.15;
                }
                if ( this.levelSelectButtons[ i + 1 ].getTag() > this.activePlanet.button.getTag() )
                {
                    width = 2;
                }
                else if( this.levelSelectButtons[i].getTag() === this.activePlanet.button.getTag() )
                {
                    startRadius *= 1.15;
                }
                if ( this.levelSelectButtons[i].getTag() > this.activePlanet.button.getTag() )
                {
                    startRadius *= .5;
                }
                if ( this.levelSelectButtons[ i + 1 ].getTag() > this.activePlanet.button.getTag() )
                {
                    endRadius *= .5;
                }

                //find the angle between the 2 planets
                var Ax = this.levelSelectButtons[i].getParent().getPosition().x;
                var Ay = this.levelSelectButtons[i].getParent().getPosition().y;
                var Bx = this.levelSelectButtons[i+1].getParent().getPosition().x;
                var By = this.levelSelectButtons[i+1].getParent().getPosition().y;
                var theta = Math.atan2( (By - Ay),(Bx - Ax) ) ;

                //find the endpoints for the line between planets
                var startX = Ax + startRadius * Math.cos( theta );
                var startY = Ay + startRadius * Math.sin( theta );
                var endX = Bx + endRadius * Math.cos( theta - Math.PI );
                var endY = By + endRadius * Math.sin( theta - Math.PI );
                var start = cc.p ( startX, startY );

                //find the distance between the points (width of segment)
                var dist = Math.sqrt( Math.pow((endX - startX),2) + Math.pow((endY - startY), 2));

                //draw a sprite
                var line = LumosityUtils.createRectangularSprite( cc.size ( dist , width ) , cc.c4b( this.settings.lockedColor.r, this.settings.lockedColor.g, this.settings.lockedColor.b, 0 ) );
                line.setAnchorPoint( cc.p (0,.5));
                line.setPosition(start);
                line.setRotation(-theta * 180/Math.PI);
                line.setOpacity(0);

                this.menu.addChild(line);
                this.levelSelectLines.push( line );
                //if we're between highest planets -- find percentage between planets
                if ( unlockedLevel > this.levelSelectButtons[i].getTag() * 20  && unlockedLevel < this.levelSelectButtons[i+1].getTag() * 20)
                {

                    percentage = unlockedLevel - (this.levelSelectButtons[i].getTag() * 20);
                    percentage = percentage / ((this.levelSelectButtons[i+1].getTag() * 20) - (this.levelSelectButtons[i].getTag() * 20));
                    var newEndX = startX + ((endX - startX) * percentage);
                    var newEndY = startY + ((endY - startY) * percentage);
                    var newEnd = cc.p ( newEndX, newEndY );
                    //find the distance between the points (width of segment)
                    dist = Math.sqrt( Math.pow((newEndX - startX),2) + Math.pow((newEndY - startY), 2));
                    //draw a sprite
                    line = LumosityUtils.createRectangularSprite( cc.size ( dist , 6 ) , cc.c4b( this.settings.lockedColor.r, this.settings.lockedColor.g, this.settings.lockedColor.b, 0 ) );
                    line.setAnchorPoint( cc.p (0,.5));
                    line.setPosition(start);
                    line.setRotation(-theta * 180/Math.PI);
                    line.setOpacity(0);

                    this.menu.addChild(line);
                    this.levelSelectLines.push( line );
                }
            }
            this.fadeInLines( unlockedLevel );
        },

        //shows level select connecting lines
        fadeInLines: function ( unlockedLevel)
        {
            for (var i = 0; i < this.levelSelectLines.length; i++)
            {
                var fadeIn = cc.FadeTo.create( .5 , this.settings.lockedColor.a * 255 );
                this.levelSelectLines[i].runAction( cc.Sequence.create( fadeIn ) );
            }
            for( var i = 0; i < this.levelSelectButtons.length; i++ )
            {
                //if the level button is lower than the user's highest record, enable it
                var levelNum = i * 20;
                if ( levelNum <= unlockedLevel )
                {
                    this.levelSelectButtons[i].setEnabled( true );
                }
            }
        },

        //removes level select connecting lines
        fadeOutLines: function ()
        {
            for (var i = 0; i < this.levelSelectLines.length; i++)
            {
                var fadeIn = cc.FadeTo.create( .5 , 0 );
                this.levelSelectLines[i].runAction( cc.Sequence.create( fadeIn ) );
            }
        },

        removeLines: function ()
        {
            for (var i = 0; i < this.levelSelectLines.length; i++)
            {
                this.levelSelectLines[i].setOpacity(0);
            }
        },

        //plays the shooting star animation in the background
        playStarShow: function( timer )
        {
            var time = Math.floor( timer * 100 );
            if ( time % 3 == 0 )
            {
                //add dot to scene
                var sprite = new LumositySprite( "dot.png" );
                sprite.setAnchorPoint( cc.p( 0.5, 0.5 ) );
                var tintAction = cc.TintTo.create( 0.0, this.settings.starEmitterColor.r, this.settings.starEmitterColor.g, this.settings.starEmitterColor.b );
                sprite.runAction( tintAction );

                var width = this.centeredContent.getContentSize().width;
                var height = this.centeredContent.getContentSize().height;

                sprite.setPosition( cc.p( width / 2, height / 2 ) );
                sprite.setVisible( true );
                sprite.setOpacity( 0 );
                this.centeredContent.addChild( sprite );

                //randomize start size
                var size = 10; //LumosityUtils.getRandomInt( 5, 20 );
                sprite.setContentSize( cc.size( size, size ), true, true );
                //set speed based on size
                var speed = Math.random() * (0.2 - 0.1) + 0.1; //LumosityUtils.getRandomInt (.001,.5);
                var end = Random.floatInclusive( 0, 2 );
                var endX = null;
                var endY = null;

                //star disappears on width
                if ( end <= 1 )
                {
                    if ( end <= .5 )
                    {
                        endX = -width;
                    }
                    else
                    {
                        endX = width;
                    }
                    endY = LumosityUtils.getRandomInt( -height, height );
                }
                //star disappears on height
                else
                {
                    if ( end <= 2 )
                    {
                        endY = -height;
                    }
                    else
                    {
                        endY = height;
                    }
                    endX = LumosityUtils.getRandomInt( -width, width );
                }

                //give dot a trajectory
                var endPoint = cc.p( endX, endY );
                var duration = 1 / speed;
                var endScale = .15 * ( speed / .2 ); //LumosityUtils.getRandomInt(.5,.5 );;

                var delay = cc.DelayTime.create( duration / 3.5 );
                var movement = cc.MoveTo.create( duration, endPoint );
                var fadeIn = cc.FadeTo.create( duration / 2,  this.settings.starEmitterColor.a * 255 * 2 *  speed / .2 );
                var scale = cc.ScaleTo.create( duration / 2, endScale );
                var destroy = cc.CallFunc.create( this.destroySprite.bind(sprite), this );

                sprite.runAction( cc.Sequence.create( movement, destroy ) );
                sprite.runAction( cc.Sequence.create( delay, scale ) );
                sprite.runAction( cc.Sequence.create( delay, fadeIn ) );
            }
        },

        destroySprite: function ( item )
        {
            this.removeChild( item );
        },
        
        _preloadStarImages: function()
        {
            if ( this.settings.shouldPreloadAllShapeImages )
            {
                var shapeImageIndex;
                for ( shapeImageIndex = 0; shapeImageIndex < this.settings.allShapeImages.length; shapeImageIndex++ )
                {
                    cc.TextureCache.getInstance().addImage( "finalShapes/" + this.settings.allShapeImages[ shapeImageIndex ] );
                }
            }
        }
    } );
}() );
