/*jslint node: true */
"use strict";

require("src/Game/BlenderController.js");
require("src/Game/BlenderView.js");
require("src/Game/CafeModel.js");
require("src/Game/CafeView.js");


/**
 * MOBGAMES-1837 #comment [~wouter], in each layout, coffee machine instance names are numbered 0 to n-1, ordered by left-to-right, top-to-bottom.  Also, instructions of developer controls button mention the developer controls button moved to top right.  2014-11-05 Last cup tilted forward.  Emma expects to predict which cup is selected.  Felt weird.  (2014-11-11 +Pranjal)
 */
var CafeController = cc.Class.extend(
{
    blenderController: undefined,
    blenderView: undefined,
    isPopulatedNow: undefined,
    isUpdating: undefined,
    model: undefined,
    onApp: undefined,
    onAppear: undefined,
    view: undefined,

    ctor: function()
    {
    },

    onBegin: function()
    {
        this.view.sounds.stopAll();
        this.isUpdating = false;
        this.isPopulatedNow = false;
    },

    /**
     * TODO Deprecate isOnApp.  On app, always true.
     */
    init: function(isOnApp)
    {
        this.model = new CafeModel();
        this.view = new CafeView();
        this.view.onApp = isOnApp;
        this.blenderView = new BlenderView();
        this.blenderController = new BlenderController();
    },

    loadLevel: function(level, overrideParams)
    {
        if (undefined === overrideParams) {
            overrideParams=null;
        }
        this.model = CafeModel.loadLevel(level, overrideParams);
        this.view.model = this.model;
        return this.model;
    },

    /**
     * GAME-2403 If is stub, save board and populate view.  If no board, then get saved board.  This helps preload stub of level.  Test case:  2014-12-18 Tap level.  Matt Leung expects to animate immediately.  Got impatient.  (+Sonia, +Beverly, +Melissa Malski, 2015-01-09 +Emma).
     */
    populate: function(board, isStub)
    {
        if (undefined === board) {
            board = this.view.board;
        }
        if (null != this.model) {
            board = this.view.populate(board, this.model);
            if (isStub) {
                if (this.blenderView.isLoaded()) {
                    isStub = false;
                }
            }
            ll.log("CafeController: populate: isStub " + isStub);
            if (isStub) {
                this.view.board = board;
            }
            else {
                this.blenderView.populate(this.model, board.rooms.layouts);
                this.blenderController.populate(this.model, 
                    this.blenderView.blenders, board.ingredients,
                    this.view.scrollButtons);
                this.isPopulatedNow = true;
            }
            this.isUpdating = true;
        }
        else {
            ll.error("CafeController: Expected model.");
        }
    },

    onStartGameplay: function()
    {
        this.model.onStartGameplay();
        this.view.sounds.gameMusicLoopPlay();
        this.isUpdating = true;
    },

    /**
     * @param   now     Milliseconds since start, compensating for pause.
     * @param   time    Seconds remaining with fraction.
     * @return  If trials were completed.
     *
     * Changelog:
     * DEPRECATED WG-699 Start an order.  Play orders started movie clip in up to 4 states, repeating the 4th state.  14/4/16 Abhishek expects to animate something else when orders started.
     */
    update: function(now, time, tutorialMode, tutorCurrentLabel)
    {
        var complete = false;
        this.isPopulatedNow = false;
        if (!this.isUpdating) {
            return complete;
        }
        if (this.model) {
            complete = this.model.update(now, time);
            if (this.model.tutor) {
                this.model.tutor.update(tutorialMode, tutorCurrentLabel);
            }
            if (this.view) {
                this.blenderView.update();
                this.view.update();
            }
        }
        return complete;
    },

    /**
     * Select tutorial.
     */
    selectTutorial: function()
    {
        if (this.model) {
            this.model.tutorial();
        }
        if (this.view) {
            this.view.tutorial();
        }
    },

    /**
     * @return  If tutoring a level is beginning now.
     */
    tutorLevelMayBegin: function()
    {
        var tutorLevelIndex = this.model.tutor.levelBegin(CafeModel.session_level);
        var isBeginning = 0 <= tutorLevelIndex;
        if (isBeginning) {
            this.model.ingredientsVisible = false;
        }
        if (this.view) {
            this.view.tutorLevelBegin(tutorLevelIndex);
        }
        return isBeginning;
    },

    setEnabled: function(enable)
    {
        ll.log("Controller.setEnabled: " + enable);
        this.model.enabled = enable;
        if (this.view && this.view.board) {
            this.view.board.mouseChildren = enable;
            this.view.board.mouseEnabled = enable;
        }
    }
});



