/*jslint node: true */
"use strict";

require("src/View/CCBReaderAdapter.js" );

require("src/Game/Blender.js");
require("src/Game/CafeModel.js");
require("src/Game/Sounds.js");
require("src/Game/IngredientsView.js");


/**
 * MOBGAMES-1837 #comment [~wouter], in each layout, coffee machine instance names are numbered 0 to n-1, ordered by left-to-right, top-to-bottom.  Also, instructions of developer controls button mention the developer controls button moved to top right.  2014-11-05 Last cup tilted forward.  Emma expects to predict which cup is selected.  Felt weird.  (2014-11-11 +Pranjal)
 */
var CafeView = cc.Class.extend(
{
    board: undefined,
    customers: undefined,
    model: undefined,
    onAppear: undefined,
    onApp: undefined,
    parent: undefined,
    scrollButtons: undefined,
    scrolls: undefined,
    sounds: undefined,

    ctor: function()
    {
        CCBReaderAdapter.loadSpriteSheets(3);
    },

    wireSounds: function(sounds)
    {
        this.sounds = sounds;
        this.sounds.wireSounds();
    },

    removeChildren: function(board)
    {
        board.removeAllChildren();
    },

    /**
     * @return  board   Adopted or created board.
     * GAME-465 Update view during populate. Test case:  2014-02-25 Select a level that has only one room.  Countdown 3-2-1. Kelsey expects to fix to not see scroll button (2014-03-07 +Erin).
     * Hide other rooms.  Test case:  2014-11-28 Level 10.  3 rooms scroll into view.  I expect to not see other room pop into view.
     * GAME-2403 Adopt playing board from CCB cache.  Lazy cache board.  A lazy cache may enable caching and reusing coffee machines without waiting for to preload all 12 coffee machines.  When reusing, adopt children instead of replacing parents, to preserve animations and parent relations with grandparents.  Test case:  2015-01-10 Play.  Restart.  How to Play.  Expect to see a coffee machine.  Skip.  Play.  Expect to see coffee machines.  Got invisible, since the playing board was orphaned.   Test case:  2014-12-18 Tap level.  Matt Leung expects to animate immediately.  Got impatient.  (+Sonia, +Beverly, +Melissa Malski, 2015-01-09 +Emma).
     * XXX GAME-2403 Sadly, cache reparent fails during reparent or restart.  Maybe there is some cleanup that could be done on cached nodes during the reparent or restart.  Test case:  2015-01-13 Level 3.  Restart.   Level 2.  Expect play.  Got game hangs.
     */
    populate: function(board, model)
    {
        this.model = model;
        this.board = board;
        var isCreated = this.populateLayouts(model.numRooms, this.board.rooms);
        var children = this.board.getChildren();
        this.populateIngredients(children);
        this.populateScrolls(this.board);
        this.customers = this.populateOrders(this.model.customers);
        this.populateHint();
        this.hideOtherRooms();
        this.update();
        return this.board;
    },

    /**
     * Retain to reuse.
     * Disable retain and remove.  Test case:  2015-01-11 Play.  How to Play.  Game stops updating.
     */
    removeLayouts: function(layouts)
    {
        if (layouts) {
            for (var p = 0; p < layouts.length; p++) {
                var layout = layouts[p];
                if (layout.getParent()) {
                    layout.setVisible(false);
                }
            }
        }
    },

    /**
     * GAME-2403 Hotspot:  Blender.ccb 0.3 sec x 12 blenders.  Test case:  2015-01-07 Xcode, iPhone 5 simulator.  Log hotspots with timestamp.
     * Trouble Brewing vs Train of Thought begin loading.  (after cocos2d info until surface size) 1.7 sec vs 0.7 sec.
     * Load time of level 1 vs level 10:  1.1 sec vs 3.6 sec.
     * If layout has a parent, do not add.  Test case:  2015-01-07 Select level.  Expect level loads.  Got crash with log: "Cocos2d: Assert failed: child already added. It can't be added again".
     *
     * Get z from first layout before removing.  Test case: 2014-12-15 iOS simulator. Select level. Wouter expects to transition.  Got app crash with log "Invalid Native Object".  The object was instantly garbage collected after being removed from parent.
     * @return isCreated    If false, then was cached.
     * 14/3/28 Wouter expects to customize tutorial layout.
     * WG-585
     * WG-1021 Index 8 or 9, for layout room 2. 14/6/3 Level 9 and 10:  Julie expects custom layout of 3 or 4 coffee machines when on the lefthand side.
     * GAME-2034 Layout customers at index equal to number of blenders.  2014-12-02 Level 5.  Vishu expects to fix layout.
     * GAME-2403 In order to load up to 0.5 seconds quicker, I removed layout from Board.ccb.  There is now an empty node named "layout_0".  This enables a layout cache and works around a l imitation in Cocos2d-x 2.2.2 that prohbited my attempts to cache and reparent graphics.
     */
    populateLayouts: function(numRooms, rooms)
    {
        this.removeLayouts(rooms.layouts);
        rooms.layouts = [];
        var offsets = this.onApp ? CafeView.layoutOffsetsOnApp : null;
        var specs = this.model.populateLayouts(rooms.layout_0.getPositionX(), 
            rooms.layout_0.getPositionY(), CafeView.width, offsets);
        var z = rooms.layout_0.getZOrder();
        var isCreated = true;
        for (var roomIndex = 0; roomIndex < numRooms; roomIndex++) {
            var spec = specs[roomIndex];
            var layoutClass = CafeView.layoutClasses[spec.category][spec.count];
            if (null == layoutClass) {
                throw new Error("Expected layout for room " + roomIndex 
                    + " with " + spec.count 
                    + " blenders or this.customers in category " 
                    + spec.category);
            }
            var name = "layout_" + roomIndex;
            var layout = this.showOrCreate(rooms, name, layoutClass);
            layout.setPosition(spec.x, spec.y);
            layout.setZOrder(z);
            layout.setVisible(true);
            rooms.layouts.push(layout);
        }
        return isCreated;
    },

    showOrCreate: function(parent, name, ccbFileName)
    {
        var key = name + "_" + ccbFileName;
        var nextChild = parent[key];
        if (!nextChild) {
            nextChild = CCBReaderAdapter.create(ccbFileName);
            parent.addChild(nextChild);
        }
        this.show(parent, key, nextChild);
        this.show(parent, name, nextChild);
        return nextChild;
    },

    show: function(parent, name, nextChild)
    {
        var previousChild = parent[name];
        if (null != previousChild && previousChild != nextChild) {
            previousChild.setVisible(false);
        }
        parent[name] = nextChild;
        nextChild.setVisible(true);
    },

    /**
     * XXX GAME-2403 Do not adopt or create.  Test case:  2015-01-11 Test reparent passed. Restart, play, how to play.  Expected animation.  Got game stops responding.
     */
    adoptOrCreate: function(parent, name, ccbFileName)
    {
        var nextChild = CCBReaderAdapter.adoptOrCreate(name, ccbFileName);
        cc.BuilderReaderHelper.replace(parent, key, nextChild);
        cc.BuilderReaderHelper.replace(parent, name, nextChild);
        return nextChild;
    },

    isCached: function(ccbFileBaseName, name)
    {
        var key = name + "_" + ccbFileBaseName;
        var cache = CafeView.layoutCache;
        return null != cache[key];
    },

    /**
     * Remove ingredients that are not being used at this level.
     * @param   board   Populate ingredients array in visible child order, which might be arbitrary.
     * @return unique, sorted instance names of display objects that are visible.
     * Example @see TestView
     */
    populateIngredients: function(children)
    {
        var enabledNames = [];
        for (var c = 0; c < children.length; c++) {
            var child = children[c];
            var name = child.getName();  // v3 and CCBReaderHelper adds getName.
            if (name && 0 == name.indexOf("ingredient_")) {
                var index = parseInt(name.split("_")[1]);
                var enabled = index < this.model.num_ingredients;
                if (enabled) {
                    if (enabledNames.indexOf(name) <= -1) {
                        enabledNames.push(name);
                    }
                }
                else {
                    child.setVisible(false);
                }
            }
        }
        enabledNames.sort();
        this.board.ingredients = [];
        for (var n = 0; n < enabledNames.length; n++) {
            child = this.board[enabledNames[n]];
            this.board.ingredients.push(child);
        }
        if (this.onApp) {
            this.distribute(this.board.ingredients, this.board);
        }
        return enabledNames;
    },

    /**
     * GAME-2403 Always distribute ingredients and cache bounds, in case of reentry with cached level.  Test case:  2015-01-07 Load level 3.  Observe ingredient positions.  Restart.  Load level 3 again.  Expect ingredient positions remain same.
     *
     * If less than saturated number of ingredients, compare least "ingredient_0", to max "ingredient_4".  Space out positions proportionally between these endpoints.
     * Special case:  If 2 ingredients, then offset as if 3 ingredients, with 2 ingredients in 2nd and 4th spots.
     * Test case:  2014-11-19 Level 3.  HeyJin expects distance between buttons.  Felt anxious.
     */
    distribute: function(ingredients, container)
    {
        var m = this.model.maxIngredientIndex();
        if (undefined == CafeView.distributeBounds) {
            CafeView.distributeBounds = [
                ingredients[0].getPositionX(),
                container["ingredient_" + m].getPositionX()];
        }
        var bounds = CafeView.distributeBounds;
        var minX = bounds[0];
        var maxX = bounds[1];
        var width = maxX - minX;
        var offset = 0;
        if (ingredients.length == 2) {
            offset = 1;
        }
        var denominator = ingredients.length - 1 + offset;
        var space = width / denominator;
        for (var i = 0; i < ingredients.length; i++) {
            var x = (space * (i + 0.5 * offset)) + minX;
            ingredients[i].setPositionX(x);
        }
    },

    /**
     * GAME-1885 While not playing, disable ingredient buttons.  Test case:  2015-01-07 During 3-2-1 countdown. Tap ingredient.  Expect no effect.  Got ingredient added.
     */
    updateShowIngredients: function(num_ingredients_visible)
    {
        for (var i = 0; i < this.board.ingredients.length; i++) {
            var child = this.board.ingredients[i];
            var visible = i < num_ingredients_visible;
            if (visible != child.isVisible()) {
                child.setVisible(visible);
            }
            var isEnabled = visible && this.model.playing;
            if (isEnabled != child.isEnabled()) {
                child.setEnabled(isEnabled);
            }
        }
    },

    /**
     * Set text of "wrongOrder".  Hide tutorLevel.
     * WG-699 Do not stop after "shouldTrash", so may restart after next mistake.
     * Stop at invisible state, "none", for tutor movie clip on left and right layouts.  14/4/30 Level 9. Left screen.  John Berry expects to fix do not see tutorial.
     */
    populateHint: function()
    {
        var tutorLevel = this.board.rooms.tutorLevel;
        tutorLevel.wrongOrder.setString(localize("tutor_wrongOrder"));
        tutorLevel.setVisible(false);
    },

    /**
     * Play end.  Room button transition completes.  Other rooms become invisible.  Scroll offscreen.  2014-11-13 level 7.  level up.  may expects transition to level up without seeing the other room.  Horizontally slide tutorial screen in and out of view.  2014-11-06 Mark Scoptur expects snappy transition.  Got laggy.
     */
    hideOtherRooms: function()
    {
        var layouts = this.board.rooms.layouts;
        for (var i = 0; i < layouts.length; i++) {
            if (this.model.roomIndex != i) {
                if (layouts[i].isVisible()) {
                    layouts[i].setVisible(false);
                }
            }
        }
    },

    showTargetRoom: function()
    {
        var layouts = this.board.rooms.layouts;
        var layout = layouts[this.model.roomIndex];
        if (!layout.isVisible()) {
            layout.setVisible(true);
        }
    },

    /**
     * WG-699 Any board shows same tutor area.
     */
    updateHint: function(model, hint)
    {
        if (model.wouldShowHint()) 
        {
            hint.setVisible(true);
            hint.wrongOrder.setString(localize("tutor_wrongOrder"));
            cc.BuilderReaderHelper.runAnimations(hint, "wrongOrder", true);
        }
    },

    tutorial: function()
    {
        if (this.sounds) {
            this.sounds.gameMusicLoopStop();
        }
        if (this.board) {
            this.board.setVisible(false);
        }
    },

    /**
     * Show the tutor clip that instructs how to use the new room.
     */
    tutorLevelBegin: function(tutorLevelIndex)
    {
        this.board.rooms.tutorLevel.setVisible(0 <= tutorLevelIndex);
        TutorScene.level(this.board.rooms.tutorLevel, 0 == tutorLevelIndex);
    },

    tutorLevelEnd: function()
    {
        this.board.rooms.tutorLevel.setVisible(false);
    },

    /**
     * Disable scroll button.  Later enable.  Test case:  2015-01-06 Level 9.  Scroll button is not at edge.  Perhaps Cocos2D does not update position until disabling and then enabling again.
     * @param   board   Assign button member of "lower" and "higher" scroll member of board, and populate these into internal "scrollButtons" variable.  Populate "scrolls" with each scroll container.  Set name of each button to "lower" or "higher" respectively.
     * GAME-1872 As in the Flash prototype and similar to How To Play buttons, in scrollButton*.png files, I widened the hit area of the scroll button to about double visible area.  I also made the hit area a little taller.  Test case:  2014-11-12 Level 5 or higher. Tap near room. Expects response.  Got no response.  (2015-01-07 +Mark Scoptur)
     * GAME-1885 When populating scroll buttons, reset room scrolling position.  Test case:  2015-01-07 Level 9.  Play.  Move room.  Restart.  Level 9.  Chris Hewitt expects to see order bar.  Got blank.  Test case:  2015-01-07 Level 9 or higher.  Move right.  Restart.  Level 3.  Expect 3 coffee machines.  Got 6.
     * GAME-1885 When populating scroll buttons, update their fill and reminder animation.  Test case:  2015-01-08 Level 9.  Play.  Move room.  Begin filling cup.  Move to another room.  Observe scroll up partially full.  Restart.  Level 9.  I expect cup is empty.  Got partially full.  Test case:  2015-01-07 Android.  Level 9.  Play.  Move room.  Fill.  Unknown steps, such as maybe offscreen reminder animation is playing.  Restart.  Brittany expects to fix to see scroll buttons are dim.  Got blue highlight on right room button, which persisted during play.
     * GAME-1885 When populating scroll fractions, nullify animation state.  Test case:  2015-01-10 Level 9 or higher.  Play.  Start filling.  Move rooms.  Observe cup on scroll button.  I expect to see it filling.  Restart.  Play.  Start filling.  Move rooms.  Observe cup on scroll button.  I expect to see it filling.  Got it is invisible until overflowing.  Sometimes only invisible in right room, such as when change level from 3 to 9.
     */
    populateScrolls: function(board)
    {
        var leftAndRight = CCBReaderAdapter.mapLeftAndRight(board);
        this.scrollButtons = [];
        this.scrolls = [];
        var names = ["lower", "higher"];
        for (var n = 0; n < names.length; n++) {
            var scroll = board[names[n]];
            var button = scroll.button;
            button.setName(names[n]);
            this.scrollButtons.push(button);
            this.scrolls.push(scroll);
            scroll.setPositionX(leftAndRight[n]);
            scroll.fraction.currentState = undefined;
            button.setEnabled(false);
        }
        board.rooms.setPositionX(0);
        this.updateScrolls();
    },

    /**
     * GAME-2178 Do not disable button.  Test case: 2014-12-29 Press scroll button.  Expect highlight state.  Got disabled state.
     */
    updateScrolls: function()
    {
        if (null != this.model) {
            this.updateScrollPosition();
            var visibles = this.model.scrollVisibles();
            var fractionMaxes = this.model.scrollFractionMaxes();
            for (var i = 0; i < visibles.length; i++) {
                var scroll = this.scrolls[i];
                var button = this.scrollButtons[i];
                var isVisible = visibles[i];
                scroll.setVisible(isVisible);
                var isEnabled = true;  
                if (isEnabled != button.isEnabled()) {
                    button.setEnabled(isEnabled);
                }
                this.updateScrollFraction(scroll.fraction, fractionMaxes[i], isVisible);
            }
            this.updateRemindRoom(CafeModel.FOREVER - this.model.now, this.model.tutor.isLevel());
        }
    },

    /**
     * The program expects the following animation structure:  
     *     scroll.fraction.content.stencil
     * If overflowing, set to "overflow" timeline.
     * Otherwise, set to "fraction" timeline 
     * and interpolate y-position of "fraction.content.stencil".
     */
    updateScrollFraction: function(fractionClip, fraction, isVisible)
    {
        var isOver = Blender.isOverflow(fraction);
        var state = isOver ? "overflow" : "fraction";
        cc.BuilderReaderHelper.setState(fractionClip, state);
        cc.BuilderReaderHelper.setState(fractionClip.content.clippingNode.root, state);
        if (!isOver && isVisible) {
            var y = this.interpolate(fraction);
            var stencil = fractionClip.content.clippingNode.root.stencil;
            stencil.setPositionY(y);
        }
    },

    /**
     * Interpolate y-position of "fraction.content.stencil" between first and last keyframe positions [-48.0, -25.0].
     * XXX Would be flexible if automated reading min and max from keyframes.
     */
    interpolate: function(fraction)
    {
        var min = -51.0;
        var max = -25.0;
        var fractionMax = 0.5;
        var ratio = Math.max(0.0, Math.min(1.0, fraction / fractionMax));
        var range = max - min;
        var y = (ratio * range) + min;
        return y;
    },

    /**
     * GAME-1885 Before move, show target room.  After move, hide other rooms.
     * Test case:  2014-12-29 Two rooms tutor. Resume.  Expect not to see first room under resume.
     */
    updateScrollPosition: function()
    {
        if (this.model.isScrollStarting) {
            var x = this.model.scrollX * CafeView.width;
            var rooms = this.board.rooms;
            var move = cc.MoveTo.create(CafeModel.scrollTime, 
                cc.p(x, rooms.getPositionY()));
            var hide = cc.CallFunc.create(this.hideOtherRooms, this);
            var moveThenHide = cc.Sequence.create(move, hide);
            this.showTargetRoom();
            rooms.runAction(moveThenHide);
        }
    },

    /**
     * Update the appearance of each room, order, and blender.
     * 
     * Changelog:
     * DEPRECATED GAME-699 Start an order.  Play orders started movie clip in up to 4 states, repeating the 4th state.  Test case:  2014-04-16 Abhishek expects to animate something else when orders started.  2014-05-01 After experimenting with this, Wouter, Abhi, and I feel distracted.
     */
    update: function()
    {
        if (this.model) {
            this.updateOrders();
            if (this.board) {
                this.updateShowIngredients(this.model.num_ingredients_visible);
                this.updateScrolls();
                this.updateHint(this.model, this.board.rooms.tutorLevel);
                if (this.model.tutor.isLevelEnding) {
                    this.tutorLevelEnd();
                }
            }
            this.updateCoffeeBrewLoop(this.model.active);
        }
    },

    /**
     * If any coffee machine is active, loop sound, unless already looping, otherwise stop.  2014-09-03 Sarah expects coffee brew sound is only heard when one or more coffee machines are flowing.
     */
    updateCoffeeBrewLoop: function(active)
    {
        if (null != this.sounds) {
            if (active) {
                this.sounds.coffeeBrewLoopPlay();
            }
            else {
                this.sounds.coffeeBrewLoopStop();
            }
        }
    },

    /**
     * Expects first room layout was configured, with member variables with prefix "order_" followed by an index.
     * This is an order container with an animated child that shows the ingredients.
     * Example structure:  order.body.ingredient_0
     *
     * @return  Array of the children OrderClip that animate the state and show the ingredients.
     *
     * GAME-2178 In SpriteBuilder Order.ccb, I positioned ingredient_0 and ingredient_1 at (0, 0).  There, I adjusted anchor to appear in center of ticket.  [~wouter], if you need to adjust, keep position at (0, 0) and adjust the anchor point instead.  2014-12-11 An order appears with a different sprite image.  Wouter expects order ingredient's anchor to retain its centering.
    * GAME-1885 When populating orders, reset animation state to "none".  Test case:  2015-10-20 Level 4.  First order appears.  Restart.  Level 4.  Before 3-2-1.  Expect no order ticket.  Got blank order.
     */
    populateOrders: function(modelCustomers)
    {
        var customers = [];
        for (var index = 0; index < modelCustomers.length; index++) {
            var name = "order_" + index.toString();
            var layout = this.board.rooms.layouts[0];
            var order = layout[name];
            CafeView.populateOrder(modelCustomers[index], order.body);
            customers.push(order);
        }
        return customers;
    },

    /**
     * When an order appears or is served, play that animation.  GAME-364 Test case:  2014-02-12 Serve 3 of 3 customers.  Emma expects to fix game does not crash.
     */
    updateOrders: function()
    {
        for (var c = 0; c < this.customers.length; c++) {
            var order = this.customers[c];
            var body = order.body;
            var state = this.model.customerState(c, order.currentState);
            IngredientsView.setIngredients(body.ingredients, this.model.customers[c],
                IngredientsView.getOrderClasses());
            if (cc.BuilderReaderHelper.setState(order, state)) {
                if ("appear" == state) {
                    if (null != this.onAppear) {
                        this.onAppear();
                    }
                }
            }
        }
    },

    /**
     * GAME-1874 During tutor, blink room button every 3 seconds.  2014-12-09 Level 5 and 9.  Chris Hewitt expects to fix remind room blinks.  (2014-12-12 +Melissa, 2014-12-15 +Emma)
     * Highlight room switch button when a new customer arrives on another screen, and hear door open.  14/1/15 Kelsey expects to realize there is a 5th blender.  (14/1/19 +Stephanie).
     * https://lumoslabs.atlassian.net/browse/WG-234
     * Example @see TestView
     */
    updateRemindRoom: function(msRemaining, alwaysRemind)
    {
        if (undefined === alwaysRemind) {
            alwaysRemind=false;
        }
        if (this.model.wouldRemindRoom(msRemaining, alwaysRemind)) {
            var scroll = this.model.numRooms <= 2 ? this.board.higher : this.board.lower;
            cc.BuilderReaderHelper.runAnimations(scroll.offscreen, "active");
        }
    }
});

CafeView.distributeBounds = CafeView.distributeBounds || undefined;

/**
 * Max.  For multipanels, set to 4.  Could extract class references to Main.
 * 14/3/4 Wouter expects unique layout for 5 blenders.
 */
CafeView.layoutClasses = [
            ["Layout0", 
             "Layout0", "Layout2", "Layout3", "Layout4", "Layout5", "Layout6",
             "Layout7", "Layout8", "Layout9", "Layout10", "Layout5", "Layout6"],
            ["0",
             "1", "2", "3", "4",
             "LayoutCustomers5", "LayoutCustomers6", "7", "8", 
             "LayoutCustomers9", "LayoutCustomers10", "LayoutCustomers11", "LayoutCustomers12"]];
/**
 * SpriteBuilder uses iPhone dimensions: 320x480.  This is 2:3 aspect ratio.  iPad aspect ratio is 3:4.  If iPad were 480 tall, then its width would be 360.
 */
CafeView.width = 360.0;
/**
 * In Flash prototype, move room 1 layout over 40 pixels, and room 2 layout -40 pixels.  2014-11-12 Level 7.  cups close to button.  Melissa Malski expects to swipe.  May expect to move machines to make room.  2014-11-13 Tap room select button.  Matt Leung expects response.  Got did not always respond.  Maybe it was disabled or occluded or too small.
 */
CafeView.layoutOffsetsOnApp = [0, 30, -30];

/**
 * 14/3/13 Wouter expects queue animates arbitrary direction.
 * https://lumoslabs.atlassian.net/browse/WG-444
 * 14/3/10 Wouter expects queue.
 * WG-699 Only stop "none" and "active", and "serve".  There is a "none" label afterward.
 */
CafeView.populateOrder = function(customerOrder, order)
{
    if (undefined == order.ingredients) {
        IngredientsView.init(order, IngredientsView.getOrderClasses());
    }
    IngredientsView.setIngredients(order.ingredients, customerOrder, 
        IngredientsView.getOrderClasses());
    cc.BuilderReaderHelper.runAnimations(order, "none", true);
}

CafeView.layoutCache = CafeView.layoutCache || {};
