/*jslint node: true */
"use strict";

var IngredientsView = {};

IngredientsView._ingredientClasses;
IngredientsView._orderClasses;

/**
 * @param    container  Interface expects parameters: ingredients, ingredient_0, ingredient_1
 */
IngredientsView.init = function(container, classes)
{
    container.ingredients = [container.ingredient_0, container.ingredient_1];
    IngredientsView.setIngredients(container.ingredients, [], classes);
}

/**
 * Lazy initialize to avoid race condition with constructor of Ingredients.
 */
IngredientsView.getIngredientClasses = function()
{
    if (null == IngredientsView._ingredientClasses) {
        IngredientsView._ingredientClasses = [
            "cupIngredientCoffee.png",
            "cupIngredientSugar.png",
            "cupIngredientChocolate.png",
            "cupIngredientCreamer.png",
            "cupIngredientCinnamon.png"
        ];
    }
    return IngredientsView._ingredientClasses;
}

/**
 * Lazy initialize to avoid race condition with constructor of Ingredients.
 */
IngredientsView.getOrderClasses = function()
{
    if (null == IngredientsView._orderClasses) {
        IngredientsView._orderClasses = [
            "orderIconCoffee.png",
            "orderIconSugar.png",
            "orderIconChocolate.png",
            "orderIconCreamer.png",
            "orderIconCinnamon.png"
        ];
    }
    return IngredientsView._orderClasses;
}

/**
 * Set sprite frame.
 */
IngredientsView.setIngredients = function(ingredients, indexes, classes)
{
    var length = 0;
    if (null != indexes) {
        length = indexes.length;
    }
    for (var i = 0; i < length; i++) {
        var ingredientClass = classes[indexes[i]];
        if (null == ingredientClass) {
            throw new Error("Expected ingredient class " + indexes[i]);
        }
        var frame = cc.SpriteFrameCache.getInstance().getSpriteFrame(ingredientClass);
        ingredients[i].setDisplayFrame(frame);
        ingredients[i].setVisible(true);
    }
    for (; i < ingredients.length; i++) {
        ingredients[i].setVisible(false);
    }
    return indexes;
}
