/*jslint node: true */
"use strict";

require("src/Utility/LumosityUtils.js" );

var LevelSelect = cc.Class.extend(
{
    model: undefined,
    scene: undefined,

    ctor: function(model, scene)
    {
        this.model = model;
        this.scene = scene;
    },

    /**
     * GAME-1603 If below highest level, show "levelUnlockText", otherwise show "maxLevelText".  
     * Read to tap and number to unlock a higher level.  Test case:  2014-10-23 Level select mode.  Ashley expects to read what the number represents and a call to tap a number.
     * GAME-1885 Do not write highest of current level or 12 on last cup.  Test case:  2015-01-20 Set level 13.  I expect to read 13 or 14 on cup.  Got 12.  End game.  Reads unlocked level 13.  Next session.  See level 13.  Reset game data.  Play again.  I expect to read level 12 on last cup.  2014-08-06 After unlocking, Aaron expects to read highest level unlocked on last tile of Disillusion.  However, unlock Disillusion, the maximum level can go down above level 12, as not to frustrate a user that whose skill has decayed.  Therefore do not upset that user by reducing the level.
     */
    localizeText: function(canLevelUp, unlockMin, parent)
    {
        parent.line1.setString( localize("levelSelectText") );
        var unlock;
        if (canLevelUp)
        {
            unlock = localize("levelUnlockText").format(unlockMin);
        }
        else
        {
            unlock = localize("levelMaxText");
        }
        parent.line2.setString( unlock );
    },

    levelUp: function(canLevelUp, nextSessionLevel, nextSessionOrders, parent)
    {
        var line1;
        var line2;
        if (canLevelUp) {
            line1 = "levelUpEndText_line1";
            line2 = localize("levelUpEndText_line2").format(nextSessionLevel);
        }
        else 
        {
            line1 = "levelUpMaxText_line1";
            line2 = localize("levelUpMaxText_line2").format(nextSessionOrders);
        }
        parent.line1.setString(localize(line1));
        parent.line2.setString(line2);
    }
});
