/*jslint node: true */
"use strict";

require("src/View/CCBReaderAdapter.js" );

var Scene = cc.Class.extend(
{
    begin: undefined,
    currentSequenceName: undefined,
    game: undefined,
    current: undefined,
    parent: undefined,
    root: undefined,

    ctor: function(onFinishDelegate, parent)
    {
        this.parent = parent;
        this.game = onFinishDelegate;
        this.beginCcbFileName = "MainBegin";
        this.ccbFileName = "Main";
        this.currentSequenceName = null;
        this.owner = this;
    },

    /**
     * GAME-2403 Instead of set animation manager delegate, poll each frame for the currently running animation.  Test case:  2015-01-17 Start game.  setDelegate.  Cocos2d-x js v2.2.2 crash.  Cocos2d-html5 ok.
     */
    addChild: function(ccbFileName, root)
    {
        if (root)
        {
            if (this.ccbFileName == ccbFileName)
            {
                this.runAnimations("none", root);
            }
        }
        else {
            root = CCBReaderAdapter.addChild(this.parent, ccbFileName, 
                root, this.owner);
            root.setZOrder(0);
        }
        return root;
    },

    /**
     * GAME-2403 I separated MainScene.ccb into MainBegin.ccb and Main.ccb.  I separated Tutor.ccb into TutorBegin.ccb and Tutor.ccb.  When the game starts in How to Play mode, TutorBegin.ccb is loaded.  After its "gameBegin" animation ends, it loads Tutor.ccb and the tutorial level in the background.  The same kind of process happens when Scene is loaded.
     *
     * GAME-2403 On begin, hide scene except beginning.  On load, show root yet reset animation.  Test case:  2015-01-10 Play.  Restart.  Expect board disappeared.  Got board remained.  How to Play.  Expect level select disappeared.  Got level select remained.  Wait 5 seconds.  Expect tutorial continues.  Got level menu appears behind tutorial.
     *
     * GAME-2403 Preload animations.  2014-12-18 Tap level.  Matt Leung expects to animate immediately.  Got impatient.  (+Sonia, +Beverly, +Melissa Malski, 2015-01-09 +Emma).
     */
    onBegin: function()
    {
        this.load();
        this.hide();
        this.begin = this.addChild(this.beginCcbFileName, this.begin);
        this.begin.setVisible(true);
        this.current = this.begin;
        cc.BuilderReaderHelper.runAnimations(this.begin, "gameBegin");
    },

    /**
     * GAME-2403 After "gameBegin", load full scene and current level.  Test case:  2015-01-07 Locomotive.  Tap "Trouble Brewing".  Sonia may expect reduce begin loading and restart loading time to 0.7 seconds (as Train of Thought).  Got impatient  (+Beverly, +Melissa Malski, +Emma).  Test case:  2014-12-18 Tap level.  Matt Leung expects to animate immediately.  Got impatient.  (+Sonia, +Beverly, +Melissa Malski, 2015-01-09 +Emma).  
     */
    load: function()
    {
        var isLoading = undefined == this.root;
        this.begin = this.addChild(this.beginCcbFileName, this.begin);
        this.root = this.addChild(this.ccbFileName, this.root);
        this.root.setVisible(true);
        this.current = this.root;
        return isLoading;
    },

    loadLevel: function()
    {
        this.load();
        this.game.loadLevel(CafeModel.session_level, this.root.board);
    },

    hide: function()
    {
        if (this.begin)
        {
            this.begin.setVisible(false);
        }
        if (this.root)
        {
            this.root.setVisible(false);
        }
    },

    update: function()
    {
        cc.BuilderReaderHelper.update(this.current, this);
    },

    /**
     * After any game end animation, ask game to play its game animation.
     */
    completedAnimationSequenceNamed: function(sequenceName)
    {
        cc.log('Scene.completedAnimationSequenceNamed: "' + sequenceName + '"');
        if (sequenceName == "gameBegin")
        {
            this.loadLevel();
        }
        else if (0 == sequenceName.indexOf("gameEnd")) 
        {
            this.game.playEndAnimation(true);
        }
        else if (0 == sequenceName.indexOf("playBegin")) 
        {
            this.game.playBegin();
        }
    },

    playBegin: function()
    {
        this.runAnimations("gameBeginEnd", this.begin);
        this.runAnimations("playBegin");
        this.isPlayBegin = true;
    },

    runAnimations: function(label, root)
    {
        if (undefined === root) {
            root = this.root;
        }
        if (cc.BuilderReaderHelper.runAnimations(root, label)) {
            root.currentState = label;
        }
        else {
            cc.log('Scene.runAnimations: "' + label + '" not found');
        }
    },

    setState: function(label)
    {
        cc.BuilderReaderHelper.setState(this.root, label);
    }
});
