/*jslint node: true */
/*global audio*/

"use strict";

require( "src/Game/Component/GameComponentAudioMixer.js" );

var Sounds = cc.Class.extend(
{
    mixer: undefined,
    preloadPaths: [
        "sounds/coffeeBrew.mp3",
        "sounds/coffeeStart.mp3",
        "sounds/coffeeStop.mp3",
        "sounds/cupMove.mp3",
        "sounds/guitarRadioAmbienceLoop.mp3",
        "sounds/ingredientClick.mp3"
    ],

    ctor: function(mixer)
    {
        this.mixer = mixer;
        this.gameMusicLoopStop();
        this.coffeeBrewLoopID = undefined;
        for (var p = 0; p < this.preloadPaths.length; p++) {
            var path = this.preloadPaths[p];
            audio.preloadEffect(path);
        }
    },

    coffeeBrewLoopPlay: function()
    {
        if (undefined === this.coffeeBrewLoopID) {
            ll.log("Sounds.coffeeBrewLoopPlay");
            this.coffeeBrewLoopID = audio.playEffect("sounds/coffeeBrew.mp3", true);
        }
    },

    coffeeBrewLoopStop: function()
    {
        if (undefined !== this.coffeeBrewLoopID)
        {
            ll.log("Sounds.coffeeBrewLoopStop");
            audio.stopEffect(this.coffeeBrewLoopID);
            this.coffeeBrewLoopID = undefined;
        }
    },

    /**
     * GAME-2394 Sometimes cocos-html5 did not play loop.
     * 14/1/30 In case of a race, if no instance, sound, and channel, Rollbar may expect to not loop sound.
     * Traceback (most recent call last): 1 File "com/lumoslabs/games/taxiPlanner//Game.as" line 0 in resetRound() 2 File "com/lumoslabs/games/taxiPlanner//Game.as" line 0 in startTrial() 3 File "com/lumoslabs/games/taxiPlanner//Sounds.as" line 0 in gameMusicLoopPlay() TypeError: Error #1009: Cannot access a property or method of a null object reference. ← Next Previous → Occurred on 2014-01-30 11:23:28 am PST
     * https://rollbar.com/item/274190143/instance/778288126/
     * Traceback (most recent call last): 1 File "com/lumoslabs/games/taxiPlanner//Game.as" line 0 in startTrial() 2 File "com/lumoslabs/games/taxiPlanner//Sounds.as" line 0 in gameMusicLoopPlay() TypeError: Error #1009: Cannot access a property or method of a null object reference. ← Next Previous → Occurred on 2014-01-30 11:23:02 am PST
     * https://rollbar.com/item/274190143/instance/778286434/
     */
    gameMusicLoopPlay: function()
    {
        this.mixer._settings.ambientSound = "sounds/guitarRadioAmbienceLoop.mp3";
        this.mixer._playMusic(this.mixer._settings.ambientSound);
    },

    /**
     * When stopping music, also stop coffee brew.  Test case:  2014-09-03 Tutorial.  Overflow.  Hear coffee brew.  Skip tutorial.  Sarah could expect to fix no longer hear coffee brew.
     * GAME-2452 End of tutorial, fade out sounds as if pausing.  End of game, program nothing.  Expect the default mixer to fade ambient sound out.  Test case:  2015-01-13 Short game.  How to Play.  Finish.  I expect ambient sound fades out.  Select level.  Complete orders.  Time rollup ends.  Game is exiting.  [~sclark] expects audio fades out.  Got audio stops abruptly.  
     */
    gameMusicLoopStop: function()
    {
        this.coffeeBrewLoopStop();
        if (this.mixer) {
            this.mixer.pauseSound();
            this.mixer._settings.ambientSound = undefined;
        }
    },

    /**
     * Delete ambient sound.  Stop coffee brew.  Stop all audio immediately.  Test case:  2015-01-07 Pregame.  Click Play.  Level select.  I expect no ambient sound.  Got ambient sound fades out.
     */
    stopAll: function()
    {
        ll.log("Sounds.stopAll");
        this.coffeeBrewLoopStop();
        if (this.mixer) {
            this.mixer._settings.ambientSound = undefined;
            this.mixer.stopSound();
        }
        audio.stopMusic();
        audio.stopAllEffects();
    },

    wireSounds: function(sounds)
    {
        Blender.onAddIngredientSelected = Sounds.onAddIngredientSelected;
        Blender.onStart = Sounds.onStart;
        Blender.onAction = Sounds.onAction;
        Blender.onSelect = Sounds.onSelect;
    },
});

Sounds.answers = {
    "answerWrong": "sounds/answerWrong.mp3",
    "answerCorrect": "sounds/answerCorrect.mp3",
    "answerCorrectGood": "sounds/answerCorrectX2.mp3",
    "answerCorrectExcellent": "sounds/answerCorrectX4.mp3"
};

/**
 * GAME-2394 During a serve before overflow, play a standard answer feedback sound.  Test case:  2015-01-07 Serve.  I expect to hear one of four sounds.  Serve immediately, I expect to hear highest sound.  Serve just before overflow, I expect to hear standard correct sound.  If wrong, I expect to hear standard wrong sound.
 */
Sounds.answer = function(state)
{
    audio.playEffect(Sounds.answers[state]);
},

Sounds.createWithMixer = function(controller)
{
    if (!controller._components[ GameComponentAudioMixer.prototype.getType() ]) {
        controller._addComponent( new GameComponentAudioMixer( controller ) );
    }
    var mixer = controller._components[ GameComponentAudioMixer.prototype.getType() ];
    var sounds = new Sounds(mixer);
    return sounds;
}

/**
 * @return  Function to play MP3 sound effect in "sounds" folder with ".mp3" extension.
 *
 * GAME-2467 Refer to sound effects in static sound function instead of sound function factory.  Test case:  2015-01-21 Play.  Hear sound.  iPhone main button.  Restart.  Hear sound.  iPhone main button.  Restart.  Tap ingredient.  Anthony Chaseendran expects to hear a sound.  Got no sound effect.  Did hear background music.
 */
Sounds.play = function(baseFileName)
{
    return audio.playEffect("sounds/" + baseFileName + ".mp3");
};

Sounds.onAddIngredientSelected = function()
{
    Sounds.play("ingredientClick");
};

Sounds.onStart = function()
{
    Sounds.play("coffeeStart");
};

Sounds.onAction = function()
{
    Sounds.play("coffeeStop");
};

Sounds.onSelect = function()
{
    Sounds.play("cupMove");
};
