/*jslint node: true */
"use strict";

require( "src/Utility/LumosityLogger.js" );
require( "src/Game/Base/GameView.js" );
require( "src/Game/Widget/GameWidgetHud.js" );

require( "src/Game/TroubleBrewingModel.js" );

var TroubleBrewingView = GameView.extend(
{
    /**
     * GAME-1874 Override creating dim layer. 2015-01-05 Tutor.  [~wouter] expects to disable standard dim layer.
     */
    createDimLayer: function()
    {
    },

    removeDimLayer: function()
    {
    },

    /**
     * Populate the hud and select the current level.
     * Show pause button if not visible.
     */
    onBegin: function()
    {
        var widget = this.getWidgetByClass(GameWidgetPauseButton);
        this.setScore( this.model.score );
        this.hidePauseButton();
        if (this.model.persistentData.isLevelSelectMode) 
        {
            this.beginPauseButton();
        }
        else if (this.model.persistentData.isPracticeMode) 
        {
            this.cue_hidePauseButton();
        }
    },

    getWidgetByClass: function(GameWidgetClass)
    {
        var widget;
        if (null != this._widgets)
        {
            var type = GameWidgetClass.prototype.getType();
            widget = this._widgets[type];
        }
        return widget;
    },

    /**
     * Tween from pause menu without animation, visually identical to onResume.
     */
    beginPauseButton: function()
    {
        var widget = this.getWidgetByClass(GameWidgetPauseButton);
        if (widget)
        {
            var playAnimation = DisplayState.PauseMenu !== this.getPrevDisplayState();
            widget.showPauseButton(true, playAnimation);
        }
    },

    hidePauseButton: function()
    {
        var widget = this.getWidgetByClass(GameWidgetPauseButton)
        if ( widget )
        {
            widget.showPauseButton(false, false);
        }
    },

    /**
     * GAME-1602 If skip tutorial, remove dim layer.  Test case:  2014-10-28 Skip tutorial.  Expect dim layer disappears. 
     * GAME-1405 Upon completing tutorial, destroy tile buttons.  Test case:  2014-11-04 First tutorial trial. Skip.  Click where spots were.  Expect no response.  Got second tutorial board appears.
     */
    onCompleteTutorial: function(skipped)
    {
        this.stopAllAnimations(skipped);
        this.cue_hideSkipTutorialButton();
    }
});
